/*

	ElectrEm (c) 2000/1 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for
	details

	6502/6502common.cpp
	===================

	The various common parts of both 6502's (the wide and regular ones)

*/
#include "6502.h"
#include "defs.h"
#include <stdlib.h>

E6502_MemoryAccess *C_6502StdBase::style_flags;
E6502_MemoryAccess **C_6502StdBase::types;
bool C_6502StdBase::quit;

C_6502StdBase::C_6502StdBase(void)
{
	allocd = false;
	o_r_mem = o_w_mem = NULL;
	f_r_mem = f_w_mem = NULL;
	style_flags = NULL;
	device = NULL;
	types = NULL;
	OwnedCosts[ONEMhz_BUS] = OwnedCosts[TWOMhz_BUS] = OwnedCosts[RAM] =
	Costs[ONEMhz_BUS] = Costs[TWOMhz_BUS] = Costs[RAM] = NULL;

	int c = 256;
	while(c--) mapped_flags[c] = false;
}

C_6502StdBase::~C_6502StdBase(void)
{
}

void C_6502StdBase::SetContentionTimes(int *ctable)
{
	Costs[RAM]	= ctable;
}

void C_6502StdBase::SetQuit(bool q)
{
	quit = q;
}

void C_6502StdBase::SetCalloutTime(int states)
{
	target = states;
}

void C_6502StdBase::SetCalloutDevice(C_6502CalloutDevice *dev)
{
	device = dev;
}

unsigned __int8 *C_6502StdBase::GetByteReadPtr(BrokenWord addr)
{
	return &o_r_mem[addr.b.h][addr.b.l];
}

unsigned __int32 *C_6502StdBase::GetDWordReadPtr(BrokenWord addr)
{
	return &f_r_mem[addr.b.h][addr.b.l];
}

void C_6502StdBase::SetAddrPtrs(unsigned __int8 start, unsigned __int8 num, unsigned __int8 *base, unsigned __int32 *bbase, bool read, bool write)
{
	while(num--)
	{
		if(read)
		{
			if(base) o_r_mem[start] = base;
			if(bbase) f_r_mem[start] = bbase;
		}

		if(write)
		{
			if(base) o_w_mem[start] = base;
			if(bbase) f_w_mem[start] = bbase;
		}

		start++;
		base += 256;
		bbase += 256;
	}
}

void C_6502StdBase::SetAddrFlags(unsigned __int8 start, unsigned __int8 num, bool device, E6502_MemoryAccess btype)
{
	while(num--)
	{
		mapped_flags[start] = device;
		style_flags[start] = btype;

		start++;
	}
}

//These are specially coded for the ROM paging mechanisms!
void C_6502StdBase::FastSetAddrPtrs(unsigned __int8 start, unsigned __int8 num, unsigned __int8 *read, unsigned __int8 *write)
{
	unsigned __int8 **rptr = &o_r_mem[start];
	unsigned __int8 **wptr = &o_w_mem[start];
	bool *mflag = &mapped_flags[start];

	if(*rptr == read && *wptr == write)
	{
		while(num--)
			*mflag++ = false;
		return;
	}

	while(num--)
	{
		*rptr++ = read;
		read += 256;

		*wptr++ = write;
		write += 256;

		*mflag++ = false;
	}
}

void C_6502StdBase::FastSetAddrFlags(unsigned __int8 start, unsigned __int8 num, bool mapped)
{
	bool *mflag = &mapped_flags[start];

	if(*mflag == mapped) return;

	while(num--)
		*mflag++ = mapped;
}

/*void C_6502StdBase::SetAddrPtrs(unsigned __int8 start, unsigned __int8 num, bool device, E6502_AccessTypeMinor btype, unsigned __int8 *base, unsigned __int32 *bbase, bool read, bool write)
{
	switch((read?1:0) | (write?2:0))
	{
		case 0 :
			while(num--)
			{
				mapped_flags[start] = device;
				style_flags[start] = btype;

				start++;
			}
		break;

		case 1 : //read pointers only
			while(num--)
			{
				if(base) { o_r_mem[start] = base; base += 256; }
				if(bbase) { f_r_mem[start] = bbase; bbase += 256; }
				mapped_flags[start] = device;
				style_flags[start] = btype;

				start++;
			}
		break;

		case 2 : //write pointers only
			while(num--)
			{
				if(base) { o_w_mem[start] = base; base += 256; }
				if(bbase) { f_w_mem[start] = bbase; bbase += 256; }
				mapped_flags[start] = device;
				style_flags[start] = btype;

				start++;
			}
		break;

		case 3 : //read & write pointers
			while(num--)
			{
				if(base) { o_w_mem[start] = o_r_mem[start] = base; base += 256; }
				if(bbase) { f_w_mem[start] = f_r_mem[start] = bbase; bbase += 256; }
				mapped_flags[start] = device;
				style_flags[start] = btype;

				start++;
			}
		break;
	}
}*/

void C_6502StdBase::Reset(void)
{
	pc.b.l = ReadByte(0xfffc);
	pc.b.h = ReadByte(0xfffd);

	SetP(0);
	SetPLoadFlags();
}

void C_6502StdBase::LeechState(C_6502Base *original)
{
	if(original != (C_6502Base *)this)
	{
		int c;

		quit = false;

		/* a couple of housekeeping things */
		target = original->ReadDWord(NMOS6502ADDR_TRGCNT);
		timecounter = original->ReadDWord(NMOS6502ADDR_TIMECNT);
		irq = original->ReadByte(NMOS6502ADDR_IRQLINE) ? true : false;

		/* the registers */
		a = original->ReadByte(NMOS6502ADDR_A);
		SetP(original->ReadByte(NMOS6502ADDR_P));
		x = original->ReadByte(NMOS6502ADDR_X);
		y = original->ReadByte(NMOS6502ADDR_Y);
		s = original->ReadByte(NMOS6502ADDR_S);
		pc.b.l = original->ReadByte(NMOS6502ADDR_PCL);
		pc.b.h = original->ReadByte(NMOS6502ADDR_PCH);

		aBig = original->ReadDWord(NMOS6502ADDR_A);
		LD_CARRY_WD(original->ReadDWord(NMOS6502ADDR_P));
		xBig = original->ReadDWord(NMOS6502ADDR_X);
		yBig = original->ReadDWord(NMOS6502ADDR_Y);

		/* the current OneCost table */
		Costs[RAM] = (int *)original->ReturnTable(NMOS6502TABLE_RAMCOST);

		/* the mmio flags */
		c = 256;
		while(c--)
			mapped_flags[c] = original->ReadByte(NMOS6502ADDR_MAPFLAGBASE + c) ? true : false;
	}
}

void C_6502StdBase::LeechTables(C_6502Base *original)
{
	o_r_mem = (unsigned __int8 **)original->ReturnTable(NMOS6502TABLE_R8PTRS);
	o_w_mem = (unsigned __int8 **)original->ReturnTable(NMOS6502TABLE_W8PTRS);
	f_r_mem = (unsigned __int32 **)original->ReturnTable(NMOS6502TABLE_R32PTRS);
	f_w_mem = (unsigned __int32 **)original->ReturnTable(NMOS6502TABLE_W32PTRS);
//	style_flags = (E6502_AccessTypeMinor *)original->ReturnTable(NMOS6502TABLE_STFLGS);
//	types = (E6502_MemoryAccess **)original->ReturnTable(NMOS6502TABLE_OPTYPES);
	device = (C_6502CalloutDevice *)original->ReturnTable(NMOS6502TABLE_COUTDEV);

	Costs[TWOMhz_BUS] = (int *)original->ReturnTable(NMOS6502TABLE_2COST);
	Costs[ONEMhz_BUS] = (int *)original->ReturnTable(NMOS6502TABLE_1COST);
}

void *C_6502StdBase::ReturnTable(int tabid)
{
	switch(tabid)
	{
		case NMOS6502TABLE_R8PTRS : return (void *)o_r_mem;
		case NMOS6502TABLE_W8PTRS : return (void *)o_w_mem;
		case NMOS6502TABLE_R32PTRS : return (void *)f_r_mem;
		case NMOS6502TABLE_W32PTRS : return (void *)f_w_mem;
//		case NMOS6502TABLE_STFLGS : return (void *)style_flags;
//		case NMOS6502TABLE_OPTYPES : return (void *)types;
		case NMOS6502TABLE_COUTDEV : return (void *)device;

		case NMOS6502TABLE_1COST : return (void *)Costs[ONEMhz_BUS];
		case NMOS6502TABLE_2COST : return (void *)Costs[TWOMhz_BUS];
		case NMOS6502TABLE_RAMCOST : return (void *)Costs[RAM];
	}

	return (void *)NULL;
}

#define setup_table(x) (E6502_MemoryAccess *)malloc(sizeof(E6502_MemoryAccess)*x)

#define enter_table(x) ptr = types[x]
#define insert_access(direction, location)	*ptr++ = location;

#define UNKNOWN	TWOMhz_BUS			//just because something has to be stored
#define VECTOR	style_flags[0xff]
#define STACK	style_flags[0x01]
#define ZERO	style_flags[0x00]

void C_6502StdBase::ReconsiderTiming(void)
{
	/*

		This function pre-stores the execution timing for cycles which always
		occur on the same page.

		Those cycles are really just zero page accesses, stack page accesses,
		and ff page accesses (e.g. IRQ vector reads

		Unknown values are filled in by the CPU core during instruction
		emulation

	*/

	E6502_MemoryAccess *ptr;

	enter_table(BRK);
		insert_access(read_op,		UNKNOWN);	//fetch opcode, inc PC
		insert_access(read_op,		UNKNOWN);	//fetch 'operand' and ignore, inc PC
		insert_access(write_op,		STACK);		//push PCH, dec s
		insert_access(write_op,		STACK);		//push PCL, dec s
		insert_access(write_op,		STACK);		//push P (with B set), dec S
		insert_access(read_op,		VECTOR);	//fetch new PCL
		insert_access(read_op,		VECTOR);	//fetch new PCH

	enter_table(RTI);
		insert_access(read_op,		UNKNOWN);	//fetch opcode, inc pc
		insert_access(read_op,		UNKNOWN);	//fetch 'operand' & ignore
		insert_access(read_op,		STACK);		//inc S
		insert_access(read_op,		STACK);		//pull P, inc S
		insert_access(read_op,		STACK);		//pull PCL, inc S
		insert_access(read_op,		STACK);		//pull PCH

	enter_table(RTS);
		insert_access(read_op,		UNKNOWN);	//fetch opcode, inc PC
		insert_access(read_op,		UNKNOWN);	//fetch 'operand' & ignore
		insert_access(read_op,		STACK);		//inc s
		insert_access(read_op,		STACK);		//pull PCL, inc s
		insert_access(read_op,		STACK);		//pull PCH
		insert_access(read_op,		UNKNOWN);	//inc PC

	enter_table(PHx);
		insert_access(read_op,		UNKNOWN);	//fetch opcode, inc PC
		insert_access(read_op,		UNKNOWN);	//fetch 'operand' & ignore
		insert_access(write_op,		STACK);		//push register, dec s

	enter_table(PLx);
		insert_access(read_op,		UNKNOWN);	//fetch opcode, inc PC
		insert_access(read_op,		UNKNOWN);	//fetch 'operand' & ignore
		insert_access(read_op,		STACK);		//inc s
		insert_access(read_op,		STACK);		//pull register

	enter_table(JSR);
		insert_access(read_op,		UNKNOWN);	//fetch opcode, inc PC
		insert_access(read_op,		UNKNOWN);	//fetch low byte, inc PC
		insert_access(read_op,		STACK);		//unknown (predec s?)
		insert_access(write_op,		STACK);		//push PCH, dec s
		insert_access(write_op,		STACK);		//push PCL, dec s
		insert_access(read_op,		UNKNOWN);	//fetch high byte, set new PC

	enter_table(IMPLIED);
		insert_access(read_op,		UNKNOWN);	//fetch opcode, inc pc
		insert_access(read_op,		UNKNOWN);	//fetch 'operand' & ignore

	enter_table(IMMEDIATE);
		insert_access(read_op,		UNKNOWN);	//fetch opcode, inc pc
		insert_access(read_op,		UNKNOWN);	//fetch operand, inc pc

	enter_table(ABS_JMP);
		insert_access(read_op,		UNKNOWN);	//fetch opcode, inc pc
		insert_access(read_op,		UNKNOWN);	//fetch low byte, inc pc
		insert_access(read_op,		UNKNOWN);	//fetch high byte, set pc

	enter_table(ABS_READ);
		insert_access(read_op,		UNKNOWN);	//fetch opcode, inc pc
		insert_access(read_op,		UNKNOWN);	//fetch low byte, inc pc
		insert_access(read_op,		UNKNOWN);	//fetch high byte, inc pc
		insert_access(read_op,		UNKNOWN);	//fetch from address

	enter_table(ABS_MODIFY);
		insert_access(read_op,		UNKNOWN);	//fetch opcode, inc pc
		insert_access(read_op,		UNKNOWN);	//fetch low byte, inc pc
		insert_access(read_op,		UNKNOWN);	//fetch high byte, inc pc
		insert_access(read_op,		UNKNOWN);	//fetch value
		insert_access(write_op,		UNKNOWN);	//write value back, modify value
		insert_access(write_op,		UNKNOWN);	//write new value back

	enter_table(ABS_WRITE);
		insert_access(read_op,		UNKNOWN);	//fetch opcode, inc pc
		insert_access(read_op,		UNKNOWN);	//fetch low byte
		insert_access(read_op,		UNKNOWN);	//fetch high byte
		insert_access(write_op,		UNKNOWN);	//write to address

	enter_table(ZERO_READ);
		insert_access(read_op,		UNKNOWN);	//fetch opcode, inc pc
		insert_access(read_op,		UNKNOWN);	//fetch address, inc pc
		insert_access(read_op,		ZERO);		//read from address

	enter_table(ZERO_MODIFY);
		insert_access(read_op,		UNKNOWN);	//fetch opcode, inc pc
		insert_access(read_op,		UNKNOWN);	//fetch address, inc pc
		insert_access(read_op,		ZERO);		//read value from address
		insert_access(write_op,		ZERO);		//write value back, modify value
		insert_access(write_op,		ZERO);		//write new value back

	enter_table(ZERO_WRITE);
		insert_access(read_op,		UNKNOWN);	//fetch opcode, inc pc
		insert_access(read_op,		UNKNOWN);	//fetch address, inc pc
		insert_access(write_op,		ZERO);		//write to address

	enter_table(ZIND_READ);
		insert_access(read_op,		UNKNOWN);	//fetch opcode, inc pc
		insert_access(read_op,		UNKNOWN);	//fetch address, inc pc
		insert_access(read_op,		ZERO);		//read from address, add index register
		insert_access(read_op,		ZERO);		//read from total address

	enter_table(ZIND_MODIFY);
		insert_access(read_op,		UNKNOWN);	//fetch opcode, inc pc
		insert_access(read_op,		UNKNOWN);	//fetch address, inc pc
		insert_access(read_op,		ZERO);		//read from address, add index
		insert_access(read_op,		ZERO);		//read value from total address
		insert_access(write_op,		ZERO);		//write value to total address, modify it
		insert_access(write_op,		ZERO);		//write new value to total address

	enter_table(ZIND_WRITE);
		insert_access(read_op,		UNKNOWN);	//fetch opcode, inc pc
		insert_access(read_op,		UNKNOWN);	//fetch address, inc pc
		insert_access(read_op,		ZERO);		//read from address, add index
		insert_access(write_op,		ZERO);		//store to total address

	enter_table(AIND_READ);
		insert_access(read_op,		UNKNOWN);	//fetch opcode, inc pc
		insert_access(read_op,		UNKNOWN);	//fetch low byte, inc pc
		insert_access(read_op,		UNKNOWN);	//fetch high byte, add index to low byte, inc pc
		insert_access(read_op,		UNKNOWN);	//read from total address. Fix high byte
		/* CONDITIONAL */
		insert_access(read_op,		UNKNOWN);	//reread from total adress if high byte changed

	enter_table(AIND_MODIFY);
		insert_access(read_op,		UNKNOWN);	//fetch opcode, inc pc
		insert_access(read_op,		UNKNOWN);
		insert_access(read_op,		UNKNOWN);
		insert_access(read_op,		UNKNOWN);
		insert_access(read_op,		UNKNOWN);
		insert_access(write_op,		UNKNOWN);
		insert_access(write_op,		UNKNOWN);

	enter_table(AIND_WRITE);
		insert_access(read_op,		UNKNOWN);	//fetch opcode, inc pc
		insert_access(read_op,		UNKNOWN);
		insert_access(read_op,		UNKNOWN);
		insert_access(read_op,		UNKNOWN);
		insert_access(write_op,		UNKNOWN);

	enter_table(REL);
		insert_access(read_op,		UNKNOWN);	//fetch opcode, inc pc
		insert_access(read_op,		UNKNOWN);	//fetch operand, inc pc
		/* CONDITIONAL */
		insert_access(read_op,		UNKNOWN);	//if branch taken, add low byte
		/* DOUBLE CONDITIONAL */
		insert_access(read_op,		UNKNOWN);	//fix high byte if necessary

	enter_table(INDI_READ);
		insert_access(read_op,		UNKNOWN);	//fetch opcode, inc pc
		insert_access(read_op,		UNKNOWN);	//fetch pointer, inc pc
		insert_access(read_op,		ZERO);		//read from pointer, add x
		insert_access(read_op,		ZERO);		//read address high byte, inc pointer
		insert_access(read_op,		ZERO);		//read address low byte
		insert_access(read_op,		UNKNOWN);	//read from address

	enter_table(INDI_MODIFY);
		insert_access(read_op,		UNKNOWN);	//fetch opcode, inc pc
		insert_access(read_op,		UNKNOWN);	//fetch pointer, inc pc
		insert_access(read_op,		ZERO);		//read from pointer, add x
		insert_access(read_op,		ZERO);		//read address low byte, inc pointer
		insert_access(read_op,		ZERO);		//read address high byte
		insert_access(read_op,		UNKNOWN);	//read from address
		insert_access(write_op,		UNKNOWN);	//write back to address, do operation
		insert_access(write_op,		UNKNOWN);	//write to address

	enter_table(INDI_WRITE);
		insert_access(read_op,		UNKNOWN);	//fetch opcode, inc pc
		insert_access(read_op,		UNKNOWN);	//fetch pointer, inc pc
		insert_access(read_op,		ZERO);		//read from pointer, add x
		insert_access(read_op,		ZERO);		//read address low byte from pointer, inc pointer
		insert_access(read_op,		ZERO);		//read address high byte
		insert_access(write_op,		UNKNOWN);	//write to address

	enter_table(IIND_READ);
		insert_access(read_op,		UNKNOWN);	//fetch opcode, inc pc
		insert_access(read_op,		UNKNOWN);	//fetch pointer, inc pc
		insert_access(read_op,		ZERO);		//read low byte from pointer, inc pointer
		insert_access(read_op,		ZERO);		//read high byte from pointer, add y to low byte
		insert_access(read_op,		UNKNOWN);	//read from address
		/* CONDITIONAL */
		insert_access(read_op,		UNKNOWN);	//read again is high byte was incorrect

	enter_table(IIND_MODIFY);
		insert_access(read_op,		UNKNOWN);	//fetch opcode, inc pc
		insert_access(read_op,		UNKNOWN);	//fetch pointer, inc pc
		insert_access(read_op,		ZERO);		//read low byte from pointer, inc pointer
		insert_access(read_op,		ZERO);		//read high byte from pointer, add y to low byte
		insert_access(read_op,		UNKNOWN);	//read from address, fix high byte
		insert_access(read_op,		UNKNOWN);	//read from address
		insert_access(write_op,		UNKNOWN);	//write back to address, do operation
		insert_access(write_op,		UNKNOWN);	//write back to address

	enter_table(IIND_WRITE);
		insert_access(read_op,		UNKNOWN);	//fetch opcode, inc pc
		insert_access(read_op,		UNKNOWN);	//fetch pointer, inc pc
		insert_access(read_op,		ZERO);		//fetch address low byte
		insert_access(read_op,		ZERO);		//fetch address high byte, add y to low
		insert_access(read_op,		UNKNOWN);	//read from address, fix high byte
		insert_access(write_op,		UNKNOWN);	//write to address

	enter_table(ABSOLUTE_IND);
		insert_access(read_op,		UNKNOWN);	//fetch opcode, inc pc
		insert_access(read_op,		UNKNOWN);	//fetch pointer low, inc pc
		insert_access(read_op,		UNKNOWN);	//fetch pointer high, inc pc
		insert_access(read_op,		UNKNOWN);	//fetch address low byte, inc pointer low byte
		insert_access(read_op,		UNKNOWN);	//fetch address high byte

	/* and this one is a guess! */

	enter_table(IRQ);
		insert_access(read_op,		UNKNOWN);	//	???
		insert_access(read_op,		UNKNOWN);	//	???
		insert_access(write_op,		STACK);		//push PCH, dec s
		insert_access(write_op,		STACK);		//push PCL, dec s
		insert_access(write_op,		STACK);		//push P, dec S
		insert_access(read_op,		VECTOR);	//fetch new PCL
		insert_access(read_op,		VECTOR);	//fetch new PCH
}

bool C_6502StdBase::Setup(bool newmem)
{
	int c;

	allocd = newmem;

	if(newmem)
	{
		timecounter = 0;

		OwnedCosts[ONEMhz_BUS] = Costs[ONEMhz_BUS] = (int *)malloc(sizeof(int)*NMOS6502_COSTLENGTH);
		OwnedCosts[TWOMhz_BUS] = Costs[TWOMhz_BUS] = (int *)malloc(sizeof(int)*NMOS6502_COSTLENGTH);
		OwnedCosts[RAM] = Costs[RAM] = (int *)malloc(sizeof(int)*NMOS6502_COSTLENGTH);

		c = NMOS6502_COSTLENGTH;
		while(c--)
		{
			OwnedCosts[RAM][c] =
			OwnedCosts[TWOMhz_BUS][c] = 1;
			OwnedCosts[ONEMhz_BUS][c] = 2 | (c&1);
		}

		o_r_mem = (unsigned __int8 **)malloc(sizeof(unsigned __int8 *)*256);
		o_w_mem = (unsigned __int8 **)malloc(sizeof(unsigned __int8 *)*256);
		style_flags = (E6502_MemoryAccess *)malloc(sizeof(E6502_MemoryAccess)*256);

		types = (E6502_MemoryAccess **)malloc(sizeof(E6502_MemoryAccess *)*NMOS6502_NUMTYPES);

		types[BRK]			= setup_table(7);
		types[RTI]			= setup_table(6);
		types[RTS]			= setup_table(6);
		types[PHx]			= setup_table(3);
		types[PLx]			= setup_table(4);
		types[JSR]			= setup_table(6);
		types[IMPLIED]		= setup_table(2);
		types[IMMEDIATE]	= setup_table(2);
		types[ABS_JMP]		= setup_table(3);
		types[ABS_READ]		= setup_table(4);
		types[ABS_MODIFY]	= setup_table(6);
		types[ABS_WRITE]	= setup_table(4);
		types[ZERO_READ]	= setup_table(3);
		types[ZERO_MODIFY]	= setup_table(5);
		types[ZERO_WRITE]	= setup_table(3);
		types[ZIND_READ]	= setup_table(4);
		types[ZIND_MODIFY]	= setup_table(6);
		types[ZIND_WRITE]	= setup_table(4);
		types[AIND_READ]	= setup_table(5);
		types[AIND_MODIFY]	= setup_table(7);
		types[AIND_WRITE]	= setup_table(5);
		types[REL]			= setup_table(4);
		types[INDI_READ]	= setup_table(6);
		types[INDI_MODIFY]	= setup_table(8);
		types[INDI_WRITE]	= setup_table(6);
		types[IIND_READ]	= setup_table(6);
		types[IIND_MODIFY]	= setup_table(8);
		types[IIND_WRITE]	= setup_table(6);
		types[ABSOLUTE_IND]	= setup_table(5);
		types[IRQ]			= setup_table(7);

		ReconsiderTiming();

		f_r_mem = (unsigned __int32 **)malloc(sizeof(unsigned __int32 *)*256);
		f_w_mem = (unsigned __int32 **)malloc(sizeof(unsigned __int32 *)*256);
	}
	quit = false;

	return true;
}

#undef ROM

#undef end_table
#undef insert_access
#undef setup_table

void C_6502StdBase::CloseDown(void)
{
	if(allocd)
	{
		int c;

		c = NMOS6502_NUMTYPES;
		while(c--)
		{
			free(types[c]);
		}
		free(types);

		free(style_flags);
		free(o_w_mem);
		free(o_r_mem);
		free(f_r_mem);
		free(f_w_mem);

		free(OwnedCosts[ONEMhz_BUS]);
		free(OwnedCosts[TWOMhz_BUS]);
		free(OwnedCosts[RAM]);
	}
}

void C_6502StdBase::Push(unsigned __int16 addr)
{
	BrokenWord tmp;
	tmp.a = addr;

	o_w_mem[1][s] = tmp.b.h; s--;
	o_w_mem[1][s] = tmp.b.l; s--;
}

void C_6502StdBase::ClearIRQ(void)
{
	irq = false;
}

void C_6502StdBase::WriteByte(unsigned __int32 addr, unsigned __int8 value)
{
	BrokenWord laddr;
	unsigned int bigcounter = 0;

	if(addr < 0x10000)
	{
		laddr.a = (unsigned __int16)addr;
		o_w_mem[laddr.b.h][laddr.b.l] = value;
	}
	else
	{
		switch(addr)
		{
			case NMOS6502ADDR_A :	a = value; break;
			case NMOS6502ADDR_P :	SetP(value); break;
			case NMOS6502ADDR_X :	x = value; break;
			case NMOS6502ADDR_Y :	y = value; break;
			case NMOS6502ADDR_S :	s = value; break;
			case NMOS6502ADDR_PCL :	pc.b.l = value; break;
			case NMOS6502ADDR_PCH :	pc.b.h = value; break;
		}
	}
}

void C_6502StdBase::WriteDWord(unsigned __int32 addr, unsigned __int32 value)
{
	BrokenWord laddr;
	unsigned int bigcounter = 0;

	if(addr < 0x10000)
	{
		laddr.a = (unsigned __int16)addr;
		f_w_mem[laddr.b.h][laddr.b.l] = value;
	}
	else
	{
		switch(addr)
		{
			case NMOS6502ADDR_A :	aBig = value; break;
			case NMOS6502ADDR_P :	LD_CARRY_WD(value); break;
			case NMOS6502ADDR_X :	xBig = value; break;
			case NMOS6502ADDR_Y :	yBig = value; break;
		}
	}
}


unsigned __int8 C_6502StdBase::ReadByte(unsigned __int32 addr)
{
	BrokenWord laddr;
	unsigned int bigcounter = 0;

	if(addr < 0x10000)
	{
		laddr.a = (unsigned __int16)addr;
		return o_r_mem[laddr.b.h][laddr.b.l];
	}

	if(addr >= NMOS6502ADDR_MAPFLAGBASE && addr < (NMOS6502ADDR_MAPFLAGBASE+256))
		return mapped_flags[addr-NMOS6502ADDR_MAPFLAGBASE] ? 1 : 0;

	switch(addr)
	{
		case NMOS6502ADDR_A :		return a;
		case NMOS6502ADDR_P :		return PStatus();
		case NMOS6502ADDR_X :		return x;
		case NMOS6502ADDR_Y :		return y;
		case NMOS6502ADDR_S :		return s;
		case NMOS6502ADDR_PCL :		return pc.b.l;
		case NMOS6502ADDR_PCH :		return pc.b.h;

		case NMOS6502ADDR_IRQLINE : return irq ? 1 : 0;
	}

	return 0;
}

unsigned __int32 C_6502StdBase::ReadDWord(unsigned __int32 addr)
{
	BrokenWord laddr;
	unsigned int bigcounter = 0;

	if(addr < 0x10000)
	{
		laddr.a = (unsigned __int16)addr;
		return f_r_mem[laddr.b.h][laddr.b.l];
	}
	else
	{
		switch(addr)
		{
			case NMOS6502ADDR_A :		return aBig;
			case NMOS6502ADDR_P :		return RD_CARRY_WD();
			case NMOS6502ADDR_X :		return xBig;
			case NMOS6502ADDR_Y :		return yBig;
			case NMOS6502ADDR_S :		return s;
			case NMOS6502ADDR_PCL :		return pc.b.l;
			case NMOS6502ADDR_PCH :		return pc.b.h;

			case NMOS6502ADDR_TRGCNT : return target;
			case NMOS6502ADDR_TIMECNT : return timecounter;
		}
	}

	return 0;
}

