#ifndef DEFS_H
#define DEFS_H

#include <stdio.h>

/* carry flag(s) */
#define RD_CARRY()					carry
#define RD_CARRY_WD()				carryBig
#define LD_CARRY(v)					carry = v
#define LD_CARRY_WD(v)				carryBig = v

/* negative flag */
#define RD_NEG()					(neg&0x80)
#define LD_NEG(v)					neg = v

/* overflow flag */
#define RD_OVERFLOW()				(over&0x80)
#define LD_OVERFLOW(v)				over = v

/* zero flag (set inverted) */
#define RD_ZERO()					zero
#define LD_ZERO(v)					zero = v

#define LD_NEGZERO(v)				neg = zero = v

#define PStatus()					(RD_NEG() ?			NMOS6502FLAG_N : 0) | \
									(RD_ZERO() ?		0 : NMOS6502FLAG_Z) | \
									(RD_OVERFLOW() ?	NMOS6502FLAG_V : 0) | \
									RD_CARRY() | \
									pmisc;

#define SetPLoadFlags()				pmisc |= NMOS6502_PLOADFLAGS;

#define SetP(v)	{	\
	LD_NEG(v&NMOS6502FLAG_N); \
	LD_ZERO((v&NMOS6502FLAG_Z)^NMOS6502FLAG_Z); \
	LD_OVERFLOW((v&NMOS6502FLAG_V)<<1); \
	LD_CARRY(v&NMOS6502FLAG_C); \
	pmisc =	v&(NMOS6502FLAG_D | NMOS6502FLAG_I | NMOS6502FLAG_B | NMOS6502FLAG_A);\
	}

#define Read(addr, val)				mapped_flags[addr.b.h] ? device->ReadFrom(addr, &val, NULL, bigcounter) : (val = o_r_mem[addr.b.h][addr.b.l]);
#define Write(addr, val)			mapped_flags[addr.b.h] ? device->WriteTo(addr, val, 0, bigcounter) : (o_w_mem[addr.b.h][addr.b.l] = val);
#define Modify(addr, val, action)	if(mapped_flags[addr.b.h]){ device->ReadFrom(addr, &val, NULL, bigcounter); action; device->WriteTo(addr, val, 0, bigcounter); } else { val = o_r_mem[addr.b.h][addr.b.l]; action; o_r_mem[addr.b.h][addr.b.l] = val;}

#define StackRead(addr, val)		val = o_r_mem[0x01][addr];
#define StackWrite(addr, val)		o_w_mem[0x01][addr] = val;
#define ZeroRead(addr, val)			val = o_r_mem[0x00][addr];
#define ZeroWrite(addr, val)		o_w_mem[0x00][addr] = val;
#define ZeroModify(addr, val, action)	val = o_r_mem[0][addr]; action; o_w_mem[0][addr] = val;

#define set_part(addr, cycle)		lblock[cycle] = style_flags[addr.b.h];
#define set_2part(addr, cycle)		lblock[cycle] = lblock[cycle+1] = style_flags[addr.b.h];
#define set_3part(addr, cycle)		lblock[cycle] = lblock[cycle+1] = lblock[cycle+2] = style_flags[addr.b.h];
#define set_4part(addr, cycle)		lblock[cycle] = lblock[cycle+1] = lblock[cycle+2] = lblock[cycle+3] = style_flags[addr.b.h];

#define set_vtype(x)				lblock = types[x]; set_part(pc, 0); pc.a++; set_part(pc, 1);
#define set_type(x, y)				set_vtype(x); len = y;

#define push(v)						StackWrite(s, v); s--;
#define pull(v)						s++; StackRead(s, v);

#define set_jsr()					set_type(JSR,6);
#define set_rts()					set_type(RTS,6);
#define set_brk()					set_type(BRK,7);
#define set_rti()					set_type(RTI,6);

#define set_jmp_indirect()			set_type(ABSOLUTE_IND,5);
#define set_jmp_absolute()			set_type(ABS_JMP,3);

#define cond_branch(v)				set_vtype(REL); Read(pc, temp16.b.l); pc.a++; if(v) {set_2part(pc, 2); temp16.b.h = (temp16.b.l&128) ? 0xff : 0; len = (((pc.a+temp16.a)^pc.a)&0xff00) ? 4 : 3; pc.a += temp16.a;} else len = 2;

#define set_nz(v)					LD_NEGZERO(v)

#define adc_dec()	\
		temp16.b.l = (a&0x0f) + (mdr&0x0f) + RD_CARRY();\
\
		temp8 = a + mdr + RD_CARRY();\
		LD_ZERO(temp8);\
\
		if(temp16.b.l > 9) temp16.b.l += 6;\
		temp16.b.h = (a >> 4) + (mdr >> 4) + (temp16.b.l > 15);\
\
		LD_NEG(temp16.b.h<<4);\
		LD_OVERFLOW((a^temp8) & ~(a^mdr));\
\
		if(temp16.b.h > 9) temp16.b.h += 6;\
\
		LD_CARRY((temp16.b.h > 15) ? 1 : 0);\
		a = (temp16.b.h << 4) | (temp16.b.l&15);

#define sbc_dec()	\
		temp16.b.l = (a&15) - (mdr&15) - (RD_CARRY()^1);\
		if(temp16.b.l&0x10) temp16.b.l -= 6;\
\
		temp16.b.h = (a >> 4) - (mdr >> 4) - ((temp16.b.l&0x10) ? 1 : 0);\
		if(temp16.b.h&0x10) temp16.b.h -= 6;\
\
		temp8 = (temp16.b.h << 4) | (temp16.b.l&15);\
\
		mdr = ~mdr;\
		temp16.a = a + mdr + RD_CARRY();\
\
		LD_NEGZERO(temp16.b.l);\
		LD_CARRY(temp16.b.h ? 1 : 0);\
		LD_OVERFLOW((a^temp16.b.l) & ~(a^mdr));\
\
		a = temp8;

#define adc_bin()	\
		temp16.a = a + mdr + RD_CARRY();\
\
		LD_NEGZERO(temp16.b.l);\
		LD_CARRY(temp16.b.h);\
		LD_OVERFLOW((a^temp16.b.l) & ~(a^mdr));\
\
		a = temp16.b.l;

#define adc()	\
	\
	if(pmisc&NMOS6502FLAG_D)\
	{\
		adc_dec();\
	}\
	else\
	{\
		adc_bin();\
	}

#define sbc()	\
	\
	if(pmisc&NMOS6502FLAG_D)\
	{\
		sbc_dec();\
	}\
	else\
	{\
		mdr = ~mdr;\
		adc_bin();\
	}

#define cp(r, v)	\
temp16.a = r - v;\
LD_NEGZERO(temp16.b.l);\
LD_CARRY(temp16.b.h ? 0 : 1);

#define do_timing()\
		count = counter;\
		while(len--)\
		{\
			counter += Costs[*lblock][counter];\
			lblock++;\
		}\
\
		bigcounter = (bigcounter+counter-count)&NMOS6502_TIMEMASK;\
\
		if(counter >= target)\
		{\
			counter -= target;\
\
			if(device->Keepalive(bigcounter))\
			{\
				device->MinimiseTimer(bigcounter);\
				bigcounter = 0;\
			}\
		}\

#define std_cpu_end()	\
		do_timing();\
\
		if(irqe)\
		{\
			irqe = false;\
\
			len = 7;\
			lblock = types[IRQ];\
			lblock[0] = lblock[1] = style_flags[pc.b.h];\
\
			push(pc.b.h);\
			push(pc.b.l);\
			pmisc &= ~NMOS6502FLAG_B;\
			temp8 = PStatus();\
			push(temp8);\
			pmisc |= NMOS6502FLAG_B | NMOS6502FLAG_I;\
\
			temp16.a = 0xfffe;\
			Read(temp16, pc.b.l);\
			temp16.a++;\
			Read(temp16, pc.b.h);\
\
			do_timing();\
		}


#define check_irq()	\
	if(irq && !(pmisc&NMOS6502FLAG_I))\
	{\
		irqe = true;\
	}

#endif
