/*

	ElectrEm (c) 2000 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for
	details

	6502/regular/control.cpp
	========================

	The 6502 core + IRQ line things + extended address space byte read /
	writes + stack push thing [useful for the vDFS & fast tape support
	and anything else that wants to control program flow via virtual
	opcodes]

*/
#include "regular.h"
#include "../defs.h"

#define set_immediate()				set_type(IMMEDIATE,2); Read(pc, mdr); pc.a++;
#define set_implied()				set_type(IMPLIED,2);

#define set_absolute_read()			set_type(ABS_READ,4); Read(pc, mar.b.l); pc.a++; set_part(pc, 2); Read(pc, mar.b.h); pc.a++; set_part(mar, 3); Read(mar, mdr);
#define set_absolute_modify()		set_type(ABS_MODIFY,6); Read(pc, mar.b.l); pc.a++; set_part(pc, 2); Read(pc, mar.b.h); pc.a++; set_3part(mar, 3);
#define set_absolute_write()		set_type(ABS_WRITE,4); Read(pc, mar.b.l); pc.a++; set_part(pc, 2); Read(pc, mar.b.h); pc.a++; set_part(mar, 3);

#define set_zero_read()				set_type(ZERO_READ,3); Read(pc, mar.b.l); pc.a++; ZeroRead(mar.b.l, mdr);
#define set_zero_modify()			set_type(ZERO_MODIFY,5); Read(pc, mar.b.l); pc.a++;
#define set_zero_write()			set_type(ZERO_WRITE,3); Read(pc, mar.b.l); pc.a++;

#define set_zero_i_read(v)			set_type(ZIND_READ,4); Read(pc, mar.b.l); pc.a++; mar.b.l += v; ZeroRead(mar.b.l, mdr);
#define set_zero_i_modify(v)		set_type(ZIND_MODIFY,6); Read(pc, mar.b.l); pc.a++; mar.b.l += v;
#define set_zero_i_write(v)			set_type(ZIND_WRITE,4); Read(pc, mar.b.l); pc.a++; mar.b.l += v;

#define set_absolute_i_read(v)		set_vtype(AIND_READ); Read(pc, mar.b.l); pc.a++; Read(pc, mar.b.h); set_part(pc, 2); pc.a++; set_part(mar, 3); len = (((mar.a+v)^mar.a) & 0xff00) ? 5 : 4; mar.a += v; set_part(mar, 4); Read(mar, mdr);
#define set_absolute_i_modify(v)	set_type(AIND_MODIFY,7); Read(pc, mar.b.l); pc.a++; set_part(pc, 2); Read(pc, mar.b.h); pc.a++; set_part(mar, 3); mar.a += v; set_3part(mar, 4);
#define set_absolute_i_write(v)		set_type(AIND_WRITE,5); Read(pc, mar.b.l); pc.a++; set_part(pc, 2); Read(pc, mar.b.h); pc.a++; set_part(mar, 3); mar.a += v; set_part(mar, 4);

//only used with x
#define set_b_zero_i_b_read(v)		set_type(INDI_READ,6); Read(pc, mdr); pc.a++; mdr += v; ZeroRead(mdr, mar.b.l); mdr++; ZeroRead(mdr, mar.b.h); Read(mar, mdr); set_part(mar, 5);
#define set_b_zero_i_b_modify(v)	set_type(INDI_MODIFY,8); Read(pc, mdr); pc.a++; mdr += v; ZeroRead(mdr, mar.b.l); mdr++; ZeroRead(mdr, mar.b.h); set_3part(mar, 5);
#define set_b_zero_i_b_write(v)		set_type(INDI_WRITE,6); Read(pc, mdr); pc.a++; mdr += v; ZeroRead(mdr, mar.b.l); mdr++; ZeroRead(mdr, mar.b.h); set_part(mar, 5);

//only used with y
#define set_b_zero_b_i_read(v)		set_vtype(IIND_READ); Read(pc, mdr); pc.a++; ZeroRead(mdr, mar.b.l); mdr++; ZeroRead(mdr, mar.b.h); set_part(mar, 4); len = (((mar.a+v)^mar.a) & 0xff00) ? 6 : 5; mar.a += v; set_part(mar, 5); Read(mar, mdr);
#define set_b_zero_b_i_modify(v)	set_type(IIND_MODIFY,8); Read(pc, mdr); pc.a++; ZeroRead(mdr, mar.b.l); mdr++; ZeroRead(mdr, mar.b.h); set_part(mar, 4); mar.a += v; set_3part(mar, 5);
#define set_b_zero_b_i_write(v)		set_type(IIND_WRITE,6); Read(pc, mdr); pc.a++; ZeroRead(mdr, mar.b.l); mdr++; ZeroRead(mdr, mar.b.h); set_part(mar, 4); mar.a += v; set_part(mar, 5);

#define push_instr(v)				StackWrite(s, v); s--; set_type(PHx,3);
#define pull_instr(v)				s++; StackRead(s, v); set_type(PLx,4);

#define ror(v)	\
temp8 = (v >> 1) | (RD_CARRY() << 7);\
LD_NEGZERO(temp8);\
LD_CARRY(v&1);\
v = temp8;

#define rol(v)	\
temp16.a = (v << 1) | RD_CARRY();\
LD_NEGZERO(temp16.b.l);\
LD_CARRY(temp16.b.h);\
v = temp16.b.l;

#define lsr(v)	\
LD_CARRY(v&1);\
v >>= 1;\
LD_NEGZERO(v);

#define asl(v)	\
LD_CARRY(v >> 7);\
v <<= 1;\
LD_NEGZERO(v);

#define bit()	\
	LD_ZERO(a&mdr);\
	LD_OVERFLOW((mdr&NMOS6502FLAG_V) << 1);\
	LD_NEG(mdr&NMOS6502FLAG_N);

#define ora()	a |= mdr; set_nz(a);
#define and()	a &= mdr; set_nz(a);
#define eor()	a ^= mdr; set_nz(a);
#define dec(v)	v--; set_nz(v);
#define inc(v)	v++; set_nz(v);

#define slo()	LD_CARRY(mdr>>7);\
				mdr <<= 1;\
				a |= mdr;\
				LD_NEGZERO(a);

#define rla()	temp16.a = (mdr << 1) | RD_CARRY();\
				LD_CARRY(temp16.b.h);\
				mdr = temp16.b.l;\
				a &= mdr;\
				LD_NEGZERO(a);

#define sre()	LD_CARRY(mdr&1);\
				mdr >>= 1;\
				a ^= mdr;\
				LD_NEGZERO(a);

#define sax()	mdr = a&x;

#define anc()	a&=mdr;\
				LD_NEGZERO(a);\
				LD_CARRY(a >> 7);

#include <ctype.h>

void C_6502Regular::Go(void)
{
	unsigned __int8 instr;
	int counter = 0, bigcounter = timecounter;
	unsigned __int8 temp8, mdr;
	BrokenWord temp16, mar;
	int len, count;
	E6502_MemoryAccess *lblock;

	irqe = false;

	while(!quit)
	{
		Read(pc, instr);

		switch(instr)
		{
			#include "../coredefs.h"

/*			default :
				if(device->TrapOpcode(instr))
				{
					set_immediate();
					device->VirtualOpcode(instr, mdr);

					pull(pc.b.l);
					pull(pc.b.h);
					pc.a++;
				}
				else
				{
					set_implied();
					device->UnknownOpcode(instr);
				}
			break;

			case 0x02 :
			case 0x12 :
			case 0x22 :
			case 0x32 :
			case 0x42 :
			case 0x52 :
			case 0x62 :
			case 0x72 :
			case 0x92 :
			case 0xb2 :
			case 0xd2 :
			case 0xf2 :
				set_implied();
				device->KillOpcode(instr);
			break;

			case 0x6c : //JMP (addr)
				set_jmp_indirect();

				Read(pc, mar.b.l); pc.a++; set_part(pc, 2);
				Read(pc, mar.b.h); set_2part(mar, 3);

				Read(mar, pc.b.l); mar.b.l++;
				Read(mar, pc.b.h);
			break;

			case 0x4c : //JMP abs
				set_jmp_absolute();

				Read(pc, mar.b.l); pc.a++; set_part(pc, 2);
				Read(pc, mar.b.h); pc.a = mar.a;
			break;*/

			case 0x00 : //BRK
				set_brk();
				pc.a++;

				push(pc.b.h);
				push(pc.b.l);
				pmisc |= NMOS6502FLAG_B;

				temp8 = PStatus();
				push(temp8);
				pmisc |= NMOS6502FLAG_I;

				temp16.a = 0xfffe;
				Read(temp16, pc.b.l);
				temp16.a++;
				Read(temp16, pc.b.h);
			break;

/*			case 0x20 : //JSR
				set_jsr();
				Read(pc, mar.b.l);
				pc.a++;

				push(pc.b.h);
				push(pc.b.l);
				set_part(pc, 5);

				Read(pc, mar.b.h);
				pc.a = mar.a;
			break;*/

			case 0x40 : //RTI
				set_rti();
				pull(temp8); SetP(temp8); SetPLoadFlags();
				pull(pc.b.l);
				pull(pc.b.h);
				check_irq();
			break;

/*			case 0x60 : //RTS
				set_rts();
				pull(pc.b.l);
				pull(pc.b.h);
				set_part(pc, 5);
				pc.a++;
			break;*/

			case 0x08 : //PHP
				SetPLoadFlags();
				temp8 = PStatus();
				push_instr(temp8);
			break;

			case 0x28 : //PLP
				pull_instr(temp8);
				SetP(temp8);
				SetPLoadFlags();
				check_irq();
			break;

			case 0x48 : //PHA
				push_instr(a);
			break;

			case 0x68 : //PLA
				pull_instr(a);
				set_nz(a);
			break;


/*			case 0x18 : //CLC
				set_implied();
				P_CARRY(p) = 0;
			break;

			case 0x38 : //SEC
				set_implied();
				P_CARRY(p) = 1;
			break;

			case 0x58 : //CLI
				set_implied();
				pmisc &= ~NMOS6502FLAG_I;

				check_irq();
			break;

			case 0x78 : //SEI
				set_implied();
				pmisc |= NMOS6502FLAG_I;
			break;

			case 0xb8 : //CLV
				set_implied();
				P_OVERFLOW(p) = 0;
			break;

			case 0xd8 : //CLD
				set_implied();
				pmisc &= ~NMOS6502FLAG_D;
			break;

			case 0xf8 : //SED
				set_implied();
				pmisc |= NMOS6502FLAG_D;
			break;


			case 0x10 : //BPL
				cond_branch(!P_NEG(p));
			break;

			case 0x30 : //BMI
				cond_branch(P_NEG(p));
			break;

			case 0x50 : //BVC
				cond_branch(!P_OVERFLOW(p));
			break;

			case 0x70 : //BVS
				cond_branch(P_OVERFLOW(p));
			break;

			case 0x90 : //BCC
				cond_branch(!P_CARRY(p));
			break;

			case 0xb0 : //BCS
				cond_branch(P_CARRY(p));
			break;

			case 0xd0 : //BNE
				cond_branch(!P_ZERO(p));
			break;

			case 0xf0 : //BEQ
				cond_branch(P_ZERO(p));
			break;

*/
			case 0x09 : //ORA immediate
				set_immediate();
				ora();
			break;

			case 0x05 : //ORA zero
				set_zero_read();
				ora();
			break;

			case 0x15 : //ORA zero, x
				set_zero_i_read(x);
				ora();
			break;

			case 0x0d : //ORA absolute
				set_absolute_read();
				ora();
			break;

			case 0x1d : //ORA absolute, x
				set_absolute_i_read(x);
				ora();
			break;

			case 0x19 : //ORA absolute, y
				set_absolute_i_read(y);
				ora();
			break;

			case 0x01 : //ORA (zero, x)
				set_b_zero_i_b_read(x);
				ora();
			break;

			case 0x11 : //ORA (zero), y
				set_b_zero_b_i_read(y);
				ora();
			break;


			case 0x29 : //AND immediate
				set_immediate();
				and();
			break;

			case 0x25 : //AND zero
				set_zero_read();
				and();
			break;

			case 0x35 : //AND zero,x
				set_zero_i_read(x);
				and();
			break;

			case 0x2d : //AND absolute
				set_absolute_read();
				and();
			break;

			case 0x3d : //AND absolute, x
				set_absolute_i_read(x);
				and();
			break;

			case 0x39 : //AND absolute, y
				set_absolute_i_read(y);
				and();
			break;

			case 0x21 : //AND (zero, x)
				set_b_zero_i_b_read(x);
				and();
			break;

			case 0x31 : //AND (zero), y
				set_b_zero_b_i_read(y);
				and();
			break;


			case 0x49 : //EOR immediate
				set_immediate();
				eor();
			break;

			case 0x45 : //EOR zero
				set_zero_read();
				eor();
			break;

			case 0x55 : //EOR zero, x
				set_zero_i_read(x);
				eor();
			break;

			case 0x4d : //EOR absolute
				set_absolute_read();
				eor();
			break;

			case 0x5d : //EOR absolute, x
				set_absolute_i_read(x);
				eor();
			break;

			case 0x59 : //EOR absolute, y
				set_absolute_i_read(y);
				eor();
			break;

			case 0x41 : //EOR (zero, x)
				set_b_zero_i_b_read(x);
				eor();
			break;

			case 0x51 : //EOR (zero), y
				set_b_zero_b_i_read(y);
				eor();
			break;

/*
			case 0x24 : //BIT zero
				set_zero_read();
				bit();
			break;

			case 0x2c : //BIT absolute
				set_absolute_read();
				bit();
			break;*/


			case 0x0a : //ASL A
				set_implied();
				asl(a);
			break;

			case 0x06 : //ASL zero
				set_zero_modify();
				ZeroModify(mar.b.l, mdr, asl(mdr));
			break;

			case 0x16 : //ASL zero, x
				set_zero_i_modify(x);
				ZeroModify(mar.b.l, mdr, asl(mdr));
			break;

			case 0x0e : //ASL absolute
				set_absolute_modify();
				Modify(mar, mdr, asl(mdr));
			break;

			case 0x1e : //ASL absolute, x
				set_absolute_i_modify(x);
				Modify(mar, mdr, asl(mdr));
			break;


			case 0x2a : //ROL A
				set_implied();
				rol(a);
			break;

			case 0x26 : //ROL zero
				set_zero_modify();
				ZeroModify(mar.b.l, mdr, rol(mdr));
			break;

			case 0x36 : //ROL zero, x
				set_zero_i_modify(x);
				ZeroModify(mar.b.l, mdr, rol(mdr));
			break;

			case 0x2e : //ROL absolute
				set_absolute_modify();
				Modify(mar, mdr, rol(mdr));
			break;

			case 0x3e : //ROL absolute,x
				set_absolute_i_modify(x);
				Modify(mar, mdr, rol(mdr));
			break;


			case 0x4a : //LSR A
				set_implied();
				lsr(a);
			break;

			case 0x46 : //LSR zero
				set_zero_modify();
				ZeroModify(mar.b.l, mdr, lsr(mdr));
			break;

			case 0x56 : //LSR zero, x
				set_zero_i_modify(x);
				ZeroModify(mar.b.l, mdr, lsr(mdr));
			break;

			case 0x4e : //LSR absolute
				set_absolute_modify();
				Modify(mar, mdr, lsr(mdr));
			break;

			case 0x5e : //LSR absolute, x
				set_absolute_i_modify(x);
				Modify(mar, mdr, lsr(mdr));
			break;


			case 0x6a : //ROR A
				set_implied();
				ror(a);
			break;

			case 0x66 : //ROR zero
				set_zero_modify();
				ZeroModify(mar.b.l, mdr, ror(mdr));
			break;

			case 0x76 : //ROR zero, x
				set_zero_i_modify(x);
				ZeroModify(mar.b.l, mdr, ror(mdr));
			break;

			case 0x6e : //ROR absolute
				set_absolute_modify();
				Modify(mar, mdr, ror(mdr));
			break;

			case 0x7e : //ROR absolute,x
				set_absolute_i_modify(x);
				Modify(mar, mdr, ror(mdr));
			break;


/*			case 0x69 : //ADC immediate
				set_immediate();
				adc();
			break;

			case 0x65 : //ADC zero
				set_zero_read();
				adc();
			break;

			case 0x75 : //ADC zero, x
				set_zero_i_read(x);
				adc();
			break;

			case 0x6d : //ADC absolute
				set_absolute_read();
				adc();
			break;

			case 0x7d : //ADC absolute,x
				set_absolute_i_read(x);
				adc();
			break;

			case 0x79 : //ADC absolute,y
				set_absolute_i_read(y);
				adc();
			break;

			case 0x61 : //ADC (zero,x)
				set_b_zero_i_b_read(x);
				adc();
			break;

			case 0x71 : //ADC (zero),y
				set_b_zero_b_i_read(y);
				adc();
			break;


			case 0xeb :
			case 0xe9 : //SBC immediate
				set_immediate();
				sbc();
			break;

			case 0xe5 : //SBC zero
				set_zero_read();
				sbc();
			break;

			case 0xf5 : //SBC zero, x
				set_zero_i_read(x);
				sbc();
			break;

			case 0xed : //SBC absolute
				set_absolute_read();
				sbc();
			break;

			case 0xfd : //SBC absolute,x
				set_absolute_i_read(x);
				sbc();
			break;

			case 0xf9 : //SBC absolute,y
				set_absolute_i_read(y);
				sbc();
			break;

			case 0xe1 : //SBC (zero,x)
				set_b_zero_i_b_read(x);
				sbc();
			break;

			case 0xf1 : //SBC (zero),y
				set_b_zero_b_i_read(y);
				sbc();
			break;


			case 0xc6 : //DEC zero
				set_zero_modify();
				ZeroModify(mar.b.l, mdr, dec(mdr));
			break;

			case 0xd6 : //DEC zero, x
				set_zero_i_modify(x);
				ZeroModify(mar.b.l, mdr, dec(mdr));
			break;

			case 0xce : //DEC absolute
				set_absolute_modify();
				Modify(mar, mdr, dec(mdr));
			break;

			case 0xde : //DEC absolute, x
				set_absolute_i_modify(x);
				Modify(mar, mdr, dec(mdr));
			break;*/


			case 0x85 : //STA zero
				set_zero_write();
				ZeroWrite(mar.b.l, a);
			break;

			case 0x95 : //STA zero,x
				set_zero_i_write(x);
				ZeroWrite(mar.b.l, a);
			break;

			case 0x8d : //STA absolute
				set_absolute_write();
				Write(mar, a);
			break;

			case 0x9d : //STA abs, x
				set_absolute_i_write(x);
				Write(mar, a);
			break;

			case 0x99 : //STA abs,y
				set_absolute_i_write(y);
				Write(mar, a);
			break;

			case 0x81 : //STA (zero, x)
				set_b_zero_i_b_write(x);
				Write(mar, a);
			break;

			case 0x91 : //STA (zero), y
				set_b_zero_b_i_write(y);
				Write(mar, a);
			break;



			case 0xa9 : //LDA immd
				set_immediate();
				a = mdr; set_nz(a);
			break;

			case 0xa5 : //LDA zero
				set_zero_read();
				a = mdr; set_nz(a);
			break;

			case 0xb5 : //LDA zero,x
				set_zero_i_read(x);
				a = mdr; set_nz(a);
			break;

			case 0xad : //LDA absolute
				set_absolute_read();
				a = mdr; set_nz(a);
			break;

			case 0xbd : //LDA absolute, x
				set_absolute_i_read(x);
				a = mdr; set_nz(a);
			break;

			case 0xb9 : //LDA absolute, y
				set_absolute_i_read(y);
				a = mdr; set_nz(a);
			break;

			case 0xa1 : //LDA (zero, x)
				set_b_zero_i_b_read(x);
				a = mdr; set_nz(a);
			break;

			case 0xb1 : //LDA (zero), y
				set_b_zero_b_i_read(y);
				a = mdr; set_nz(a);
			break;


			case 0xa2 : //LDX immd
				set_immediate();
				x = mdr; set_nz(x);
			break;

			case 0xa6 : //LDX zero
				set_zero_read();
				x = mdr; set_nz(x);
			break;

			case 0xb6 : //LDX zero, y
				set_zero_i_read(y);
				x = mdr; set_nz(x);
			break;

			case 0xae : //LDX absolute
				set_absolute_read();
				x = mdr; set_nz(x);
			break;

			case 0xbe : //LDX absolute, y
				set_absolute_i_read(y);
				x = mdr; set_nz(x);
			break;


			case 0xa0 : //LDY immd
				set_immediate();
				y = mdr; set_nz(y);
			break;

			case 0xa4 : //LDY zero
				set_zero_read();
				y = mdr; set_nz(y);
			break;

			case 0xb4 : //LDY zero,x
				set_zero_i_read(x);
				y = mdr; set_nz(y);
			break;

			case 0xac : //LDY absolute
				set_absolute_read();
				y = mdr; set_nz(y);
			break;

			case 0xbc : //LDY absolute, x
				set_absolute_i_read(x);
				y = mdr; set_nz(y);
			break;


/*			case 0xe6 : //INC zero
				set_zero_modify();
				ZeroModify(mar.b.l, mdr, inc(mdr));
			break;

			case 0xf6 : //INC zero, x
				set_zero_i_modify(x);
				ZeroModify(mar.b.l, mdr, inc(mdr));
			break;

			case 0xee : //INC absolute
				set_absolute_modify();
				Modify(mar, mdr, inc(mdr));
			break;

			case 0xfe : //INC absolute, x
				set_absolute_i_modify(x);
				Modify(mar, mdr, inc(mdr));
			break;


			case 0xca : //DEX
				set_implied();
				dec(x);
			break;

			case 0xe8 : //INX
				set_implied();
				inc(x);
			break;

			case 0x88 : //DEY
				set_implied();
				dec(y);
			break;

			case 0xc8 : //INY
				set_implied();
				inc(y);
			break;*/


			case 0x9a : //TXS
				set_implied();
				s = x;
			break;

			case 0xba : //TSX
				set_implied();
				x = s; set_nz(x);
			break;

			case 0x98 : //TYA
				set_implied();
				a = y; set_nz(a);
			break;

			case 0x8a : //TXA
				set_implied();
				a = x; set_nz(a);
			break;

			case 0xa8 : //TAY
				set_implied();
				y = a; set_nz(y);
			break;

			case 0xaa : //TAX
				set_implied();
				x = a; set_nz(x);
			break;


			case 0x84 : //STY zero
				set_zero_write();
				ZeroWrite(mar.b.l, y);
			break;

			case 0x94 : //STY zero,x
				set_zero_i_write(x);
				ZeroWrite(mar.b.l, y);
			break;

			case 0x8c : //STY absolute
				set_absolute_write();
				Write(mar, y);
			break;


			case 0x86 : //STX zero
				set_zero_write();
				ZeroWrite(mar.b.l, x);
			break;

			case 0x96 : //STX zero,y
				set_zero_i_write(y);
				ZeroWrite(mar.b.l, x);
			break;

			case 0x8e : //STX absolute
				set_absolute_write();
				Write(mar, x);
			break;


/*			case 0xc0 : //CPY immediate
				set_immediate();
				cp(y, mdr);
			break;

			case 0xc4 : //CPY zero
				set_zero_read();
				cp(y, mdr);
			break;

			case 0xcc : //CPY absolute
				set_absolute_read();
				cp(y, mdr);
			break;


			case 0xe0 : //CPX immediate
				set_immediate();
				cp(x, mdr);
			break;

			case 0xe4 : //CPX zero
				set_zero_read();
				cp(x, mdr);
			break;

			case 0xec : //CPX absolute
				set_absolute_read();
				cp(x, mdr);
			break;


			case 0xc9 : //CMP immediate
				set_immediate();
				cp(a, mdr);
			break;

			case 0xc5 : //CMP zero
				set_zero_read();
				cp(a, mdr);
			break;

			case 0xd5 : //CMP zero, x
				set_zero_i_read(x);
				cp(a, mdr);
			break;

			case 0xcd : //CMP absolute
				set_absolute_read();
				cp(a, mdr);
			break;

			case 0xdd : //CMP absolute,x
				set_absolute_i_read(x);
				cp(a, mdr);
			break;

			case 0xd9 : //CMP absolute,y
				set_absolute_i_read(y);
				cp(a, mdr);
			break;

			case 0xc1 : //CMP (zero, x)
				set_b_zero_i_b_read(x);
				cp(a, mdr);
			break;

			case 0xd1 : //CMP (zero), y
				set_b_zero_b_i_read(y);
				cp(a, mdr);
			break;


			case 0x80 : //NOP [immediate]
			case 0x82 : //NOP [immediate]
			case 0xc2 : //NOP [immediate]
			case 0xe2 : //NOP [immediate]
				set_immediate();
			break;

			case 0x04 : //NOP [zero]
			case 0x44 : //NOP [zero]
			case 0x64 : //NOP [zero]
				set_zero_read();
			break;

			case 0x0c : //NOP [absolute]
				set_absolute_read();
			break;

			case 0x14 : //NOP [zero, x]
			case 0x34 : //NOP [zero, x]
			case 0x54 : //NOP [zero, x]
			case 0x74 : //NOP [zero, x]
			case 0xf4 : //NOP [zero, x]
				set_zero_i_read(x);
			break;

			case 0x1c : //NOP [absolute, x]
			case 0x3c : //NOP [absolute, x]
			case 0x5c : //NOP [absolute, x]
			case 0x7c : //NOP [absolute, x]
			case 0xdc : //NOP [absolute, x]
			case 0xfc : //NOP [absolute, x]
				set_absolute_i_read(x);
			break;

			case 0x1a : //NOP [implied]
			case 0x3a : //NOP [implied]
			case 0x5a : //NOP [implied]
			case 0x7a : //NOP [implied]
			case 0xda : //NOP [implied]
			case 0xea : //NOP [implied]
			case 0xfa : //NOP [implied]
				set_implied();
			break;*/

/* UNOFFICIAL OPCODES START HERE ...

	Electron uses a SynerTek SY6502A, so all this information is taken
	from Atari rather than Commodore docs

	SLO -

		memory is read, shifted left, & or'd with accumulator.
		Sets the N,Z and C flags.

	RLA -

		memory is rolled left, then and'd with the accumulator.
		Sets N, Z and C.

	SRE -

		memory is shifted right, then eor'd with the accumulator.
		Sets N, Z and C.

	RRA -

		memory is rolled right, then adc'd with the accumulator.
		Sets N, Z, V and C.

	SAX -

		a register and x register are and'd and stored in memory (neither of
		the registers are affected). Sets the N and Z flags

	LAX -

		a register and x register are both loaded simultaneously. Sets the N
		and Z flags

	DCP -

		memory is dec'd, then cmp'd with the accumulator.
		Sets N, Z and C.

	ISC -

		memory is rolled right, then adc'd with the accumulator.
		Sets N, Z, V and C.

	ANC -

		exactly as AND, except that carry is set equal to the negative flag

	ALR -

		operand is and'd with accumulator, then accumulator is lsr'd.

	ARR -

		operand os and'd with accumulator, then accumulator is ror'd.

	XAA[2] -

		puts x into a, then and's with the operand

	AXS -

		computes sbc of a&x and operand, and stores in x

	AXA[1] -

		stores a&x&(high byte of addr)

	SAY[1] -

 		stores y&(high byte of addr)

	XAS[1] -

 		stores x&(high byte of addr)

	TAS[1] -

		pushes a&x to the stack, then stores a&x&(high byte of addr)

	LAS -

 		stores value&s in a, x and y. Sets N & Z.

*/

/*			case 0x03 : //SLO (zero,x)
				set_b_zero_i_b_modify(x);
				Modify(mar, mdr, slo());
			break;

			case 0x07 : //SLO zero
				set_zero_modify();
				ZeroModify(mar.b.l, mdr, slo());
			break;

			case 0x0f : //SLO absolute
				set_absolute_modify();
				Modify(mar, mdr, slo());
			break;

			case 0x13 : //SLO (zero),y
				set_b_zero_b_i_modify(y);
				Modify(mar, mdr, slo());
			break;

			case 0x17 : //SLO zero, x
				set_zero_i_modify(x);
				ZeroModify(mar.b.l, mdr, slo());
			break;

			case 0x1f : //SLO absolute,x
				set_absolute_i_modify(x);
				Modify(mar, mdr, slo());
			break;

			case 0x1b : //SLO absolute,y
				set_absolute_i_modify(y);
				Modify(mar, mdr, slo());
			break;



			case 0x23 : //RLA (zero,x)
				set_b_zero_i_b_modify(x);
				Modify(mar, mdr, rla());
			break;

			case 0x27 : //RLA zero
				set_zero_modify();
				ZeroModify(mar.b.l, mdr, rla());
			break;

			case 0x2f : //RLA absolute
				set_absolute_modify();
				Modify(mar, mdr, rla());
			break;

			case 0x33 : //RLA (zero),y
				set_b_zero_b_i_modify(y);
				Modify(mar, mdr, rla());
			break;

			case 0x37 : //RLA zero, x
				set_zero_i_modify(x);
				ZeroModify(mar.b.l, mdr, rla());
			break;

			case 0x3f : //RLA absolute,x
				set_absolute_i_modify(x);
				Modify(mar, mdr, rla());
			break;

			case 0x3b : //RLA absolute,y
				set_absolute_i_modify(y);
				Modify(mar, mdr, rla());
			break;



			case 0x43 : //SRE (zero,x)
				set_b_zero_i_b_modify(x);
				Modify(mar, mdr, sre());
			break;

			case 0x47 : //SRE zero
				set_zero_modify();
				ZeroModify(mar.b.l, mdr, sre());
			break;

			case 0x4f : //SRE absolute
				set_absolute_modify();
				Modify(mar, mdr, sre());
			break;

			case 0x53 : //SRE (zero),y
				set_b_zero_b_i_modify(y);
				Modify(mar, mdr, sre());
			break;

			case 0x57 : //SRE zero, x
				set_zero_i_modify(x);
				ZeroModify(mar.b.l, mdr, sre());
			break;

			case 0x5f : //SRE absolute,x
				set_absolute_i_modify(x);
				Modify(mar, mdr, sre());
			break;

			case 0x5b : //SRE absolute,y
				set_absolute_i_modify(y);
				Modify(mar, mdr, sre());
			break;*/


			case 0x63 : //RRA (zero,x)
				set_b_zero_i_b_modify(x);
				Modify(mar, mdr, ror(mdr));
				adc();
			break;

			case 0x67 : //RRA zero
				set_zero_modify();
				ZeroModify(mar.b.l, mdr, ror(mdr));
				adc();
			break;

			case 0x6f : //RRA absolute
				set_absolute_modify();
				Modify(mar, mdr, ror(mdr));
				adc();
			break;

			case 0x73 : //RRA (zero),y
				set_b_zero_b_i_modify(y);
				Modify(mar, mdr, ror(mdr));
				adc();
			break;

			case 0x77 : //RRA zero, x
				set_zero_i_modify(x);
				ZeroModify(mar.b.l, mdr, ror(mdr));
				adc();
			break;

			case 0x7f : //RRA absolute,x
				set_absolute_i_modify(x);
				Modify(mar, mdr, ror(mdr));
				adc();
			break;

			case 0x7b : //RRA absolute,y
				set_absolute_i_modify(y);
				Modify(mar, mdr, ror(mdr));
				adc();
			break;


			case 0x87 : //SAX zero
				set_zero_write();
				sax();
				ZeroWrite(mar.b.l, mdr);
			break;

			case 0x8f : //SAX absolute
				set_absolute_write();
				sax();
				Write(mar, mdr);
			break;

			case 0x9f : //SAX abs, x
				set_absolute_i_write(x);
				sax();
				Write(mar, mdr);
			break;

			case 0x97 : //SAX zero,y
				set_zero_i_write(y);
				sax();
				ZeroWrite(mar.b.l, mdr);
			break;



			case 0xa7 : //LAX zero
				set_zero_read();
				x = a = mdr; set_nz(a);
			break;

			case 0xb7 : //LAX zero,x
				set_zero_i_read(x);
				x = a = mdr; set_nz(a);
			break;

			case 0xaf : //LAX absolute
				set_absolute_read();
				x = a = mdr; set_nz(a);
			break;

			case 0xbf : //LAX absolute, x
				set_absolute_i_read(x);
				x = a = mdr; set_nz(a);
			break;

			case 0xa3 : //LAX (zero, x)
				set_b_zero_i_b_read(x);
				x = a = mdr; set_nz(a);
			break;

			case 0xb3 : //LAX (zero), y
				set_b_zero_b_i_read(y);
				x = a = mdr; set_nz(a);
			break;



/*			case 0xc3 : //DCP (zero,x)
				set_b_zero_i_b_modify(x);
				Modify(mar, mdr, dec(mdr));
				cp(a, mdr);
			break;

			case 0xc7 : //DCP zero
				set_zero_modify();
				ZeroModify(mar.b.l, mdr, dec(mdr));
				cp(a, mdr);
			break;

			case 0xcf : //DCP absolute
				set_absolute_modify();
				Modify(mar, mdr, dec(mdr));
				cp(a, mdr);
			break;

			case 0xd3 : //DCP (zero),y
				set_b_zero_b_i_modify(y);
				Modify(mar, mdr, dec(mdr));
				cp(a, mdr);
			break;

			case 0xd7 : //DCP zero, x
				set_zero_i_modify(x);
				ZeroModify(mar.b.l, mdr, dec(mdr));
				cp(a, mdr);
			break;

			case 0xdf : //DCP absolute,x
				set_absolute_i_modify(x);
				Modify(mar, mdr, dec(mdr));
				cp(a, mdr);
			break;

			case 0xdb : //DCP absolute,y
				set_absolute_i_modify(y);
				Modify(mar, mdr, dec(mdr));
				cp(a, mdr);
			break;


			case 0xe3 : //ISC (zero,x)
				set_b_zero_i_b_modify(x);
				Modify(mar, mdr, inc(mdr));
				sbc();
			break;

			case 0xe7 : //ISC zero
				set_zero_modify();
				ZeroModify(mar.b.l, mdr, inc(mdr));
				sbc();
			break;

			case 0xef : //ISC absolute
				set_absolute_modify();
				Modify(mar, mdr, inc(mdr));
				sbc();
			break;

			case 0xf3 : //ISC (zero),y
				set_b_zero_b_i_modify(y);
				Modify(mar, mdr, inc(mdr));
				sbc();
			break;

			case 0xf7 : //ISC zero, x
				set_zero_i_modify(x);
				ZeroModify(mar.b.l, mdr, inc(mdr));
				sbc();
			break;

			case 0xff : //ISC absolute,x
				set_absolute_i_modify(x);
				Modify(mar, mdr, inc(mdr));
				sbc();
			break;

			case 0xfb : //ISC absolute,y
				set_absolute_i_modify(y);
				Modify(mar, mdr, inc(mdr));
				sbc();
			break;


			case 0x0b :
			case 0x2b : //ANC immediate
				set_immediate();
				anc();
			break;*/
		}

		std_cpu_end();
	}

	timecounter = bigcounter+counter;
}

void C_6502Regular::SetIRQ(void)
{
	irq = true;
	check_irq();
}

#undef eor
#undef and
#undef ora

#undef cp
#undef asl
#undef lsr
#undef rol
#undef ror
#undef sbc
#undef adc
#undef add

#undef set_type
#undef set_part
#undef set_3part

#undef set_immediate
#undef set_implied

#undef set_absolute_read
#undef set_absolute_modify
#undef set_absolute_write

#undef set_zero_read
#undef set_zero_modify
#undef set_zero_write

#undef set_zero_i_read
#undef set_zero_i_modify
#undef set_zero_i_write

#undef set_absolute_i_read
#undef set_absolute_i_modify
#undef set_absolute_i_write

#undef set_b_zero_i_b_read
#undef set_b_zero_i_b_modify
#undef set_b_zero_i_b_write

#undef set_b_zero_b_i_read
#undef set_b_zero_b_i_modify
#undef set_b_zero_b_i_write

#undef push_instr
#undef pull_instr
#undef push
#undef pull

#undef set_jsr
#undef set_rts
#undef set_brk
#undef set_rti

#undef set_jmp_indirect
#undef set_jmp_absolute
#undef cond_branch
#undef set_nz

#undef StackRead
#undef StackWrite
#undef ZeroRead
#undef ZeroWrite
#undef Read
#undef Write