/*

	ElectrEm (c) 2000 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for
	details

	6502/wide/wcontrol.cpp
	======================

	The multiplexing 6502 core + IRQ line set

*/
#include "wide.h"
#include "../defs.h"

#define ReadWide(addr, val, bval)	if(mapped_flags[addr.b.h]) device->ReadFrom(addr, &val, &bval, bigcounter); else {val = o_r_mem[addr.b.h][addr.b.l]; bval = f_r_mem[addr.b.h][addr.b.l];}
#define WriteWide(addr, val, bval)	if(mapped_flags[addr.b.h]) device->WriteTo(addr, val, bval, bigcounter); else {o_r_mem[addr.b.h][addr.b.l] = val; f_r_mem[addr.b.h][addr.b.l] = bval;}
#define WriteWideOnly(addr, bval)	mapped_flags[addr.b.h] ? 0 : (f_r_mem[addr.b.h][addr.b.l] = bval)

#define StackReadWide(addr, val, bval)	val = o_r_mem[0x01][addr]; bval = f_r_mem[0x01][addr];
#define StackWriteWide(addr, val, bval)	o_w_mem[0x01][addr] = val; f_w_mem[0x01][addr] = bval;
#define ZeroReadWide(addr, val, bval)	val = o_r_mem[0x00][addr]; bval = f_r_mem[0x00][addr];
#define ZeroWriteWide(addr, val, bval)	o_w_mem[0x00][addr] = val; f_w_mem[0x00][addr] = bval;

#define ModifyWide(addr, val, bval, action)		if(mapped_flags[addr.b.h]){ device->ReadFrom(addr, &val, &bval, bigcounter); action; device->WriteTo(addr, val, bval, bigcounter);} else { val = o_r_mem[addr.b.h][addr.b.l]; bval = f_r_mem[addr.b.h][addr.b.l]; action; o_r_mem[addr.b.h][addr.b.l] = val; f_r_mem[addr.b.h][addr.b.l] = bval;}
#define ZeroModifyWide(addr, val, bval, action)	val = o_r_mem[0][addr]; bval = f_r_mem[0][addr]; action; o_w_mem[0][addr] = val; f_w_mem[0][addr] = bval;

/* MEMORY ADDRESSING MODE READS, COMMON PARTS */

#define setup_absolute_read()		set_type(ABS_READ,4); Read(pc, mar.b.l); pc.a++; set_part(pc, 2); Read(pc, mar.b.h); pc.a++; set_part(mar, 3);
#define setup_zero_read()			set_type(ZERO_READ,3); Read(pc, mar.b.l); pc.a++;
#define setup_zero_i_read(v)		set_type(ZIND_READ,4); Read(pc, mar.b.l); pc.a++; mar.b.l += v;
#define setup_absolute_i_read(v)	set_vtype(AIND_READ); Read(pc, mar.b.l); pc.a++; Read(pc, mar.b.h); set_part(pc, 2); pc.a++; set_part(mar, 3); len = (((mar.a+v)^mar.a) & 0xff00) ? 5 : 4; mar.a += v; set_part(mar, 4);
#define setup_b_zero_i_b_read(v)	set_type(INDI_READ,6); Read(pc, mdr); pc.a++; mdr += v; ZeroRead(mdr, mar.b.l); mdr++; ZeroRead(mdr, mar.b.h); set_part(mar, 5);
#define setup_b_zero_b_i_read(v)	set_vtype(IIND_READ); Read(pc, mdr); pc.a++; ZeroRead(mdr, mar.b.l); mdr++; ZeroRead(mdr, mar.b.h); set_part(mar, 4); len = (((mar.a+v)^mar.a) & 0xff00) ? 6 : 5; mar.a += v; set_part(mar, 5);

/* STANDARD MEMORY ADDRESSING MODES */

#define set_immediate()				set_type(IMMEDIATE,2); Read(pc, mdr); pc.a++;
#define set_implied()				set_type(IMPLIED,2);

#define set_absolute_read()			setup_absolute_read(); Read(mar, mdr);
#define set_absolute_modify()		set_type(ABS_MODIFY,6); Read(pc, mar.b.l); pc.a++; set_part(pc, 2); Read(pc, mar.b.h); pc.a++; set_3part(mar, 3);
#define set_absolute_write()		set_type(ABS_WRITE,4); Read(pc, mar.b.l); pc.a++; set_part(pc, 2); Read(pc, mar.b.h); pc.a++; set_part(mar, 3);

#define set_zero_read()				setup_zero_read(); ZeroRead(mar.b.l, mdr);
#define set_zero_modify()			set_type(ZERO_MODIFY,5); Read(pc, mar.b.l); pc.a++;
#define set_zero_write()			set_type(ZERO_WRITE,3); Read(pc, mar.b.l); pc.a++;

#define set_zero_i_read(v)			setup_zero_i_read(v); ZeroRead(mar.b.l, mdr);
#define set_zero_i_modify(v)		set_type(ZIND_MODIFY,6); Read(pc, mar.b.l); pc.a++; mar.b.l += v;
#define set_zero_i_write(v)			set_type(ZIND_WRITE,4); Read(pc, mar.b.l); pc.a++; mar.b.l += v;

#define set_absolute_i_read(v)		setup_absolute_i_read(v); Read(mar, mdr);
#define set_absolute_i_modify(v)	set_type(AIND_MODIFY,7); Read(pc, mar.b.l); pc.a++; set_part(pc, 2); Read(pc, mar.b.h); pc.a++; set_part(mar, 3); mar.a += v; set_3part(mar, 4);
#define set_absolute_i_write(v)		set_type(AIND_WRITE,5); Read(pc, mar.b.l); pc.a++; set_part(pc, 2); Read(pc, mar.b.h); pc.a++; set_part(mar, 3); mar.a += v; set_part(mar, 4);

//only used with x
#define set_b_zero_i_b_read(v)		setup_b_zero_i_b_read(v); Read(mar, mdr); 
#define set_b_zero_i_b_modify(v)	set_type(INDI_MODIFY,8); Read(pc, mdr); pc.a++; mdr += v; ZeroRead(mdr, mar.b.l); mdr++; ZeroRead(mdr, mar.b.h); set_3part(mar, 5);
#define set_b_zero_i_b_write(v)		set_type(INDI_WRITE,6); Read(pc, mdr); pc.a++; mdr += v; ZeroRead(mdr, mar.b.l); mdr++; ZeroRead(mdr, mar.b.h); set_part(mar, 5);

//only used with y
#define set_b_zero_b_i_read(v)		setup_b_zero_b_i_read(v); Read(mar, mdr);
#define set_b_zero_b_i_modify(v)	set_type(IIND_MODIFY,8); Read(pc, mdr); pc.a++; ZeroRead(mdr, mar.b.l); mdr++; ZeroRead(mdr, mar.b.h); set_part(mar, 4); mar.a += v; set_3part(mar, 5);
#define set_b_zero_b_i_write(v)		set_type(IIND_WRITE,6); Read(pc, mdr); pc.a++; ZeroRead(mdr, mar.b.l); mdr++; ZeroRead(mdr, mar.b.h); set_part(mar, 4); mar.a += v; set_part(mar, 5);

/* WIDE MEMORY ADDRESSING MODES FOR READS */

#define set_immediate_w()			set_type(IMMEDIATE,2); ReadWide(pc, mdr, mdrBig); pc.a++;
#define set_absolute_read_w()		setup_absolute_read(); ReadWide(mar, mdr, mdrBig);
#define set_zero_read_w()			setup_zero_read(); ZeroReadWide(mar.b.l, mdr, mdrBig);
#define set_zero_i_read_w(v)		setup_zero_i_read(v); ZeroReadWide(mar.b.l, mdr, mdrBig);
#define set_absolute_i_read_w(v)	setup_absolute_i_read(v); ReadWide(mar, mdr, mdrBig);
#define set_b_zero_i_b_read_w(v)	setup_b_zero_i_b_read(v); ReadWide(mar, mdr, mdrBig); 
#define set_b_zero_b_i_read_w(v)	setup_b_zero_b_i_read(v); ReadWide(mar, mdr, mdrBig);

/* MISC INSTRUCTION STUFF */

#define push_instr(v, bv)			StackWriteWide(s, v, bv); s--; set_type(PHx,3);
#define pull_instr(v, bv)			s++; StackReadWide(s, v, bv); set_type(PLx,4);
#define pushWide(v, bv)				StackWriteWide(s, v, bv); s--;
#define pullWide(v, bv)				s++; StackReadWide(s, v, bv);

#define ror(v, vb)	\
temp8 = (v >> 1) | (RD_CARRY() << 7);\
LD_CARRY(v&1);\
v = temp8;\
LD_NEGZERO(v);\
temp32 = (vb >> 4) | (RD_CARRY_WD() << 28);\
LD_CARRY_WD(vb&15);\
vb = temp32;

#define rol(v, vb)	\
temp16.a = (v << 1) | RD_CARRY();\
LD_CARRY(temp16.b.h);\
v = temp16.b.l;\
LD_NEGZERO(v);\
temp32 = (vb << 4) | RD_CARRY_WD();\
LD_CARRY_WD(vb >> 28);\
vb = temp32;

#define lsr(v, vb)	\
LD_CARRY(v&1);\
v >>= 1;\
LD_NEGZERO(v);\
LD_CARRY_WD(vb&15);\
vb >>= 4;

#define asl(v, vb)	\
LD_CARRY(v>>7);\
v <<= 1;\
LD_NEGZERO(v);\
LD_CARRY_WD(vb>>28);\
vb <<= 4;

#define bit()	\
	LD_ZERO(a&mdr);\
	LD_OVERFLOW((mdr&NMOS6502FLAG_V) << 1);\
	LD_NEG(mdr&NMOS6502FLAG_N);

#define ora()	aBig |= mdrBig; a |= mdr; set_nz(a);
#define and()	aBig &= mdrBig; a &= mdr; set_nz(a);
#define eor()	aBig ^= mdrBig; a ^= mdr; set_nz(a);
#define dec(v)	v--; set_nz(v);
#define inc(v)	v++; set_nz(v);

#define slo()	LD_CARRY(mdr>>7);\
				mdr <<= 1;\
				a |= mdr;\
				LD_NEGZERO(a);\
				LD_CARRY_WD(mdrBig>>28);\
				mdrBig <<= 4;\
				aBig |= mdrBig;

#define rla()	temp8 = (mdr << 1) | RD_CARRY();\
				LD_CARRY(temp8&1);\
				a &= (mdr = temp8);\
				LD_NEGZERO(a);\
				temp32 = (mdrBig << 4) | RD_CARRY_WD();\
				LD_CARRY_WD(temp32&15);\
				aBig &= (mdrBig = temp32);

#define sre()	LD_CARRY(mdr&1);\
				mdr >>= 1;\
				a ^= mdr;\
				LD_NEGZERO(a);\
				LD_CARRY_WD(mdrBig&15);\
				mdrBig >>= 4;\
				aBig ^= mdrBig;

#define sax()	mdr = a&x; mdrBig = aBig^xBig;

#define anc()	a&=mdr;\
				LD_NEGZERO(a);\
				LD_CARRY(a >> 7);\
				aBig &= mdrBig;\
				LD_CARRY_WD(aBig >> 28);

void C_6502Wide::Go(void)
{
	unsigned __int8 instr;
	int counter = 0, bigcounter = timecounter;
	unsigned __int8 temp8, mdr;
	unsigned __int32 temp32, mdrBig;
	BrokenWord temp16, mar;
	int len, count;
	E6502_MemoryAccess *lblock;

	irqe = false;

	while(!quit)
	{
		Read(pc, instr);

		switch(instr)
		{
			#include "../coredefs.h"
/*			default :
				if(device->TrapOpcode(instr))
				{
					set_immediate();
					device->VirtualOpcode(instr, mdr);

					pull(pc.b.l);
					pull(pc.b.h);
					pc.a++;
				}
				else
				{
					set_implied();
					device->UnknownOpcode(instr);
				}
			break;

			case 0x6c : //JMP (addr)
				set_jmp_indirect();

				Read(pc, mar.b.l); pc.a++; set_part(pc, 2);
				Read(pc, mar.b.h); set_2part(mar, 3);

				Read(mar, pc.b.l); mar.b.l++;
				Read(mar, pc.b.h);
			break;

			case 0x4c : //JMP abs
				set_jmp_absolute();

				Read(pc, mar.b.l); pc.a++; set_part(pc, 2);
				Read(pc, mar.b.h); pc.a = mar.a;
			break;*/

			case 0x00 : //BRK
				set_brk();
				pc.a++;

				push(pc.b.h);
				push(pc.b.l);
				pmisc |= NMOS6502FLAG_B;

				temp8 = PStatus();
				pushWide(temp8, RD_CARRY_WD());
				pmisc |= NMOS6502FLAG_I;

				temp16.a = 0xfffe;
				Read(temp16, pc.b.l);
				temp16.a++;
				Read(temp16, pc.b.h);
			break;

/*			case 0x20 : //JSR
				set_jsr();
				Read(pc, mar.b.l);
				pc.a++;

				push(pc.b.h);
				push(pc.b.l);
				set_part(pc, 5);

				Read(pc, mar.b.h);
				pc.a = mar.a;
			break;*/

			case 0x40 : //RTI
				set_rti();
				pullWide(temp8, RD_CARRY_WD()); SetP(temp8); SetPLoadFlags();
				pull(pc.b.l);
				pull(pc.b.h);
				check_irq();
			break;

/*			case 0x60 : //RTS
				set_rts();
				pull(pc.b.l);
				pull(pc.b.h);
				set_part(pc, 5);
				pc.a++;
			break;*/

			case 0x08 : //PHP
				SetPLoadFlags();
				temp8 = PStatus();
				push_instr(temp8, RD_CARRY_WD());
			break;

			case 0x28 : //PLP
				pull_instr(temp8, temp32);
				LD_CARRY_WD(temp32);
				SetP(temp8);
				SetPLoadFlags();
				check_irq();
			break;

			case 0x48 : //PHA
				push_instr(a, aBig);
			break;

			case 0x68 : //PLA
				pull_instr(a, aBig);
				set_nz(a);
			break;


/*			case 0x18 : //CLC
				set_implied();
				P_CARRY(p) = 0;
			break;

			case 0x38 : //SEC
				set_implied();
				P_CARRY(p) = 1;
			break;

			case 0x58 : //CLI
				set_implied();
				pmisc &= ~NMOS6502FLAG_I;

				check_irq();
			break;

			case 0x78 : //SEI
				set_implied();
				pmisc |= NMOS6502FLAG_I;
			break;

			case 0xb8 : //CLV
				set_implied();
				P_OVERFLOW(p) = 0;
			break;

			case 0xd8 : //CLD
				set_implied();
				pmisc &= ~NMOS6502FLAG_D;
			break;

			case 0xf8 : //SED
				set_implied();
				pmisc |= NMOS6502FLAG_D;
			break;


			case 0x10 : //BPL
				cond_branch(!P_NEG(p));
			break;

			case 0x30 : //BMI
				cond_branch(P_NEG(p));
			break;

			case 0x50 : //BVC
				cond_branch(!P_OVERFLOW(p));
			break;

			case 0x70 : //BVS
				cond_branch(P_OVERFLOW(p));
			break;

			case 0x90 : //BCC
				cond_branch(!P_CARRY(p));
			break;

			case 0xb0 : //BCS
				cond_branch(P_CARRY(p));
			break;

			case 0xd0 : //BNE
				cond_branch(!P_ZERO(p));
			break;

			case 0xf0 : //BEQ
				cond_branch(P_ZERO(p));
			break;

*/
			case 0x09 : //ORA immediate
				set_immediate_w();
				ora();
			break;

			case 0x05 : //ORA zero
				set_zero_read_w();
				ora();
			break;

			case 0x15 : //ORA zero, x
				set_zero_i_read_w(x);
				ora();
			break;

			case 0x0d : //ORA absolute
				set_absolute_read_w();
				ora();
			break;

			case 0x1d : //ORA absolute, x
				set_absolute_i_read_w(x);
				ora();
			break;

			case 0x19 : //ORA absolute, y
				set_absolute_i_read_w(y);
				ora();
			break;

			case 0x01 : //ORA (zero, x)
				set_b_zero_i_b_read_w(x);
				ora();
			break;

			case 0x11 : //ORA (zero), y
				set_b_zero_b_i_read_w(y);
				ora();
			break;


			case 0x29 : //AND immediate
				set_immediate_w();
				and();
			break;

			case 0x25 : //AND zero
				set_zero_read_w();
				and();
			break;

			case 0x35 : //AND zero,x
				set_zero_i_read_w(x);
				and();
			break;

			case 0x2d : //AND absolute
				set_absolute_read_w();
				and();
			break;

			case 0x3d : //AND absolute, x
				set_absolute_i_read_w(x);
				and();
			break;

			case 0x39 : //AND absolute, y
				set_absolute_i_read_w(y);
				and();
			break;

			case 0x21 : //AND (zero, x)
				set_b_zero_i_b_read_w(x);
				and();
			break;

			case 0x31 : //AND (zero), y
				set_b_zero_b_i_read_w(y);
				and();
			break;


			case 0x49 : //EOR immediate
				set_immediate_w();
				eor();
			break;

			case 0x45 : //EOR zero
				set_zero_read_w();
				eor();
			break;

			case 0x55 : //EOR zero, x
				set_zero_i_read_w(x);
				eor();
			break;

			case 0x4d : //EOR absolute
				set_absolute_read_w();
				eor();
			break;

			case 0x5d : //EOR absolute, x
				set_absolute_i_read_w(x);
				eor();
			break;

			case 0x59 : //EOR absolute, y
				set_absolute_i_read_w(y);
				eor();
			break;

			case 0x41 : //EOR (zero, x)
				set_b_zero_i_b_read_w(x);
				eor();
			break;

			case 0x51 : //EOR (zero), y
				set_b_zero_b_i_read_w(y);
				eor();
			break;

/*
			case 0x24 : //BIT zero
				set_zero_read();
				bit();
			break;

			case 0x2c : //BIT absolute
				set_absolute_read();
				bit();
			break;*/


			case 0x0a : //ASL A
				set_implied();
				asl(a, aBig);
			break;

			case 0x06 : //ASL zero
				set_zero_modify();
				ZeroModifyWide(mar.b.l, mdr, mdrBig, asl(mdr, mdrBig));
			break;

			case 0x16 : //ASL zero, x
				set_zero_i_modify(x);
				ZeroModifyWide(mar.b.l, mdr, mdrBig, asl(mdr, mdrBig));
			break;

			case 0x0e : //ASL absolute
				set_absolute_modify();
				ModifyWide(mar, mdr, mdrBig, asl(mdr, mdrBig));
			break;

			case 0x1e : //ASL absolute, x
				set_absolute_i_modify(x);
				ModifyWide(mar, mdr, mdrBig, asl(mdr, mdrBig));
			break;


			case 0x2a : //ROL A
				set_implied();
				rol(a, aBig);
			break;

			case 0x26 : //ROL zero
				set_zero_modify();
				ZeroModifyWide(mar.b.l, mdr, mdrBig, rol(mdr, mdrBig));
			break;

			case 0x36 : //ROL zero, x
				set_zero_i_modify(x);
				ZeroModifyWide(mar.b.l, mdr, mdrBig, rol(mdr, mdrBig));
			break;

			case 0x2e : //ROL absolute
				set_absolute_modify();
				ModifyWide(mar, mdr, mdrBig, rol(mdr, mdrBig));
			break;

			case 0x3e : //ROL absolute,x
				set_absolute_i_modify(x);
				ModifyWide(mar, mdr, mdrBig, rol(mdr, mdrBig));
			break;


			case 0x4a : //LSR A
				set_implied();
				lsr(a, aBig);
			break;

			case 0x46 : //LSR zero
				set_zero_modify();
				ZeroModifyWide(mar.b.l, mdr, mdrBig, lsr(mdr, mdrBig));
			break;

			case 0x56 : //LSR zero, x
				set_zero_i_modify(x);
				ZeroModifyWide(mar.b.l, mdr, mdrBig, lsr(mdr, mdrBig));
			break;

			case 0x4e : //LSR absolute
				set_absolute_modify();
				ModifyWide(mar, mdr, mdrBig, lsr(mdr, mdrBig));
			break;

			case 0x5e : //LSR absolute, x
				set_absolute_i_modify(x);
				ModifyWide(mar, mdr, mdrBig, lsr(mdr, mdrBig));
			break;


			case 0x6a : //ROR A
				set_implied();
				ror(a, aBig);
			break;

			case 0x66 : //ROR zero
				set_zero_modify();
				ZeroModifyWide(mar.b.l, mdr, mdrBig, ror(mdr, mdrBig));
			break;

			case 0x76 : //ROR zero, x
				set_zero_i_modify(x);
				ZeroModifyWide(mar.b.l, mdr, mdrBig, ror(mdr, mdrBig));
			break;

			case 0x6e : //ROR absolute
				set_absolute_modify();
				ModifyWide(mar, mdr, mdrBig, ror(mdr, mdrBig));
			break;

			case 0x7e : //ROR absolute,x
				set_absolute_i_modify(x);
				ModifyWide(mar, mdr, mdrBig, ror(mdr, mdrBig));
			break;


/*			case 0x69 : //ADC immediate
				set_immediate();
				adc();
			break;

			case 0x65 : //ADC zero
				set_zero_read();
				adc();
			break;

			case 0x75 : //ADC zero, x
				set_zero_i_read(x);
				adc();
			break;

			case 0x6d : //ADC absolute
				set_absolute_read();
				adc();
			break;

			case 0x7d : //ADC absolute,x
				set_absolute_i_read(x);
				adc();
			break;

			case 0x79 : //ADC absolute,y
				set_absolute_i_read(y);
				adc();
			break;

			case 0x61 : //ADC (zero,x)
				set_b_zero_i_b_read(x);
				adc();
			break;

			case 0x71 : //ADC (zero),y
				set_b_zero_b_i_read(y);
				adc();
			break;


			case 0xe9 : //SBC immediate
				set_immediate();
				sbc();
			break;

			case 0xe5 : //SBC zero
				set_zero_read();
				sbc();
			break;

			case 0xf5 : //SBC zero, x
				set_zero_i_read(x);
				sbc();
			break;

			case 0xed : //SBC absolute
				set_absolute_read();
				sbc();
			break;

			case 0xfd : //SBC absolute,x
				set_absolute_i_read(x);
				sbc();
			break;

			case 0xf9 : //SBC absolute,y
				set_absolute_i_read(y);
				sbc();
			break;

			case 0xe1 : //SBC (zero,x)
				set_b_zero_i_b_read(x);
				sbc();
			break;

			case 0xf1 : //SBC (zero),y
				set_b_zero_b_i_read(y);
				sbc();
			break;


			case 0xc6 : //DEC zero
				set_zero_modify();
				ZeroModify(mar.b.l, mdr, dec(mdr));
			break;

			case 0xd6 : //DEC zero, x
				set_zero_i_modify(x);
				ZeroModify(mar.b.l, mdr, dec(mdr));
			break;

			case 0xce : //DEC absolute
				set_absolute_modify();
				Modify(mar, mdr, dec(mdr));
			break;

			case 0xde : //DEC absolute, x
				set_absolute_i_modify(x);
				Modify(mar, mdr, dec(mdr));
			break;*/


			case 0x85 : //STA zero
				set_zero_write();
				ZeroWriteWide(mar.b.l, a, aBig);
			break;

			case 0x95 : //STA zero,x
				set_zero_i_write(x);
				ZeroWriteWide(mar.b.l, a, aBig);
			break;

			case 0x8d : //STA absolute
				set_absolute_write();
				WriteWide(mar, a, aBig);
			break;

			case 0x9d : //STA abs, x
				set_absolute_i_write(x);
				WriteWide(mar, a, aBig);
			break;

			case 0x99 : //STA abs,y
				set_absolute_i_write(y);
				WriteWide(mar, a, aBig);
			break;

			case 0x81 : //STA (zero, x)
				set_b_zero_i_b_write(x);
				WriteWide(mar, a, aBig);
			break;

			case 0x91 : //STA (zero), y
				set_b_zero_b_i_write(y);
				WriteWide(mar, a, aBig);
			break;


			case 0xa9 : //LDA immd
				set_immediate_w();
				a = mdr; aBig = mdrBig; set_nz(a);
			break;

			case 0xa5 : //LDA zero
				set_zero_read_w();
				a = mdr; aBig = mdrBig; set_nz(a);
			break;

			case 0xb5 : //LDA zero,x
				set_zero_i_read_w(x);
				a = mdr; aBig = mdrBig; set_nz(a);
			break;

			case 0xad : //LDA absolute
				set_absolute_read_w();
				a = mdr; aBig = mdrBig; set_nz(a);
			break;

			case 0xbd : //LDA absolute, x
				set_absolute_i_read_w(x);
				a = mdr; aBig = mdrBig; set_nz(a);
			break;

			case 0xb9 : //LDA absolute, y
				set_absolute_i_read_w(y);
				a = mdr; aBig = mdrBig; set_nz(a);
			break;

			case 0xa1 : //LDA (zero, x)
				set_b_zero_i_b_read_w(x);
				a = mdr; aBig = mdrBig; set_nz(a);
			break;

			case 0xb1 : //LDA (zero), y
				set_b_zero_b_i_read_w(y);
				a = mdr; aBig = mdrBig; set_nz(a);
			break;


			case 0xa2 : //LDX immd
				set_immediate_w();
				x = mdr; xBig = mdrBig; set_nz(x);
			break;

			case 0xa6 : //LDX zero
				set_zero_read_w();
				x = mdr; xBig = mdrBig; set_nz(x);
			break;

			case 0xb6 : //LDX zero, y
				set_zero_i_read_w(y);
				x = mdr; xBig = mdrBig; set_nz(x);
			break;

			case 0xae : //LDX absolute
				set_absolute_read_w();
				x = mdr; xBig = mdrBig; set_nz(x);
			break;

			case 0xbe : //LDX absolute, y
				set_absolute_i_read_w(y);
				x = mdr; xBig = mdrBig; set_nz(x);
			break;


			case 0xa0 : //LDY immd
				set_immediate_w();
				y = mdr; yBig = mdrBig; set_nz(y);
			break;

			case 0xa4 : //LDY zero
				set_zero_read_w();
				y = mdr; yBig = mdrBig; set_nz(y);
			break;

			case 0xb4 : //LDY zero,x
				set_zero_i_read_w(x);
				y = mdr; yBig = mdrBig; set_nz(y);
			break;

			case 0xac : //LDY absolute
				set_absolute_read_w();
				y = mdr; yBig = mdrBig; set_nz(y);
			break;

			case 0xbc : //LDY absolute, x
				set_absolute_i_read_w(x);
				y = mdr; yBig = mdrBig; set_nz(y);
			break;


/*			case 0xe6 : //INC zero
				set_zero_modify();
				ZeroModify(mar.b.l, mdr, inc(mdr));
			break;

			case 0xf6 : //INC zero, x
				set_zero_i_modify(x);
				ZeroModify(mar.b.l, mdr, inc(mdr));
			break;

			case 0xee : //INC absolute
				set_absolute_modify();
				Modify(mar, mdr, inc(mdr));
			break;

			case 0xfe : //INC absolute, x
				set_absolute_i_modify(x);
				Modify(mar, mdr, inc(mdr));
			break;


			case 0xca : //DEX
				set_implied();
				x--; set_nz(x);
			break;

			case 0xe8 : //INX
				set_implied();
				x++; set_nz(x);
			break;

			case 0x88 : //DEY
				set_implied();
				y--; set_nz(y);
			break;

			case 0xc8 : //INY
				set_implied();
				y++; set_nz(y);
			break;*/


			case 0x9a : //TXS
				set_implied();
				s = x;
			break;

			case 0xba : //TSX
				set_implied();
				x = s; set_nz(x);
			break;

			case 0x98 : //TYA
				set_implied();
				a = y; aBig = yBig; set_nz(a);
			break;

			case 0x8a : //TXA
				set_implied();
				a = x; aBig = xBig; set_nz(a);
			break;

			case 0xa8 : //TAY
				set_implied();
				y = a; yBig = aBig; set_nz(y);
			break;

			case 0xaa : //TAX
				set_implied();
				x = a; xBig = aBig; set_nz(x);
			break;


			case 0x84 : //STY zero
				set_zero_write();
				ZeroWriteWide(mar.b.l, y, yBig);
			break;

			case 0x94 : //STY zero,x
				set_zero_i_write(x);
				ZeroWriteWide(mar.b.l, y, yBig);
			break;

			case 0x8c : //STY absolute
				set_absolute_write();
				WriteWide(mar, y, yBig);
			break;


			case 0x86 : //STX zero
				set_zero_write();
				ZeroWriteWide(mar.b.l, x, xBig);
			break;

			case 0x96 : //STX zero.y
				set_zero_i_write(y);
				ZeroWriteWide(mar.b.l, x, xBig);
			break;

			case 0x8e : //STX absolute
				set_absolute_write();
				WriteWide(mar, x, xBig);
			break;


/*			case 0xc0 : //CPY immediate
				set_immediate();
				cp(y, mdr);
			break;

			case 0xc4 : //CPY zero
				set_zero_read();
				cp(y, mdr);
			break;

			case 0xcc : //CPY absolute
				set_absolute_read();
				cp(y, mdr);
			break;


			case 0xe0 : //CPX immediate
				set_immediate();
				cp(x, mdr);
			break;

			case 0xe4 : //CPX zero
				set_zero_read();
				cp(x, mdr);
			break;

			case 0xec : //CPX absolute
				set_absolute_read();
				cp(x, mdr);
			break;


			case 0xc9 : //CMP immediate
				set_immediate();
				cp(a, mdr);
			break;

			case 0xc5 : //CMP zero
				set_zero_read();
				cp(a, mdr);
			break;

			case 0xd5 : //CMP zero, x
				set_zero_i_read(x);
				cp(a, mdr);
			break;

			case 0xcd : //CMP absolute
				set_absolute_read();
				cp(a, mdr);
			break;

			case 0xdd : //CMP absolute,x
				set_absolute_i_read(x);
				cp(a, mdr);
			break;

			case 0xd9 : //CMP absolute,y
				set_absolute_i_read(y);
				cp(a, mdr);
			break;

			case 0xc1 : //CMP (zero, x)
				set_b_zero_i_b_read(x);
				cp(a, mdr);
			break;

			case 0xd1 : //CMP (zero), y
				set_b_zero_b_i_read(y);
				cp(a, mdr);
			break;


			case 0x04 : //NOP [zero]
			case 0x44 : //NOP [zero]
			case 0x64 : //NOP [zero]
				set_zero_read();
			break;

			case 0x0c : //NOP [absolute]
				set_absolute_read();
			break;

			case 0x14 : //NOP [zero, x]
			case 0x34 : //NOP [zero, x]
			case 0x54 : //NOP [zero, x]
			case 0x74 : //NOP [zero, x]
			case 0xf4 : //NOP [zero, x]
				set_zero_i_read(x);
			break;

			case 0x1c : //NOP [absolute, x]
			case 0x3c : //NOP [absolute, x]
			case 0x5c : //NOP [absolute, x]
			case 0x7c : //NOP [absolute, x]
			case 0xdc : //NOP [absolute, x]
			case 0xfc : //NOP [absolute, x]
				set_absolute_i_read(x);
			break;

			case 0x1a : //NOP [implied]
			case 0x3a : //NOP [implied]
			case 0x5a : //NOP [implied]
			case 0x7a : //NOP [implied]
			case 0xda : //NOP [implied]
			case 0xea : //NOP [implied]
			case 0xfa : //NOP [implied]
				set_implied();
			break;

			case 0x03 : //SLO (zero,x)
				set_b_zero_i_b_modify(x);
				Read(mar, mdr);
				asl(mdr);
				ora();
				Write(mar, mdr);
			break;

			case 0x07 : //SLO zero
				set_zero_modify();
				ZeroRead(mar.b.l, mdr);
				asl(mdr);
				ora();
				ZeroWrite(mar.b.l, mdr);
			break;

			case 0x0f : //SLO absolute
				set_absolute_modify();
				Read(mar, mdr);
				asl(mdr);
				ora();
				Write(mar, mdr);
			break;

			case 0x13 : //SLO (zero),y
				set_b_zero_b_i_modify(y);
				Read(mar, mdr);
				asl(mdr);
				ora();
				Write(mar, mdr);
			break;

			case 0x17 : //SLO zero, x
				set_zero_i_modify(x);
				ZeroRead(mar.b.l, mdr);
				asl(mdr);
				ora();
				ZeroWrite(mar.b.l, mdr);
			break;

			case 0x1b : //SLO absolute,y
				set_absolute_i_modify(y);
				Read(mar, mdr);
				asl(mdr);
				ora();
				Write(mar, mdr);
			break;

			case 0x1f : //SLO absolute,x
				set_absolute_i_modify(x);
				Read(mar, mdr);
				ora();
				asl(mdr);
				Write(mar, mdr);
			break;

			case 0x2f : //RLA absolute
				set_absolute_modify();
				ReadWide(mar, mdr, mdrBig);
				rol(mdr, mdrBig);
				and();
				WriteWide(mar, mdr, mdrBig);
			break;*/

			case 0x63 : //RRA (zero,x)
				set_b_zero_i_b_modify(x);
				ModifyWide(mar, mdr, mdrBig, ror(mdr, mdrBig));
				adc();
			break;

			case 0x67 : //RRA zero
				set_zero_modify();
				ZeroModifyWide(mar.b.l, mdr, mdrBig, ror(mdr, mdrBig));
				adc();
			break;

			case 0x6f : //RRA absolute
				set_absolute_modify();
				ModifyWide(mar, mdr, mdrBig, ror(mdr, mdrBig));
				adc();
			break;

			case 0x73 : //RRA (zero),y
				set_b_zero_b_i_modify(y);
				ModifyWide(mar, mdr, mdrBig, ror(mdr, mdrBig));
				adc();
			break;

			case 0x77 : //RRA zero, x
				set_zero_i_modify(x);
				ZeroModifyWide(mar.b.l, mdr, mdrBig, ror(mdr, mdrBig));
				adc();
			break;

			case 0x7f : //RRA absolute,x
				set_absolute_i_modify(x);
				ModifyWide(mar, mdr, mdrBig, ror(mdr, mdrBig));
				adc();
			break;

			case 0x7b : //RRA absolute,y
				set_absolute_i_modify(y);
				ModifyWide(mar, mdr, mdrBig, ror(mdr, mdrBig));
				adc();
			break;


			case 0x87 : //SAX zero
				set_zero_write();
				sax();
				ZeroWriteWide(mar.b.l, mdr, mdrBig);
			break;

			case 0x8f : //SAX absolute
				set_absolute_write();
				sax();
				WriteWide(mar, mdr, mdrBig);
			break;

			case 0x9f : //SAX abs, x
				set_absolute_i_write(x);
				sax();
				WriteWide(mar, mdr, mdrBig);
			break;

			case 0x97 : //SAX zero,y
				set_zero_i_write(y);
				sax();
				ZeroWriteWide(mar.b.l, mdr, mdrBig);
			break;



			case 0xa7 : //LAX zero
				set_zero_read_w();
				x = a = mdr; set_nz(a);
				xBig = aBig = mdrBig;
			break;

			case 0xb7 : //LAX zero,x
				set_zero_i_read_w(x);
				x = a = mdr; set_nz(a);
				xBig = aBig = mdrBig;
			break;

			case 0xaf : //LAX absolute
				set_absolute_read_w();
				x = a = mdr; set_nz(a);
				xBig = aBig = mdrBig;
			break;

			case 0xbf : //LAX absolute, x
				set_absolute_i_read_w(x);
				x = a = mdr; set_nz(a);
				xBig = aBig = mdrBig;
			break;

			case 0xa3 : //LAX (zero, x)
				set_b_zero_i_b_read_w(x);
				x = a = mdr; set_nz(a);
				xBig = aBig = mdrBig;
			break;

			case 0xb3 : //LAX (zero), y
				set_b_zero_b_i_read_w(y);
				x = a = mdr; set_nz(a);
				xBig = aBig = mdrBig;
			break;
		}

		std_cpu_end();
	}

	timecounter = bigcounter+counter;
}

void C_6502Wide::SetIRQ(void)
{
	irq = true;
	check_irq();
}

#undef eor
#undef and
#undef ora

#undef cp
#undef asl
#undef lsr
#undef rol
#undef ror
#undef sbc
#undef adc
#undef add

#undef set_type
#undef set_part
#undef set_3part

#undef set_immediate
#undef set_implied

#undef set_absolute_read
#undef set_absolute_modify
#undef set_absolute_write

#undef set_zero_read
#undef set_zero_modify
#undef set_zero_write

#undef set_zero_i_read
#undef set_zero_i_modify
#undef set_zero_i_write

#undef set_absolute_i_read
#undef set_absolute_i_modify
#undef set_absolute_i_write

#undef set_b_zero_i_b_read
#undef set_b_zero_i_b_modify
#undef set_b_zero_i_b_write

#undef set_b_zero_b_i_read
#undef set_b_zero_b_i_modify
#undef set_b_zero_b_i_write

#undef push_instr
#undef pull_instr
#undef push
#undef pull

#undef set_jsr
#undef set_rts
#undef set_brk
#undef set_rti

#undef set_jmp_indirect
#undef set_jmp_absolute
#undef cond_branch
#undef set_nz

#undef StackRead
#undef StackWrite
#undef ZeroRead
#undef ZeroWrite
#undef Read
#undef Write