/*

	ElectrEm (c) 2000 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for
	details

	fs/posix/platfs.cpp
	===================

	Contains platform specific parts of C_FSAccess class, specifically related
	to genuine POSIX (UNIX) and half POSIX (DJGPP on DOS) targets

*/
#include "../../targets.h"

#if defined(TRG_FILE_POSIX_D) || defined(TRG_FILE_POSIX)

#include <malloc.h>
#include <stdio.h>
#include <string.h>
#include <dirent.h>
#include <sys/stat.h>
#include <unistd.h>

#include "../fs.h"

void C_FSAccess::Erase(char *path, char *setting, char *name)
{
	unlink(name);
}

#ifndef TRG_FILE_POSIX_D

#include <pwd.h>
#include <stdlib.h>

char *C_FSAccess::GetTrueName(char *name)
{
	char *firstname = name;
	char *newname;

	/* follow BASH convention that ~ means 'home directory' */

	if(*name == '~')
	{
		char *tmp = getenv("HOME");

		if(!tmp)
		{
			/*

			if HOME environment variable is not set, read system allocated
			home path for current user

			*/

			struct passwd *p = getpwuid(getuid());
			tmp = p->pw_dir;
		}

		newname = (char *)malloc(strlen(tmp)+strlen(name)+1);
		sprintf(newname, "%s%s", tmp, name+1);

		name = newname;
	}

	return (name == firstname) ? strdup(name) : name;
}

#endif

bool C_FSAccess::ValidPath(char *path, char *setting)
{
	char *tmp;
	DIR *cdir;
	char *dir = CorrectPath(path, setting);
	bool ret = false;

	if(!dir) return false;

	if(strlen(dir) > 1)
	{
		if(dir[strlen(dir)-1] != '/')
		{
			free(dir);
			return false;
		}
	}

	if(!strcmp(dir, "/"))
		ret = true;
	else
	{
		tmp = GetTrueName(dir);

		if(cdir = opendir(tmp))
		{
			closedir(cdir);
			ret = true;
		}
		free(tmp);
	}

	free(dir);

	return ret;
}

FileTree *C_FSAccess::GetFileTree(char *pth, char *setting, char **extensions)
{
	DIR *cdir;
	struct dirent *f;
	struct stat s;
	char *name, *ext, *path, **checkext, *pname;
	FileTree *first, *last, *current;
	unsigned int c;
	bool moveon;

	pname = CorrectPath(pth, setting);
	if(!pname) pname = strdup("");

	#ifdef TRG_FILE_POSIX_D
	if(!strcmp(pname, "/"))	//assumptions : drives a and b are present.
	{
		last = NULL;
		current = first = (FileTree *)malloc(sizeof(FileTree));

		current->fname = strdup("A:");
		current->dir = true;
		current->next = (FileTree *)malloc(sizeof(FileTree));
		last = current;
		current = current->next;

		current->fname = strdup("B:");
		current->dir = true;
		current->next = (FileTree *)malloc(sizeof(FileTree));
		last = current;
		current = current->next;

		name = (char *)malloc(4);

		for(c=0;c<24;c++)
		{
			sprintf(name, "%c:/", c+'C');
			if(!access(name, R_OK))
			{
				current->fname = (char *)malloc(3);
				sprintf(current->fname, "%c:", c+'C');
				current->dir = true;
				current->next = (FileTree *)malloc(sizeof(FileTree));
				last = current;
				current = current->next;
			}
		}

		if(last)
		{
			free(last->next);
				last->next = NULL;
		}

		free(name);
	}
	else
	{
	#endif
		path = GetTrueName(pname);

		if((cdir = opendir(path)))
		{
			last = NULL;
			current = first = (FileTree *)malloc(sizeof(FileTree));

			while((f = readdir(cdir)))
			{
				if(strcmp(f->d_name, "."))
				{
					if((!strcmp(f->d_name, "..") && strcmp(path, "/")) || *f->d_name != '.')
					{
						moveon = true;

						name = GetCompositePath(path, NULL, NULL, NULL, f->d_name);

						stat(name, &s);

						if(S_ISDIR(s.st_mode))
							current->dir = true;
						else
						{
							current->dir = false;

							moveon = ExtensionIncluded(f->d_name, extensions);
						}

						free(name);

						if(moveon)
						{
							current->fname = strdup(f->d_name);
							current->next = (FileTree *)malloc(sizeof(FileTree));
							last = current;
							current = current->next;
						}
					}
				}
			}

			closedir(cdir);

			if(last)
			{
				free(last->next);
				last->next = NULL;
			}
		}
		else
		{
			first = NULL;
		}
	#ifdef TRG_FILE_POSIX_D
	}
	#endif

	free(pname);
	return (first != NULL) ? SortFileTree(first) : NULL;
}

bool C_FSAccess::Access(char *path, char *setting, char *name, int r)
{
	char *fname;
	bool res;

	fname = GetCompositePath(path, setting, NULL, NULL, name);

	res = access(fname, r) ? false : true;
	free(fname);

	return res;
}

#ifdef TRG_FILE_POSIX_D
#include <fcntl.h>

char *C_FSAccess::GetCleanFilename(char *fname, char *extension)
{
	char *e, *r;
	char *exts[] = {extension, NULL};

	if(!ExtensionIncluded(fname, exts))
	{
		e = (char *)malloc(strlen(fname)+strlen(extension)+2);
		sprintf(e, "%s.%s", fname, extension);
	}
	else
		e = strdup(fname);

	if(!_USE_LFN)
	{
		_lfn_gen_short_fname(e, r);
	}
	else
		r = strdup(e);

	free(e);

	return r;
}
#else

char *C_FSAccess::GetCleanFilename(char *fname, char *extension)
{
	char *e, *ep;
	char *exts[] = {extension, NULL};

	if(!ExtensionIncluded(fname, exts))
	{
		e = (char *)malloc(strlen(fname)+strlen(extension)+2);
		sprintf(e, "%s.%s", fname, extension);
	}
	else
		e = strdup(fname);

	ep = e;
	while(*ep)
	{
		if(*ep == ' ') *ep = '_';
		ep++;
	}

	return e;
}
#endif

#endif

