/*

	ElectrEm (c) 2000 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for
	details

	fs/Win32/platfs.cpp
	===================

	Platform dependant parts of FS class for Windows target

*/
#include "../../targets.h"

#ifdef TRG_FILE_WIN32

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <malloc.h>
#include <stdio.h>
#include <direct.h>

#include "../fs.h"

void C_FSAccess::Erase(char *path, char *setting, char *name)
{
	_unlink(name);
}

bool C_FSAccess::ValidPath(char *path, char *setting)
{
	char *name;
	HANDLE fsystem;
	WIN32_FIND_DATA entry;
	bool ret = false;
	char *fdir, *dir;

	fdir = CorrectPath(path, setting);
	if(!fdir) return false;
	if(fdir[strlen(fdir)-1] != '/')
	{
		free(fdir);
		return false;
	}

	dir = GetTrueName(fdir);
	free(fdir);
	fdir = dir;

	name = (char *)malloc(strlen(dir)+4);
	sprintf(name, "%s*.*", dir);

	fsystem = FindFirstFile(name, &entry);
	if(fsystem != INVALID_HANDLE_VALUE)
	{
		FindClose(fsystem);
		ret= true;
	}

	free(name);
	free(fdir);

	return ret;
}

FileTree *C_FSAccess::GetFileTree(char *pth, char *setting, char **extensions)
{
	int drive;
	FileTree *current, *first, *last;
	HANDLE fsystem;
	WIN32_FIND_DATA entry;
	char *name, *temp, *tpath, *path;
	bool moveon;
	char *oldpath;
	int olddrive;

	tpath = CorrectPath(pth, setting);
	if(!tpath) tpath = strdup("");

	olddrive = _getdrive();
	oldpath = _getdcwd(olddrive, NULL, 1);

	if(strlen(tpath) == 1)
	{
		current = first = (FileTree *)malloc(sizeof(FileTree));
		last = NULL;

		for( drive = 1; drive <= 26; drive++ )
			if( !_chdrive( drive ) )
			{
				current->fname = (char *)malloc(3);
				sprintf(current->fname, "%c:", drive + 'A' - 1 );
				current->dir = true;
				current->next = (FileTree *)malloc(sizeof(FileTree));
				last = current;
				current = current->next;
			}

		if(last)
		{
			free(last->next);
			last->next = NULL;
		}
	}
	else
	{
		path = GetTrueName(tpath);

		temp = name = (char *)malloc(strlen(path)+4);
		sprintf(name, "%s*.*", path);

		last = NULL;
		current = first = (FileTree *)malloc(sizeof(FileTree));

		current->fname = strdup("..");
		current->dir = true;
		current->next = (FileTree *)malloc(sizeof(FileTree));
		last = current;
		current = current->next;

		fsystem = FindFirstFile(name, &entry);
		do
		{
			if(strcmp(entry.cFileName, ".") && strcmp(entry.cFileName, ".."))
			{
				moveon = true;

				if(entry.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY)
					current->dir = true;
				else
				{
					current->dir = false;
					moveon = ExtensionIncluded(entry.cFileName, extensions);
				}

				if(moveon)
				{
					current->fname = strdup(entry.cFileName);
					current->next = (FileTree *)malloc(sizeof(FileTree));
					last = current;
					current = current->next;
				}
			}
		}while(FindNextFile(fsystem, &entry));

		FindClose(fsystem);

		if(last)
		{
			free(last->next);
			last->next = NULL;
		}
		else
			return NULL;

		free(path);
	}

	_chdrive(olddrive);
	_chdir(oldpath);
	free(oldpath);
	free(tpath);

	return SortFileTree(first);
}

char *C_FSAccess::GetCleanFilename(char *fname, char *extension)
{
	char *e, *ep;
	char *exts[] = {extension, NULL};

	if(!ExtensionIncluded(fname, exts))
	{
		e = (char *)malloc(strlen(fname)+strlen(extension)+2);
		sprintf(e, "%s.%s", fname, extension);
	}
	else
		e = strdup(fname);

	ep = e;
	while(*ep)
	{
		if(	*ep == '\\' ||
			*ep == '/' ||
			*ep == ':' ||
			*ep == '*' ||
			*ep == '?' ||
			*ep == '<' ||
			*ep == '>' ||
			*ep == '|') *ep = '_';
		ep++;
	}

	return e;
}

#endif