/*

	ElectrEm (c) 2000 Thomas Harte - an Acorn tape-> Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for
	details

	fs/fs.cpp
	=========

	This source file is used to request pointers to UEF files. It is therefore
	used principly to allow several different bits (e.g. the tape interface,
	the disk drive and the snapshot mechanisms) of ElectrEm to access the
	same UEF files simultaneously, without the operating system file locks
	causing difficulties

	This code is new as of beta 6, and I still consider it a little suspect

*/
#include "../misc/config.h"
#include "../uef/uef.h"
#include <malloc.h>
#include <ctype.h>
#include "string.h"
#include "fs.h"

/* this is the highest UEF revision that will be opened */
#define UEF_VERSIONNUMBER 0x0008

C_FSAccess::C_FSAccess(C_config *conf, char *configname)
{
	char *name;

	cfg = conf;
	cfg->Open(name = GetTrueName(configname), this);
	free(name);

	uefs = NULL;
}

C_FSAccess::~C_FSAccess(void)
{
	cfg->Close();

	while(uefs)
	{
		ReleaseUEFFile(uefs->file);
	}
	uefs = NULL;
}

bool C_FSAccess::ExtensionIncluded(char *name, char **options)
{
	bool result = false;
	char *comp1, *comp2;

	while(*options && !result)
	{
		comp1 = name + strlen(name) - 1;
		comp2 = *options + strlen(*options) - 1;

		/*

			\a is used as a special character to signify that an extension
			may also be followed by .gz

		*/
		if(*comp2 == '\a')
		{
			comp2--;

			if((comp1-name) >=3 && !strcmp(comp1-2, ".gz")) comp1 -= 3;
		}

		while(comp2 > *options && comp1 > name && tolower(*comp2) == tolower(*comp1))
		{
			comp2--;
			comp1--;
		}

		if((comp2 == *options) && (tolower(*comp2) == tolower(*comp1))) result = true;

		options++;
	}

	return result;
}

char *C_FSAccess::CorrectPath(char *path, char *setting)
{
	char *ret = NULL;

	if(setting)
	{
		if(cfg->FindSetting(setting, STRING_RESULT))
		{
			if(ValidPath(cfg->ReadSetting().result_string, NULL))
				ret = strdup(cfg->ReadSetting().result_string);
		}
	}

	if(!ret && path)
	{
		ret = strdup(path);
	}

	return ret;
}

char *C_FSAccess::GetCompositePath(char *path1, char *setting1, char *path2, char *setting2, char *fname)
{
	char *pth1, *pth2, *newpath;

	pth1 = CorrectPath(path1, setting1);
	pth2 = CorrectPath(path2, setting2);

	if(pth2)
	{
		if(pth1[strlen(pth1)-1] == '/')
		{
			newpath = (char *)malloc(strlen(pth1) + strlen(pth2) + 1);
			sprintf(newpath, "%s%s", pth1, pth2);
		}
		else
		{
			newpath = (char *)malloc(strlen(pth1) + strlen(pth2) + 2);
			sprintf(newpath, "%s/%s", pth1, pth2);
		}

		free(pth1);
		free(pth2);
	}
	else
	{
		newpath = pth1;
	}

	if(newpath[strlen(newpath)-1] != '/')
	{
		char *tmp = (char *)malloc(strlen(newpath)+2), *old;
		sprintf(tmp, "%s/", newpath);
		old = newpath; newpath = tmp;
		free(old);
	}

	if(newpath[strlen(newpath)-2] == '.')
	{
		char *it;

		it = newpath + strlen(newpath) - 2;
		while(*it == '.') it--;
		it--;
		while(*it != '/') it--;
		it[1] = '\0';
	}

	if(fname)
	{
		char *old, *tmp = (char *)malloc(strlen(newpath)+strlen(fname)+1);
		sprintf(tmp, "%s%s", newpath, fname);
		old = newpath; newpath = tmp;
		free(old);
	}

	return newpath;
}

char *C_FSAccess::GetTrueName(char *path, char *name)
{
	char *ret2, *ret;

	if(!strlen(path))
		return strdup(name);

	if(path[strlen(path)-1] == '/')
	{
		ret = (char *)malloc(strlen(path) + strlen(name) + 1);
		sprintf(ret, "%s%s", path, name);
	}
	else
	{
		ret = (char *)malloc(strlen(path) + strlen(name) + 2);
		sprintf(ret, "%s/%s", path, name);
	}
	ret2 = GetTrueName(ret);
	free(ret);

	return ret2;
}

void C_FSAccess::SplitName(char *name, char **newname, char **newpath)
{
	/* must split the name 'name' into a path and filename */

	char *parser;

	parser = name + strlen(name) - 1;
	while(parser > name && *parser != '/') parser--;

	if(parser == name)
	{
		*newname = strdup(name);
		*newpath = strdup("");
	}
	else
	{
		parser++;
		*newname = strdup(parser);

		*newpath = (char *)calloc(parser-name+1, 1);
		memcpy(*newpath, name, parser-name);
	}
}

void C_FSAccess::GetName(char *path, char *setting, char *name, char **newname, char **newpath)
{
	if(!(*newpath = CorrectPath(path, setting)))
	{
		SplitName(name, newname, newpath);
	}
	else
		*newname = strdup(name);
}

C_UEFFile *C_FSAccess::RequestUEFFile(char *path, char *setting, char *ofname, char *mode)
{
	char *loc, *fname;

	GetName(path, setting, ofname, &fname, &loc);

	/* see if the requested UEF is already open */
	bool found = false;
	uefstore *ueffind = uefs;

	while(ueffind && !found)
	{
		if(!strcmp(ueffind->name, fname) && !strcmp(ueffind->path, loc))
			found = true;
		else
			ueffind = ueffind->next;
	}

	/* if this UEF file is already open, then work here is done */
	if(found)
	{
		free(loc);
		free(fname);

		ueffind->refcount++;

		return ueffind->file;
	}

	/* if this UEF is not already open, it must be opened */
	C_UEFFile *newuef;
	char *tmp;

	newuef = new C_UEFFile;
	if(newuef->Open(tmp = GetTrueName(loc, fname), UEF_VERSIONNUMBER, mode))
	{
		if(!uefs)
		{
			ueffind = uefs = new uefstore;
		}
		else
		{
			ueffind = uefs;
			while(ueffind->next)
				ueffind = ueffind->next;

			ueffind->next = new uefstore;
			ueffind = ueffind->next;
		}

		ueffind->next = NULL;
		ueffind->refcount = 1;
		ueffind->name = strdup(fname);
		ueffind->path = strdup(loc);
		ueffind->file = newuef;
	}
	else
	{
		delete newuef;
		newuef = NULL;
	}

	free(tmp);
	free(loc);
	free(fname);

	return newuef;
}

gzFile C_FSAccess::GetGZ(char *path, char *setting, char *ofname, char *mode)
{
	char *loc, *fname, *tmp;
	gzFile f;

	GetName(path, setting, ofname, &fname, &loc);

	f = gzopen(tmp = GetTrueName(loc, fname), mode);

	free(loc);
	free(fname);
	free(tmp);

	return f;
}

void C_FSAccess::ReleaseGZ(gzFile f)
{
	gzclose(f);
}

FILE *C_FSAccess::GetFile(char *path, char *setting, char *ofname, char *mode)
{
	char *loc, *fname, *tmp;
	FILE * f;

	GetName(path, setting, ofname, &fname, &loc);

	f = fopen(tmp = GetTrueName(loc, fname), mode);

	free(loc);
	free(fname);
	free(tmp);

	return f;
}

void C_FSAccess::ReleaseFile(FILE *f)
{
	fclose(f);
}

void C_FSAccess::ReleaseUEFFile(C_UEFFile *selt)
{
	/* find which file this is referring to */
	if(uefs)
	{
		uefstore *ueffind = uefs;

		while(ueffind && ueffind->file != selt)
		{
			ueffind = ueffind->next;
		}

		if(ueffind)
		{
			if(ueffind->file == selt)
			{
				ueffind->refcount--;

				if(!ueffind->refcount)
				{
					if(!uefs->next)
					{
						uefs = NULL;
					}
					else
					{
						if(ueffind == uefs)
						{
							uefs = uefs->next;
						}
						else
						{
							uefstore *uef2 = uefs;
							while(uef2->next != ueffind)
							{
								uef2 = uef2->next;
							}
							uef2->next = ueffind->next;
						}
					}

					free(ueffind->name);
					free(ueffind->path);
					ueffind->file->Close();
					delete ueffind->file;
					free(ueffind);
				}
			}
		}
	}
}

C_UEFChunkSelector *C_FSAccess::RequestUEFChunkSelector(char *path, char *setting, char *fname, char *mode)
{
	C_UEFFile *local;

	if(local = RequestUEFFile(path, setting, fname, mode))
		return local->GetSelectorPtr();
	else
		return NULL;
}

void C_FSAccess::ReleaseUEFChunkSelector(C_UEFChunkSelector *selt)
{
	C_UEFFile *local = selt->GetFather();

	local->ReleaseSelectorPtr(selt);
	ReleaseUEFFile(local);
}

bool C_FSAccess::FLower(FileTree *st1, FileTree *st2)
{
	char *t1, *t2;

	if(st1->dir && !st2->dir)
		return true;

	if(!st1->dir && st2->dir)
		return false;

	t1 = st1->fname;
	t2 = st2->fname;

	while(*t1 && *t2 && tolower(*t1) == tolower(*t2))
	{
		t1++;
		t2++;
	}

	return tolower(*t1) < tolower(*t2);
}

FileTree *C_FSAccess::SortFileTree(FileTree *in)
{
	FileTree *newtree, *s, *l, *next;

	newtree = in;

	if(in)
	{
		in = in->next;
		newtree->next = NULL;

		while(in)
		{
			next = in->next;

			l = NULL;
			s = newtree;
			while(s && FLower(s, in))
			{
				l = s;
				s = s->next;
			}

			if(l)
			{
				in->next = l->next;
				l->next = in;
			}
			else
			{
				in->next = newtree;
				newtree = in;
			}
			in = next;
		}
	}

	return newtree;
}

void C_FSAccess::FreeFileTree(FileTree *first)
{
	FileTree *next;

	while(first)
	{
		next = first->next;
		free(first->fname);
		free(first);
		first = next;
	}
}

void C_FSAccess::SetPath(char *path, char *setting)
{
	if(!ValidPath(NULL, setting))
	{
		bool valid = false;;
		config_setting newresult;

		/* okay, but perhaps just the final '/' was missing */
		if(cfg->FindSetting(setting, STRING_RESULT))
		{
			char *tmp;
			tmp = cfg->ReadSetting().result_string;
			if(tmp[strlen(tmp)-1] != '/')
			{
				tmp = (char *)malloc(strlen(cfg->ReadSetting().result_string)+2);
				sprintf(tmp, "%s/", cfg->ReadSetting().result_string);

				if(valid = ValidPath(tmp, NULL))
				{
					newresult.result_string = tmp;
					newresult.result_type = STRING_RESULT;
					cfg->WriteSetting(newresult);
				}

				free(tmp);
			}
		}

		if(!valid)
		{
			/* nope, either the setting doesn't exist, or it is totally wrong */
			newresult.result_string = path;
			newresult.result_type = STRING_RESULT;

			cfg->FindSetting(setting, ANY_RESULT);
			cfg->WriteSetting(newresult);
		}
	}
}
