#ifndef FS_H
#define FS_H

struct FileTree
{
	char *fname;
	bool dir;
	FileTree *next;
};

#include <stdio.h>
#include <zlib.h>
class C_UEFChunkSelector;
class C_UEFFile;
class C_config;

#if defined(TRG_FILE_POSIX) || defined(TRG_FILE_POSIX_D)
#include <unistd.h>
#define ACCESS_READ		R_OK
#define ACCESS_WRITE	W_OK
#define ACCESS_EXISTS	F_OK
#endif

#ifdef TRG_FILE_WIN32
#define ACCESS_READ		4
#define ACCESS_WRITE	2
#define ACCESS_EXISTS	0
#endif

class C_FSAccess
{
	public :
		C_FSAccess(C_config *settings, char *configname);
		~C_FSAccess(void);

		bool Access(char *path, char *setting, char *name, int r);
		void Erase(char *path, char *setting, char *name);
		void Rename(char *path1, char *setting1, char *name1, char *path2, char *setting2, char *name2);

		char *GetCleanFilename(char *fname, char *extension);

		char *GetCompositePath(char *path1, char *setting1, char *path2, char *setting2, char *fname);
		bool ExtensionIncluded(char *name, char **options);
		bool ValidPath(char *path, char *setting);

		FileTree *GetFileTree(char *path, char *setting, char **extensions);
		void FreeFileTree(struct FileTree *first);

		C_UEFChunkSelector *RequestUEFChunkSelector(char *path, char *setting, char *fname, char *mode);
		void ReleaseUEFChunkSelector(C_UEFChunkSelector *selt);

		C_UEFFile *RequestUEFFile(char *path, char *setting, char *fname, char *mode);
		void ReleaseUEFFile(C_UEFFile *selt);

		gzFile GetGZ(char *path, char *setting, char *ofname, char *mode);
		void ReleaseGZ(gzFile f);

		FILE *GetFile(char *path, char *setting, char *ofname, char *mode);
		void ReleaseFile(FILE *f);

		void SetPath(char *path, char *setting);
		char *GetPath(char *setting);

	private :
		char *GetTrueName(char *name);
		char *GetTrueName(char *path, char *name);
		void SplitName(char *name, char **newname, char **newpath);
		void GetName(char *path, char *setting, char *name, char **newname, char **newpath);

		FileTree *SortFileTree(FileTree *in);
		bool FLower(FileTree *st1, FileTree *st2);
		char *CorrectPath(char *path, char *setting);

		C_config *cfg;

		struct uefstore
		{
			C_UEFFile *file;
			char *name;
			char *path;
			int refcount;
			uefstore *next;
		} *uefs;
};

#endif