/*

	ElectrEm (c) 2000 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for
	details

	gfx/Allegro/platbitmap.cpp
	==========================

	This file makes up my 'bitmap' class Allegro dependant bits, i.e. pixel
	access to the bitmap and the create/destroy bits

*/

#include "../../targets.h"
#ifdef TRG_GFX_ALLEGRO

#include "../gfx.h"

bool C_bitmap::Destroy(void)
{
	if(surface)
	{
		destroy_bitmap(surface);
		surface = NULL;
	}

	return false;
}

bool C_bitmap::PlatformCreate(C_gfx_api *apiref, int w, int h, DSPSTRUCT intended, int pal)
{
	if((surface = create_system_bitmap(w, h)))
		clear(surface);
	else
		return false;

	return true;
}

void C_bitmap::Blit(void) // blits to the 'centre' of the screen, or stretches as appropriate
{
	if(slevel >= rslevel)
		stretch_blit(surface, screen, 0, 0, surface->w, surface->h, StretchArea.left, StretchArea.top, StretchArea.right-StretchArea.left, StretchArea.bottom-StretchArea.top);
	else
		blit(surface, screen, 0, 0, tx, ty, surface->w, surface->h);
}

bool C_bitmap::BeginDraw(void)
{
	acquire_bitmap(surface);
	bmp_select(surface);
	pitch = surface->w;
	return true;
}

bool C_bitmap::BeginRead(void)
{
	acquire_bitmap(surface);
	bmp_select(surface);
	pitch = surface->w;
	return true;
}

void C_bitmap::EndOperation(void)
{
	release_bitmap(surface);
}

unsigned __int8 *C_bitmap::GetLinePtr(int y)
{
	return (unsigned __int8 *)bmp_write_line(surface, y);
}

void C_bitmap::ReleaseLinePtr(void)
{
	bmp_unwrite_line(surface);
}

/*

these next two are incorrect for a general Allegro BITMAP, but as I'm only
using memory bitmaps are okay

*/

unsigned __int8 *C_bitmap::GetDLinePtr(int y)
{
	return (unsigned __int8 *)bmp_write_line(surface, y);
}

void C_bitmap::ReleaseDLinePtr(void)
{
	bmp_unwrite_line(surface);
}

#endif
