/*

	ElectrEm (c) 2000 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for
	details

	gfx/allegro/platgfx.cpp
	=======================

	Contains platform specific basics of C_gfx_api for the Allegro target

*/
#include "../../targets.h"
#ifdef TRG_GFX_ALLEGRO

#include "../../ula/ula.h"
#include "../../misc/config.h"

#include "../gfx.h"
#include <allegro.h>
#include <string.h>

BEGIN_COLOR_DEPTH_LIST
COLOR_DEPTH_8
END_COLOR_DEPTH_LIST

#ifdef TARG_OS_DOS
BEGIN_GFX_DRIVER_LIST
GFX_DRIVER_VBEAF
GFX_DRIVER_VESA3
GFX_DRIVER_VESA2L
GFX_DRIVER_VESA2B
GFX_DRIVER_VESA1
GFX_DRIVER_MODEX
END_GFX_DRIVER_LIST
#endif

void C_gfx_api::PlatformConstructor(void)
{
}

void C_gfx_api::PlatformShutdown(void)
{
}

struct known_modes
{
	int w, h;
};

/* modes searched when aspect ratio must be good */
known_modes some_modes_odd[] =
{
	{800, 600},
	{640, 400},
	{400, 300},
	{-1, -1}
},
some_modes_safe[] =
{
	{800, 600},
	{640, 480},
	{320, 400},
	{-1, -1}
},
*some_modes;

DSPSTRUCT C_gfx_api::GetGenuineSize(int w, int h)
{
	known_modes *c = some_modes;

	while(c->w != -1 && c->w >= w && c->h >= h)
	{
		c++;
	}
	c--;

	DSPSTRUCT rtangle;

	rtangle.width = c->w;
	rtangle.height = c->h;

	return rtangle;
}

bool C_gfx_api::SetResolution(DSPSTRUCT newsize)
{
	if(ScreenArea.width != newsize.width || ScreenArea.height != newsize.height)
	{
		set_color_depth(8);
		if(set_gfx_mode(vesa ? GFX_VESA2L : GFX_AUTODETECT, newsize.width, newsize.height, 0, 0) < 0)
		{
			errstring = "[Allegro] Failed at set_gfx_mode";
			return false;
		}
		set_window_title(name); //just in case someone compiles for not DOS

		ScreenArea = newsize;
		SetLastPalette();
	}

	clear(screen);
	return true;
}

bool C_gfx_api::PlatformSetup(char *appname)
{
	bool prefsafe = false;

	/* check for platform specific option! */
	if(elk->GetConfig()->FindSetting("DOS_usevesa", BOOLEAN_RESULT))
	{
		vesa = elk->GetConfig()->ReadSetting().result_bool;
	}
	else
		vesa = false;

	if(elk->GetConfig()->FindSetting("DOS_preferorthodox", BOOLEAN_RESULT))
	{
		prefsafe = elk->GetConfig()->ReadSetting().result_bool;
	}

	some_modes = prefsafe ? some_modes_safe : some_modes_odd;

	name = strdup(appname);
	if(install_mouse() < 1)
	{
		errstring = "[Allegro] Failed at install_mouse";
		return false;
	}
	remove_mouse();
	CursorRequired = true;

	return true;
}

void C_gfx_api::PlatformSetPalette(int palnum)
{
	PALETTE sccols;
	int c;

	c = 256;
	while(c--)
	{
		sccols[c].r = (int)(63*colsets[palnum][c].red);
		sccols[c].g = (int)(63*colsets[palnum][c].green);
		sccols[c].b = (int)(63*colsets[palnum][c].blue);
	}
	set_palette(sccols);
}

void C_gfx_api::PlatformFinishPalette(int pal)
{
}

/* Mouse stuff from here onwards */

void C_gfx_api::EnableMouse(void)
{
	install_mouse();
	set_mouse_range(0, 0, 640, 512);
}

void C_gfx_api::DisableMouse(void)
{
	remove_mouse();
}

MOUSE_STRUCT C_gfx_api::GetMouseState(void)
{
	MOUSE_STRUCT minfo;

	minfo.x = mouse_x;
	minfo.y = mouse_y;
	minfo.lbutton = mouse_b&1;

	return minfo;
}

#endif