/*

	ElectrEm (c) 2000 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for details

	gfx/DirectX/platbitmap.cpp
	==========================

	Contains platform dependant parts of C_bitmap for DirectX
*/
#ifdef TARGET_WIN32

#include "../gfx.h"

bool C_bitmap::Destroy(void)
{
	if(surface)
	{
		surface->Release();
		surface = NULL;
	}

	return true;
}

bool C_bitmap::PlatformCreate(C_gfx_api *apiref, int w, int h, DSPSTRUCT intended, int pal)
{
	DDRAW_INIT_STRUCT(sdesc);

	sdesc.dwFlags = DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH;
	sdesc.dwHeight = h;
	sdesc.dwWidth = w;
	sdesc.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN | DDSCAPS_SYSTEMMEMORY;

	if(FAILED(apiref->lpdd->CreateSurface(&sdesc, &surface, NULL)))
		return false;

	sdesc.dwFlags = 0;

	return true;
}

void C_bitmap::Blit(void) // blits to the 'centre' of the screen, or stretches as appropriate
{
	HRESULT res;

	if(slevel >= rslevel)
	{
		res = api->front->Blt(&StretchArea, surface, NULL, DDBLT_WAIT, NULL);

		if(res == DDERR_SURFACELOST)
		{
			api->front->Restore();
			api->front->Blt(&StretchArea, surface, NULL, DDBLT_WAIT, NULL);
		}
	}
	else
	{
		res = api->front->BltFast(tx, ty, surface, NULL, DDBLTFAST_NOCOLORKEY | DDBLTFAST_WAIT);

		if(res == DDERR_SURFACELOST)
		{
			api->front->Restore();
			api->front->BltFast(tx, ty, surface, NULL, DDBLTFAST_NOCOLORKEY | DDBLTFAST_WAIT);
		}
	}
}

bool C_bitmap::BeginDraw(void)
{
	HRESULT res;

	res = surface->Lock(NULL, &sdesc, DDLOCK_WAIT | DDLOCK_SURFACEMEMORYPTR | DDLOCK_WRITEONLY | DDLOCK_NOSYSLOCK, NULL);

	if(res == DDERR_SURFACELOST)
	{
		surface->Restore();
		res = surface->Lock(NULL, &sdesc, DDLOCK_WAIT | DDLOCK_SURFACEMEMORYPTR | DDLOCK_WRITEONLY | DDLOCK_NOSYSLOCK, NULL);
	}

	if(res != DD_OK)
		return false;
	else
	{
		pitch = sdesc.lPitch;
		base = (unsigned __int8 *)sdesc.lpSurface;
		return true;
	}
}

bool C_bitmap::BeginRead(void)
{
	HRESULT res;

	res = surface->Lock(NULL, &sdesc, DDLOCK_WAIT | DDLOCK_SURFACEMEMORYPTR | DDLOCK_READONLY | DDLOCK_NOSYSLOCK, NULL);

	if(res == DDERR_SURFACELOST)
	{
		surface->Restore();
		res = surface->Lock(NULL, &sdesc, DDLOCK_WAIT | DDLOCK_SURFACEMEMORYPTR | DDLOCK_READONLY | DDLOCK_NOSYSLOCK, NULL);
	}

	if(res != DD_OK)
		return false;
	else
	{
		pitch = sdesc.lPitch;
		base = (unsigned __int8 *)sdesc.lpSurface;
		return true;
	}
}

void C_bitmap::EndOperation(void)
{
	surface->Unlock(NULL);
}

unsigned __int8 *C_bitmap::GetLinePtr(int y)
{
	return base + (y*pitch);
}

unsigned __int8 *C_bitmap::GetDLinePtr(int y)
{
	return base + (y*pitch);
}

void C_bitmap::ReleaseLinePtr(void)
{
}

void C_bitmap::ReleaseDLinePtr(void)
{
}

#endif