/*

	ElectrEm (c) 2000 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for details

	gfx/DirectX/platgfx.cpp
	=======================

	Contains platform dependant basics of the C_gfx_api class for DirectX.
	Recently downgraded to DirectDraw 1.0 code, although that was hardly
	the greatest programming challenge of my life . . . .
*/
#ifdef TARGET_WIN32

#include "../gfx.h"
#include "../../defines.h"
#include "../../ula/ula.h"
#include <stdio.h>

#define WIN32_LEAN_AND_MEAN
#include <windows.h>

bool lbutton;
C_gfx_api *gfx_api;

LRESULT CALLBACK DefaultWndProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam)
{
	RECT rect;

	switch(msg)
	{	
		case WM_LBUTTONDBLCLK :
		case WM_LBUTTONDOWN :
			lbutton = true;
		return 0;

		case WM_LBUTTONUP :
			lbutton = false;
		return 0;

		case WM_CREATE: 
		return 0;

		case WM_ACTIVATE:
			switch(LOWORD(wparam))
			{
				case WA_ACTIVE : //activated by some means other than a mouse click
				case WA_CLICKACTIVE : //activated by a mouse click
					if(gfx_api)
						gfx_api->Restore();
				break;

				case WA_INACTIVE : //deactivated
				break;
			}
		return 0;

		case WM_CHAR :
			if(gfx_api->gui_active)
			{
				aqueue[qpos] = (TCHAR)wparam;
				qpos = (qpos+1)&255;
			}
		break;

		case WM_PAINT: 
			GetClientRect(hwnd, &rect);
			ValidateRect(hwnd, &rect);
		return 0;

/*		case WM_KEYDOWN :
			if(gui_active)
			{
				MSG t;

				t.hwnd = hwnd;
				t.message = msg;
				t.wParam = wparam;
				t.lParam = lparam;

				TranslateMessage(&t);
			}
		return 0;*/

		case WM_CLOSE: 
			PostQuitMessage(0);
		return 0;

		default:break;
	}

	return DefWindowProc(hwnd, msg, wparam, lparam);
}

void C_gfx_api::Restore(void)
{
	if(gui_active)
		SetCursor(LoadCursor(NULL, IDC_ARROW));
	else
		elk->WakeUp();
}

void C_gfx_api::PlatformConstructor(void)
{
	lpdd = NULL;
	front = NULL;
	className = NULL;
	pal = NULL;
	gui_active = false;
	gfx_api = NULL;
}

void C_gfx_api::PlatformShutdown(void)
{
	gfx_api = NULL;
	ShowCursor(true);

	if(pal)
	{
		pal->Release();
		pal = NULL;
	}

	if(front)
	{
		front->Release();
		front = NULL;
	}

	if(lpdd)
	{
		lpdd->Release();
		lpdd = NULL;
	}

	if(hWnd)
	{
		DestroyWindow(hWnd);
		hWnd = winfo.hWnd = NULL;
	}

	if(className)
	{
		UnregisterClass(className, className+3); //winfo.hInstance ??
		delete className;
		className = NULL;
	}
}

unsigned int rw, rh, tw, th;
int closedist;

HRESULT WINAPI displaycheck(LPDDSURFACEDESC sdesc, LPVOID lpContext)
{
	int difx, dify, dist;

	if((sdesc->dwHeight >= rh) && (sdesc->dwWidth >= rw) && (sdesc->ddpfPixelFormat.dwFlags&DDPF_PALETTEINDEXED8))
	{
		difx = sdesc->dwWidth - rw;
		dify = sdesc->dwHeight - rh;
		dist = difx*difx + dify*dify;

		if(closedist == -1 || dist < closedist)
		{
			tw = sdesc->dwWidth;
			th = sdesc->dwHeight;
			closedist = dist;
		}
	}

	return DDENUMRET_OK;
}

DSPSTRUCT C_gfx_api::GetGenuineSize(int w, int h)
{
	closedist = -1;
	rw = w;
	rh = h;
	lpdd->EnumDisplayModes(0, NULL, NULL, displaycheck);

	DSPSTRUCT ret;
	ret.width = tw; ret.height = th;

	return ret;
}

bool C_gfx_api::SetResolution(DSPSTRUCT newsize)
{
	if(ScreenArea.width != newsize.width || ScreenArea.height != newsize.height)
	{
		ScreenArea = newsize;

		if(pal)
		{
			pal->Release();
			pal = NULL;
		}

		if(front)
		{
			front->Release();
			front = NULL;
		}

		if(FAILED(lpdd->SetDisplayMode(newsize.width, newsize.height, 8)))
		{
			errstring = "[DirectX] failed at SetDisplayMode";
			return false;
		}

		DDRAW_INIT_STRUCT(ddsd);

		ddsd.dwFlags = DDSD_CAPS;
		ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE;

		if(FAILED(lpdd->CreateSurface(&ddsd, &front, NULL)))
		{
			errstring = "[DirectX] failed at CreateSurface";
			return false;
		}

		if(FAILED(lpdd->CreatePalette(DDPCAPS_8BIT, pcols, &pal, NULL)))
		{
			errstring = "[DirectX] failed at CreatePalette";
			return false;
		}

		if(FAILED(front->SetPalette(pal)))
		{
			errstring = "[DirectX] failed at SetPalette";
			return false;
		}

		SetLastPalette();
	}

	return true;
}

bool C_gfx_api::PlatformSetup(char *appname)
{
	WNDCLASSEX winclass =
	{
		sizeof(WNDCLASSEX),
		CS_DBLCLKS | CS_OWNDC | CS_HREDRAW | CS_VREDRAW,
		DefaultWndProc,
		0, 0,
		winfo.hInstance,
		LoadIcon(NULL, IDI_APPLICATION),
		NULL,
		GetStockObject(BLACK_BRUSH),
		NULL,
		NULL,
		LoadIcon(NULL, IDI_APPLICATION)
	};

	gfx_api = this;
	gui_active = false;
	CursorRequired = false;
	className = new char[strlen(appname)+4];
	sprintf(className, "WC_%s", appname);
	winclass.lpszClassName = className;

	if (!RegisterClassEx(&winclass))
	{
		errstring = "[Windows] failed at RegisterClassEx";
		return false;
	}

	if (!(hWnd = CreateWindowEx(	NULL,
									className,
									appname,
									WS_POPUP | WS_VISIBLE,
									0,0,
									100,100,
									NULL,
									NULL,
									winfo.hInstance,
									NULL)))
	{
		errstring = "[Windows] failed at CreateWindowEx";
		return false;
	}

	winfo.hWnd = hWnd;

	if(FAILED(DirectDrawCreate(NULL, &lpdd, NULL)))
	{
		errstring = "[DirectX] failed at DirectDrawCreate";
		return false;
	}

	if(FAILED(lpdd->SetCooperativeLevel(hWnd, DDSCL_FULLSCREEN | DDSCL_EXCLUSIVE | DDSCL_ALLOWREBOOT)))
	{
		errstring = "[DirectX] failed at SetCooperativeLevel";
		return false;
	}

	ShowCursor(false);

	return true;
}

void C_gfx_api::PlatformFinishPalette(int pal)
{
}

void C_gfx_api::PlatformSetPalette(int palnum)
{
	int c;

	c = 256;
	while(c--)
	{
		pcols[c].peRed = (int)(255*colsets[palnum][c].red);
		pcols[c].peGreen = (int)(255*colsets[palnum][c].green);
		pcols[c].peBlue = (int)(255*colsets[palnum][c].blue);
	}

	pal->SetEntries(0, 0, 256, pcols);
}

/* And the mouse related stuff . . . . */

MOUSE_STRUCT C_gfx_api::GetMouseState(void)
{
	POINT details;
	MOUSE_STRUCT minfo;
	GetCursorPos(&details);

	minfo.x = details.x - backupscr.tx;
	minfo.y = details.y - backupscr.ty;
	minfo.lbutton = lbutton;

	return minfo;
}

void C_gfx_api::EnableMouse(void)
{
	ShowCursor(TRUE);
	SetCursor(LoadCursor(NULL, IDC_ARROW));
	gui_active = true;
}

void C_gfx_api::DisableMouse(void)
{
	ShowCursor(FALSE);
	gui_active = false;
}

#endif