/*

	ElectrEm (c) 2000 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for details

	gfx/GGI/platgfx.cpp
	===================

	Contains GGI dependant basics of the C_gfx_api class
*/
#include "../../targets.h"

#ifdef TRG_GFX_GGI

#include "../gfx.h"
#include "../../defines.h"

#include <malloc.h>
#include <stdio.h>

int mx, my;

void C_gfx_api::PlatformConstructor(void)
{
	mode = NULL;
}

void C_gfx_api::PlatformShutdown(void)
{
	if(mode)
	{
		ggiClose(mode);
		mode = NULL;
	}
	ggiExit();
}

DSPSTRUCT C_gfx_api::GetGenuineSize(int w, int h)
{
	char text[100];
	DSPSTRUCT rt;

	sprintf(text, "%dx%d.[8]", w, h);
	ggiParseMode(text, &rt.mode);
	ggiCheckMode(mode, &rt.mode);

	rt.width = rt.mode.visible.x;
	rt.height = rt.mode.visible.y;

	return rt;
}

bool C_gfx_api::SetResolution(DSPSTRUCT newsize)
{
	ggi_mode mde;
	char text[100];

	if(newsize.width != ScreenArea.width || newsize.height != ScreenArea.height)
	{
		ScreenArea = newsize;

		if(ggiSetMode(mode, &newsize.mode))
		{
			errstring = "[GGI] unable to set any mode";
			return false;
		}

		if(GT_DEPTH(newsize.mode.graphtype) != 8)
		{
			errstring = "[GGI] Not an 8bit mode. If you cannot switch to an 8bpp mode, try setting the environment variable 'GGI_DISPLAY' to 'palemu' if you are running at over 8bpp";
			return false;
		}
	}

	return true;
}

bool C_gfx_api::PlatformSetup(char *appname)
{
	if(ggiInit())
	{
		errstring = "[GGI] unable to init";
		return false;
	}

	if(!(mode = ggiOpen(NULL)))
	{
		errstring = "[GGI] failed at ggiOpen";
		return false;
	}

	ggiSetFlags(mode, GGIFLAG_ASYNC);
	ggiVIS = mode;
	CursorRequired = true;

	mbutton = false;
	mx = my = 0;

	return true;
}

void C_gfx_api::PlatformFinishPalette(int pal)
{
}

void C_gfx_api::PlatformSetPalette(int pal)
{
	ggi_color pale;
	int c;
	float multiplier;

	multiplier = (float)((1 << GGI_COLOR_PRECISION) - 1);

	c = 256;
	while(c--)
	{
		if(colsets[pal][c].set)
		{
			pale.r = (int)(multiplier*colsets[pal][c].red);
			pale.g = (int)(multiplier*colsets[pal][c].green);
			pale.b = (int)(multiplier*colsets[pal][c].blue);
			ggiSetPalette(mode, c, 1, &pale);
		}
	}
}

/*

Mouse stuff from here onwards.

*/

void C_gfx_api::EnableMouse(void)
{
}

void C_gfx_api::DisableMouse(void)
{
}

MOUSE_STRUCT C_gfx_api::GetMouseState(void)
{
	gii_event event;
	gii_event_mask mask;
	struct timeval t={0,0};
	int x, y;
	bool bresult = mbutton;

	x = mx; y = my;

	/* with GII, must parse mouse events. Which isn't terribly difficult */

	while((mask = ggiEventPoll(mode, emPointer, &t)))
	{
		ggiEventRead(mode, &event, emPointer);

		switch(event.any.type)
		{
			default :
				fprintf(stderr, "unknown event type : %d\n", event.any.type);
			break;

			case evPtrRelative :
				mx += event.pmove.x;
				my += event.pmove.y;
			break;

			case evPtrAbsolute :
				mx = event.pmove.x;
				my = event.pmove.y;
			break;

			case evPtrButtonPress :
				bresult = mbutton = true;
			break;

			case evPtrButtonRelease :
				mbutton = false;
			break;
		}
	}

	if(mx < 0) mx = 0;
	if(my < 0) my = 0;
	if(mx >= backupscr.width) mx = backupscr.width-1;
	if(my >= backupscr.height) my = backupscr.height-1;

	MOUSE_STRUCT minfo;
	minfo.x = mx;
	minfo.y = my;
	minfo.lbutton = bresult;

	return minfo;
}

#endif
