/*

	ElectrEm (c) 2000/1 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for
	details

	gfx/SDL/platbitmap.cpp
	======================

	This is the NULL graphics driver. This file acts both as a useful starter
	for and as documentation for what needs to be done when start a new port

*/

#include "../../targets.h"
#ifdef TRG_GFX_SDL

#include "../gfx.h"

/*

causes a C_bitmap to clean up after itself, returning resources to
the system, etc

theoretically returns true on success false on failure, but the result isn't
checked at this time

*/
bool C_bitmap::Destroy(void)
{
	if(surface)
	{
		SDL_FreeSurface(surface);
		surface = NULL;
	}

	return true;
}

/*

this is a call to generate a new bitmap, of size w and h which is to be used
by the C_gfx_api pointed to, on the display described in DSPSTRUCT.

returns true on success, false on failure

*/
bool C_bitmap::PlatformCreate(C_gfx_api *apiref, int w, int h, DSPSTRUCT intended, int pal)
{
	surface = SDL_CreateRGBSurface(0, w, h, 8, 0, 0, 0, 0);
	SDL_SetColors(surface, apiref->screen->format->palette->colors, 0, 256);

	return true;
}

/*

this procuedure acts to copy the bitmap to the screen, and has to perform
scaling of the bitmap if required.

If the test (slevel >= rslevel) returns true, the bitmap should be scaled
to fit the area described by class member RECT StretchArea

Otherwise it should be blitted at 1:1 scale to the location (tx, ty)

*/
void C_bitmap::Blit(void)
{
	SDL_Rect scr;

/*	if(slevel >= rslevel)
	{
		scr.x = StretchArea.left;
		scr.y = StretchArea.top;
		scr.w = StretchArea.right - StretchArea.left;
		scr.h = StretchArea.bottom - StretchArea.top;
	}
	else
	{*/
		scr.x = tx;
		scr.y = ty;
		scr.w = width;
		scr.h = height;
//	}

	SDL_BlitSurface(surface, NULL, api->screen, &scr);
	SDL_UpdateRects(api->screen, 1, &scr);
}

/*

BeginDraw signifies that a load of drawing operations are about to begin, and
gives a chance to lock or acquire the bitmap if required on this target.

Result is true on success, false on failure

*/
bool C_bitmap::BeginDraw(void)
{
	SDL_LockSurface(surface);
	return true;
}

/*

BeginDraw signifies that a load of reading operations are about to begin, and
gives a chance to lock or acquire the bitmap if required on this target.

Result is true on success, false on failure

*/
bool C_bitmap::BeginRead(void)
{
	SDL_LockSurface(surface);
	return true;
}

/*

EndOperation is used to signify that reading or drawing is finished, and is
called after one of the two above functions

*/
void C_bitmap::EndOperation(void)
{
	SDL_UnlockSurface(surface);
}

/*

GetLinePtr returns a pointer to the memory representing the scanline of pixels
located at line y. Returns NULL on error, a valid pointer otherwise.

This is only called inbetween a Begin??? and EndOperation pair

*/
unsigned __int8 *C_bitmap::GetLinePtr(int y)
{
	return (unsigned __int8 *)surface->pixels + surface->pitch*y;
}

/*

A call to ReleaseLinePtr signifies that operations on the pointer last
returned by GetLinePtr are all done

*/
void C_bitmap::ReleaseLinePtr(void)
{
}

/*

GetDLinePtr is as GetLinePtr, except that it must provide a pointer at which
a first line of pixels may be written or read, and from which the line at
y+1 follows 'pitch' (a public class member) bytes later

*/
unsigned __int8 *C_bitmap::GetDLinePtr(int y)
{
	pitch = surface->pitch;
	return (unsigned __int8 *)surface->pixels + surface->pitch*y;
}

/*

as for ReleaseLinePtr, but to GetDLinePtr

*/
void C_bitmap::ReleaseDLinePtr(void)
{
}

#endif
