/*

	ElectrEm (c) 2000 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for details

	gfx/SVGALib/platgfx.cpp
	=======================

	Contains SVGALib dependant basics of the C_gfx_api class
*/
#include "../../targets.h"

#ifdef TRG_GFX_SVGALIB

#include "../gfx.h"

#include <malloc.h>
#include <vga.h>
#include <vgamouse.h>

C_gfx_api *mouse;

void C_gfx_api::PlatformConstructor(void)
{
}

void C_gfx_api::PlatformShutdown(void)
{
}

DSPSTRUCT C_gfx_api::GetGenuineSize(int w, int h)
{
	vga_modeinfo *inf;
	int c, dist, lowdist = -1;
	DSPSTRUCT ret;

	c = vga_lastmodenumber();

	while(c--)
	{
		if(vga_hasmode(c))
		{
			inf = vga_getmodeinfo(c);

			if((inf->width >= w) && (inf->height >= h) && ((inf->bytesperpixel == 1) || (inf->colors == 256)))
			{
				dist = (w-inf->width)*(w-inf->width) + (h-inf->height)*(h-inf->height);
				if((lowdist == -1) || (dist < lowdist))
				{
					lowdist = dist;
					ret.modenum = c;
					ret.width = inf->width;
					ret.height = inf->height;
				}
			}
		}
	}

	return ret;
}

bool C_gfx_api::SetResolution(DSPSTRUCT area)
{
	vga_modeinfo *inf;
	int c, dist, lowdist = -1;
	unsigned __int8 *tmp;

	if(ScreenArea.width != area.width || ScreenArea.height != area.height)
	{
		ScreenArea = area;

		if(vga_setmode(area.modenum))
		{
			errstring = "[SVGALib] unable to set selected mode";
			return false;
		}
		else
		{
			inf = vga_getmodeinfo(area.modenum);
			if(inf->flags & CAPABLE_LINEAR)
				vga_setlinearaddressing();

			SetLastPalette();
			tmx = (inf->width >> 1);
			tmy = (inf->height >> 1);
		}
	}

	inf = vga_getmodeinfo(vga_getcurrentmode());
	tmp = (unsigned __int8 *)calloc(inf->width, 1);
	for(c = 0; c < inf->height; c++)
		vga_drawscansegment(tmp, 0, c, inf->width);
	free(tmp);

	return true;
}

bool C_gfx_api::PlatformSetup(char *appname)
{
	if(vga_init())
	{
		errstring = "[SVGALib] failed at vga_init";
		return false;
	}

	CursorRequired = true;
	return true;
}

void C_gfx_api::PlatformSetPalette(int pal)
{
	int c;
	c = 256;
	while(c--)
		vga_setpalette(c, (int)(63*colsets[pal][c].red), (int)(63*colsets[pal][c].green), (int)(63*colsets[pal][c].blue));
}

void C_gfx_api::PlatformFinishPalette(int pal)
{
}

/*

Mouse stuff from here onwards.

*/

void C_gfx_api::UpdateMouse(int button, int dx, int dy, int dz, int drx, int dry, int drz)
{
	tmx += dx >> 5;
	tmy += dy >> 5;

	if(tmy < mylbound)
		tmy = mylbound;
	if(tmy > myhbound)
		tmy = myhbound;
	if(tmx < mxlbound)
		tmx = mxlbound;
	if(tmx > mxhbound)
		tmx = mxhbound;

	mbutton = button;
}

void mouse_handler(int button, int dx, int dy, int dz, int drx, int dry, int drz)
{
	mouse->UpdateMouse(button, dx, dy, dz, drx, dry, drz);
}

void C_gfx_api::FixMouseArea(void)
{
	mxlbound = backupscr.tx; mxhbound = backupscr.tx+backupscr.width-1;
	mylbound = backupscr.ty; myhbound = backupscr.ty+backupscr.height-1;

	if(tmx < mxlbound || tmx > mxhbound)
		tmx = mxlbound+mxhbound >> 1;

	if(tmy < mylbound || tmy > myhbound)
		tmy = mylbound+myhbound >> 1;
}

void C_gfx_api::EnableMouse(void)
{
	FixMouseArea();
	if(!mouse_init("", vga_getmousetype(), MOUSE_DEFAULTSAMPLERATE))
	{
		mouse_seteventhandler(mouse_handler);
		mouse = this;

		mbutton = 0;
	}
}

void C_gfx_api::DisableMouse(void)
{
	mouse_close();
}

MOUSE_STRUCT C_gfx_api::GetMouseState(void)
{
	mouse_update();

	MOUSE_STRUCT minfo;

	minfo.x = tmx - mxlbound;
	minfo.y = tmy - mylbound;
	minfo.lbutton = (mbutton&MOUSE_LEFTBUTTON) ? true : false;
	return minfo;
}

#endif
