/*

	ElectrEm (c) 2000 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for
	details

	gfx/bitmap.cpp
	==============

	This file makes up platform independant part of my 'bitmap' class. That
	is really just the PCX dump code & the GetError thing.

*/
#include "gfx.h"
#include <sys/stat.h>
#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <math.h>

C_bitmap::C_bitmap(void)
{
	surface = NULL;
}

C_bitmap::~C_bitmap(void)
{
	Destroy();
}

unsigned char C_bitmap::GetPel(int x, int y)
{
	unsigned char col;

	BeginRead();

	col = GetLinePtr(y)[x];
	ReleaseLinePtr();

	EndOperation();

	return col;
}

void C_bitmap::DumpToFile(char *name)
{
	bool mallocd = false;
	struct stat r;
	int c;
	FILE *outpcx;
	unsigned char col, count;
	unsigned short pitch, wval;
	unsigned char rubbish[54], *vptr, basecol;
	int x, y, tc, basex, basey;
	int scalex, scaley;
	int twidth, theight;

	if(!name)
	{
		mallocd = true;
		name = (char *)malloc(13);
		c = 0;

		do
		{
			sprintf(name, "snap%04d.pcx", c);
			c++;
		} while(!stat(name, &r));
	}

	/*
		calculate the minimum size this image can be scaled down to before
		loosing any pixels
	*/
	scalex = width;
	scaley = height;

	BeginRead();
	for(y = 0; y < height; y++)
	{
		vptr = GetLinePtr(y);
		x = 0;

		while(x < width)
		{
			basex = x;
			basecol = *vptr;
			while((*vptr == basecol) && (x < width))
			{
				vptr++;
				x++;
			}

			if((x-basex) < scalex) scalex = x-basex;
		}
	}

	for(x = 0; x < width; x++)
	{
		y = 0;

		while(y < height)
		{
			basey = y;
			basecol = GetPel(x, y);
			while((GetPel(x, y) == basecol) && (y < height))
			{
				y++;
			}

			if((y-basey) < scaley) scaley = y-basey;
		}
	}


	EndOperation();

	/*

		ensure scalex and scaley are powers of 2

	*/

	tc = scalex;
	c = 32;
	while(c--)
	{
		if((tc&1) && (tc&~1))
		{
			scalex = 1;
			break;
		}
		tc >>= 1;
	}

	tc = scaley;
	c = 32;
	while(c--)
	{
		if((tc&1) && (tc&~1))
		{
			scaley = 1;
			break;
		}
		tc >>= 1;
	}

	/* */

	twidth = width / scalex;
	theight = height / scaley;

	memset(rubbish, 0, 54);

	if((outpcx = fopen(name, "wb")))
	{
		fputc(10, outpcx);
		fputc(5, outpcx);
		fputc(1, outpcx);
		fputc(8, outpcx);

		pitch = 0; fwrite(&pitch, 2, 1, outpcx);
		pitch = 0; fwrite(&pitch, 2, 1, outpcx);
		pitch = twidth-1; fwrite(&pitch, 2, 1, outpcx);
		pitch = theight-1; fwrite(&pitch, 2, 1, outpcx);

		///assume a 15" screen	=> 15^2 = x^2 + y^2, and x = 4*y/3
		//						=> 15^2 = (4*y/3)^2 + y^2
		//						=> 225 = (25/9)y^2
		//						=> y = 9.581231654 => x = 11.54123044
		pitch = (int)((float)twidth / 11.54123044f);
		fwrite(&pitch, 2, 1, outpcx);
		pitch = (int)((float)theight / 9.581231654f);
		fwrite(&pitch, 2, 1, outpcx);

		fwrite(rubbish, 1, 48, outpcx); //16 colour palette (unimportant)
		fputc(0, outpcx); //reserved

		//65 to here

		fputc(1, outpcx);
		wval = twidth; //should check width is a power of two really
		fwrite(&wval, 2, 1, outpcx);

		wval = 1;
		fwrite(&wval, 2, 1, outpcx);

		wval = twidth;
		fwrite(&wval, 2, 1, outpcx);
		wval = theight;
		fwrite(&wval, 2, 1, outpcx);
		
		fwrite(rubbish, 1, 54, outpcx); //header padding (unimportant)

		BeginRead();
		for(y = 0; y < height; y+= scaley)
		{
			tc = 0;

			vptr = GetLinePtr(y);
			x = twidth;
			while(tc < twidth)
			{
				count = 0;
				col = *vptr;

				while((count < 63) && x && (*vptr == col))
				{
					x--;
					count++;
					vptr+= scalex;
				}

				tc += count;

				if((count > 1) || (col&192))
				{
					count |= 192;

					fputc(count, outpcx);
					fputc(col, outpcx);
				}
				else
					fputc(col, outpcx);
			}

			ReleaseLinePtr();
		}
		EndOperation();

		//palette now
		putc(12, outpcx);
		for(x = 0; x < 256; x++)
		{
			putc((int)(api->GetPaletteR(x, false)*255), outpcx);
			putc((int)(api->GetPaletteG(x, false)*255), outpcx);
			putc((int)(api->GetPaletteB(x, false)*255), outpcx);
		}

		fclose(outpcx);
	}

	if(mallocd)
		free(name);
}

char *C_bitmap::GetError()
{
	return errtext;
}

int C_bitmap::GetStretchLevel(void)
{
	return slevel;
}

bool C_bitmap::GetCleanScale(void)
{
	return cleanscale;
}

void C_bitmap::SetStretchLevel(int newlev, bool clean)
{
	int wastedpels, v, a;

	slevel = newlev;
	cleanscale = clean;

	rslevel = 2;

	//TEST 1 : scaling level goes to 1 if plenty of pixels are wasted
	wastedpels = (ScreenArea.width*ScreenArea.height) - (width*height);
	if(((float)wastedpels / (float)(width*height)) > 0.5f)
		rslevel = 1;

	//TEST 1 : scaling level goes to 1 if aspect ratio is particularly bad
	if( fabs( ((float)ScreenArea.width / (float)width) - ((float)ScreenArea.height / (float)height) ) > 0.1)
		rslevel = 1;

	if(cleanscale)
	{
		v = width;
		a = width >> 1;
		while((v+a) <= ScreenArea.width)
			v += a;

		StretchArea.left = (ScreenArea.width >> 1) - (v >> 1);
		StretchArea.right = StretchArea.left + v;

		v = height;
		a = height >> 1;
		while((v+a) <= ScreenArea.height)
			v += a;

		StretchArea.top = (ScreenArea.height >> 1) - (v >> 1);
		StretchArea.bottom = StretchArea.top + v;
	}
	else
	{
		StretchArea.left = StretchArea.top = 0;
		StretchArea.right = ScreenArea.width;
		StretchArea.bottom = ScreenArea.height;
	}
}

bool C_bitmap::Create(C_gfx_api *apiref, int w, int h, DSPSTRUCT intended, int pal)
{
	api = apiref;

	width = w;
	height = h;
	ScreenArea = intended;
	tx = ( (intended.width >> 1) - (w >> 1) );
	ty = ( (intended.height >> 1) - (h >> 1) );

	return PlatformCreate(apiref, w, h, intended, pal);
}
