/*

	ElectrEm (c) 2000 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for
	details

	gfx/gfx.cpp
	===========

	Contains platform independant basics of the C_gfx_api class

*/
#include "gfx.h"
#include <math.h>
#include <malloc.h>

C_gfx_api::C_gfx_api(void)
{
	PlatformConstructor();

	lastcols = PAL_ELK;
	fonts = NULL;
	imgs = NULL;
}

C_gfx_api::~C_gfx_api(void)
{
	Shutdown();
}

void C_gfx_api::Shutdown(void)
{
	backupscr.Destroy();
	FreeData();

	PlatformShutdown();
}

bool C_gfx_api::Setup(int w, int h, char *appname, C_ULA *elec)
{
	int c;
	DSPSTRUCT tmp;

	elk = elec;

	if(!PlatformSetup(appname))
		return false;

	ScreenArea.width = ScreenArea.height = 0;

	tmp = GetGenuineSize(w, h);
	if(!SetResolution(tmp))
	{
		return false;
	}

	ScreenArea = tmp;

	tmp = GetGenuineSize(640, 512);
	if(!backupscr.Create(this, 640, 512, tmp, PAL_GUI))
	{
		errstring = backupscr.GetError();
		return false;
	}

	backupscr.SetStretchLevel(0, false);

	c = 256;
	while(c--)
	{
		colsets[0][c].red = colsets[0][c].green = colsets[0][c].blue =
		colsets[1][c].red = colsets[1][c].green = colsets[1][c].blue = 0.0f;

		colsets[0][c].set = colsets[1][c].set = false;
	}

	return true;
}

void C_gfx_api::SetPaletteEntry(int id, float r, float g, float b, int pal)
{
	colsets[pal][id].red = r;
	colsets[pal][id].green = g;
	colsets[pal][id].blue = b;
	colsets[pal][id].set = true;
}

float C_gfx_api::GetPaletteR(int id, int pal)
{
	return colsets[pal][id].red;
}

float C_gfx_api::GetPaletteG(int id, int pal)
{
	return colsets[pal][id].green;
}

float C_gfx_api::GetPaletteB(int id, int pal)
{
	return colsets[pal][id].blue;
}

void C_gfx_api::FinishPalette(int pal)
{
	PlatformFinishPalette(pal);

	if(pal == lastcols)
		SetLastPalette();
}

void C_gfx_api::SetLastPalette(void)
{
	SetPalette(lastcols);
}

void C_gfx_api::SetPalette(int pal)
{
	lastcols = pal;
	PlatformSetPalette(pal);
}

void C_gfx_api::SetNormalResolution(int w, int h)
{
	BackupArea = GetGenuineSize(w, h);
}

char *C_gfx_api::GetError(void)
{
	return errstring;
}