#ifndef GFX_H
#define GFX_H

#include "../types.h"
#include "../targets.h"
#include <string.h>

#define TRG_GFX_GLOBALS
#include TRG_GFX_INCLUDE
#undef TRG_GFX_GLOBALS

#ifndef RECT_DEFINED
struct RECT
{
	int top, bottom, left, right;
};
#endif

#define PAL_GUI 0
#define PAL_ELK 1
#define GFX_NUMPALS 2

class C_bitmap;
class C_gfx_api;

class C_ULA;

struct DSPSTRUCT
{
	int width, height;
#define TRG_GFX_DSPSTRUCT_MEMBERS
#include TRG_GFX_INCLUDE
#undef TRG_GFX_DSPSTRUCT_MEMBERS
};

struct Image
{
	unsigned __int16 w, h;
	unsigned __int8 *data;
	int *runs, *mask;
	bool mnew;
};

struct Font
{
	Image *chars;
	int charstart, charend, fonth;
};

class C_bitmap
{
	public:
		C_bitmap(void);
		~C_bitmap(void);

		bool Create(C_gfx_api *apiref, int w, int h, DSPSTRUCT intended, int pal);
		bool Destroy(void);

		bool BeginDraw(void);
		bool BeginRead(void);
		void EndOperation(void);

		void DumpToFile(char *name);

		unsigned __int8 *GetLinePtr(int y);
		void ReleaseLinePtr(void);

		unsigned __int8 *GetDLinePtr(int y);
		void ReleaseDLinePtr(void);

		unsigned char GetPel(int x, int y);
		void Blit(void);

		char *GetError(void);

		int pitch;

		void SetStretchLevel(int newlev, bool cleanscale);
		int GetStretchLevel(void);
		bool GetCleanScale(void);

#define TRG_GFX_BITMAP_PUMEMBERS
#include TRG_GFX_INCLUDE
#undef TRG_GFX_BITMAP_PUMEMBERS

	private :
		C_gfx_api *api;

#define TRG_GFX_BITMAP_PRMEMBERS
#include TRG_GFX_INCLUDE
#undef TRG_GFX_BITMAP_PRMEMBERS

		bool PlatformCreate(C_gfx_api *apiref, int w, int h, DSPSTRUCT intended, int pal);

		RECT StretchArea;
		DSPSTRUCT ScreenArea;

		int width, height, tx, ty;
		int slevel, rslevel;
		bool cleanscale;
		friend C_gfx_api;

		char *errtext;
};

struct MOUSE_STRUCT
{
	int x, y;
	bool lbutton;
};

#ifndef C_FSAccess
class C_FSAccess;
#endif

class C_gfx_api
{
	public :
		char *GetError(void);

		~C_gfx_api(void);
		C_gfx_api(void);

		bool Setup(int w, int h, char *appname, C_ULA *elec);
		void Shutdown(void);
		void SetNormalResolution(int w, int h);

		void FinishPalette(int pal);
		void SetPalette(int pal);
		void SetPaletteEntry(int id, float r, float g, float b, int pal);
		float GetPaletteR(int id, int pal);
		float GetPaletteG(int id, int pal);
		float GetPaletteB(int id, int pal);

		/*

		GUI draw things

		*/

		bool LoadData(char *psetting, char *name, C_FSAccess *fs);

		void Line(int x1, int y1, int x2, int y2, int colour);
		void AALine(int x1, int y1, int x2, int y2, int colour);
		void Rectangle(int x1, int y1, int x2, int y2, int colour);
		void PutImage(int x, int y, int imgid, bool transparent);
		int ImgWidth(int imgid);
		int ImgHeight(int imgid);

		void SetMaskByColours(int dest, int src, int ccount, int *cols);
		int GetColour(int bmp, int x, int y);
		void ResetMask(int dest);

		bool ImgPresent(int id);
		bool FontPresent(int id);

		int WriteChar(int font, char letter, int x, int y, int cbase);
		int WriteString(int font, char *text, int x, int y, int cbase);
		int StrWidth(int font, char *text);
		int StrHeight(int font, char *text);

		void SetClipArea(int x1, int y1, int x2, int y2);

		bool EnableGUI(void);
		bool DisableGUI(void);

		void BeginRedraw(void);
		void EndRedraw(void);

		/* These next two implemented on a platform by platform basis */

		MOUSE_STRUCT GetMouseState(void);

		DSPSTRUCT GetGenuineSize(int w, int h);

#define TRG_GFX_API_PUMEMBERS
#include TRG_GFX_INCLUDE
#undef TRG_GFX_API_PUMEMBERS

	friend C_bitmap;

	private :
		C_bitmap backupscr;
		C_ULA *elk;

		Font *fonts;
		Image *imgs;

		struct colours{
			float red, green, blue;
			bool set;
#define TRG_GFX_API_CLBONUSMEMS
#include TRG_GFX_INCLUDE
#undef TRG_GFX_API_CLBONUSMEMS
		}colsets[2][256];

		DSPSTRUCT ScreenArea, BackupArea;
		RECT ClipArea;
		char *errstring;

		void PutImage(int x, int y, Image *img, bool transparent, void (* mcpy)(void *d, const void *s, size_t n));
		void PreprocessImage(Image *img);
		void SetMaskByColours(Image *dest, Image *src, int ccount, int *cols, bool inc);

		int lastcols;
		void SetLastPalette(void);
		void FreeData(void);
		bool ColIncluded(int colour, int ccount, int *cols);

		/*

		these things actually implemented on a platform by platform basis

		*/

		bool SetResolution(DSPSTRUCT newsize);
		bool PlatformSetup(char *appname);
		void PlatformConstructor(void);
		void PlatformShutdown(void);
		void PlatformSetPalette(int pal);
		void PlatformFinishPalette(int pal);

		bool CursorRequired;
		void EnableMouse(void);
		void DisableMouse(void);

#define TRG_GFX_API_PRMEMBERS
#include TRG_GFX_INCLUDE
#undef TRG_GFX_API_PRMEMBERS
};

#endif
