/*

	ElectrEm (c) 2000 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for
	details

	gfx/gfxdraw.cpp
	===============

	Does the GUI related drawing - rectangles, text, images and lines. Also
	loads the fonts and images and that.

	NOTICE : this file is not intended to be the most optimised functions in
	the world. It has some optimisations over the very most basic cases (e.g.
	RLE sprites, rather than simple ones) but still is primarily designed
	just to work!

*/
#include "gfx.h"
#include "../uef/uef.h"
#include "../fs/fs.h"
#include <string.h>
#include <malloc.h>
#include "gfxdefs.h"

void C_gfx_api::SetClipArea(int x1, int y1, int x2, int y2)
{
	if(x2 < x1)
	{
		x2 ^= x1; x1 ^= x2; x2 ^= x1;
	}
	if(y2 < y1)
	{
		y2 ^= y1; y1 ^= y2; y2 ^= y1;
	}

	ClipArea.top = y1;
	ClipArea.bottom = y2;
	ClipArea.left = x1;
	ClipArea.right = x2;
}

unsigned __int32 cb32;
unsigned __int16 cb16;
unsigned __int8 cb8;

void setbase(int c)
{
	cb8 = c;
	cb16 = c | (c << 8);
	cb32 = cb16 | (cb16 << 16);
}

void cbasecpy(void *dest, const void *src, size_t num)
{
	unsigned __int8 *d = (unsigned __int8 *)dest, *s = (unsigned __int8 *)src;
	int c;

	// become DWORD aligned
	while(((int)d&3) && num)
	{
		*d++ = cb8 | *s++;
		num--;
	}

	if(!num) return;

	// do as many DWORD copies as possible
	c = num >> 2;
	while(c--)
	{
		*(unsigned __int32 *)d = cb32 | *(unsigned __int32 *)s;
		d += 4; s += 4;
	}

	// finish off with bytes
	c = num&3;
	while(c--)
	{
		*d++ = cb8 | *s++;
	}
}

int C_gfx_api::WriteChar(int font, char l, int x, int y, int cbase)
{
	int letter = (l >= 0) ? l : 256+l;

	if(letter == 156)
		letter = 163;

	if(letter == '\t')
	{
		return 32-(x&31);
	}
	else
		if(letter >= fonts[font].charstart && letter <= fonts[font].charend)
		{
			setbase(cbase);
			letter -= fonts[font].charstart;
			PutImage(x, y, &fonts[font].chars[letter], true, cbasecpy);

			return fonts[font].chars[letter].w;
		}

	return 0;
}

int C_gfx_api::WriteString(int font, char *text, int x, int y, int cbase)
{
	int ox = x;

	while(*text)
	{
		x += WriteChar(font, *text, x, y, cbase);
		if(*text == '\n') {x = ox; y += fonts[font].fonth;}
		text++;
	}

	return x - ox;
}

int C_gfx_api::StrWidth(int font, char *text)
{
	int w = 0;
	int highw = 0;

	while(*text)
	{
		if( (*text >= fonts[font].charstart) && (*text <= fonts[font].charend))
			w += fonts[font].chars[(*text) - fonts[font].charstart].w;

		if( *text == '\t')
			w += 32 - (w&31);

		if( *text == '\n')
		{
			if(w > highw)
			{
				highw = w;
			}
			w = 0;
		}

		text++;
	}

	return (w > highw) ? w : highw;
}

int C_gfx_api::StrHeight(int font, char *text)
{
	/* value is (1 + quantity of \n's)*font height */
	int c = 1;

	while(*text)
	{
		if(*text == '\n') c++;
		text++;
	}

	return c*fonts[font].fonth;
}

int C_gfx_api::ImgWidth(int imgid)
{
	return imgs[imgid].w;
}

int C_gfx_api::ImgHeight(int imgid)
{
	return imgs[imgid].h;
}

#define Bound(v, ll, ul) {if(v < ll) v = ll; if(v > ul) v = ul;}

void C_gfx_api::Rectangle(int x1, int y1, int x2, int y2, int colour)
{
	int y;

	Bound(x1, ClipArea.left, ClipArea.right);
	Bound(x2, ClipArea.left, ClipArea.right);

	Bound(y1, ClipArea.top, ClipArea.bottom);
	Bound(y2, ClipArea.top, ClipArea.bottom);

	if(!(x2-x1))
		return;

	backupscr.BeginDraw();

	for(y = y1; y < y2; y++)
	{
		memset(&backupscr.GetLinePtr(y)[x1], colour, x2-x1);
	}

	backupscr.EndOperation();
}

void C_gfx_api::Line(int x1, int y1, int x2, int y2, int colour)
{
	int xdiff, ydiff;
	int xadd, yadd;
	int c;
	int tx, ty;

	xdiff = abs(x2 - x1);
	ydiff = abs(y2 - y1);

	x1 <<= 8;
	x2 <<= 8;
	y1 <<= 8;
	y2 <<= 8;

	if(!xdiff && !ydiff)
		return;

	if(xdiff < ydiff)
	{
		c = ydiff;
		yadd = (y2 < y1) ? -256 : +256;
		xadd = (x2 - x1) / ydiff;
	}
	else
	{
		c = xdiff;
		xadd = (x2 < x1) ? -256 : +256;
		yadd = (y2 - y1) / xdiff;
	}

	c++;
	backupscr.BeginDraw();
	while(c--)
	{
		tx = x1 >> 8;
		ty = y1 >> 8;

		backupscr.GetLinePtr(ty)[tx] = colour;
		backupscr.ReleaseLinePtr();

		x1 += xadd;
		y1 += yadd;
	}
	backupscr.EndOperation();
}

#define modpel(element, add, base) \
					element = (element&7) + add;\
					if(element > 7) element = 7;\
					element += base;

void C_gfx_api::AALine(int x1, int y1, int x2, int y2, int colour)
{
	int xdiff, ydiff;
	int xadd, yadd;
	int c;
	bool horiz;

	xdiff = abs(x2 - x1);
	ydiff = abs(y2 - y1);

	if(!xdiff || !ydiff)
	{
		Line(x1, y1, x2, y2, colour+7);
		return;
	}

	x1 <<= 8;
	x2 <<= 8;
	y1 <<= 8;
	y2 <<= 8;

	if(xdiff < ydiff)
	{
		c = ydiff;
		yadd = (y2 < y1) ? -256 : +256;
		xadd = (x2 - x1) / ydiff;

		//draw horizontal segments
		horiz = true;
	}
	else
	{
		c = xdiff;
		xadd = (x2 < x1) ? -256 : +256;
		yadd = (y2 - y1) / xdiff;

		//draw vertical segments
		horiz = false;
	}

	unsigned __int8 *l;
	int tx, ty;
	int index;

	c++;
	backupscr.BeginDraw();
	while(c--)
	{
		tx = x1 >> 8;
		ty = y1 >> 8;

		if(horiz)
		{
			if(ty >= ClipArea.top && ty < ClipArea.bottom)
			{
				l = backupscr.GetLinePtr(ty);
				index = (x1 >> 5)&7;

				if(tx >= ClipArea.left && tx < ClipArea.right)
				{
					modpel(l[tx], 7-index, colour);
				}

				tx ++;
				if(tx >= ClipArea.left && tx < ClipArea.right)
				{
					modpel(l[tx], index, colour);
				}

				backupscr.ReleaseLinePtr();
			}
		}
		else
		{
			if(tx >= ClipArea.left && tx < ClipArea.right)
			{
				index = (y1 >> 5)&7;

				if(ty >= ClipArea.top && ty < ClipArea.bottom)
				{
					l = backupscr.GetLinePtr(ty);
					modpel(l[tx], 7-index, colour);
					backupscr.ReleaseLinePtr();
				}

				ty ++;
				if(ty >= ClipArea.top && ty < ClipArea.bottom)
				{
					l = backupscr.GetLinePtr(ty);
					modpel(l[tx], index, colour);
					backupscr.ReleaseLinePtr();
				}
			}
		}

		x1 += xadd;
		y1 += yadd;
	}
	backupscr.EndOperation();
}

#undef modpel

void C_gfx_api::PutImage(int x, int y, Image *img, bool transparent, void (* mcpy)(void *d, const void *s, size_t n))
{
	/*

	This draws one of the pixmaps, optionally with colour 0 transparent

	*/

	if(!backupscr.BeginDraw()) return;

	if(!transparent)
	{
		int c;
		unsigned __int8 *line, *write;
		int xc, xp;

		c = img->h;
		line = img->data;

		while(c-- && y < ClipArea.bottom)
		{
			if(y >= ClipArea.top)
			{
				write = backupscr.GetLinePtr(y);

				xc = img->w;
				xp = x;

				if(x < ClipArea.left)
				{
					xp = ClipArea.left;
					xc -= ClipArea.left - x;
				}

				if(x + xc > ClipArea.right)
				{
					xc -= x + xc - ClipArea.right;
				}

				memcpy(&write[xp], &line[xp-x], xc);
				line += img->w;

				backupscr.ReleaseLinePtr();
			}
			else
				line += img->w;

			y++;
		}
	}
	else
	{
		int *parser, count, xp, tc;
		unsigned __int8 *sbase, *sprbase, *scr, *scrl, *scrr, *oscr, *ospr;

		parser = img->mask;
		sbase = img->data;
		count = 0;

		while(*parser != -2 && y < ClipArea.bottom)
		{
			xp = x;

			if(y >= ClipArea.top && *parser >= 0)
			{
				sprbase = sbase;

				scr = backupscr.GetLinePtr(y);
				scrl = scr + ClipArea.left;
				scrr = scr + ClipArea.right - 1;
				scr += x;

				while(*parser >= 0)
				{
					//skip *parser pixels
					sprbase += *parser;
					scr += *parser;
					parser++;

					//draw *parser pixels
					tc = *parser;
					oscr = scr;
					ospr = sprbase;

					if(scr < scrl && (scr+tc) >= scrl)
					{
						sprbase += scrl - scr;
						tc -= scrl - scr;
						scr = scrl;
					}

					if((scr + tc) > scrr)
					{
						tc = scrr-scr;
					}

					if(scr >= scrl && scr <= scrr)
						mcpy(scr, sprbase, tc);

					//advance pointers
					scr = oscr + *parser;
					sprbase = ospr + *parser;
					parser++;
				}

				backupscr.ReleaseLinePtr();
			}
			else
			{
				while(*parser >= 0)
					parser++;

				sprbase += img->w;
			}

			y++;
			sbase += img->w;
			if(*parser == -1) parser++;
		}
	}

	backupscr.EndOperation();
}

typedef void (* mptr)(void *d, const void *s, size_t n);

void C_gfx_api::PutImage(int x, int y, int imgid, bool transparent)
{
	PutImage(x, y, &imgs[imgid], transparent, (mptr)memcpy);
}

void C_gfx_api::FreeData(void)
{
	int c, c2;

	if(fonts)
	{
		c = 16;
		while(c--)
		{
			if(fonts[c].chars)
			{
				c2 = fonts[c].charend - fonts[c].charstart;
				while(c2--)
				{
					free(fonts[c].chars[c2].data);
					free(fonts[c].chars[c2].runs);
				}

				free(fonts[c].chars);
			}
		}
		free(fonts);
		fonts = NULL;
	}

	if(imgs)
	{
		c = 256;
		while(c--)
		{
			if(imgs[c].data)
			{
				free(imgs[c].data);
				free(imgs[c].runs);

				if(imgs[c].mnew) free(imgs[c].mask);
			}
		}

		free(imgs);
		imgs = NULL;
	}
}

bool C_gfx_api::LoadData(char *psetting, char *name, C_FSAccess *fs)
{
	C_UEFChunkSelector *in;
	int c, charpos;
	char *temp;
	bool res;
	Font *cfont;
	Image *cimage;
	int idminor;
	unsigned __int8 r, g, b;

	if(!(in = fs->RequestUEFChunkSelector(NULL, psetting, name, "r"))) return false;

	if(!in->FindId(0xff00))
		return false;

	temp = (char *)malloc(in->CurrentChunk()->GetLength());
	in->CurrentChunk()->Read((unsigned char *)temp, in->CurrentChunk()->GetLength());

	res = strcmp(temp, "ElectrEm") == 0;
	free(temp);

	if(!res)
		return false;

	/* allocate some space */
	fonts = (Font *)malloc(sizeof(Font)*16);
	c = 16;
	while(c--)
		fonts[c].chars = NULL;

	imgs = (Image *)malloc(sizeof(Image)*256);
	c = 256;
	while(c--)
		imgs[c].data = NULL;

	in->ResetOverRan();
	in->FindIdMajor(0xff);

	while(!in->OverRan())
	{
		idminor = in->CurrentChunk()->GetId()&255;

		/* all ff1? entries are fonts, others are images */

		if((idminor&0xf0) == 0x10)
		{
			cfont = &fonts[idminor-0x10];

			cfont->charstart = 32;
			cfont->charend = 165;
			cfont->fonth = 0;

			cfont->chars = (Image *)calloc(cfont->charend-cfont->charstart, sizeof(Image));
			charpos = 0;

			c = cfont->charend-cfont->charstart;
			while(c--)
			{
				cfont->chars[charpos].w = in->CurrentChunk()->GetC();
				cfont->chars[charpos].h = in->CurrentChunk()->GetC();

				if(cfont->chars[charpos].h > cfont->fonth)
					cfont->fonth = cfont->chars[charpos].h;

				cfont->chars[charpos].data = (unsigned char *)malloc(cfont->chars[charpos].w*cfont->chars[charpos].h);
				in->CurrentChunk()->Read(cfont->chars[charpos].data, cfont->chars[charpos].w*cfont->chars[charpos].h);
				PreprocessImage(&cfont->chars[charpos]);
				charpos++;
			}
		}
		else

		if(idminor < 0xf0)
		{
			/* load an image here */
			cimage = &imgs[idminor];

			cimage->w = in->CurrentChunk()->GetC();
			cimage->w |= in->CurrentChunk()->GetC() << 8;

			cimage->h = in->CurrentChunk()->GetC();
			cimage->h |= in->CurrentChunk()->GetC() << 8;

			c = cimage->w * cimage->h;
			cimage->data = (unsigned __int8 *)calloc(c, sizeof(unsigned __int8));
			in->CurrentChunk()->Read(cimage->data, c);

			PreprocessImage(cimage);
		}

		in->FindIdMajor(0xff);
	}

	/* now search for and set the palette */
	if(in->FindId(0xfff0))
	{
		for(c = 0; c < 256; c++)
		{
			r = in->CurrentChunk()->GetC();
			g = in->CurrentChunk()->GetC();
			b = in->CurrentChunk()->GetC();

			if(r || g || b || !c)
				SetPaletteEntry(c, (float)r/256.0f, (float)g/256.0f, (float)b/256.0f, PAL_GUI);
		}
	}
	else return false;

	fs->ReleaseUEFChunkSelector(in);

	return true;
}

void C_gfx_api::BeginRedraw(void)
{
	backupscr.BeginDraw();
	SetClipArea(0, 0, backupscr.width, backupscr.height);
	Rectangle(0, 0, backupscr.width, backupscr.height, 0);
}

void C_gfx_api::EndRedraw(void)
{
	if(CursorRequired)
	{
		//draw a mouse cursor
		MOUSE_STRUCT mdata;

		mdata = GetMouseState();

		SetClipArea(0, 0, backupscr.width, backupscr.height);
		PutImage(mdata.x, mdata.y, IMG_CURSOR, true);
	}

	backupscr.EndOperation();
	backupscr.Blit();
}

bool C_gfx_api::EnableGUI(void)
{
	DSPSTRUCT tmp;

	BackupArea = ScreenArea;

	tmp = GetGenuineSize(640, 512);
	SetResolution(tmp);

	SetPalette(PAL_GUI);
	EnableMouse();

	return true;
}

bool C_gfx_api::DisableGUI(void)
{
	DisableMouse();
	SetResolution(BackupArea);
	ScreenArea = BackupArea;
	SetPalette(PAL_ELK);

	return true;
}

void C_gfx_api::PreprocessImage(Image *img)
{
	/* does some run-length stuff */
	int col = 0;

	img->mnew = false;
	SetMaskByColours(img, img, 1, &col, false);

	img->runs = img->mask;
	img->mnew = false;
}

int C_gfx_api::GetColour(int i, int x, int y)
{
	return imgs[i].data[(y*imgs[i].w)+x];
}

void C_gfx_api::ResetMask(int dest)
{
	if(imgs[dest].mnew)
	{
		free(imgs[dest].mask);
		imgs[dest].mnew = false;
	}

	imgs[dest].mask = imgs[dest].runs;
}

bool C_gfx_api::ColIncluded(int colour, int ccount, int *cols)
{
	while(ccount--)
	{
		if(colour == *cols) return true;
		cols++;
	}

	return false;
}

void C_gfx_api::SetMaskByColours(Image *dest, Image *src, int ccount, int *cols, bool inc)
{
	int swaps = 0;
	bool oldstate = false, newstate;
	int cx, cy, count;
	int *r;

	if(src->h != dest->h) return;

	for(cy = 0; cy < src->h; cy++)
	{
		oldstate = false;
		for(cx = 0; cx < src->w; cx++)
		{
			newstate = ColIncluded(src->data[(cy*src->w)+cx], ccount, cols) ? inc : !inc;
			if(newstate != oldstate)
				swaps++;

			oldstate = newstate;
		}
		if(oldstate) swaps++;
	}

	if(dest->mnew) free(dest->mask);
	r = dest->mask = (int *)malloc(sizeof(int)*(swaps + dest->h));
	dest->mnew = true;

	for(cy = 0; cy < src->h; cy++)
	{
		count = 0;
		oldstate = false;

		for(cx = 0; cx < src->w; cx++)
		{
			newstate = ColIncluded(src->data[(cy*src->w)+cx], ccount, cols) ? inc : !inc;
			if(newstate != oldstate)
			{
				*r++ = count;
				count = 0;
			}

			oldstate = newstate;
			count++;
		}

		if(oldstate)
			*r++ = count;

		*r++ = -1;
	}
	*(r-1) = -2;
}

void C_gfx_api::SetMaskByColours(int dest, int src, int ccount, int *cols)
{
	SetMaskByColours(&imgs[dest], &imgs[src], ccount, cols, true);
}
