/*

	ElectrEm (c) 2000/1 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for
	details

	gui/about.cpp
	=============

	This is the 'info' screen. It displays the 'ElectrEm is an open source
	emulator . . .' message and the credits screen, and does absolutely
	nothing else. This is not particularly smart or hard to follow code!

*/
#include "about.h"
#include "../defines.h"
#include "../gfx/gfxdefs.h"
#include "../gfx/gfx.h"
#include "../misc/config.h"
#include "../ula/ula.h"
#include "guidefs.h"
#include <stdio.h>
#include <malloc.h>
#include <string.h>

char DESC_STRING[] = "The Electron system ROMs are copyright Pace Micro Technology PLC, http://www.pace.co.uk\n\nElectrEm is a portable, open source, moderately C++ emulator of the Acorn Electron which attempts to pay good attention to timing. Also, it is able to simulate an extended data bus, allowing the addition new graphics to old games.\n\nThis is only a beta release, but if you find any problems then please don't hesitate to contact me via T.Harte@excite.com. And keep an eye on http://electrem.emuunlim.com for updates!";

struct credit
{
	char *name;
	char *job;
	char *email;
	char *url;
};

#define NUM_CREDITS 14
credit creds[] =
{
	{"gARetH baBB", "Provided information regarding the MRB & First Byte joystick", "hick@pobox.co.uk", ""},
	{"John Belson", "Help with ULA tape information", "", ""},
	{"David Boddie", "UEF conversion scripts and GGI testing", "david@boddie.net", "http://www-solar.mcs.st-and.ac.uk/~davidb"},
	{"Paul Boddie", "GGI/Unix distribution maintainer, programming", "paul@boddie.net", ""},
	{"Ricky Grant", "Provided information concerning clock rates & interrupt timing", "", ""},
	{"Thomas Harte", "Programming, Project Leader & Originator", "T.Harte@excite.com", "http://electrem.emuunlim.com"},
	{"R. Henderson", "Wrote a useful article for Electron User [in 1990!]", "", ""},
	{"Wouter Hobers", "Provided information concerning the Electron CPU", "", ""},
	{"Simon Irwin", "Help with debugging of the CPU core (+ drive control specs)", "", ""},
	{"Dave M", "Support & occasional uploading", "dllm@usa.net", "http://www.stairwaytohell.com"},
	{"Ian Marshall", "Translucent electron image & emulator name", "ian@imarshall.karoo.co.uk", "http://www.imarshall.karoo.net"},
	{"David Raven", "Free (secondhand) copy of MSVC++", "", ""},
	{"Patrick Shoenmakers", "Important hints about mode 0-3 timing", "", ""},
	{"Paul Walker", "MRB and Mode 7 information", "", ""},
};

/* a thing for displaying credits */

class C_GUICredit : public C_GUIElementBase
{
	public :
		void Draw(void);

		void Setup(C_GUI *g, GUI_BOX area, credit *crd);

		void SetPosition(int x, int y);
		GUI_BOX GetShape(void);

	private :
		GUI_BOX clip;

		int x, y;
		credit *cred;
};

#define CREDIT_NAMEWIDTH	155

void C_GUICredit::Draw(void)
{
	gfx->SetClipArea(clip.x1, clip.y1, clip.x2, clip.y2);

	gfx->WriteString(FONT_SIZE12, cred->name, x, y, FCOL_RED | BCOL_BLACK);
	gfx->WriteString(FONT_SIZE12, cred->job, x+CREDIT_NAMEWIDTH, y, FCOL_WHITE | BCOL_BLACK);

/*	int y2 = y + gfx->StrHeight(FONT_SIZE12, "");

	gfx->WriteString(FONT_SIZE12, cred->email, x, y2, FCOL_WHITE | BCOL_BLACK);
	gfx->WriteString(FONT_SIZE12, cred->url, x+CREDIT_NAMEWIDTH, y2, FCOL_WHITE | BCOL_BLACK);*/
}

void C_GUICredit::Setup(C_GUI *g, GUI_BOX area, credit *crd)
{
	gui = g;
	gfx = gui->GetPartialGFX();
	desc.loc = clip = area;
	gui->SetDefault(&desc);

	cred = crd;
}

void C_GUICredit::SetPosition(int xx, int yy)
{
	x = xx;
	y = yy;

	desc.loc.y1 = y;
	desc.loc.y2 = y+gfx->StrHeight(FONT_SIZE12, "t");

	if(desc.loc.y1 <= clip.y1)
		desc.loc.y1 = clip.y1;
	if(desc.loc.y2 > clip.y2)
		desc.loc.y2 = clip.y2;
}

GUI_BOX C_GUICredit::GetShape(void)
{
	GUI_BOX t;
	int h = gfx->StrHeight(FONT_SIZE12, "t");

	t = clip;
	t.y2 = t.y1 + h + (h >> 2);

	return t;
}

/* the menu */

#define ELK_ID		1
#define BUTTON_ID	2
#define GUI_ID		3

bool C_AboutMenu::Initialise(C_GUI *parent, int x1, int y1, int x2, int y2)
{
	GUI_BOX box, newbox, sarea;
	char *ttext;

	gui = parent;
	InitialiseStack();
	InitialiseList();

	box.x1 = x1;
	box.y1 = y1;
	box.x2 = x2;
	box.y2 = y2;

	ttext = (char *)malloc(strlen(name_string)+2);
	sprintf(ttext, "%s ", name_string);
	title[0].Setup(parent, box, ttext, FCOL_WHITE | BCOL_BLACK, FONT_SIZE24);
	free(ttext);

	newbox = title[0].MinimiseShape();

	box.x1 = newbox.x2;
	box.y2 = newbox.y2;
	title[1].Setup(parent, box, version_string, FCOL_WHITE | BCOL_BLACK, FONT_SIZE18);
	newbox = title[1].MinimiseShape();
	box.y1 += box.y2 - newbox.y2 - 2;
	title[1].SetShape(box);

	box.x1 = x1;
	box.x2 = x2;
	box.y1 = newbox.y2+5;
	box.y2 = y2;
	body.Setup(parent, box, DESC_STRING, FCOL_WHITE | BCOL_BLACK, FONT_SIZE12);
	sarea = box;

	box.x1 = x1;
	box.y1 = y1;
	box.x2 = x2;
	box.y2 = y2;
	elk.Setup(parent, this, ELK_ID, box, IMG_XRAYELEC, false);
	newbox = elk.MinimiseShape();
	box.y1 = box.y2 - (newbox.y2 - newbox.y1);
	box.x1 = ((box.x1+box.x2) >> 1) - ((newbox.x1 + newbox.x2) >> 1);
	box.x2 = box.x1 + (newbox.x2 - newbox.x1);
	elk.SetShape(box);

	guifirst.Setup(parent, this, GUI_ID, box, "enter GUI first", FONT_SIZE12, FCOL_WHITE | BCOL_BLACK);
	newbox = guifirst.MinimiseShape();
	newbox.x1 = box.x2 - (newbox.x2 - newbox.x1);
	newbox.x2 = box.x2;
	newbox.y1 = box.y1 - (newbox.y2 - newbox.y1);
	newbox.y2 = box.y1;
	guifirst.SetShape(newbox);
//	AddToList(&guifirst);

	box.x1 = x1;
	box.y1 = y1;
	box.x2 = x2;
	box.y2 = y2;
	flip.Setup(parent, this, BUTTON_ID, box, "Credits >>", 0, FCOL_WHITE | BCOL_BLACK, FONT_SIZE12, true, false);
	newbox = flip.MinimiseShape();

	box.x1 = x2 - (newbox.x2 - newbox.x1);
	box.x2 = x2;
	box.y2 = y1 + (newbox.y2 - newbox.y1);
	box.y1 = y1;
	flip.SetShape(box);

	credits = false;

	/* credits */
	CreditElements = new C_GUICredit[NUM_CREDITS];
	creditbox.Setup(parent, sarea);

	for(int i = 0; i < NUM_CREDITS; i++)
	{
		CreditElements[i].Setup(parent, sarea, &creds[i]);
	}
	creditbox.SetElements(CreditElements, NUM_CREDITS, sizeof(C_GUICredit));

	/* seed value of 'enter GUI first' box */
	C_config *cfg = gui->GetElectron()->GetConfig();

	if(cfg->FindSetting("guifirst", BOOLEAN_RESULT))
	{
		guifirst.SetTicked(cfg->ReadSetting().result_bool);
	}
	else
		guifirst.SetTicked(false);

	return true;
}

void C_AboutMenu::DeInitialise(void)
{
	C_config *cfg = gui->GetElectron()->GetConfig();
	config_setting s;

	s.result_type = BOOLEAN_RESULT;
	s.result_bool = guifirst.GetTicked();

	cfg->FindSetting("guifirst", ANY_RESULT);
	cfg->WriteSetting(s);

	/* and kill */

	title[0].Kill();
	title[1].Kill();
	body.Kill();
	elk.Kill();
	flip.Kill();
	creditbox.Kill();
	guifirst.Kill();
}

void C_AboutMenu::SendMsg(GUI_MSG msg)
{
	BACK_MSG w;

	C_GUIScreen::SendMsg(msg);
	flip.SendMsg(msg);

	if(!credits) guifirst.SendMsg(msg);

	if(credits) creditbox.SendMsg(msg);

	while(MessagesWaiting())
	{
		w = PopMessage();

		if(w.owner == BUTTON_ID)
		{
			if(w.val == GUIBUTTON_PERFORM)
			{
				credits = !credits;
				if(credits)
				{
					flip.ChangeText("<< About");
				}
				else
				{
					flip.ChangeText("Credits >>");
				}
			}
		}
	}
}

void C_AboutMenu::Draw(void)
{
	C_GUIScreen::Draw();

	title[0].Draw();
	title[1].Draw();

	if(!credits)
	{
		body.Draw();
		elk.Draw();
		guifirst.Draw();
	}
	else
	{
		creditbox.Draw();
	}

	flip.Draw();
}
