/*

	ElectrEm (c) 2000/1 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for
	details

	gui/bseelmnts.cpp
	=================

	The basic GUI elements (text boxes, lines & buttons) & default class
	members

*/
#include "guilocal.h"
#include "../gfx/gfx.h"
#include "guidefs.h"
#include "../gfx/gfxdefs.h"
#include <string.h>
#include <malloc.h>
#include <stdio.h>

#define SetClip() gfx->SetClipArea((desc.loc.x1 < desc.loc.x2) ? desc.loc.x1 : desc.loc.x1+1, (desc.loc.y1 < desc.loc.y2) ? desc.loc.y1 : desc.loc.y1+1, (desc.loc.x1 < desc.loc.x2) ? desc.loc.x2+1 : desc.loc.x2, (desc.loc.y1 < desc.loc.y2) ? desc.loc.y2+1 : desc.loc.y2)

/* shared features */

GUI_BOX C_GUIElementBase::GetShape(void)
{
	return desc.loc;
}

void C_GUIElementBase::SetShape(GUI_BOX box)
{
	desc.loc = box;
}

void C_GUIElementBase::SendMsg(GUI_MSG msg)
{
}

GUI_BOX C_GUIElementBase::MinimiseShape(void)
{
	return GetShape();
}

void C_GUIElementBase::Kill(void)
{
}

void C_GUIElementBase::Reset(void)
{
}

void C_GUIElementBase::SetPosition(int x, int y)
{
}

/* The TextBox element */

C_GUITextBox::C_GUITextBox(void)
{
	string = copy = NULL;
}

void C_GUITextBox::InitialiseState(void)
{
	start = nextpos = pos = copy;
	memcpy(copy, string, strlen(string));
	y = desc.loc.y1;
}

bool C_GUITextBox::GenerateNextLine(void)
{
	int w = 0;
	char oldval;

	if(pos != copy) y += gfx->StrHeight(font, start);
	start = pos;

	if(! *nextpos) return false;

	while(w < (desc.loc.x2 - desc.loc.x1) && *nextpos)
	{
		pos = nextpos;

		while(*nextpos > 32) nextpos++;

		oldval = *nextpos;
		*nextpos = '\0';

		w = gfx->StrWidth(font, start);

		*nextpos = oldval;
		if(*nextpos) nextpos++;
	}

	if(!*nextpos)
		pos = nextpos;
	else
		*(pos-1) = '\0';

	return true;
}

GUI_BOX C_GUITextBox::MinimiseShape(void)
{
	int nw, w = -1;
	InitialiseState();

	while(GenerateNextLine())
	{
		nw = gfx->StrWidth(font, start);
		if(nw > w) w = nw;
	}

	desc.loc.x2 = desc.loc.x1 + w;

	if(y <= desc.loc.y2)
		desc.loc.y2 = y;

	return desc.loc;
}

void C_GUITextBox::Draw(void)
{
	SetClip();

	InitialiseState();
	while(GenerateNextLine())
		gfx->WriteString(font, start, desc.loc.x1, y, colour);
}

void C_GUITextBox::Kill(void)
{
	if(string)
	{
		free(string);
		string = NULL;
	}

	if(copy)
	{
		free(copy);
		copy = NULL;
	}
}

void C_GUITextBox::Setup(C_GUI *g, GUI_BOX area, char *text, int col, int fnt)
{
	gui = g;
	gfx = gui->GetPartialGFX();
	desc.loc = area;
	colour = col;
	font = fnt;

	ChangeText(text);
}

void C_GUITextBox::ChangeText(char *newtext)
{
	Kill();
	string = strdup(newtext);
	copy = strdup(string);
}

/* the line element */

void C_GUILine::Draw(void)
{
	SetClip();
	gfx->AALine(desc.loc.x1, desc.loc.y1, desc.loc.x2, desc.loc.y2, colour);
}

void C_GUILine::Setup(C_GUI *g, GUI_BOX area, int col)
{
	gui = g;
	gfx = gui->GetPartialGFX();

	SetShape(area);
	ChangeColour(col);
}

void C_GUILine::ChangeColour(int col)
{
	colour = col;
}

/* the image element */

GUI_BOX C_GUIImage::MinimiseShape(void)
{
	desc.loc.x2 = desc.loc.x1 + gfx->ImgWidth(imgid);
	desc.loc.y2 = desc.loc.y1 + gfx->ImgHeight(imgid);

	return desc.loc;
}

void C_GUIImage::SendMsg(GUI_MSG msg)
{
	if(gui->PreHandling(&desc, msg))
	{
		if(desc.mouseover)
		{
			if(desc.newclicks == 1)
			{
				bmsg.val = gfx->GetColour(imgid, msg.x - desc.loc.x1, msg.y - desc.loc.y1);

				if(bmsg.val || !trans)
					screen->PushMessage(bmsg);
			}
		}
	}
}

void C_GUIImage::Draw(void)
{
	SetClip();
	gfx->PutImage(desc.loc.x1, desc.loc.y1, imgid, trans);
}

void C_GUIImage::Setup(C_GUI *g, C_GUIMessageCollector *scrn, int id, GUI_BOX area, int img, bool t)
{
	gui = g;
	gfx = gui->GetPartialGFX();
	screen = scrn;
	bmsg.owner = id;

	gui->SetDefault(&desc);
	desc.loc = area;
	imgid = img;
	trans = t;
}

/* the outline box element */

void C_GUIOutlineBox::Draw(void)
{
	SetClip();

	gfx->Line(desc.loc.x1, desc.loc.y1, desc.loc.x2, desc.loc.y1, colour);
	gfx->Line(desc.loc.x1, desc.loc.y2, desc.loc.x2, desc.loc.y2, colour);
	gfx->Line(desc.loc.x1, desc.loc.y1, desc.loc.x1, desc.loc.y2, colour);
	gfx->Line(desc.loc.x2, desc.loc.y1, desc.loc.x2, desc.loc.y2, colour);
}

void C_GUIOutlineBox::Setup(C_GUI *g, GUI_BOX area, int col)
{
	gui = g;
	gfx = gui->GetPartialGFX();

	desc.loc = area;
	colour = col;
}

/* the button element */

C_GUIButton::C_GUIButton(void)
{
	label = NULL;
}

GUI_BOX C_GUIButton::MinimiseShape(void)
{
	desc.loc = over.GetShape();

	desc.loc.x1 -= 2;
	desc.loc.x2 += 2;
	desc.loc.y1 -= 2;
	desc.loc.y2 += 2;

	return desc.loc;
}

void C_GUIButton::Draw(void)
{
	SetClip();

	if(border)
	{
		gfx->Line(desc.loc.x1, desc.loc.y1, desc.loc.x2, desc.loc.y1, pressed ? SCOL_GREY : SCOL_WHITE);
		gfx->Line(desc.loc.x1, desc.loc.y1, desc.loc.x1, desc.loc.y2, pressed ? SCOL_GREY : SCOL_WHITE);

		gfx->Line(desc.loc.x1, desc.loc.y2, desc.loc.x2, desc.loc.y2, pressed ? SCOL_WHITE : SCOL_GREY);
		gfx->Line(desc.loc.x2, desc.loc.y1, desc.loc.x2, desc.loc.y2, pressed ? SCOL_WHITE : SCOL_GREY);
	}

	if(desc.mouseover)
		over.Draw();

	if(label)
		gfx->WriteString(font, label, wx, wy, colour);
	else
		gfx->PutImage(wx, wy, imgid, true);

}

void C_GUIButton::SendMsg(GUI_MSG msg)
{
	if(gui->PreHandling(&desc, msg))
	{
		if(desc.mouseover)
		{
			if(desc.newclicks == 1)
			{
				pressed = true;

				if(!border)
					screen->PushMessage(bmsg);
			}

			if(desc.newclicks == -1 && pressed)
			{
				pressed = persistant;

				if(border)
					screen->PushMessage(bmsg);
			}
		}
		else
		{
			if(pressed) pressed = persistant;
		}
	}
}

void C_GUIButton::Kill(void)
{
	if(label)
	{
		free(label);
		label = NULL;
	}

	over.Kill();
}

void C_GUIButton::Setup(C_GUI *g, C_GUIMessageCollector *scrn, int id, GUI_BOX area, char *text, int img, int col, int fnt, bool brder, bool persist)
{
	gui = g;
	gfx = gui->GetPartialGFX();
	screen = scrn;

	gui->SetDefault(&desc);

	bmsg.owner = id;
	bmsg.val = GUIBUTTON_PERFORM;
	colour = col;
	font = fnt;
	imgid = img;

	ChangeText(text);

	border = brder;
	persistant = persist;
	pressed = false;

	SetShape(area);
}

void C_GUIButton::Set(bool v)
{
	pressed = v;
}

void C_GUIButton::ChangeText(char *newtext)
{
	Kill();
	if(newtext)
	{
		label = strdup(newtext);
		int width = gfx->StrWidth(font, label);
		int height = gfx->StrHeight(font, label);

		wx = ((desc.loc.x1 + desc.loc.x2) >> 1) - (width >> 1);
		wy = ((desc.loc.y1 + desc.loc.y2) >> 1) - (height >> 1);

		desc.loc.x1 = wx;
		desc.loc.y1 = wy;
		desc.loc.x2 = wx+width;
		desc.loc.y2 = wy+height;
		over.Setup(gui, desc.loc, SCOL_GREY);

		desc.loc.x1-=2; desc.loc.x2+= 2;
		desc.loc.y1-=2; desc.loc.y2+= 2;
	}
}

void C_GUIButton::ChangeColour(int newcol)
{
	colour = newcol;
}

void C_GUIButton::SetShape(GUI_BOX box)
{
	int width, height;

	over.Kill();

	desc.loc = box;

	if(label)
	{
		width = gfx->StrWidth(font, label);
		height = gfx->StrHeight(font, label);
	}
	else
	{
		width = gfx->ImgWidth(imgid);
		height = gfx->ImgHeight(imgid);
	}

	wx = ((desc.loc.x1 + desc.loc.x2) >> 1) - (width >> 1);
	wy = ((desc.loc.y1 + desc.loc.y2) >> 1) - (height >> 1);

	box.x1 = wx;
	box.y1 = wy;
	box.x2 = wx+width;
	box.y2 = wy+height;
	over.Setup(gui, box, SCOL_GREY);
}

/* the tickbox element */

C_GUITickBox::C_GUITickBox(void)
{
	label = NULL;
}

GUI_BOX C_GUITickBox::MinimiseShape(void)
{
	desc.loc = over.GetShape();

	desc.loc.x1 -= 2 + gfx->ImgWidth(IMG_TICKBOX_Y);
	SetShape(desc.loc);

	return desc.loc;
}

void C_GUITickBox::Draw(void)
{
	SetClip();

	gfx->PutImage(desc.loc.x1, ty, ticked ? IMG_TICKBOX_Y : IMG_TICKBOX_N, false);

	if(desc.mouseover)
		over.Draw();

	gfx->WriteString(font, label, wx, wy, colour);
}

void C_GUITickBox::SendMsg(GUI_MSG msg)
{
	if(gui->PreHandling(&desc, msg))
	{
		if(desc.newclicks == 1)
		{
			ticked = !ticked;
			screen->PushMessage(bmsg);
		}
	}
}

void C_GUITickBox::Kill(void)
{
	if(label)
	{
		free(label);
		label = NULL;
	}

	over.Kill();
}

void C_GUITickBox::Setup(C_GUI *g, C_GUIMessageCollector *scrn, int id, GUI_BOX area, char *text, int fnt, int col)
{
	gui = g;
	gfx = gui->GetPartialGFX();
	screen = scrn;

	gui->SetDefault(&desc);

	bmsg.owner = id;
	bmsg.val = GUITICKBOX_CHANGE;

	colour = col;
	font = fnt;

	label = strdup(text);
	SetShape(area);

	ticked = false;
}

void C_GUITickBox::SetTicked(bool t)
{
	ticked = t;
}

bool C_GUITickBox::GetTicked(void)
{
	return ticked;
}

void C_GUITickBox::ChangeText(char *newtext)
{
	Kill();
	label = strdup(newtext);

	SetShape(desc.loc);
}

void C_GUITickBox::ChangeColour(int newcol)
{
	colour = newcol;
}

void C_GUITickBox::SetShape(GUI_BOX box)
{
	int width, height;

	over.Kill();

	desc.loc = box;

	width = gfx->StrWidth(font, label);
	height = gfx->StrHeight(font, label);

	wx = desc.loc.x1 + 2 + gfx->ImgWidth(IMG_TICKBOX_Y);
	wy = desc.loc.y1;

	box.x1 = wx;
	box.y1 = wy;
	box.x2 = wx+width;
	box.y2 = wy+height;
	over.Setup(gui, box, SCOL_GREY);

	ty = ((desc.loc.y1 + desc.loc.y2) >> 1) - (gfx->ImgHeight(IMG_TICKBOX_Y) >> 1);
}

/* the radio button element */

C_GUIRadioButton::C_GUIRadioButton(void)
{
	label = NULL;
}

GUI_BOX C_GUIRadioButton::MinimiseShape(void)
{
	desc.loc = over.GetShape();

	desc.loc.x1 -= 2 + gfx->ImgWidth(IMG_RADIO_Y);
	SetShape(desc.loc);

	return desc.loc;
}

void C_GUIRadioButton::Draw(void)
{
	SetClip();

	gfx->PutImage(desc.loc.x1, ty, (*var == bmsg.val) ? IMG_RADIO_Y : IMG_RADIO_N, false);

	if(desc.mouseover)
		over.Draw();

	gfx->WriteString(font, label, wx, wy, colour);
}

void C_GUIRadioButton::SendMsg(GUI_MSG msg)
{
	if(gui->PreHandling(&desc, msg))
	{
		if(desc.newclicks == 1)
		{
			*var = bmsg.val;
			screen->PushMessage(bmsg);
		}
	}
}

void C_GUIRadioButton::Kill(void)
{
	if(label)
	{
		free(label);
		label = NULL;
	}

	over.Kill();
}

void C_GUIRadioButton::Setup(C_GUI *g, C_GUIMessageCollector *scrn, int id, int number, int *v, GUI_BOX area, char *text, int fnt, int col)
{
	gui = g;
	gfx = gui->GetPartialGFX();
	screen = scrn;

	gui->SetDefault(&desc);

	bmsg.owner = id;
	bmsg.val = number;
	var = v;

	colour = col;
	font = fnt;

	label = strdup(text);
	SetShape(area);
}

void C_GUIRadioButton::ChangeText(char *newtext)
{
	Kill();
	label = strdup(newtext);

	SetShape(desc.loc);
}

void C_GUIRadioButton::ChangeColour(int newcol)
{
	colour = newcol;
}

void C_GUIRadioButton::SetShape(GUI_BOX box)
{
	int width, height;

	over.Kill();

	desc.loc = box;

	width = gfx->StrWidth(font, label);
	height = gfx->StrHeight(font, label);

	wx = desc.loc.x1 + 2 + gfx->ImgWidth(IMG_RADIO_Y);
	wy = desc.loc.y1;

	box.x1 = wx;
	box.y1 = wy;
	box.x2 = wx+width;
	box.y2 = wy+height;
	over.Setup(gui, box, SCOL_GREY);

	ty = ((desc.loc.y1 + desc.loc.y2) >> 1) - (gfx->ImgHeight(IMG_RADIO_Y) >> 1);
}
