/*

	ElectrEm (c) 2000/1 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for
	details

	gui/elements.cpp
	================

	Some more 'complicated' GUI elements - the folder tabs & the text input
	string builder

*/
#include "guilocal.h"
#include "../gfx/gfx.h"
#include "guidefs.h"
#include "../gfx/gfxdefs.h"
#include <string.h>
#include <malloc.h>
#include <stdio.h>
#include <stdarg.h>
#include <time.h>

#define SetClip() gfx->SetClipArea((desc.loc.x1 < desc.loc.x2) ? desc.loc.x1 : desc.loc.x1+1, (desc.loc.y1 < desc.loc.y2) ? desc.loc.y1 : desc.loc.y1+1, (desc.loc.x1 < desc.loc.x2) ? desc.loc.x2+1 : desc.loc.x2, (desc.loc.y1 < desc.loc.y2) ? desc.loc.y2+1 : desc.loc.y2)

/* the folder tabs */

void C_GUIFolderTabs::Draw(void)
{
	int c = numlines;
	while(c--)
		lines[c].Draw();

	c = numbtns;
	while(c--)
		btns[c].Draw();
}

void C_GUIFolderTabs::Kill(void)
{
	int c = numlines;
	while(c--)
		lines[c].Kill();

	c = numbtns;
	while(c--)
		btns[c].Kill();

	delete[] lines;
	delete[] btns;
}

void C_GUIFolderTabs::SetFront(int c)
{
	int newc, count = numbtns;
	c = numbtns-c-1;
	while(count--)
	{
		newc = BCOL_BLACK | ((count == c) ? FCOL_WHITE : FCOL_GREY);

		lines[count*3 + 5].ChangeColour(newc);
		lines[count*3 + 6].ChangeColour(newc);
		lines[count*3 + 7].ChangeColour(newc);
		btns[count].ChangeColour(newc);
	}

	GUI_BOX box, box2;

	box2 = lines[c*3 + 5].GetShape();

	box = lines[3].GetShape();
	box.x2 = box2.x2;
	lines[3].SetShape(box);

	box2 = lines[c*3 + 7].GetShape();

	box = lines[4].GetShape();
	box.x1 = box2.x1;
	lines[4].SetShape(box);
}

void C_GUIFolderTabs::SendMsg(GUI_MSG msg)
{
	int c = numbtns;
	while(c--)
		btns[c].SendMsg(msg);

	BACK_MSG w;

	while(MessagesWaiting())
	{
		w = PopMessage();
		bmsg.val = w.owner;
		screen->PushMessage(bmsg);

		SetFront(bmsg.val);
	}
}

C_GUIFolderTabs::C_GUIFolderTabs()
{
	numbtns = numlines = 0;
}

GUI_BOX C_GUIFolderTabs::GetEnclosedArea(void)
{
	GUI_BOX box1, box2;

	box1 = lines[0].GetShape();
	box2 = lines[1].GetShape();

	box1.x2 = box2.x2-2;
	box1.x1 += 2;
	box1.y1 += 2;
	box1.y2 -= 2;

	return box1;
}

GUI_BOX C_GUIFolderTabs::GetExcludedArea(void)
{
	GUI_BOX box1, box2;

	box1 = lines[7].GetShape();

	box2.x2 = desc.loc.x2;
	box2.x1 = box1.x1;
	box2.y1 = desc.loc.y1;
	box2.y2 = box1.y1;

	return box2;
}

void C_GUIFolderTabs::FinishTabs(void)
{
	GUI_BOX box;

	numbtns = numtops;
	btns = new C_GUIButton[numbtns];

	numlines = (numtops*3) +5;
	lines = new C_GUILine[numlines];

	/* three entirely static lines */

	box.y1 = desc.loc.y1+gfx->StrHeight(font, tops[0]) + 8;
	box.y2 = desc.loc.y2-1;

	box.x1 =
	box.x2 = desc.loc.x1;
	lines[0].Setup(gui, box, FCOL_WHITE | BCOL_BLACK);

	box.x1 =
	box.x2 = desc.loc.x2-1;
	lines[1].Setup(gui, box, FCOL_WHITE | BCOL_BLACK);

	box.x1 = desc.loc.x1;
	box.y1 = desc.loc.y2-1;
	lines[2].Setup(gui, box, FCOL_WHITE | BCOL_BLACK);

	/* add card tabs */
	int x = desc.loc.x1;
	int c = numtops;
	while(c--)
	{
		box.x1 = x+4;
		box.y1 = desc.loc.y1;
		box.x2 = x;
		box.y2 = desc.loc.y1+gfx->StrHeight(font, tops[c]) + 8;
		lines[c*3 + 5].Setup(gui, box, FCOL_WHITE | BCOL_BLACK);

		x += 12+gfx->StrWidth(font, tops[c]);
		box.x2 = x-4;
		btns[c].Setup(gui, this, numbtns-c-1, box, tops[c], 0, FCOL_WHITE | BCOL_BLACK, font, false, false);

		box.y2 = box.y1;
		lines[c*3 + 6].Setup(gui, box, FCOL_WHITE | BCOL_BLACK);

		box.x1 = x;
		box.y1 = desc.loc.y1+gfx->StrHeight(font, tops[c]) + 8;
		lines[c*3 + 7].Setup(gui, box, FCOL_WHITE | BCOL_BLACK);
	}

	box.x1 = desc.loc.x1;
	box.x2 = desc.loc.x2;
	box.y2 = box.y1;
	lines[3].Setup(gui, box, FCOL_WHITE | BCOL_BLACK);
	lines[4].Setup(gui, box, FCOL_WHITE | BCOL_BLACK);

	SetFront(0);

	while(numtops--)
	{
		free(tops[numtops]);
	}
	free(tops);
}

void C_GUIFolderTabs::StartEntryList(int numentries)
{
	numtops = numentries;
	tops = (char **)malloc(sizeof(char *)*numentries);
	topcounter = numtops;
}

void C_GUIFolderTabs::AddEntry(char *title)
{
	topcounter--;
	tops[topcounter] = strdup(title);
	if(!topcounter)
		FinishTabs();
}

void C_GUIFolderTabs::Setup(C_GUI *g, C_GUIMessageCollector *scrn, int id, GUI_BOX area, int fnt, int numentries, ...)
{
	va_list marker;

	gui = g;
	gfx = gui->GetPartialGFX();
	desc.loc = area;
	screen = scrn;

	gui->SetDefault(&desc);
	InitialiseStack();
	bmsg.owner = id;
	font = fnt;

	/* do a tiny bit of variable argument stuff */
	if(numentries)
	{
		/* list is here, now */
		numtops = numentries;
		tops = (char **)malloc(sizeof(char *)*numtops);

		va_start(marker, numentries);

		int c = numtops;
		while(c--)
		{
			tops[c] = strdup(va_arg(marker, char *));
		}

		va_end(marker);

		FinishTabs();
	}
}

/* the text input line */

C_GUITextInputLine::C_GUITextInputLine()
{
	string = NULL;
}

void C_GUITextInputLine::Draw(void)
{
	SetClip();

	gfx->Line(desc.loc.x1, desc.loc.y1, desc.loc.x2, desc.loc.y1, SCOL_GREY);
	gfx->Line(desc.loc.x1, desc.loc.y2, desc.loc.x2, desc.loc.y2, SCOL_GREY);
	gfx->Line(desc.loc.x1, desc.loc.y1, desc.loc.x1, desc.loc.y2, SCOL_GREY);
	gfx->Line(desc.loc.x2, desc.loc.y1, desc.loc.x2, desc.loc.y2, SCOL_GREY);

	int lx, w;
	w = gfx->StrWidth(font, string);
	if(w < (desc.loc.x2 - desc.loc.x1 - 10))
	{
		gfx->WriteString(font, string, desc.loc.x1+1, desc.loc.y1+1, colour);
		lx = desc.loc.x1 + w;
	}
	else
	{
		gfx->WriteString(font, string, desc.loc.x2-w-9, desc.loc.y1+1, colour);
		lx = desc.loc.x2 - 8;
	}

	time_t tmp = time(NULL);

	if((active || desc.mouseover) && (tmp&1))
		gfx->Rectangle(lx, desc.loc.y2-3, lx+8, desc.loc.y2-1, FCOL_WHITE|BCOL_BLUE);
}

void C_GUITextInputLine::SendMsg(GUI_MSG msg)
{
	if(gui->PreHandling(&desc, msg))
	{
		if(msg.msg == GUIM_LBUTTONDOWN)
		{
			active = true;
		}
	}

	if(msg.msg == GUIM_KEYPRESS && (active || desc.mouseover))
	{
		char *tpos;
		tpos = string + strlen(string);

		if(msg.ch >= 32 && msg.ch < 127)
		{
			if(strlen(string) < strlength-1)
			{
				*tpos++ = msg.ch;
				*tpos = '\0';
			}

			return;
		}

		switch(msg.ch)
		{
			case '\r' :
			case '\n' :
				bmsg.val = GUITEXTINPUTLINE_PERFORM;
				screen->PushMessage(bmsg);
			case 27 : // \e - escape
				active = activealways;
			break;

			case '\b' :
				if(tpos != string)
					*--tpos = '\0';
			break;
		}
	}
}

void C_GUITextInputLine::Setup(C_GUI *g, C_GUIMessageCollector *scrn, int id, GUI_BOX area, char *text, unsigned int length, int col, int fnt, bool aa)
{
	gui = g;
	gfx = gui->GetPartialGFX();
	desc.loc = area;
	screen = scrn;

	gui->SetDefault(&desc);
	bmsg.owner = id;

	string = text;
	strlength = length;
	font = fnt;
	colour = col;

	active = activealways = aa;
}

GUI_BOX C_GUITextInputLine::MinimiseShape(void)
{
	desc.loc.y2 = desc.loc.y1 + 2+ gfx->StrHeight(font, string);
	return desc.loc;
}

