/*

	ElectrEm (c) 2000/1 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for
	details

	gui/elements.cpp
	================

	The 'complicated' GUI elements - sliders & scroll fields

*/
#include "guilocal.h"
#include "../gfx/gfx.h"
#include "guidefs.h"
#include "../gfx/gfxdefs.h"
#include <string.h>
#include <malloc.h>
#include <stdio.h>

#define SetClip() gfx->SetClipArea((desc.loc.x1 < desc.loc.x2) ? desc.loc.x1 : desc.loc.x1+1, (desc.loc.y1 < desc.loc.y2) ? desc.loc.y1 : desc.loc.y1+1, (desc.loc.x1 < desc.loc.x2) ? desc.loc.x2+1 : desc.loc.x2, (desc.loc.y1 < desc.loc.y2) ? desc.loc.y2+1 : desc.loc.y2)

/* a slide bar all alone - used for the slider and scroll field */

GUI_BOX C_GUISlideBar::MinimiseShape(void)
{
	int y2 = desc.loc.y1 + gfx->ImgHeight(IMG_SLIDER);

	if(desc.loc.y2 > y2)
		desc.loc.y2 = y2;

	return desc.loc;
}

void C_GUISlideBar::Draw(void)
{
	SetClip();

	if(horiz)
	{
		gfx->PutImage(position, draw, imgid, false);
		gfx->Line(desc.loc.x1, desc.loc.y1, desc.loc.x2, desc.loc.y1, colour);
		gfx->Line(desc.loc.x1, desc.loc.y2, desc.loc.x2, desc.loc.y2, colour);
	}
	else
	{
		gfx->PutImage(draw, position, imgid, false);
		gfx->Line(desc.loc.x1, desc.loc.y1, desc.loc.x1, desc.loc.y2, colour);
		gfx->Line(desc.loc.x2, desc.loc.y1, desc.loc.x2, desc.loc.y2, colour);
	}
}

void C_GUISlideBar::SendMsg(GUI_MSG msg)
{
	if(gui->PreHandling(&desc, msg))
	{
		if(desc.newclicks == 1)
		{
			desc.ignoremouse = true;
		}

		if(desc.newclicks == -1)
		{
			desc.ignoremouse = false;
		}

		if(desc.ignoremouse)
		{
			position = horiz ? (msg.x - (gfx->ImgWidth(imgid) >> 1)) : (msg.y - (gfx->ImgHeight(imgid) >> 1));
			RecalcValue(false);

			screen->PushMessage(bmsg);
		}
	}
}

void C_GUISlideBar::Setup(C_GUI *g, C_GUIMessageCollector *scrn, int id, GUI_BOX area, int img, bool horizontal, int col)
{
	gui = g;
	gfx = gui->GetPartialGFX();
	screen = scrn;

	gui->SetDefault(&desc);

	bmsg.owner = id;
	bmsg.val = SLIDEBAR_MOVE;
	horiz = horizontal;

	colour = col;
	imgid = img;

	desc.loc = area;

	if(horiz)
	{
		position = left = area.x1;
		right = area.x2 - gfx->ImgWidth(imgid);

		draw = ((desc.loc.y1 + desc.loc.y2) >> 1) - (gfx->ImgHeight(imgid) >> 1);
	}
	else
	{
		position = left = area.y1;
		right = area.y2 - gfx->ImgHeight(imgid);

		draw = ((desc.loc.x1 + desc.loc.x2) >> 1) - (gfx->ImgWidth(imgid) >> 1);
	}

	value = 0.0f;
}

void C_GUISlideBar::SetValue(float val)
{
	value = val;
	RecalcValue(true);
}

float C_GUISlideBar::GetValue(void)
{
	return value;
}

void C_GUISlideBar::MovePixels(int adder)
{
	position += adder;
	RecalcValue(false);
}

void C_GUISlideBar::RecalcValue(bool valleader)
{
	if(valleader)
		position = left + (int)((float)(right - left)*value);
	else
		value = (float)(position - left) / (float)(right - left);

	if(position > right)
	{
		position = right;
		value = 1.0f;
	}

	if(position < left)
	{
		position = left;
		value = 0.0f;
	}
}

/* the (horizontal) slider element */

C_GUISlider::C_GUISlider(void)
{
	label = NULL;
}

void C_GUISlider::Draw(void)
{
	SetClip();

	gfx->WriteString(font, label, desc.loc.x1, desc.loc.y1, colour);

	sprintf(qtext, " %0.1f", base + slider.GetValue()*mult);
	gfx->WriteString(font, qtext, qx, desc.loc.y1, colour);

	slider.Draw();
	dec.Draw();
	inc.Draw();
}

void C_GUISlider::SendMsg(GUI_MSG msg)
{
	BACK_MSG m;
	bool ch;

	dec.SendMsg(msg);
	inc.SendMsg(msg);
	slider.SendMsg(msg);

	while(MessagesWaiting())
	{
		ch = true;
		m = PopMessage();

		if(m.owner)
		{
			slider.MovePixels(m.owner);
		}
	}

	if(ch) screen->PushMessage(bmsg);
}

void C_GUISlider::Kill(void)
{
	inc.Kill();
	dec.Kill();
	slider.Kill();

	if(label)
	{
		free(label);
		label = NULL;
	}
}

void C_GUISlider::Setup(C_GUI *g, C_GUIMessageCollector *scrn, int id, GUI_BOX area, char *text, int fnt, int col, int range, int bse)
{
	gui = g;
	gfx = gui->GetPartialGFX();
	screen = scrn;
	InitialiseStack();

	bmsg.owner = id;
	bmsg.val = 0;//SLIDER_MOVE;

	colour = col;
	font = fnt;
	mult = (float)range;
	base = (float)bse;

	label = strdup(text);
	SetShape(area);
}

void C_GUISlider::ChangeText(char *newtext)
{
	Kill();
	label = strdup(newtext);

	SetShape(desc.loc);
}

void C_GUISlider::ChangeColour(int newcol)
{
	colour = newcol;
}

void C_GUISlider::SetShape(GUI_BOX box)
{
	GUI_BOX shape;
	int x1, x2, y;
	char text[10];

	slider.Kill();
	dec.Kill();
	inc.Kill();

	desc.loc = box;
	y = desc.loc.y1 + gfx->StrHeight(font, label);
	if(y < desc.loc.y2)
		desc.loc.y2 = y;

	shape = desc.loc;
	x1 = shape.x1 += 2 + gfx->StrWidth(font, label);

	//now, shape is the region for slider + buttons

	dec.Setup(gui, this, -1, shape, NULL, IMG_ARROW_L, 0, 0, true, false);
	shape = dec.MinimiseShape();

	shape.x2 -= shape.x1-x1;
	shape.x1 = x1;

	y = ((shape.y2-shape.y1) >> 1);
	shape.y1 = ((desc.loc.y2+desc.loc.y1) >> 1) - y;
	shape.y2 = ((desc.loc.y2+desc.loc.y1) >> 1) + y;

	dec.SetShape(shape);

	//assumption (for now) : letters are taller than wide
	sprintf(text, "%0.0f", mult);
	qx = desc.loc.x2 - strlen(text)*gfx->StrHeight(font, text) - 1;

	x1 = shape.x2+1;
	x2 = qx - (shape.x2 - shape.x1) - 1;

	shape.x1 = x2+1;
	shape.x2 = qx;
	inc.Setup(gui, this, 1, shape, NULL, IMG_ARROW_R, 0, 0, true, false);

	shape.x1 = x1;
	shape.x2 = x2;
	slider.Setup(gui, this, 0, shape, IMG_SLIDER, true, SCOL_GREY);
}

void C_GUISlider::SetValue(float val)
{
	slider.SetValue(val);
}

float C_GUISlider::GetValue(void)
{
	return slider.GetValue();
}

/* the (vertical) scroll (bar &) field element */

C_GUIScrollField::C_GUIScrollField(void)
{
}

void C_GUIScrollField::Draw(void)
{
	SetClip();

	/* draw some elements ! */
	int cvar = total, num = pos;
	C_GUIElementBase *lpos = list;

	lpos = (C_GUIElementBase *)(((__int8 *)lpos) + size*pos);

	while(cvar-- && (num<listlen))
	{
		lpos->Draw();
		lpos = (C_GUIElementBase *)(((__int8 *)lpos) + size);
		num++;
	}

	/* box up */
	gfx->Line(desc.loc.x1, desc.loc.y1, maxx, desc.loc.y1, SCOL_GREY);
	gfx->Line(desc.loc.x1, desc.loc.y2, maxx, desc.loc.y2, SCOL_GREY);
	gfx->Line(desc.loc.x1, desc.loc.y1, desc.loc.x1, desc.loc.y2, SCOL_GREY);

	/* add all three components of slider */
	slider.Draw();
	dec.Draw();
	inc.Draw();
}

void C_GUIScrollField::ReseedPosition(void)
{
	if(listlen)
	{
		float v = slider.GetValue();
		int pelset, slen, tlines;

		tlines = ((desc.loc.y2 - desc.loc.y1) / height)-1;

		if((tlines*numperline) < listlen)
		{
			slen = listlen - (tlines*numperline);
			pelset = (int)(((float)(slen*height)*v)/(float)numperline);

			offset = pelset%height;
			pos = (pelset / height)*numperline;
		}

		/* reset positions */
		int cvar = total, num = pos, x = desc.loc.x1, y = desc.loc.y1;
		C_GUIElementBase *lpos = list;

		lpos = (C_GUIElementBase *)(((__int8 *)lpos) + size*pos);

		while(cvar-- && (num<listlen))
		{
			lpos->SetPosition(xoffset+x, y-offset);
			lpos = (C_GUIElementBase *)(((__int8 *)lpos) + size);
			num++;

			x += width;
			if(x >= (desc.loc.x1+(numperline*width)))
			{
				x = desc.loc.x1;
				y += height;
			}
		}
	}
}

void C_GUIScrollField::SendMsg(GUI_MSG msg)
{
	BACK_MSG m;

	dec.SendMsg(msg);
	inc.SendMsg(msg);
	slider.SendMsg(msg);

	/* and send message to some elements ! */
	int cvar = total, num = pos;
	C_GUIElementBase *lpos = list;

	lpos = (C_GUIElementBase *)(((__int8 *)lpos) + size*pos);

	while(cvar-- && (num<listlen))
	{
		lpos->SendMsg(msg);
		lpos = (C_GUIElementBase *)(((__int8 *)lpos) + size);
		num++;
	}

	/* now handle messages queued */
	bool checkpos = MessagesWaiting();

	while(MessagesWaiting())
	{
		m = PopMessage();

		if(m.owner)
		{
			slider.MovePixels(m.owner);
		}
	}

	if(checkpos)
	{
		ReseedPosition();
	}
}

void C_GUIScrollField::Kill(void)
{
	inc.Kill();
	dec.Kill();
	slider.Kill();
}

void C_GUIScrollField::Setup(C_GUI *g, GUI_BOX area)
{
	gui = g;
	gfx = gui->GetPartialGFX();
	desc.loc = area;

	//put an 'up button' in the top right
	GUI_BOX box;
	box = area;

	dec.Setup(gui, this, -1, area, NULL, IMG_ARROW_U, 0, 0, true, false);
	box = dec.MinimiseShape();

	box.x1 = area.x2 - (box.x2 - box.x1);
	box.x2 = area.x2;
	sbox.y1 = box.y2 = area.y1 + (box.y2 - box.y1);
	box.y1 = area.y1;

	dec.SetShape(box);

	inc.Setup(gui, this, 1, box, NULL, IMG_ARROW_D, 0, 0, true, false);
	box = dec.MinimiseShape();

	sbox.y2 = box.y1 = area.y2 - (box.y2 - box.y1);
	box.y2 = area.y2;

	sbox.x1 = box.x1;
	maxx = sbox.x2 = box.x2;

	inc.SetShape(box);
	slider.Setup(gui, this, 0, sbox, IMG_SCROLLER, false, SCOL_GREY);

	listlen = 0;
}

void C_GUIScrollField::SetElements(C_GUIElementBase *l, int num, int elsize)
{
	list = l;
	listlen = num;

	if(listlen)
	{
		offset = pos = 0;
		slider.SetValue(0.0f);

		GUI_BOX box;
		box = l[0].GetShape();

		width = box.x2 - box.x1;
		height = box.y2 - box.y1;

		numperline = (maxx - desc.loc.x1) / width;
		lines = (desc.loc.y2 - desc.loc.y1) / height;

		if(lines*height < (desc.loc.y2 - desc.loc.y1))
			lines++;

		xoffset = ((maxx-desc.loc.x1) - (numperline*width)) >> 1;

		total = numperline*(lines+1);

		size = elsize;
		ReseedPosition();
	}
}

int C_GUIScrollField::GetTopLeft(void)
{
	return pos;
}

GUI_BOX C_GUIScrollField::GetEnclosedArea(void)
{
	GUI_BOX tmp, box;

	box = dec.MinimiseShape();

	tmp = desc.loc;
	tmp.x2 -= box.x2 - box.x1 + 1;
	tmp.x1++;
	tmp.y1++;
	tmp.y2--;

	return tmp;
}

