/*

	ElectrEm (c) 2000/1 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for
	details

	gui/filemenu.cpp
	================

	The file screen & a special GUI element for displaying files

*/

#include <stdio.h>
#include <malloc.h>
#include <string.h>

#include "../misc/config.h"
#include "../defines.h"

#include "../gfx/gfx.h"
#include "../wd1770/drive.h"
#include "../tape/tape.h"

#include "../fs/fs.h"
#include "../misc/config.h"
#include "../screen/screen.h"

#include "filemenu.h"

#include "../gfx/gfxdefs.h"
#include "guidefs.h"

#include "romclasses.h"

#include "../ula/ula.h"
#include "../6502/6502.h"

/* the file element */

class C_FSAccess;

class C_GUIFileElement : public C_GUIElementBase, public C_GUIMessageCollector
{
	public :
		C_GUIFileElement();

		void Draw(void);
		void SendMsg(GUI_MSG msg);
		void Kill(void);

		void Setup(C_GUI *g, C_GUIMessageCollector *scrn, int id, GUI_BOX area, char *fname, bool dir);
		bool FixIcons(char *path, int c);
		bool HasIcons(void);

		void SetPosition(int x, int y);

		bool HoldsDir(void);
		char *GetName(void);

		GUI_BOX GetShape(void);
		int GetStats(void);

		void SetIconographic(int m);

	private :
		static C_GUIMessageCollector *screen;
		BACK_MSG bmsg;
		GUI_BOX clip;

		/* for display purposes */

		int ix, iy, ty, x, y, hx, hy;
		int fground, bground, flags;
		bool icon;

		/* some information that needs to be stored */
		char *name;
		bool dir;

		/* this is used for figuring out what to draw */

		static C_UEFChunkSelector *cnk;
		static C_FSAccess *fs;
		static int iconographic;
};

C_UEFChunkSelector *C_GUIFileElement::cnk;
C_FSAccess *C_GUIFileElement::fs;
C_GUIMessageCollector *C_GUIFileElement::screen;
int C_GUIFileElement::iconographic;

C_GUIFileElement::C_GUIFileElement()
{
	name = NULL;
}

#define FSTAT_TAPE	(1 << 0)
#define FSTAT_ROM	(1 << 1)
#define FSTAT_DISC	(1 << 2)
#define FSTAT_SNAP	(1 << 3)
#define FSTAT_MASTER	(1 << 4)

#define FELE_WIDTH 80
#define FELE_HEIGHT 65
#define FELE_BORDER 20

#define FELE_TWIDTH 300
#define FELE_TWWIDTH 600

#define ICN_ICONS		0
#define ICN_WORDSTHIN	1
#define ICN_WORDSWIDE	2

GUI_BOX C_GUIFileElement::GetShape(void)
{
	GUI_BOX t;

	switch(iconographic)
	{
		case ICN_ICONS :
			t.x1 = x;
			t.y1 = y;
			t.x2 = x + FELE_WIDTH + FELE_BORDER;
			t.y2 = y + FELE_HEIGHT + FELE_BORDER;
		break;

		case ICN_WORDSTHIN :
			t.x1 = x;
			t.y1 = y;
			t.x2 = x + FELE_TWIDTH;
			t.y2 = y + 1+gfx->ImgHeight(IMG_FILE_TAPE_S);
		break;

		case ICN_WORDSWIDE :
			t.x1 = x;
			t.y1 = y;
			t.x2 = x + FELE_TWWIDTH;
			t.y2 = y + 1+gfx->ImgHeight(IMG_FILE_TAPE_S);
		break;
	}

	return t;
}

void C_GUIFileElement::Draw(void)
{
	int iw, lix;
	char *start, *pos, oldval;
	int w, y = ty, yc = 0;

	switch(iconographic)
	{
		case ICN_ICONS :
			gfx->SetClipArea(clip.x1, clip.y1, clip.x2, clip.y2);

			if(bground != -1)
				gfx->PutImage(ix, iy, bground, false);

			if(fground != -1)
				gfx->PutImage(ix, iy, fground, true);

			if(flags&FSTAT_MASTER)
			{
				if(flags&FSTAT_TAPE)	gfx->PutImage(ix, iy, IMG_FILE_TAPE_S, true);
				if(flags&FSTAT_ROM)		gfx->PutImage(hx, iy, IMG_FILE_ROM_S, true);
				if(flags&FSTAT_SNAP)	gfx->PutImage(ix, hy, IMG_FILE_SNAP_S, true);
				if(flags&FSTAT_DISC)	gfx->PutImage(hx, hy, IMG_FILE_DISC_S, true);
			}

			/* now add string */
			start = pos = name;
			oldval = *pos;

			while(oldval && yc < 2)
			{
				yc++;

				w = 0;
				while(w < FELE_WIDTH && oldval)
				{
					*pos = oldval;
					pos++;

					oldval = *pos;
					*pos = '\0';
					w = gfx->StrWidth(FONT_SIZE8, start);
				}

				if(w >= FELE_WIDTH)
				{
					*pos = oldval;
					pos--;
					oldval = *pos;
					*pos = '\0';
					w = gfx->StrWidth(FONT_SIZE8, start);
				}

				gfx->WriteString(FONT_SIZE8, start, x + ((FELE_WIDTH >> 1) - (w >> 1)), y, FCOL_WHITE | BCOL_BLACK);
				y += gfx->StrHeight(FONT_SIZE8, start);

				start = pos;
			}

			*pos = oldval;
		break;

		case ICN_WORDSTHIN :
		case ICN_WORDSWIDE :
			iw = gfx->ImgWidth(IMG_FILE_TAPE_S);

			gfx->SetClipArea(desc.loc.x1, desc.loc.y1, desc.loc.x2, desc.loc.y2);
			lix = desc.loc.x2-10-4*iw;
			if(flags&FSTAT_TAPE){	gfx->PutImage(lix, iy, IMG_FILE_TAPE_S, false);	lix += iw;}
			if(flags&FSTAT_DISC){	gfx->PutImage(lix, iy, IMG_FILE_DISC_S, false);	lix += iw;}
			if(flags&FSTAT_ROM){	gfx->PutImage(lix, iy, IMG_FILE_ROM_S, false);	lix += iw;}
			if(flags&FSTAT_SNAP){	gfx->PutImage(lix, iy, IMG_FILE_SNAP_S, false);	lix += iw;}

			gfx->SetClipArea(desc.loc.x1, desc.loc.y1, desc.loc.x2-10-4*iw, desc.loc.y2);
			gfx->WriteString(FONT_SIZE12, name, x, ty, ((bground == IMG_FILE_FOLDER) ? FCOL_RED : ((bground == IMG_FILE_MULTI) ? ((hx&8) ? FCOL_WHITE : FCOL_GREY) : FCOL_WHITE)) | BCOL_BLACK);
			hx++;
		break;
	}

	if(desc.mouseover)
	{
		gfx->Line(desc.loc.x1, desc.loc.y1, desc.loc.x2, desc.loc.y1, SCOL_GREY);
		gfx->Line(desc.loc.x2, desc.loc.y1, desc.loc.x2, desc.loc.y2, SCOL_GREY);
		gfx->Line(desc.loc.x2, desc.loc.y2, desc.loc.x1, desc.loc.y2, SCOL_GREY);
		gfx->Line(desc.loc.x1, desc.loc.y2, desc.loc.x1, desc.loc.y1, SCOL_GREY);
	}
}

void C_GUIFileElement::SendMsg(GUI_MSG msg)
{
	if(gui->PreHandling(&desc, msg))
	{
		if(desc.newclicks == 1)
			screen->PushMessage(bmsg);
	}
}

void C_GUIFileElement::Kill(void)
{
	if(name)
	{
		free(name);
		name = NULL;
	}
}

void C_GUIFileElement::Setup(C_GUI *g, C_GUIMessageCollector *scrn, int id, GUI_BOX area, char *fname, bool d)
{
	Kill();

	gui = g;
	screen = scrn;
	gfx = gui->GetPartialGFX();
	fs = gui->GetElectron()->GetFS();
	desc.loc = clip = area;
	gui->SetDefault(&desc);

	fground = -1;
	flags = 0;
	bground = IMG_FILE_WAITING;
	name = strdup(fname);
	dir = d;

	bmsg.owner = id;

	if(d)
	{
		bground = IMG_FILE_FOLDER;
	}
}

bool C_GUIFileElement::FixIcons(char *path, int c)
{
	char *dscexts[] = {"ssd\a", "dsd\a", "img\a", "adf\a", "adm\a", "adl\a", NULL};
	char *uefexts[] = {"uef", NULL};
	char *tapeexts[] = {"t2p\a", "t2p3\a", "t2peg\a", "tape\a", "tap\a", "wav\a", "wave\a", "raw\a", "aif\a", "aiff\a", "au\a", NULL};

	switch(c)
	{
		/* check extension */
		case 0 :

			/* check if a simple disc image is being dealt with */
			if(fs->ExtensionIncluded(name, dscexts))
			{
				//just a disc image, no big deal
				bground = -1;
				fground = IMG_FILE_DISC_L;
				flags = FSTAT_DISC;
				return true;
			}

			/* tape (sound or T2P???) maybe ? */
			if(fs->ExtensionIncluded(name, tapeexts))
			{
				//a tape if anything
				bground = -1;
				fground = IMG_FILE_TAPE_L;
				flags = FSTAT_TAPE;
				return true;
			}

			/* check if a simple ROM image is being dealt with */

			/* check if a UEF is being dealt with */
			if(fs->ExtensionIncluded(name, uefexts))
			{
				if(cnk = fs->RequestUEFChunkSelector(path, NULL, name, "r"))
				{
					fground = bground = -1;
					flags = 0;

					if(gui->GetElectron()->Multiplexable(cnk))
						bground = IMG_FILE_MULTI;

					if(gui->GetElectron()->GetTape()->Useful(cnk))
						flags |= FSTAT_TAPE;

					/* doesn't really matter which drive */
					if(gui->GetElectron()->GetCurrentDrive()->Useful(cnk))
						flags |= FSTAT_DISC;

					if(gui->GetElectron()->SnapshotAvailable(cnk, 0))
						flags |= FSTAT_SNAP;

					/* clean up */
					fs->ReleaseUEFChunkSelector(cnk);

					if(!flags)
					{
						bground = IMG_FILE_UNKNOWN;
						fground = -1;
					}
					else
					{
						if(
							(flags == FSTAT_TAPE) || (flags == FSTAT_ROM) ||
							(flags == FSTAT_SNAP) || (flags == FSTAT_DISC))
						{
							if(flags == FSTAT_TAPE)	fground = IMG_FILE_TAPE_L;
							if(flags == FSTAT_ROM)	fground = IMG_FILE_ROM_L;
							if(flags == FSTAT_SNAP)	fground = IMG_FILE_SNAP_L;
							if(flags == FSTAT_DISC)	fground = IMG_FILE_DISC_L;

							if(bground == -1)
							{
								bground = fground;
								fground = -1;
							}
						}
						else
							flags |= FSTAT_MASTER;
					}

					return true;
				}
			}

			/* still haven't found it, then haven't a clue */
			flags = 0;
			fground = -1;
			bground = IMG_FILE_UNKNOWN;

		return true;
	}

	return true;
}

bool C_GUIFileElement::HasIcons(void)
{
	return (bground == IMG_FILE_WAITING) ? false : true;
}

bool C_GUIFileElement::HoldsDir(void)
{
	return dir;
}

char *C_GUIFileElement::GetName(void)
{
	return name;
}

void C_GUIFileElement::SetPosition(int qx, int qy)
{
	x = qx;
	y = qy;

	switch(iconographic)
	{
		case ICN_ICONS :
			ix = x + (FELE_WIDTH >> 1) - (gfx->ImgWidth(IMG_FILE_WAITING) >> 1);
			iy = y;
			hx = ix + (gfx->ImgWidth(IMG_FILE_WAITING) >> 1);
			hy = iy + (gfx->ImgHeight(IMG_FILE_WAITING) >> 1);

			ty = y + gfx->ImgHeight(IMG_FILE_WAITING) + 5;

			desc.loc.x1 = qx;
			desc.loc.y1 = qy;
			desc.loc.x2 = qx+FELE_WIDTH;
			desc.loc.y2 = qy+FELE_HEIGHT+FELE_BORDER-5;
		break;

		case ICN_WORDSTHIN :
		case ICN_WORDSWIDE :
			desc.loc = GetShape();

			iy = 1+qy;
			ty = qy + (gfx->ImgWidth(IMG_FILE_TAPE_S) >> 1) - (gfx->StrHeight(FONT_SIZE12, name) >> 1);
		break;
	}

	if(desc.loc.x1 < clip.x1) desc.loc.x1 = clip.x1;
	if(desc.loc.x2 > clip.x2) desc.loc.x2 = clip.x2;
	if(desc.loc.y1 < clip.y1) desc.loc.y1 = clip.y1;
	if(desc.loc.y2 > clip.y2) desc.loc.y2 = clip.y2;

	gui->SetDefault(&desc);
}

int C_GUIFileElement::GetStats(void)
{
	return flags;
}

void C_GUIFileElement::SetIconographic(int s)
{
	iconographic = s;
}

/* a generic save sub-screen */

#define FMENUSAVE_SAVE	0
#define FMENUSAVE_QSAVE	1

class C_FileMenu_Save : public C_GUIScreen
{
	public :
		bool Initialise(C_GUI *parent, int x1, int y1, int x2, int y2, C_GUIMessageCollector *scrn, char *string, int strlength, int id);
		void SendMsg(GUI_MSG msg);

	private :
		C_GUI *gui;
		C_GUIMessageCollector *screen;
		BACK_MSG bmsg;

		C_GUITextBox title;
		C_GUITextInputLine name;
		C_GUIButton save, qsave;
};

#define TEXT_ID	0
#define BTN_ID	1
#define QBTN_ID	2

bool C_FileMenu_Save::Initialise(C_GUI *parent, int x1, int y1, int x2, int y2, C_GUIMessageCollector *scrn, char *string, int strlength, int id)
{
	GUI_BOX area, area2;

	screen = scrn;
	gui = parent;
	bmsg.owner = id;
	InitialiseList();
	InitialiseStack();

	area.x1 = x1; area.x2 = x2; area.y1 = y1; area.y2 = y2;
	title.Setup(gui, area, "File name : ", FCOL_WHITE | BCOL_BLACK, FONT_SIZE8);
	AddToList(&title);
	area = title.MinimiseShape();
	area.y2 += area.y2-area.y1;
	area.y1 += (area.y2-area.y1) >> 1;
	title.SetShape(area);
	area.x1 = area.x2;

	qsave.Setup(gui, this, QBTN_ID, area, "Quicksave", 0, FCOL_WHITE | BCOL_BLACK, FONT_SIZE8, true, false);
	AddToList(&qsave);
	area2 = qsave.MinimiseShape();
	area2.x1 = x2 - (area2.x2 - area2.x1);
	area2.x2 = x2;

	save.Setup(gui, this, BTN_ID, area2, "Save", 0, FCOL_WHITE | BCOL_BLACK, FONT_SIZE8, true, false);
	AddToList(&save);

	area2.y2 += area2.y2 - area2.y1;
	area2.y1 += (area2.y2 - area.y1) >> 1;
	area2.y2 += 4;
	area2.y1 += 4;

	qsave.SetShape(area2);

	area.x2 = area2.x1-3;
	name.Setup(gui, this, TEXT_ID, area, string, strlength, FCOL_WHITE | BCOL_BLACK, FONT_SIZE8, true);
	area = name.MinimiseShape();
	AddToList(&name);

	return true;
}

void C_FileMenu_Save::SendMsg(GUI_MSG msg)
{
	C_GUIScreen::SendMsg(msg);

	BACK_MSG m;

	while(MessagesWaiting())
	{
		m = PopMessage();

		switch(m.owner)
		{
			case BTN_ID :
			case TEXT_ID :
				bmsg.val = FMENUSAVE_SAVE;
				screen->PushMessage(bmsg);
			break;

			case QBTN_ID :
				bmsg.val = FMENUSAVE_QSAVE;
				screen->PushMessage(bmsg);
			break;
		}
	}
}

/* the 'new' */

#define FMENUNEW_DFS1	1
#define FMENUNEW_DFS2	2
#define FMENUNEW_ADFS1	3
#define FMENUNEW_ADFS2	4
#define FMENUNEW_TAPE	5

#define FTYPE_SELECTOR	8

#define FMENUNEW_SAVE	0

class C_FileMenu_New : public C_GUIScreen
{
	public :
		bool Initialise(C_GUI *parent, int x1, int y1, int x2, int y2, C_GUIMessageCollector *scrn, char *string, int strlength, int id);
		void SendMsg(GUI_MSG msg);

	private :
		C_GUI *gui;
		C_GUIMessageCollector *screen;
		BACK_MSG bmsg;

		C_GUITextBox title;
		C_GUITextInputLine name;
		C_GUIButton save;
		C_GUIRadioButton dfs1, dfs2, adfs1, adfs2, tape;

		int fsval;
};

bool C_FileMenu_New::Initialise(C_GUI *parent, int x1, int y1, int x2, int y2, C_GUIMessageCollector *scrn, char *string, int strlength, int id)
{
	GUI_BOX area, area2;

	screen = scrn;
	gui = parent;
	bmsg.owner = id;
	InitialiseList();
	InitialiseStack();

	area.x1 = x1; area.x2 = x2; area.y1 = y1; area.y2 = y2;
	title.Setup(gui, area, "File name : ", FCOL_WHITE | BCOL_BLACK, FONT_SIZE8);
	AddToList(&title);
	area = title.MinimiseShape();
	area.y2 += area.y2-area.y1;
	area.y1 += (area.y2-area.y1) >> 1;
	title.SetShape(area);
	area.x1 = area.x2;

	save.Setup(gui, this, BTN_ID, area, "Create", 0, FCOL_WHITE | BCOL_BLACK, FONT_SIZE8, true, false);

	area2 = save.MinimiseShape();
	area2.x1 = x2 - (area2.x2 - area2.x1);
	area2.x2 = x2;
	save.SetShape(area2);

	AddToList(&save);

	area.x2 = area2.x1-3;
	name.Setup(gui, this, TEXT_ID, area, string, strlength, FCOL_WHITE | BCOL_BLACK, FONT_SIZE8, true);
	area = name.MinimiseShape();
	AddToList(&name);

	area = title.MinimiseShape();
	area.x2 = x2;
	area.y2 += area.y2 - area.y1;
	area.y1 = (area.y2 + area.y1) >> 1;
	area.y2 += area.y2 - area.y1;
	area.y1 = (area.y2 + area.y1) >> 1;

	dfs1.Setup(parent, this, FTYPE_SELECTOR, FMENUNEW_DFS1, &fsval, area, "DFS disc : Single Sided", FONT_SIZE8, FCOL_WHITE | BCOL_BLACK);
	dfs1.MinimiseShape();
	AddToList(&dfs1);
	area.y2 += area.y2 - area.y1;
	area.y1 = (area.y2 + area.y1) >> 1;
	dfs2.Setup(parent, this, FTYPE_SELECTOR, FMENUNEW_DFS2, &fsval, area, "DFS disc : Double Sided", FONT_SIZE8, FCOL_WHITE | BCOL_BLACK);
	dfs2.MinimiseShape();
	AddToList(&dfs2);

	area.y1 -= area.y2 - area.y1;
	area.y2 = (area.y2 + area.y1) >> 1;
	area.x1 += (x2 - x1) / 3;

	adfs1.Setup(parent, this, FTYPE_SELECTOR, FMENUNEW_ADFS1, &fsval, area, "ADFS disc : Single Sided", FONT_SIZE8, FCOL_WHITE | BCOL_BLACK);
	adfs1.MinimiseShape();
	AddToList(&adfs1);
	area.y2 += area.y2 - area.y1;
	area.y1 = (area.y2 + area.y1) >> 1;
	adfs2.Setup(parent, this, FTYPE_SELECTOR, FMENUNEW_ADFS2, &fsval, area, "ADFS disc : Double Sided", FONT_SIZE8, FCOL_WHITE | BCOL_BLACK);
	adfs2.MinimiseShape();
	AddToList(&adfs2);

	area.y1 -= area.y2 - area.y1;
	area.y2 = (area.y2 + area.y1) >> 1;
	area.x1 += (x2 - x1) / 3;

	tape.Setup(parent, this, FTYPE_SELECTOR, FMENUNEW_TAPE, &fsval, area, "Cassette", FONT_SIZE8, FCOL_WHITE | BCOL_BLACK);
	tape.MinimiseShape();
	AddToList(&tape);

	/* default is double sided ADFS */
	bmsg.val = fsval = FMENUNEW_ADFS2;
	screen->PushMessage(bmsg);

	return true;
}

void C_FileMenu_New::SendMsg(GUI_MSG msg)
{
	C_GUIScreen::SendMsg(msg);

	BACK_MSG m;

	while(MessagesWaiting())
	{
		m = PopMessage();

		switch(m.owner)
		{
			case BTN_ID :
			case TEXT_ID :
				bmsg.val = FMENUNEW_SAVE;
				screen->PushMessage(bmsg);
			break;

			case FTYPE_SELECTOR :
				bmsg.val = m.val;
				screen->PushMessage(bmsg);
			break;
		}
	}
}

/* the general options sub-screen */

class C_FileMenu_General : public C_GUIScreen
{
	public :
		bool Initialise(C_GUI *parent, int x1, int y1, int x2, int y2, C_GUIMessageCollector *scrn, int id);
		void DeInitialise(void);

		void SendMsg(GUI_MSG msg);

	private :
		C_GUI *gui;

		C_GUITextBox title;
		C_GUIRadioButton icons, wordsthin, wordswide;
		int rval;
		C_GUIRadioButton drive1, drive2;
		int drive;
		C_GUITickBox loadreturn, savereturn, autoload;
		C_GUIMessageCollector *screen;
		BACK_MSG bmsg;
};

#define DISPLAY_SELECTOR	0
#define LOAD_SELECTOR		1
#define SAVE_SELECTOR		2
#define AUTO_SELECTOR		3
#define DRIVE_SELECTOR		4

#define GENERALVALUE_SETICONS		0
#define GENERALVALUE_SETWORDSTHIN	1
#define GENERALVALUE_SETWORDSWIDE	2

#define GENERALVALUE_LQUIT_Y		3
#define GENERALVALUE_LQUIT_N		4
#define GENERALVALUE_SQUIT_Y		5
#define GENERALVALUE_SQUIT_N		6
#define GENERALVALUE_ALOAD_Y		7
#define GENERALVALUE_ALOAD_N		8
#define GENERALVALUE_DRIVE_1		9
#define GENERALVALUE_DRIVE_2		10

bool C_FileMenu_General::Initialise(C_GUI *parent, int x1, int y1, int x2, int y2, C_GUIMessageCollector *scrn, int id)
{
	GUI_BOX b;
	gui = parent;
	InitialiseStack();
	InitialiseList();
	screen = scrn;
	bmsg.owner = id;

	rval = 0;
	b.x1 = x1;
	b.x2 = x2;
	b.y1 = y1;
	b.y2 = y2;

	title.Setup(parent, b, "Display format", FCOL_WHITE | BCOL_BLACK, FONT_SIZE8);
	AddToList(&title);
	b = title.MinimiseShape();
	b.y2 += b.y2 - b.y1;
	b.y1 += (b.y2 - b.y1) >> 1;
	title.SetShape(b);
	b.y2 += b.y2 - b.y1;
	b.y1 += (b.y2 - b.y1) >> 1;
	b.x1 += 5;
	b.x2 = x2;

	icons.Setup(parent, this, DISPLAY_SELECTOR, GENERALVALUE_SETICONS, &rval, b, "Icons", FONT_SIZE8, FCOL_WHITE | BCOL_BLACK);
	b = icons.MinimiseShape();
	AddToList(&icons);

	b.x2 = x2;
	b.y2 += b.y2 - b.y1;
	b.y1 += (b.y2 - b.y1) >> 1;
	wordsthin.Setup(parent, this, DISPLAY_SELECTOR, GENERALVALUE_SETWORDSTHIN, &rval, b, "Double column words", FONT_SIZE8, FCOL_WHITE | BCOL_BLACK);
	b = wordsthin.MinimiseShape();
	AddToList(&wordsthin);

	b.x2 = x2;
	b.y2 += b.y2 - b.y1;
	b.y1 += (b.y2 - b.y1) >> 1;
	wordswide.Setup(parent, this, DISPLAY_SELECTOR, GENERALVALUE_SETWORDSWIDE, &rval, b, "Single column words", FONT_SIZE8, FCOL_WHITE | BCOL_BLACK);
	b = wordswide.MinimiseShape();
	AddToList(&wordswide);

	b.x1 = (x2 + x1) /3;
	b.x2 = x2;
	savereturn.Setup(parent, this, SAVE_SELECTOR, b, "Exit on save/create", FONT_SIZE8, FCOL_WHITE | BCOL_BLACK);
	savereturn.MinimiseShape();
	AddToList(&savereturn);

	b.y1 -= b.y2 - b.y1;
	b.y2 -= (b.y2 - b.y1) >> 1;
	loadreturn.Setup(parent, this, LOAD_SELECTOR, b, "Exit on load", FONT_SIZE8, FCOL_WHITE | BCOL_BLACK);
	loadreturn.MinimiseShape();
	AddToList(&loadreturn);

	b.x2 = x2;
	b.y1 -= b.y2 - b.y1;
	b.y2 -= (b.y2 - b.y1) >> 1;
	autoload.Setup(parent, this, AUTO_SELECTOR, b, "attempt automatic load", FONT_SIZE8, FCOL_WHITE | BCOL_BLACK);
	autoload.MinimiseShape();
	AddToList(&autoload);

	b.x1 = 2*(x2 + x1) /3;
	b.x2 = x2;
	drive1.Setup(parent, this, DRIVE_SELECTOR, GENERALVALUE_DRIVE_1, &drive, b, "Drive 1", FONT_SIZE8, FCOL_WHITE | BCOL_BLACK);
	b = drive1.MinimiseShape();
	AddToList(&drive1);
	drive = GENERALVALUE_DRIVE_1;

	b.x2 = x2;
	b.y2 += b.y2 - b.y1;
	b.y1 += (b.y2 - b.y1) >> 1;
	drive2.Setup(parent, this, DRIVE_SELECTOR, GENERALVALUE_DRIVE_2, &drive, b, "Drive 2", FONT_SIZE8, FCOL_WHITE | BCOL_BLACK);
	b = drive2.MinimiseShape();
	AddToList(&drive2);

	/* initialise load / save quit states */
	if(gui->GetElectron()->GetConfig()->FindSetting("exitonload", BOOLEAN_RESULT))
	{
		loadreturn.SetTicked(gui->GetElectron()->GetConfig()->ReadSetting().result_bool);// ? GENERALVALUE_LQUIT_Y : GENERALVALUE_LQUIT_N;
	}
	else
		loadreturn.SetTicked(false);

	bmsg.val = loadreturn.GetTicked() ? GENERALVALUE_LQUIT_Y : GENERALVALUE_LQUIT_N;
	screen->PushMessage(bmsg);

	if(gui->GetElectron()->GetConfig()->FindSetting("exitonsave", BOOLEAN_RESULT))
	{
		savereturn.SetTicked(gui->GetElectron()->GetConfig()->ReadSetting().result_bool);// ? GENERALVALUE_LQUIT_Y : GENERALVALUE_LQUIT_N;
	}
	else
		savereturn.SetTicked(false);

	bmsg.val = loadreturn.GetTicked() ? GENERALVALUE_SQUIT_Y : GENERALVALUE_SQUIT_N;
	screen->PushMessage(bmsg);

	/* initialise iconographic state */

	if(gui->GetElectron()->GetConfig()->FindSetting("fileicons", INTEGER_RESULT))
	{
		switch(gui->GetElectron()->GetConfig()->ReadSetting().result_int)
		{
			case 0 : rval = GENERALVALUE_SETICONS; break;
			case 1 : rval = GENERALVALUE_SETWORDSTHIN; break;
			case 2 : rval = GENERALVALUE_SETWORDSWIDE; break;
		}
	}
	else
		rval = GENERALVALUE_SETICONS;

	bmsg.val = rval;
	screen->PushMessage(bmsg);

	/* initialise autorun state */

	if(gui->GetElectron()->GetConfig()->FindSetting("autoload", BOOLEAN_RESULT))
	{
		autoload.SetTicked(gui->GetElectron()->GetConfig()->ReadSetting().result_bool);
	}
	else
		autoload.SetTicked(true);

	bmsg.val = autoload.GetTicked() ? GENERALVALUE_ALOAD_Y : GENERALVALUE_ALOAD_N;
	screen->PushMessage(bmsg);

	return true;
}

void C_FileMenu_General::DeInitialise(void)
{
	config_setting s;

	/* save iconographic state */
	s.result_type = INTEGER_RESULT;
	s.result_int = rval;
	gui->GetElectron()->GetConfig()->FindSetting("fileicons", ANY_RESULT);
	gui->GetElectron()->GetConfig()->WriteSetting(s);

	/* save 'quit on ...' options */
	s.result_type = BOOLEAN_RESULT;
	s.result_bool = loadreturn.GetTicked();
	gui->GetElectron()->GetConfig()->FindSetting("exitonload", ANY_RESULT);
	gui->GetElectron()->GetConfig()->WriteSetting(s);

	s.result_bool = savereturn.GetTicked();
	gui->GetElectron()->GetConfig()->FindSetting("exitonsave", ANY_RESULT);
	gui->GetElectron()->GetConfig()->WriteSetting(s);

	/* and autoload option */
	s.result_bool = autoload.GetTicked();
	gui->GetElectron()->GetConfig()->FindSetting("autoload", ANY_RESULT);
	gui->GetElectron()->GetConfig()->WriteSetting(s);
}

void C_FileMenu_General::SendMsg(GUI_MSG msg)
{
	C_GUIScreen::SendMsg(msg);

	BACK_MSG m;
	while(MessagesWaiting())
	{
		m = PopMessage();
		switch(m.owner)
		{
			case DISPLAY_SELECTOR :
				bmsg.val = m.val;
				screen->PushMessage(bmsg);
			break;

			case DRIVE_SELECTOR :
				bmsg.val = m.val;
				screen->PushMessage(bmsg);
			break;

			case LOAD_SELECTOR :
				bmsg.val = loadreturn.GetTicked() ? GENERALVALUE_LQUIT_Y : GENERALVALUE_LQUIT_N;
				screen->PushMessage(bmsg);
			break;

			case SAVE_SELECTOR :
				bmsg.val = savereturn.GetTicked() ? GENERALVALUE_SQUIT_Y : GENERALVALUE_SQUIT_N;
				screen->PushMessage(bmsg);
			break;

			case AUTO_SELECTOR :
				bmsg.val = autoload.GetTicked() ? GENERALVALUE_ALOAD_Y : GENERALVALUE_ALOAD_N;
				screen->PushMessage(bmsg);
			break;
		}
	}
}

/* the main part of the menu screen */

#define TABS_ID		1
#define GENERAL_ID	2
#define NEW_ID		3
#define STATE_ID	4
#define SCREEN_ID	5

C_FileMenu_General general;
C_FileMenu_Save SaveState, SaveScreen;
C_FileMenu_New NewFile;
C_GUITextBox upd;
C_GUIScreen *baby, *childlist[] = {NULL, NULL, NULL, NULL};
char str1[256], str2[256], str3[256];
bool lstate, sstate, aload, newdisc;
int driveid;
FSTypes newtype;

bool C_FileMenu::Initialise(C_GUI *parent, int x1, int y1, int x2, int y2)
{
	GUI_BOX area2;

	gui = parent;
	elk = gui->GetElectron();
	fs = elk->GetFS();

	InitialiseStack();

	if(elk->GetFS()->ValidPath(NULL, "startdir"))
	{
		elk->GetConfig()->FindSetting("startdir", STRING_RESULT);
		path = strdup(elk->GetConfig()->ReadSetting().result_string);
	}
	else
	{
		path = strdup("/");
	}

	files = NULL;
	filearea.x1 = x1;
	filearea.y1 = y1;
	filearea.x2 = x2;
	filearea.y2 = filearea.y1+385;

	field.Setup(gui, filearea);

	area2.y1 = filearea.y2+2;
	area2.y2 = y2;
	area2.x1 = x1;
	area2.x2 = x2;

	tabs.Setup(gui, this, TABS_ID, area2, FONT_SIZE8, 4, "General", "New", "Save State", "Save Picture");
	area2 = tabs.GetExcludedArea();
	upd.Setup(gui, area2, "  Last action : none", FCOL_RED | BCOL_BLACK, FONT_SIZE12);

	area2 = tabs.GetEnclosedArea();

	childlist[0] = &general;
	general.Initialise(gui, area2.x1, area2.y1, area2.x2, area2.y2, this, GENERAL_ID);

	childlist[1] = &NewFile;
	NewFile.Initialise(gui, area2.x1, area2.y1, area2.x2, area2.y2, this, str3, 256, NEW_ID);

	childlist[2] = &SaveState;
	SaveState.Initialise(gui, area2.x1, area2.y1, area2.x2, area2.y2, this, str1, 256, STATE_ID);

	childlist[3] = &SaveScreen;
	SaveScreen.Initialise(gui, area2.x1, area2.y1, area2.x2, area2.y2, this, str2, 256, SCREEN_ID);

	baby = childlist[0];

	NewDir();

	driveid = 0;

	/* fix initial state of various flags */
	GUI_MSG m;
	m.msg = GUIM_NONE;
	SendMsg(m);

	return true;
}

void C_FileMenu::DeInitialise(void)
{
	config_setting s;

	elk->GetConfig()->FindSetting("startdir", ANY_RESULT);
	s.result_string = path;
	s.result_type = STRING_RESULT;
	elk->GetConfig()->WriteSetting(s);

	free(path);

	if(files)
	{
		while(numfiles--)
		{
			files[numfiles].Kill();
		}
		delete files;
		files = NULL;
	}

	general.DeInitialise();
	SaveState.DeInitialise();
	SaveScreen.DeInitialise();
}

void C_FileMenu::NewDir(void)
{
	/* kill any file elements already alloc'd */

	if(files)
	{
		while(numfiles--)
		{
			files[numfiles].Kill();
		}
		delete files;
		files = NULL;
	}

	/* inspect the new directory */

	FileTree *newpath, *parser;
	char *extensions[] = {"uef", "t2p\a", "t2p3\a", "t2peg\a", "wav\a", "wave\a", "raw\a", "au\a", "aif\a", "aiff\a", "ssd\a", "dsd\a", "img\a", "adf\a", "adm\a", "adl\a", NULL};
	int c;

	if(parser = newpath = fs->GetFileTree(path, NULL, extensions))
	{
		bool hasup = false;

		/* determine the number of files */
		numfiles = 0;
		while(parser)
		{
			numfiles++;
			parser = parser->next;
		}

		/* allocate enough file elements */
		files = new C_GUIFileElement[numfiles];

		/* generate file elements */
		parser = newpath;
		c = 0;
		while(parser)
		{
			files[c].Setup(gui, this, -c, filearea, parser->fname, parser->dir);
			c++;
			parser = parser->next;
		}

		/* reseed scroll field */
		field.SetElements(files, numfiles, sizeof(C_GUIFileElement));

		/* free filetree */
		fs->FreeFileTree(newpath);

		/* prepare icon fixer */
		idone = false;
		icount = icid = 0;
	}
}

void C_FileMenu::Enter(void)
{
	upd.ChangeText("  Last action : none");

	if(baby)
		baby->Enter();
}

void C_FileMenu::Exit(void)
{
	if(baby)
		baby->Exit();
}

void C_FileMenu::SendMsg(GUI_MSG msg)
{
	field.SendMsg(msg);
	tabs.SendMsg(msg);

	if(baby)
		baby->SendMsg(msg);

	BACK_MSG m;
	char tstring[256];

	while(MessagesWaiting())
	{
		m = PopMessage();

		if(m.owner <= 0)
		{
			m.owner = -m.owner;

			if(files[m.owner].HoldsDir())
			{
				char *p2;
				p2 = fs->GetCompositePath(path, NULL, files[m.owner].GetName(), NULL, NULL);

				free(path);
				path = p2;
				NewDir();
			}
			else
			{
				//load file

				if(!files[m.owner].HasIcons())
				{
					while(icount)
						icount = files[icid].FixIcons(path, icount) ? 0 : (icount+1);

					if(icid != m.owner)
					{
						do
							icount = files[m.owner].FixIcons(path, icount) ? 0 : (icount+1);
						while(icount);
					}
				}

				AttemptLoad(path, &files[m.owner]);
				sprintf(tstring, "  Last action : loaded %s", files[m.owner].GetName());
				upd.ChangeText(tstring);

				if(lstate)
					gui->SignalFinished();
			}
		}
		else
		{
			char *name;

			switch(m.owner)
			{
				case TABS_ID :
					if(baby)
						baby->Exit();
					baby = childlist[m.val];
					if(baby)
						baby->Enter();
				break;

				case GENERAL_ID :
					switch(m.val)
					{
						case GENERALVALUE_SETICONS :
						case GENERALVALUE_SETWORDSTHIN :
						case GENERALVALUE_SETWORDSWIDE :
							files[0].SetIconographic(m.val);
							field.SetElements(files, numfiles, sizeof(C_GUIFileElement));
						break;

						case GENERALVALUE_LQUIT_Y : lstate = true;	break;
						case GENERALVALUE_LQUIT_N : lstate = false;	break;
						case GENERALVALUE_SQUIT_Y : sstate = true;	break;
						case GENERALVALUE_SQUIT_N : sstate = false;	break;
						case GENERALVALUE_ALOAD_Y : aload = true; break;
						case GENERALVALUE_ALOAD_N : aload = false; break;
						case GENERALVALUE_DRIVE_1 : driveid = 0; break;
						case GENERALVALUE_DRIVE_2 : driveid = 1; break;
					}
				break;

				case NEW_ID :
					switch(m.val)
					{
						case FMENUNEW_SAVE :
							upd.ChangeText("  Last action : created new file");

							if(newdisc)
								elk->GetCurrentDrive()->New(path, str3, 0, newtype);
							else
								elk->GetTape()->New(path, str3, 0, 0);

							NewDir();
							if(sstate)
								gui->SignalFinished();
						break;

						case FMENUNEW_DFS1	: newdisc = true; newtype = FS_DFS;		break;
						case FMENUNEW_DFS2	: newdisc = true; newtype = FS_DBLDFS;	break;
						case FMENUNEW_ADFS1	: newdisc = true; newtype = FS_ADFS;	break;
						case FMENUNEW_ADFS2	: newdisc = true; newtype = FS_DBLADFS;	break;
						case FMENUNEW_TAPE	: newdisc = false;						break;
					}
				break;

				case SCREEN_ID :
					switch(m.val)
					{
						case FMENUSAVE_SAVE :
							name = fs->GetCleanFilename(str2, "pcx");
							elk->GetScreen()->SaveImage(str2);
							sprintf(tstring, "  Last action : saved screen as %s", name);
							upd.ChangeText(tstring);
							free(name);
						break;

						case FMENUSAVE_QSAVE :
							elk->GetScreen()->SaveImage(NULL);
							upd.ChangeText("  Last action : quicksaved image");
						break;
					}
				break;

				case STATE_ID :
					switch(m.val)
					{
						case FMENUSAVE_SAVE :
							name = fs->GetCleanFilename(str1, "uef");
							elk->SaveSnapshot(path, name, 0);
							sprintf(tstring, "  Last action : saved state as %s", name);
							upd.ChangeText(tstring);
							free(name);
						break;

						case FMENUSAVE_QSAVE :
							elk->SaveSnapshot(path, "kwiksave.uef", 0);
							upd.ChangeText("  Last action : quicksaved state");
						break;
					}

					NewDir();
					if(sstate)
						gui->SignalFinished();
				break;
			}
		}
	}
}

void C_FileMenu::Draw(void)
{
	field.Draw();
	tabs.Draw();
	upd.Draw();

	if(baby)
	{
		baby->Draw();
	}

	if(!icount && !idone)
	{
		int topleft = field.GetTopLeft();

		icid = topleft;

		// try and find the first thing after the top left which doesn't yet
		// have icons - a good plan for keeping up with the scroll bar

		while(icid < numfiles)
		{
			if(!files[icid].HasIcons())
				break;
			icid++;
		}

		if(icid == numfiles)
		{
			// if no files were found afterwards, check before

			icid = 0;
			while(icid < topleft)
			{
				if(!files[icid].HasIcons())
					break;
				icid++;
			}

			if(icid == topleft)
			{
				// everything has icons
				idone = true;
			}
		}
	}

	if(!idone)
	{
		icount = files[icid].FixIcons(path, icount) ? 0 : (icount+1);
	}
}

void C_FileMenu::AttemptLoad(char *path, C_GUIFileElement *file)
{
	int t = file->GetStats(), LoadID = 0xff;
	char *name = file->GetName();

	if(t&FSTAT_SNAP)
		elk->LoadSnapshot(path, name, 0);

	if(t&FSTAT_DISC)
	{
		elk->GetParticularDrive(driveid)->Open(path, name, 0);
		LoadID = elk->GetCurrentDrive()->GetAutoloadID();
	}

	if(t&FSTAT_TAPE)
	{
		elk->GetTape()->Open(path, name, 0, 0);
		LoadID = elk->GetTape()->GetAutoloadID();
	}

	if(aload && (LoadID != 0xff))
	{
		gui->PassROMHint(false, ROMCLASS_IGNORERESET, NULL);
		elk->SuperReset();
		elk->SetAutoloadID(LoadID);
	}
}

void C_FileMenu::EvaluatePreload(void)
{
	/* parse preload config file section */
	if(elk->GetConfig()->FindSetting("load", STRING_RESULT))
	{
		char *ptemp, *ltemp, *t = strdup(elk->GetConfig()->ReadSetting().result_string);
		ptemp = ltemp = t;

		while(*ltemp != '\0')
		{
			while(*ptemp != ';' && *ptemp != '\0')
				ptemp++;

			if(*ptemp)
			{
				*ptemp++ = '\0';
			}

			if(strlen(ltemp))
			{
				/* determine type of file */
				C_GUIFileElement tfile;
				GUI_BOX temp;
				int ic = 0;

				tfile.Setup(gui, this, 0, temp, ltemp, false);
				while(!tfile.FixIcons(NULL, ic)) ic++;

				/* load as necessary */
				AttemptLoad(NULL, &tfile);
				tfile.Kill();
			}

			ltemp = ptemp;
		}
		free(t);
	}

	/* parse program arguments */
	int c;
	char *name, *namep, *copy;

	for(c = 1; c < parcount; c++)
	{
		copy = parlist[c];
		namep = name = (char *)malloc(strlen(copy)+1);

		while(*copy)
		{
			if(*copy != '"') *name++ = *copy;
			copy++;
		}
		*name = '\0';

		C_GUIFileElement tfile;
		GUI_BOX temp;
		int ic = 0;

		tfile.Setup(gui, this, 0, temp, namep, false);
		while(!tfile.FixIcons(NULL, ic)) ic++;

		/* load as necessary */
		AttemptLoad(NULL, &tfile);
		tfile.Kill();
		free(namep);
	}
}

