#ifndef GUILOCAL_H
#define GUILOCAL_H

#include "gui.h"

/* some structs which move between the GUI & the GUI elements */

struct GUI_BOX
{
	int x1, y1;
	int x2, y2;
};

struct GUI_DSCRPTR
{
	GUI_BOX loc;
	bool mouseover, ignoremouse;

	int newclicks;
};

/* this is used by GUI elements to comunicate a change of state back to the
GUI screen */

struct BACK_MSG
{
	int owner, val;
};

/*

a simple message collecting anything - all C_GUIScreen derived classes collect
messages, and some of the C_GUIElementBase derived classes do also

*/

class C_GUIMessageCollector
{
	public :
		void PushMessage(BACK_MSG msg);
		BACK_MSG PopMessage(void);
		void InitialiseStack(void);
		bool MessagesWaiting(void);

	protected :
		int sptr;
		BACK_MSG stack[32];
};

/* the basics of any GUI screen */

class C_GUI;
class C_GUIElementBase;

class C_GUIScreen : public C_GUIMessageCollector
{
	public :
		virtual bool Initialise(C_GUI *parent, int x1, int y1, int x2, int y2);
		virtual void DeInitialise(void);

		virtual void Enter(void);
		virtual void Exit(void);

		virtual void SendMsg(GUI_MSG msg);
		virtual	void Draw(void);

		void InitialiseList(void);
		void AddToList(C_GUIElementBase *elem);
		void KillList(void);

	private :
		struct Elem
		{
			Elem *next;
			C_GUIElementBase *e;
		} *list;
};

/* the basics of a GUI element */

class C_GUIElementBase
{
	public :
		virtual GUI_BOX MinimiseShape(void);
		virtual void Draw(void) = 0;
		virtual void SendMsg(GUI_MSG msg);
		virtual void Kill(void);
		virtual void Reset(void);
		virtual void SetPosition(int x, int y);

		virtual GUI_BOX GetShape(void);
		virtual void SetShape(GUI_BOX box);

	protected :
		GUI_DSCRPTR desc;
		C_gfx_api *gfx;
		C_GUI *gui;
};

/* the GUI elements */

class C_GUITextBox : public C_GUIElementBase
{
	public :
		C_GUITextBox(void);

		GUI_BOX MinimiseShape(void);
		void Draw(void);
		void Kill(void);

		void Setup(C_GUI *g, GUI_BOX area, char *text, int col, int fnt);
		void ChangeText(char *newtext);

	private :
		void InitialiseState(void);
		bool GenerateNextLine(void);

		int y;
		char *pos, *nextpos, *start;

		char *string, *copy;
		int colour;
		int font;
};

class C_GUILine : public C_GUIElementBase
{
	public :
		void Draw(void);

		void Setup(C_GUI *g, GUI_BOX area, int col);
		void ChangeColour(int col);

	private :
		int colour;
};

class C_GUIImage : public C_GUIElementBase
{
	public :
		GUI_BOX MinimiseShape(void);
		void Draw(void);
		void SendMsg(GUI_MSG msg);
		void Setup(C_GUI *g, C_GUIMessageCollector *scrn, int id, GUI_BOX area, int img, bool t);

	private :
		int imgid;
		bool trans;
		C_GUIMessageCollector *screen;
		BACK_MSG bmsg;
};

class C_GUIOutlineBox : public C_GUIElementBase
{
	public :
		void Draw(void);
		void Setup(C_GUI *g, GUI_BOX area, int col);

	private :
		int colour;
};

#define GUIBUTTON_PERFORM 0

class C_GUIButton : public C_GUIElementBase
{
	public :
		C_GUIButton(void);

		GUI_BOX MinimiseShape(void);
		void Draw(void);
		void SendMsg(GUI_MSG msg);
		void Kill(void);
		void Set(bool);

		void Setup(C_GUI *g, C_GUIMessageCollector *scrn, int id, GUI_BOX area, char *text, int img, int col, int fnt, bool brder, bool persist);
		void ChangeText(char *newtext);
		void ChangeColour(int newcol);
		void SetShape(GUI_BOX box);

	private :
		C_GUIMessageCollector *screen;
		BACK_MSG bmsg;
		char *label;
		int imgid, font, colour;
		bool border, persistant;

		int wx, wy;
		bool pressed;
		C_GUIOutlineBox over;
};

#define GUITICKBOX_CHANGE 0

class C_GUITickBox : public C_GUIElementBase
{
	public :
		C_GUITickBox();

		GUI_BOX MinimiseShape(void);
		void Draw(void);
		void SendMsg(GUI_MSG msg);
		void Kill(void);

		void Setup(C_GUI *g, C_GUIMessageCollector *scrn, int id, GUI_BOX area, char *text, int font, int colour);

		void SetTicked(bool set);
		bool GetTicked(void);
		void SetShape(GUI_BOX box);
		void ChangeColour(int newcol);
		void ChangeText(char *nt);

		void SetActive(bool set);

	private :
		C_GUIMessageCollector *screen;
		BACK_MSG bmsg;
		char *label;
		int font, colour;
		bool ticked, active;

		int wx, wy, ty;
		C_GUIOutlineBox over;
};

class C_GUIRadioButton : public C_GUIElementBase
{
	public :
		C_GUIRadioButton();

		GUI_BOX MinimiseShape(void);
		void Draw(void);
		void SendMsg(GUI_MSG msg);
		void Kill(void);

		void Setup(C_GUI *g, C_GUIMessageCollector *scrn, int id, int number, int *var, GUI_BOX area, char *text, int font, int colour);

		void SetShape(GUI_BOX box);
		void ChangeColour(int newcol);
		void ChangeText(char *nt);

		void SetActive(bool set);

	private :
		C_GUIMessageCollector *screen;
		BACK_MSG bmsg;

		/* for state purposes */
		bool selected, active;
		int *var;

		/* for display purposes */
		char *label;
		int font, colour;
		int wx, wy, ty;
		C_GUIOutlineBox over;
};

#define SLIDEBAR_MOVE 0

class C_GUISlideBar : public C_GUIElementBase
{
	public :
		GUI_BOX MinimiseShape(void);
		void Draw(void);
		void SendMsg(GUI_MSG msg);

		void Setup(C_GUI *g, C_GUIMessageCollector *scrn, int id, GUI_BOX area, int img, bool horizontal, int col);

		void SetValue(float value);
		float GetValue(void);
		void MovePixels(int adder);

	private :
		C_GUIMessageCollector *screen;
		BACK_MSG bmsg;

		/* for state purposes */
		int position;
		bool horiz;

		int left, right, draw;

		/* for display purposes */

		int imgid, colour;

		float value;
		void RecalcValue(bool valleader);
};

class C_GUISlider : public C_GUIElementBase, public C_GUIMessageCollector
{
	public :
		C_GUISlider();

		void Draw(void);
		void SendMsg(GUI_MSG msg);
		void Kill(void);

		void Setup(C_GUI *g, C_GUIMessageCollector *scrn, int id, GUI_BOX area, char *text, int font, int colour, int range, int base);

		void SetShape(GUI_BOX box);
		void ChangeColour(int newcol);
		void ChangeText(char *nt);

		void SetActive(bool set);

		void SetValue(float value);
		float GetValue(void);

	private :
		C_GUIMessageCollector *screen;
		BACK_MSG bmsg;

		/* for display purposes */

		//name
		char *label;
		int font, colour;
		int wx, wy;
		C_GUIOutlineBox over;

		//numeric quantity
		int qx;
		char qtext[10];

		//(de/in)crease button
		C_GUIButton dec, inc;

		//slider itself
		C_GUISlideBar slider;

		//numeric indicator
		int npos;
		float mult, base;
};

class C_GUIScrollField : public C_GUIElementBase, public C_GUIMessageCollector
{
	public :
		C_GUIScrollField();

		void Draw(void);
		void SendMsg(GUI_MSG msg);
		void Kill(void);

		void Setup(C_GUI *g, GUI_BOX area);

		void SetElements(C_GUIElementBase *list, int num, int elsize);
		int GetTopLeft(void);

		GUI_BOX GetEnclosedArea(void);

	private :
		/* for display purposes */
		void ReseedPosition(void);

		//(de/in)crease button
		C_GUIButton dec, inc;

		//slider itself
		C_GUISlideBar slider;
		GUI_BOX sbox;

		//children
		C_GUIElementBase *list;
		int listlen;

		int maxx, pos, offset, xoffset;
		int width, height;
		int numperline, lines, total;
		int size;
};

#define FOLDERTABS_BASE 0

class C_GUIFolderTabs : public C_GUIElementBase, public C_GUIMessageCollector
{
	public :
		C_GUIFolderTabs();

		void Draw(void);
		void SendMsg(GUI_MSG msg);
		void Kill(void);

		void Setup(C_GUI *g, C_GUIMessageCollector *scrn, int id, GUI_BOX area, int font, int numentries, ...);
		void SetFront(int c);

		void StartEntryList(int numentries);
		void AddEntry(char *title);


		GUI_BOX GetEnclosedArea(void);
		GUI_BOX GetExcludedArea(void);

	private :
		C_GUIMessageCollector *screen;
		C_GUILine *lines;
		C_GUIButton *btns;

		BACK_MSG bmsg;
		int numlines, numbtns;

		int numtops, font, topcounter;
		char **tops;
		void FinishTabs(void);
};

#define GUITEXTINPUTLINE_PERFORM 0

class C_GUITextInputLine : public C_GUIElementBase
{
	public :
		C_GUITextInputLine();

		GUI_BOX MinimiseShape(void);
		void Draw(void);
		void SendMsg(GUI_MSG msg);

		void Setup(C_GUI *g, C_GUIMessageCollector *scrn, int id, GUI_BOX area, char *text, unsigned int length, int col, int fnt, bool aa);

	private :
		C_GUIMessageCollector *screen;
		BACK_MSG bmsg;
		int font, colour;

		char *string;
		unsigned int strlength;
		bool active, activealways;
};

#endif
