/*

	ElectrEm (c) 2000/1 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for
	details

	gui/about.cpp
	=============

	This is the 'Hardware Options' screen.

*/
#include "hwsetup.h"

#include "../gfx/gfxdefs.h"
#include "../gfx/gfx.h"
#include "guidefs.h"

#include "../misc/timing.h"
#include "../screen/screen.h"
#include "../wd1770/drive.h"
#include "../sound/speaker.h"
#include "../ula/ula.h"
#include "../6502/6502.h"
#include "../tape/tape.h"
#include <string.h>
#include <malloc.h>

#define AddBox(name, v)	\
	boxes[v].Setup(parent, box, SCOL_GREY);\
	AddToList(&boxes[v]);\
	box.x1 += 2;\
	box.y1 += 2;\
	names[v].Setup(parent, box, name, FCOL_WHITE | BCOL_BLACK, FONT_SIZE18);\
	AddToList(&names[v]);

#define Begin(v)\
	box = names[v].MinimiseShape();\
	box2 = boxes[v].GetShape();\
	box.y1 += (box.y2 - box.y1) + 5;\
	box.y2 = box2.y2;\
	box.x1 += 5;\
	box.x2 = box2.x2;

#define DISCEJECT1_ID	2
#define DISCEJECT2_ID	3
#define RESET_ID		4
#define SPEEDSET_ID		5
#define FASTTAPE_ID		6
#define JIM_ID			7
#define REWIND_ID		8

#define TAPE_OUTPUTOPTS 9

bool C_HardwareMenu::Initialise(C_GUI *parent, int x1, int y1, int x2, int y2)
{
	int y, c;
	GUI_BOX box, box2;

	gui = parent;
	elk = parent->GetElectron();
	InitialiseStack();
	InitialiseList();

	/* speed slider */
	box.x1 = x1;
	box.x2 = x2;
	box.y1 = y1;
	box.y2 = y2;
	speed.Setup(parent, this, 0, box, "Speed: ", FONT_SIZE12, FCOL_WHITE | BCOL_BLACK, 750, 50);
	AddToList(&speed);
	speedreset.Setup(parent, this, SPEEDSET_ID, box, "100%", 0, FCOL_WHITE | BCOL_BLACK, FONT_SIZE12, true, false);
	AddToList(&speedreset);

	box = speed.MinimiseShape();
	box2 = speedreset.MinimiseShape();

	c = box2.x2 - box2.x1;
	box2.x2 = x2 - 6;
	box2.x1 = box2.x2 - c;
	y = box2.y2 - box2.y1;
	box2.y2 = y2 - 3;
	box2.y1 = box2.y2 - y;
	speedreset.SetShape(box2);

	y = box.y2 - box.y1;
	box.y2 = y2 - 3;
	box.y1 = box.y2 - y;
	box.x2 = box2.x1 - 3;
	speed.SetShape(box);

	y2 = box.y1-2;

	/* screen box */
	box.x1 = x1+6;
	box.y1 = y1+6;
	box.x2 = ((x2+x1) >> 1) - 3;
	box.y2 = ((y2+y1) >> 1) - 3 - 30;
	AddBox("Screen", 0);

	/* speaker box */
	box = boxes[0].MinimiseShape();
	box.x1 = box.x2 + 6;
	box.x2 = x2 - 6;
	box.y2 = ((box.y1 + box.y2) >> 1) - 3;
	AddBox("Speaker", 1);

	/* disc box */
	box = boxes[1].MinimiseShape();
	box.y1 = box.y2 + 6;
	box.y2 = ((y2+y1) >> 1) - 3 - 30;
	AddBox("Disc", 2);

	/* misc. box */
	box = boxes[0].MinimiseShape();
	box.y1 = box.y2 + 6;
	box.y2 = y2 - 6;
	AddBox("Miscellaneous", 3);

	/* tape box */
	box = boxes[3].MinimiseShape();
	box.x1 = box.x2 + 6;
	box.x2 = x2 - 6;
	AddBox("Tape", 4);

	/* screen options */
	char *scrstrings[] = {"Use high resolution output", "Allow multiplexing", "Accept changes to authentic palette", "Limit to half-multiple scaling"};

	Begin(0);

	for(c = 0; c < 4; c++)
	{
		screen[c].Setup(parent, this, 0, box, scrstrings[c], FONT_SIZE12, FCOL_WHITE | BCOL_BLACK);
		AddToList(&screen[c]);
		box = screen[c].MinimiseShape();

		y = box.y2 - box.y1;
		box.y1 += y;
		box.y2 += y;
		box.x2 = x2;
	}

	box.y1 += y;
	box.y2 += y;

	char *scalestrings[] = {"Disable output scaling", "Scale only in dire cases", "Always scale"};

	for(c = 0; c < 3; c++)
	{
		scale[c].Setup(parent, this, 0, c, &scaling, box, scalestrings[c], FONT_SIZE12, FCOL_WHITE | BCOL_BLACK);
		AddToList(&scale[c]);
		box = scale[c].MinimiseShape();

		y = box.y2 - box.y1;
		box.y1 += y;
		box.y2 += y;
		box.x2 = x2;
	}

	/* speaker options */
	Begin(1);

	volume.Setup(parent, this, 0, box, "Volume: ", FONT_SIZE12, FCOL_WHITE | BCOL_BLACK, 100, 0);
	AddToList(&volume);
	box = volume.MinimiseShape();

	y = box.y2 - box.y1 + 7;
	box.y1 += y;
	box.y2 += y;
	box.x2 = x2-8;
	buffering.Setup(parent, this, 0, box, "High fidelity sound reproduction", FONT_SIZE12, FCOL_WHITE | BCOL_BLACK);
	AddToList(&buffering);

	/* create 'disc' options */
	Begin(2);

	dwrprot[0].Setup(parent, this, 0, box, "Write Prot. disc 1", FONT_SIZE12, FCOL_WHITE | BCOL_BLACK);
	AddToList(&dwrprot[0]);
	dwrprot[0].MinimiseShape();

	box.x1 = (box.x1+box.x2) >> 1;
	dwrprot[1].Setup(parent, this, 0, box, "Write Prot. disc 2", FONT_SIZE12, FCOL_WHITE | BCOL_BLACK);
	AddToList(&dwrprot[1]);
	dwrprot[1].MinimiseShape();

	box = boxes[2].GetShape();
	disceject[0].Setup(parent, this, DISCEJECT1_ID, box2, "Eject disc 1", 0, FCOL_WHITE | BCOL_BLACK, FONT_SIZE12, true, false);
	disceject[1].Setup(parent, this, DISCEJECT2_ID, box2, "Eject disc 2", 0, FCOL_WHITE | BCOL_BLACK, FONT_SIZE12, true, false);

	AddToList(&disceject[0]);
	AddToList(&disceject[1]);
	box2 = disceject[0].MinimiseShape();

	y = box2.y2 - box2.y1;
	box2.y1 = box.y2 - 5 - y;
	box2.y2 = box.y2 - 5;

	c = (box.x1 + box.x2) >> 1;

	box2.x2 = c -2;
	box2.x1 = box.x1 + 5;
	disceject[0].SetShape(box2);

	box2.x1 = box2.x2 +4;
	box2.x2 = box.x2-5;
	disceject[1].SetShape(box2);

	/* create 'cpu & misc' options */
	char *mpstrings[] = {"Allow multiplexing", "Limit multiplexing by state"};

	Begin(3);

	for(c = 0; c < 2; c++)
	{
		misc[c].Setup(parent, this, 0, box, mpstrings[c], FONT_SIZE12, FCOL_WHITE | BCOL_BLACK);
		AddToList(&misc[c]);
		box = misc[c].MinimiseShape();

		y = box.y2 - box.y1;
		box.y1 += y;
		box.y2 += y;
		box.x2 = x2;
	}

	box.y1 += y;
	box.y2 += y;

	MRBTitle.Setup(parent, box, "Slogger Master RAM Board :", FCOL_WHITE | BCOL_BLACK, FONT_SIZE12);
	AddToList(&MRBTitle);
	box2 = MRBTitle.MinimiseShape();

	y = box2.y2 - box2.y1;
	box.y1 += y;
	box.y2 += y;
	box.x1 += 5;

	char *mrbstrings[] = {"off", "turbo", "shadow"};

	for(c = 0; c < 3; c++)
	{
		mrb[c].Setup(parent, this, 0, c, &mrbmode, box, mrbstrings[c], FONT_SIZE12, FCOL_WHITE | BCOL_BLACK);
		AddToList(&mrb[c]);
		box = mrb[c].MinimiseShape();

		y = box.y2 - box.y1;
		box.y1 += y;
		box.y2 += y;
		box.x2 = x2;
	}

	box.y1 += y;
	box.y2 += y;
	box.x2 = x2;
	box.x1 -= 5;
	jim.Setup(parent, this, JIM_ID, box, "enable JIM page emulation", FONT_SIZE12, FCOL_WHITE | BCOL_BLACK);
	AddToList(&jim);
	jim.MinimiseShape();

	box = boxes[3].GetShape();
	reset.Setup(parent, this, RESET_ID, box, "Super-hard Reset", 0, FCOL_WHITE | BCOL_BLACK, FONT_SIZE12, true, false);
	AddToList(&reset);
	box2 = reset.MinimiseShape();

	y = box2.y2 - box2.y1;
	box2.y1 = box.y2 - 5 - y;
	box2.y2 = box.y2 - 5;
	box2.x1 = box.x1 + 5;
	box2.x2 = box.x2 - 5;
	reset.SetShape(box2);

	/* tape option(s) */
	Begin(4);

	struct
	{
		char *name;
		int id;
	} tins[] = {
		{"Input from sound card", TAPE_INPUTMODE_SOUND},
		{"Input from file", TAPE_INPUTMODE_FILE}
	};
	char * touts[] = {
		"Output to sound card",
		"Output to UEF"
	};

	for(c = 0; c < 2; c++)
	{
		input[c].Setup(parent, this, 0, tins[c].id, &imode, box, tins[c].name, FONT_SIZE12, FCOL_WHITE | BCOL_BLACK);
		AddToList(&input[c]);
		box = input[c].MinimiseShape();

		y = box.y2 - box.y1;
		box.y1 += y;
		box.y2 += y;
		box.x2 = x2;
	}

	box.y1 += y;
	box.y2 += y;

	for(c = 0; c < 2; c++)
	{
		output[c].Setup(parent, this, 0, box, touts[c], FONT_SIZE12, FCOL_WHITE | BCOL_BLACK);
		AddToList(&output[c]);
		box = output[c].MinimiseShape();

		y = box.y2 - box.y1;
		box.y1 += y;
		box.y2 += y;
		box.x2 = x2;
	}

	box.y1 += y;
	box.y2 += y;
	box.x2 = x2;
	tspeed.Setup(parent, this, 0, box, "Use fast loading where possible", FONT_SIZE12, FCOL_WHITE | BCOL_BLACK);
	AddToList(&tspeed);
	tspeed.MinimiseShape();
	box.y1 += y;
	box.y2 += y;
	box.x2 = x2;
	tpipe.Setup(parent, this, 0, box, "Enable input to output piping", FONT_SIZE12, FCOL_WHITE | BCOL_BLACK);
	AddToList(&tpipe);
	tpipe.MinimiseShape();

	box = boxes[4].GetShape();
	taperewind.Setup(parent, this, REWIND_ID, box, "Rewind Cassette to start", 0, FCOL_WHITE | BCOL_BLACK, FONT_SIZE12, true, false);
	AddToList(&taperewind);
	box2 = taperewind.MinimiseShape();

	y = box2.y2 - box2.y1;
	box2.y1 = box.y2 - 5 - y;
	box2.y2 = box.y2 - 5;
	box2.x1 = box.x1 + 5;
	box2.x2 = box.x2 - 5;
	taperewind.SetShape(box2);

	return true;
}

void C_HardwareMenu::Enter(void)
{
	MultiState oldstate;

	oldstate = elk->GetMultiState();

	screen[0].SetTicked(elk->GetScreen()->GetSize());
	screen[1].SetTicked(oldstate.multiscr);
	screen[2].SetTicked(oldstate.palonly);
	screen[3].SetTicked(elk->GetScreen()->GetBitmap()->GetCleanScale());
	scaling = elk->GetScreen()->GetBitmap()->GetStretchLevel();

	buffering.SetTicked(elk->GetSpeaker()->GetAccuracy());
	volume.SetValue(elk->GetSpeaker()->GetVolume());

	dwrprot[0].SetTicked(elk->GetParticularDrive(0)->GetWriteProtect());
	dwrprot[1].SetTicked(elk->GetParticularDrive(1)->GetWriteProtect());

	misc[0].SetTicked(oldstate.multicpu);
	misc[1].SetTicked(oldstate.usestate);

	switch(elk->GetSloggerMRBMode())
	{
		case MRBMODE_OFF : mrbmode = 0; break;
		case MRBMODE_TURBO : mrbmode = 1; break;
		case MRBMODE_SHADOW : mrbmode = 2; break;
	}

	speed.SetValue((elk->GetTimer()->GetMultiplier()-0.5f) / 7.5f);

	tspeed.SetTicked(elk->TapeHackPresent());

	output[0].SetTicked(elk->GetTape()->GetOperationMode(TAPE_OUTPUTMODE_SOUND));
	output[1].SetTicked(elk->GetTape()->GetOperationMode(TAPE_OUTPUTMODE_UEF));
	tpipe.SetTicked(elk->GetTape()->GetOperationMode(TAPE_IO_PIPING));

	if(elk->GetTape()->GetOperationMode(TAPE_INPUTMODE_SOUND))
		imode = TAPE_INPUTMODE_SOUND;
	else
		imode = TAPE_INPUTMODE_FILE;


	jim.SetTicked(elk->GetJimEnabled());
}

void C_HardwareMenu::Exit(void)
{
	MultiState newstate;

	newstate.multicpu = misc[0].GetTicked();
	newstate.multiscr = screen[1].GetTicked();
	newstate.usestate = misc[1].GetTicked();
	newstate.palonly = screen[2].GetTicked();

	elk->SetMultiState(newstate);

	elk->GetScreen()->SetSize(screen[0].GetTicked(), scaling, screen[3].GetTicked());
	elk->GetSpeaker()->SetAccuracy(buffering.GetTicked());
	elk->GetSpeaker()->SetVolume(volume.GetValue());

	elk->GetParticularDrive(0)->SetWriteProtect(dwrprot[0].GetTicked());
	elk->GetParticularDrive(1)->SetWriteProtect(dwrprot[1].GetTicked());

	switch(mrbmode)
	{
		case 0 : elk->SetSloggerMRBMode(MRBMODE_OFF); break;
		case 1 : elk->SetSloggerMRBMode(MRBMODE_TURBO); break;
		case 2 : elk->SetSloggerMRBMode(MRBMODE_SHADOW); break;
	}

	elk->GetTimer()->SetMultiplier(0.5f + (7.5f * speed.GetValue()));

	tspeed.GetTicked() ? elk->InstallTapeHack() : elk->RemoveTapeHack();

	elk->GetTape()->SetOperationMode(TAPE_OUTPUTMODE_SOUND, output[0].GetTicked(), 0, elk->GetCPU()->ReadDWord(NMOS6502ADDR_TIMECNT));
	elk->GetTape()->SetOperationMode(TAPE_OUTPUTMODE_UEF, output[1].GetTicked(), 0, elk->GetCPU()->ReadDWord(NMOS6502ADDR_TIMECNT));
	elk->GetTape()->SetOperationMode(TAPE_IO_PIPING, tpipe.GetTicked(), 0, elk->GetCPU()->ReadDWord(NMOS6502ADDR_TIMECNT));

	elk->GetTape()->SetOperationMode(imode, true, 0, elk->GetCPU()->ReadDWord(NMOS6502ADDR_TIMECNT));

	elk->SetJimEnabled(jim.GetTicked());
}

void C_HardwareMenu::SendMsg(GUI_MSG msg)
{
	C_GUIScreen::SendMsg(msg);

	BACK_MSG m;

	while(MessagesWaiting())
	{
		m = PopMessage();

		switch(m.owner)
		{
			default : break;

			case DISCEJECT1_ID : elk->GetParticularDrive(0)->Close(); break;
			case DISCEJECT2_ID : elk->GetParticularDrive(1)->Close(); break;
			case REWIND_ID : elk->GetTape()->Rewind(0, elk->GetCPU()->ReadDWord(NMOS6502ADDR_TIMECNT)); break;

			case RESET_ID :
				elk->SuperReset();
				gui->SignalFinished();
			break;

			case SPEEDSET_ID :
				speed.SetValue(0.5f / 7.5f);
			break;
		}
	}
}