/*

	ElectrEm (c) 2000/1 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for
	details

	gui/iptsetup.cpp
	================

	This is actually just the keyboard screen nowadays

*/
#include "iptsetup.h"

#include "../gfx/gfxdefs.h"
#include "../gfx/gfx.h"
#include "guidefs.h"

#include "../ula/ula.h"
#include "../misc/input.h"
#include "../plus1/plus1.h"
#include "../fs/fs.h"
#include <string.h>
#include <stdio.h>
#include <malloc.h>

class C_GUIKeymaps : public C_GUIElementBase, public C_GUIMessageCollector
{
	public :
		C_GUIKeymaps(void);

		void Draw(void);
		void SendMsg(GUI_MSG msg);
		void Kill(void);

		void Setup(C_GUI *g, C_GUIMessageCollector *scrn, int id, GUI_BOX area, char *name);
		char *GetName(void);
		void SetSelected(bool t);

		void SetPosition(int x, int y);
		GUI_BOX GetShape(void);

	private :
		static C_GUIMessageCollector *screen;
		BACK_MSG bmsg;
		GUI_BOX clip;

		int x, y;
		bool sel;
		char *name;
};

C_GUIMessageCollector *C_GUIKeymaps::screen;

void C_GUIKeymaps::SetSelected(bool t)
{
	sel =t;
}

C_GUIKeymaps::C_GUIKeymaps(void)
{
	name = NULL;
}

#define NAME_OFFSET 300

void C_GUIKeymaps::Draw(void)
{
	gfx->SetClipArea(desc.loc.x1, desc.loc.y1, desc.loc.x2, desc.loc.y2);

	if(sel)
	{
		gfx->Rectangle(desc.loc.x1, desc.loc.y1, desc.loc.x2, desc.loc.y2, FCOL_WHITE | BCOL_BLUE);
	}

	gfx->WriteString(FONT_SIZE12, name, x, y, FCOL_WHITE | (sel ? BCOL_BLUE : BCOL_BLACK));

	if(desc.mouseover)
	{
		gfx->Line(desc.loc.x1, desc.loc.y1, desc.loc.x2, desc.loc.y1, SCOL_GREY);
		gfx->Line(desc.loc.x2, desc.loc.y1, desc.loc.x2, desc.loc.y2, SCOL_GREY);
		gfx->Line(desc.loc.x2, desc.loc.y2, desc.loc.x1, desc.loc.y2, SCOL_GREY);
		gfx->Line(desc.loc.x1, desc.loc.y2, desc.loc.x1, desc.loc.y1, SCOL_GREY);
	}
}

void C_GUIKeymaps::SendMsg(GUI_MSG msg)
{
	if(gui->PreHandling(&desc, msg))
	{
		if(desc.newclicks == 1)
			screen->PushMessage(bmsg);
	}
}

void C_GUIKeymaps::Kill(void)
{
	if(name)
	{
		free(name);
		name = NULL;
	}
}

#define KEYMAP_CLICK 0

void C_GUIKeymaps::Setup(C_GUI *g, C_GUIMessageCollector *scrn, int id, GUI_BOX area, char *n)
{
	gui = g;
	screen = scrn;
	gfx = gui->GetPartialGFX();
	desc.loc = clip = area;
	gui->SetDefault(&desc);

	bmsg.owner = id;
	bmsg.val = KEYMAP_CLICK;

	name = strdup(n);
}

char *C_GUIKeymaps::GetName(void)
{
	return name;
}

void C_GUIKeymaps::SetPosition(int xx, int yy)
{
	x = xx;
	y = yy;

	desc.loc.y1 = y;
	desc.loc.y2 = y+gfx->StrHeight(FONT_SIZE12, "");

	if(desc.loc.y1 <= clip.y1)
		desc.loc.y1 = clip.y1;
	if(desc.loc.y2 > clip.y2)
		desc.loc.y2 = clip.y2;
}

GUI_BOX C_GUIKeymaps::GetShape(void)
{
	GUI_BOX t;

	t = clip;
	t.y2 = t.y1 + gfx->StrHeight(FONT_SIZE12, "");

	return t;
}


#define KDARK_ID	1
#define KLIGHT_ID	2
#define KMASK_ID	3

#define MENU_ID		4
#define SBREAK_ID	5
#define QUIT_ID		6

#define SHIFTS_ID	7
#define NOSHIFTS_ID	8
#define CLEAR_ID	9

#define NAME_ID		10
#define CREATE_ID	11

bool C_InputMenu::Initialise(C_GUI *parent, int x1, int y1, int x2, int y2)
{
	GUI_BOX box, box2;
	int x, xadd;

	gui = parent;
	ipt = parent->GetElectron()->GetInput();
	InitialiseStack();
	InitialiseList();

	maps = NULL;
	nummaps = 0;

	box.x1 = x1;
	box.y1 = y1;
	box.x2 = x2;
	box.y2 = y2;
	box2 = box;

	box2.y1 += 20;

	delk.Setup(parent, this, KDARK_ID, box2, IMG_KEYBDARK, false);
	lelk.Setup(parent, this, KLIGHT_ID, box2, IMG_KEYBLIGHT, true);
	elkm.Setup(parent, this, KMASK_ID, box2, IMG_KEYMASK, false);

	AddToList(&elkm);
	AddToList(&delk);
	AddToList(&lelk);

	box2 = delk.MinimiseShape();

	int tx = ((box.x2+box.x1) >>1)- ((box2.x2 + box2.x1) >>1);
	box2.x2 = tx + (box2.x2 - box2.x1);
	box2.x1 = tx;

	delk.SetShape(box2);
	lelk.SetShape(box2);
	elkm.SetShape(box2);

	box2.y1 = box2.y2 + 20;
	x = box2.x1; xadd = (box2.x2 - box2.x1)/3;
	box2.x2 = x+xadd;

	sbreak.Setup(parent, this, SBREAK_ID, box2, "Super-break key", 0, FCOL_GREY | BCOL_BLACK, FONT_SIZE12, false, false);
	sbreak.MinimiseShape();
	AddToList(&sbreak);

	box2.x1 += xadd; box2.x2 += xadd;
	menu.Setup(parent, this, MENU_ID, box2, "Menu key", 0, FCOL_GREY | BCOL_BLACK, FONT_SIZE12, false, false);
	menu.MinimiseShape();
	AddToList(&menu);

	box2.x1 += xadd; box2.x2 += xadd;
	quit.Setup(parent, this, QUIT_ID, box2, "Quit key", 0, FCOL_GREY | BCOL_BLACK, FONT_SIZE12, false, false);
	box = quit.MinimiseShape();
	AddToList(&quit);

	box.x1 = x1; box.x2 = x2;
	box.y2 += box.y2 - box.y1; box.y1 = (box.y1 + box.y1) >> 1;
	box.y2 += box.y2 - box.y1;

	allshifts.Setup(parent, this, SHIFTS_ID, box, "Select all shifts", 0, FCOL_WHITE | BCOL_BLACK, FONT_SIZE12, true, true);
	noshifts.Setup(parent, this, NOSHIFTS_ID, box, "Ignore shifts", 0, FCOL_WHITE | BCOL_BLACK, FONT_SIZE12, true, true);
	clear.Setup(parent, this, CLEAR_ID, box, "Clear key", 0, FCOL_WHITE | BCOL_BLACK, FONT_SIZE12, true, true);

	x = x1 + x2 >> 1;
	box2 = clear.MinimiseShape();
	box2.x1 = x - ((box2.x2 - box2.x1) >> 1); box2.x2 = x + (x - box2.x1);
	allshifts.SetShape(box2);

	box = allshifts.MinimiseShape();
	box.x1 = box2.x1 - (box.x2 - box.x1) - 3; box.x2 = box2.x1 - 3;
	allshifts.SetShape(box);

	box = noshifts.MinimiseShape();
	box.x2 = box2.x2 + (box.x2 - box.x1) + 3; box.x1 = box2.x2 + 3;
	noshifts.SetShape(box);

	AddToList(&allshifts);
	AddToList(&noshifts);
	AddToList(&clear);

	filearea.y1 = box.y2+5;

	box2.y2 = y2; box2.x1 = x1; box2.x2 = x2; box2.y1 = y1;
	command.Setup(parent, box2, "Next keyboard prompt", FCOL_WHITE | BCOL_BLACK, FONT_SIZE12);
	box2 = command.MinimiseShape();

	box2.y1 = y2 - (box2.y2 - box2.y1) - 3;
	box2.y2 = y2 - 3;
	box2.x1 = x1 + 3;
	box2.x2 = x2;
	command.SetShape(box2);

	AddToList(&command);
	filearea.y2 = box2.y1-5;

	filearea.x1 = x1; filearea.x2 = x2;
	loadfile.Setup(parent, filearea, "Select a different keyboard map :", FCOL_WHITE | BCOL_BLACK, FONT_SIZE12);
	AddToList(&loadfile);
	box2 = loadfile.MinimiseShape();

	filearea.y1 = box2.y2+5;

	filearea.x1 = x1;
	filearea.x2 = (x1+x2 >> 1);
	mapfield.Setup(parent, filearea);
	filearea = mapfield.GetEnclosedArea();
	AddToList(&mapfield);

	maps = NULL;
	ReseedMaplist();

	//save = button, fname = textinputline, newfile = text indicator

	save.Setup(parent, this, CREATE_ID, filearea, "Create", 0, FCOL_WHITE | BCOL_BLACK, FONT_SIZE12, true, false);
	box = save.MinimiseShape();

	box.y1 = filearea.y2 - (box.y2 - box.y1);
	box.y2 = filearea.y2;
	box.x1 = x2 - (box.x2 - box.x1);
	box.x2 = x2;
	save.SetShape(box);
	AddToList(&save);

	box.x2 = box.x1 - 2;
	box.x1 = ((x1 +x2) >> 1) + 3;

	sprintf(namestr, "");
	fname.Setup(parent, this, NAME_ID, box, namestr, 256, FCOL_WHITE | BCOL_BLACK, FONT_SIZE12, false);
	AddToList(&fname);

	box.y1 -= (box.y2 - box.y1);
	box.y2 = (box.y1 + box.y2) >> 1;
	newfile.Setup(parent, box, "Create a new map :", FCOL_WHITE | BCOL_BLACK, FONT_SIZE12);
	AddToList(&newfile);

	return true;
}

void C_InputMenu::Enter(void)
{
	ClearKeys();
}

void C_InputMenu::Exit(void)
{
	ipt->ClearKeyPressedDescriptor();
}

void C_InputMenu::SendMsg(GUI_MSG msg)
{
	C_GUIScreen::SendMsg(msg);

	BACK_MSG m;

	while(MessagesWaiting())
	{
		m = PopMessage();

		if(m.owner <= 0)
		{
			m.owner = -m.owner;

			if(ipt->LoadMap(maps[m.owner].GetName()))
			{
				int c = nummaps;

				command.ChangeText("New map loaded");
				while(c--)
					maps[c].SetSelected((c == m.owner) ? true : false);
			}
		}
		else
		{
			char *name;

			switch(m.owner)
			{
				case KMASK_ID :

					if(m.val) AddKey(m.val-1);

				break;

				case MENU_ID :		AddKey(ADDKEY_GUI); break;
				case SBREAK_ID :	AddKey(ADDKEY_SBREAK); break;
				case QUIT_ID :		AddKey(ADDKEY_QUIT); break;
				case SHIFTS_ID :	AddKey(ADDKEY_SETALLSHIFTS); break;
				case NOSHIFTS_ID :	AddKey(ADDKEY_SETNOSHIFTS); break;
				case CLEAR_ID :		AddKey(ADDKEY_SETCLEAR); break;

				case CREATE_ID :
				case NAME_ID :

					name = gui->GetElectron()->GetFS()->GetCleanFilename(namestr, "kyb");
					if(ipt->CreateMap(name))
					{
						command.ChangeText("New map created");
						ReseedMaplist();
					}
					else
					{
						command.ChangeText("Unable to create new map - check validity of filename");
					}
					free(name);

				break;
			}
		}
	}
}

void C_InputMenu::Draw(void)
{
	C_GUIScreen::Draw();

	if(ipt->GetBindDone())
	{
		ClearKeys();
	}
}

#define KTEXT "Electron keypress expected : "

void C_InputMenu::AddKey(int id)
{
	char *nkey;
	char *str;
	int *c, *ltop, len, flags;

	nkey = ipt->AddToKeyPressedDescriptor(id);
	str = (char *)malloc(strlen(KTEXT) + strlen(nkey) + 1);

	sprintf(str, "%s%s", KTEXT, nkey);
	command.ChangeText(str);

	free(str);
	free(nkey);

	menu.ChangeColour(FCOL_GREY | BCOL_BLACK);
	sbreak.ChangeColour(FCOL_GREY | BCOL_BLACK);
	quit.ChangeColour(FCOL_GREY | BCOL_BLACK);

	flags = ipt->GetKeyFlags();
	allshifts.Set((flags&IPTFLG_ALLSHIFT) ? true : false);
	noshifts.Set((flags&IPTFLG_NOSHIFT) ? true : false);
	clear.Set((flags&IPTFLG_CLEAR) ? true : false);

	len = 0;
	ltop = c = ipt->GetCurrentKeyPressedList();
	while(*c != -1)
	{
		switch(*c)
		{
			default : (*c) ++; break;

			case ADDKEY_GUI :		(*c) = 57; menu.ChangeColour(FCOL_WHITE | BCOL_BLACK); break;
			case ADDKEY_QUIT :		(*c) = 57; quit.ChangeColour(FCOL_WHITE | BCOL_BLACK); break;
			case ADDKEY_SBREAK :	(*c) = 57; sbreak.ChangeColour(FCOL_WHITE | BCOL_BLACK); break;
		}
		c++;
		len++;
	}

	gui->GetPartialGFX()->SetMaskByColours(IMG_KEYBLIGHT, IMG_KEYMASK, len, ltop);
}

#undef KTEXT

void C_InputMenu::ClearKeys(void)
{
	int c1 = 250;

//	if(allowing remapping)
//	{
//		/* mask off all of light image */
		gui->GetPartialGFX()->SetMaskByColours(IMG_KEYBLIGHT, IMG_KEYMASK, 1, &c1);
		ipt->ClearKeyPressedDescriptor();
		command.ChangeText("Select keys");

		menu.ChangeColour(FCOL_GREY | BCOL_BLACK);
		sbreak.ChangeColour(FCOL_GREY | BCOL_BLACK);
		quit.ChangeColour(FCOL_GREY | BCOL_BLACK);

		allshifts.Set(false);
		noshifts.Set(false);
		clear.Set(false);
//	}
//	else
//	{
//		/* enable all of light image */
//		parent->GetPartialGFX()->ResetMask(IMG_ELKLIGHT);
//	}
}

void C_InputMenu::ReseedMaplist(void)
{
	C_FSAccess *fs = gui->GetElectron()->GetFS();
	FileTree *tree, *pass;
	char *extensions[] = {"kyb", NULL}, *cmap = ipt->GetMapName();

	nummaps = 0;
	if(maps)
	{
		delete[] maps;
		maps = NULL;
	}

	if(pass = tree = fs->GetFileTree(NULL, "keybpath", extensions))
	{
		while(pass)
		{
			if(!pass->dir)	nummaps++;
			pass = pass->next;
		}

		maps = new C_GUIKeymaps[nummaps];
		nummaps = 0;
		pass = tree;
		while(pass)
		{
			if(!pass->dir)
			{
				maps[nummaps].Setup(gui, this, -nummaps, filearea, pass->fname);
				maps[nummaps].SetSelected(strcmp(pass->fname, cmap) ? false : true);
				nummaps++;
			}
			pass = pass->next;
		}

		mapfield.SetElements(maps, nummaps, sizeof(C_GUIKeymaps));

		fs->FreeFileTree(tree);
	}
}

