#include "gui.h"
#include "mainmenu.h"
#include "../gfx/gfxdefs.h"
#include "../gfx/gfx.h"
#include "guidefs.h"
#include <stdio.h>
#include <malloc.h>
#include <string.h>

#include "about.h"
#include "romsetup.h"
#include "hwsetup.h"
#include "filemenu.h"
#include "iptsetup.h"
#include "optsetup.h"

C_AboutMenu *about;
C_ROMMenu *rom;
C_HardwareMenu *hardware;
C_FileMenu *filemenu;
C_InputMenu *input;
C_OutputMenu *output;

struct SubMenu
{
	char *name;
	C_GUIScreen *menu;
};

SubMenu subordinates[] =
{
	{"File", NULL},
	{"Hardware", NULL},
	{"ROMs", NULL},
	{"Keyboard", NULL},
	{"Printer/Joystick", NULL},
	{"Info", NULL},
};

#define SUBMENUS 6

bool C_CardMenu::Initialise(C_GUI *parent, int x1, int y1, int x2, int y2)
{
	int bnum = 0;
	SubMenu *p = subordinates;

	InitialiseStack();
	gui = parent;
	baby = NULL;

	/* create the right number of elements */
	int c = 5 + 3*SUBMENUS;
	GUI_BOX box;

	lines = new C_GUILine[c];
	btns = new C_GUIButton[SUBMENUS];

	p = subordinates;

	/* three entirely static lines */

	box.y1 = y1+parent->GetPartialGFX()->StrHeight(FONT_SIZE12, p->name) + 8;
	box.y2 = y2-1;

	box.x1 =
	box.x2 = x1;
	lines[0].Setup(parent, box, FCOL_WHITE | BCOL_BLACK);

	box.x1 =
	box.x2 = x2-1;
	lines[1].Setup(parent, box, FCOL_WHITE | BCOL_BLACK);

	box.x1 = x1;
	box.y1 = y2-1;
	lines[2].Setup(parent, box, FCOL_WHITE | BCOL_BLACK);

	/* add card tabs */
	int x = x1;
	c = SUBMENUS;
	while(c--)
	{
		box.x1 = x+4;
		box.y1 = y1;
		box.x2 = x;
		box.y2 = y1+parent->GetPartialGFX()->StrHeight(FONT_SIZE12, p->name) + 8;
		lines[c*3 + 5].Setup(parent, box, FCOL_WHITE | BCOL_BLACK);

		x += 12+parent->GetPartialGFX()->StrWidth(FONT_SIZE12, p->name);
		box.x2 = x-4;
		btns[c].Setup(parent, this, c, box, p->name, 0, FCOL_WHITE | BCOL_BLACK, FONT_SIZE12, false, false);

		box.y2 = box.y1;
		lines[c*3 + 6].Setup(parent, box, FCOL_WHITE | BCOL_BLACK);

		box.x1 = x;
		box.y1 = y1+parent->GetPartialGFX()->StrHeight(FONT_SIZE12, p->name) + 8;
		lines[c*3 + 7].Setup(parent, box, FCOL_WHITE | BCOL_BLACK);

		p++;
	}

	box.x1 = x1;
	box.x2 = x2;
	box.y2 = box.y1;
	lines[3].Setup(parent, box, FCOL_WHITE | BCOL_BLACK);
	lines[4].Setup(parent, box, FCOL_WHITE | BCOL_BLACK);

	box.x1 = x2 - 7 - parent->GetPartialGFX()->StrWidth(FONT_SIZE12, "Back to Electron");
	box.y1 = y1;
	box.x2 = x2 - 2;
	box.y2 = y1 + 4 + parent->GetPartialGFX()->StrHeight(FONT_SIZE12, "Back to Electron");
	qbutton.Setup(parent, this, -1, box, "Back to Electron", 0, FCOL_WHITE | BCOL_BLACK, FONT_SIZE12, true, false);

	/* create subordinates */
	output = new C_OutputMenu();
	subordinates[4].menu = output;

	about = new C_AboutMenu();
	subordinates[5].menu = about;

	rom = new C_ROMMenu();
	subordinates[2].menu = rom;

	hardware = new C_HardwareMenu();
	subordinates[1].menu = hardware;

	filemenu = new C_FileMenu();
	subordinates[0].menu = filemenu;

	input = new C_InputMenu();
	subordinates[3].menu = input;

	/* pass on initialise message to all subordinates */
	bool result = true;

	p = subordinates;
	c = SUBMENUS;
	while(c--)
	{
		if(p->menu)
			if(!p->menu->Initialise(parent, x1+3, y1+parent->GetPartialGFX()->StrHeight(FONT_SIZE12, p->name) + 11, x2-3, y2-3))
				result = false;

		p++;
	}

	/* set default state */

	BringToFore(0);

	return result;
}

void C_CardMenu::DeInitialise(void)
{
	/* pass on deinitialise message */
	SubMenu *p = subordinates;
	int c = SUBMENUS;

	while(c--)
	{
		if(p->menu)
		{
			p->menu->DeInitialise();
		}

		p++;
	}

	c = 3*SUBMENUS + 5;
	while(c--)
		lines[c].Kill();
	delete lines;

	c = SUBMENUS;
	while(c--)
		btns[c].Kill();
	delete btns;

	qbutton.Kill();

	/* and now free memory */
	delete about;
	delete rom;
	delete hardware;
	delete filemenu;
	delete input;
	delete output;
}

void C_CardMenu::Enter(void)
{
	qbutton.Reset();

	if(baby)
		baby->Enter();
}

void C_CardMenu::Exit(void)
{
	if(baby)
		baby->Exit();
}

void C_CardMenu::SendMsg(GUI_MSG msg)
{
	/* handle own elements first */
	qbutton.SendMsg(msg);

	int c = SUBMENUS;
	while(c--)
		btns[c].SendMsg(msg);

	BACK_MSG w;

	while(MessagesWaiting())
	{
		w = PopMessage();

		if(w.owner == -1)
			gui->SignalFinished();
		else
			BringToFore(w.owner);
	}

	/* now pass message on */
	if(baby)
		baby->SendMsg(msg);
}

void C_CardMenu::BringToFore(int c)
{
	int count, newc;

	if((subordinates[SUBMENUS-1-c].menu == NULL) || (baby != subordinates[SUBMENUS-1-c].menu))
	{
		count = SUBMENUS;
		while(count--)
		{
			newc = BCOL_BLACK | ((count == c) ? FCOL_WHITE : FCOL_GREY);

			lines[count*3 + 5].ChangeColour(newc);
			lines[count*3 + 6].ChangeColour(newc);
			lines[count*3 + 7].ChangeColour(newc);
			btns[count].ChangeColour(newc);
		}

		GUI_BOX box, box2;

		box2 = lines[c*3 + 5].GetShape();

		box = lines[3].GetShape();
		box.x2 = box2.x2;
		lines[3].SetShape(box);

		box2 = lines[c*3 + 7].GetShape();

		box = lines[4].GetShape();
		box.x1 = box2.x1;
		lines[4].SetShape(box);

		/* replace 'baby' (the current card) */

		if(baby)
			baby->Exit();

		baby = subordinates[SUBMENUS - 1 - c].menu;

		if(baby)
			baby->Enter();
	}
}

void C_CardMenu::Draw(void)
{
	int c;

	c = 3*SUBMENUS + 5;
	while(c--)
		lines[c].Draw();

	c = SUBMENUS;
	while(c--)
		btns[c].Draw();

	qbutton.Draw();

	if(baby)
		baby->Draw();
}

void C_CardMenu::EvaluatePreload(void)
{
	filemenu->EvaluatePreload();
}

bool C_CardMenu::SetAutoId(unsigned __int8 id)
{
	return rom->SetAutoId(id);
}

void C_CardMenu::PassROMHint(bool set, int id, char *name)
{
	rom->PassROMHint(set, id, name);
}

