/*

	ElectrEm (c) 2000/1 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for
	details

	gui/operror.cpp
	===============

	This is just the 'Operation error' & 'Halt Opcode' screens

*/
#include "gui.h"
#include "operror.h"
#include "../gfx/gfxdefs.h"
#include "../gfx/gfx.h"
#include "../ula/ula.h"
#include "guidefs.h"
#include <stdio.h>
#include <malloc.h>
#include <string.h>

/* Opcode Error screen */

#define BACK_ID		1
#define ONCE_ID		2
#define ALWAYS_ID	3

bool C_OpcodeErrorMenu::Initialise(C_GUI *parent, int x1, int y1, int x2, int y2)
{
	InitialiseStack();
	InitialiseList();
	gui = parent;

	GUI_BOX box;
	int ny2;

	box.x1 = x1+5;
	box.x2 = ((x2-x1)/3)-5;
	box.y1 = y1;
	box.y2 = y2;

	back.Setup(parent, this, BACK_ID, box, "Reset Electron", 0, FCOL_WHITE | BCOL_BLACK, FONT_SIZE12, true, false);
	AddToList(&back);
	box = back.MinimiseShape();

	box.x1 = x1+5;
	box.x2 = ((x2-x1)/3)-5;
	ny2 = box.y1 = y2 - box.y2 + box.y1 - 5;
	box.y2 = y2 - 5;
	back.SetShape(box);

	box.x1 = ((x2-x1)/3)+5;
	box.x2 = (2*(x2-x1)/3)-5;
	once.Setup(parent, this, ONCE_ID, box, "Ignore Opcode once", 0, FCOL_WHITE | BCOL_BLACK, FONT_SIZE12, true, false);
	AddToList(&once);

	box.x1 = (2*(x2-x1)/3)+5;
	box.x2 = x2-5;
	always.Setup(parent, this, ONCE_ID, box, "Ignore Opcode always", 0, FCOL_WHITE | BCOL_BLACK, FONT_SIZE12, true, false);
	AddToList(&always);

	box.y2 = box.y1 - 5;
	box.y1 = y1 + 3;
	box.x1 = x1 + 3;
	box.x2 = x2 - 3;
	title.Setup(parent, box, "Operation Error", FCOL_WHITE | BCOL_BLACK, FONT_SIZE24);
	AddToList(&title);

	box = title.MinimiseShape();
	box.x1 = x1 + 3;
	box.x2 = x2 - 3;
	box.y1 = box.y2;
	box.y2 = ny2- 5;

	body.Setup(parent, box, "pl", FCOL_WHITE | BCOL_BLACK, FONT_SIZE12);
	AddToList(&body);

	return true;
}

#define BASE_STR "\nError %02x at %02x%02x [%02x]\n\n\
An undefined operation has been attempted by the program currently running. This could be a genuine problem with the software you are running, but is most probably an error in ElectrEm.\n\n\
It may be possible for the piece of software to run near-correctly if this error is simply ignored. Further, you can instruct ElectrEm to ignore this error from now on until the next time you run the emulator.\n\n\
However, it is most likely that ElectrEm is just not compatible with the piece of software you are trying to use. Hopefully this will be fixed in a later release, but in the mean time it would be useful to report the problem to T.Harte@excite.com, naming the software being run and the numbers mentioned above.\n\n\
Notice that in some cases with tape software this error can occur when fast loading is enabled, but not otherwise, which may be worth looking into.\n\n\
Sorry!"

void C_OpcodeErrorMenu::Enter(void)
{
	words = (char *)malloc(strlen(BASE_STR));
	sprintf(words, BASE_STR, gui->GetElectron()->ReadValue(ULAVALUE_OPCODE), gui->GetElectron()->GetCPU()->ReadByte(NMOS6502ADDR_PCH), gui->GetElectron()->GetCPU()->ReadByte(NMOS6502ADDR_PCL), gui->GetElectron()->ReadValue(ULAVALUE_PAGED));
	body.ChangeText(words);
	free(words);
}

void C_OpcodeErrorMenu::SendMsg(GUI_MSG msg)
{
	C_GUIScreen::SendMsg(msg);

	BACK_MSG w;

	while(MessagesWaiting())
	{
		w = PopMessage();

		if(w.val == GUIBUTTON_PERFORM)
		{
			switch(w.owner)
			{
				case BACK_ID :
					gui->GetElectron()->SuperReset();
					gui->SignalFinished();
				break;

				case ALWAYS_ID :
					gui->GetElectron()->SetToIgnore(gui->GetElectron()->ReadValue(ULAVALUE_OPCODE));

				case ONCE_ID :
					gui->SignalFinished();
				break;
			}
		}
	}
}

/* Halt Opcode screen */

#define HALT_STR "\nThe Electron has entered a state where it will not respond to any activity of any sort.\n\n\
Although it is possible that such a state was intended, it is more likely to be down to a bug in this release of the emulator. Please e-mail me (T.Harte@excite.com) and tell me which software you were running, and at what point this error occurred.\n\n\
The only real option now is to reset the Electron and start again."

bool C_HaltOpcodeMenu::Initialise(C_GUI *parent, int x1, int y1, int x2, int y2)
{
	InitialiseStack();
	InitialiseList();
	gui = parent;

	GUI_BOX box;
	int ny2;

	box.x1 = x1+5;
	box.x2 = ((x2-x1)/3)-5;
	box.y1 = y1;
	box.y2 = y2;

	back.Setup(parent, this, BACK_ID, box, "Reset Electron", 0, FCOL_WHITE | BCOL_BLACK, FONT_SIZE12, true, false);
	AddToList(&back);
	box = back.MinimiseShape();

	box.x1 = x1+5;
	box.x2 = ((x2-x1)/3)-5;
	ny2 = box.y1 = y2 - box.y2 + box.y1 - 5;
	box.y2 = y2 - 5;
	back.SetShape(box);

	box.y2 = box.y1 - 5;
	box.y1 = y1 + 3;
	box.x1 = x1 + 3;
	box.x2 = x2 - 3;
	title.Setup(parent, box, "Electron Halted", FCOL_WHITE | BCOL_BLACK, FONT_SIZE24);
	AddToList(&title);

	box = title.MinimiseShape();
	box.x1 = x1 + 3;
	box.x2 = x2 - 3;
	box.y1 = box.y2;
	box.y2 = ny2- 5;

	body.Setup(parent, box, HALT_STR, FCOL_WHITE | BCOL_BLACK, FONT_SIZE12);
	AddToList(&body);

	return true;
}

void C_HaltOpcodeMenu::SendMsg(GUI_MSG msg)
{
	C_GUIScreen::SendMsg(msg);

	BACK_MSG w;

	while(MessagesWaiting())
	{
		w = PopMessage();
		gui->GetElectron()->SuperReset();
		gui->SignalFinished();
	}
}
