/*

	ElectrEm (c) 2000/1 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for
	details

	gui/optsetup.cpp
	================

	This is the 'Output Devices' screen.

*/
#include "optsetup.h"

#include "../gfx/gfxdefs.h"
#include "../gfx/gfx.h"
#include "guidefs.h"

#include "../ula/ula.h"
#include "../misc/input.h"
#include "../plus1/plus1.h"
#include <string.h>
#include <stdio.h>
#include <malloc.h>

void C_GUIJoyDisplay::Draw(void)
{
	JOY_STATE i = input->GetJoystickState(joyid);
	int x, y;

	gfx->SetClipArea(desc.loc.x1, desc.loc.y1, desc.loc.x2, desc.loc.y2);

	gfx->Rectangle(desc.loc.x1, desc.loc.y1, desc.loc.x2, desc.loc.y2, i.button ? SCOL_RED : SCOL_BLACK);

	x = hx - ((w*(i.x-128)) >> 8);
	y = hy - ((h*(i.y-128)) >> 8);

	gfx->Line(desc.loc.x1, hy, desc.loc.x2, hy, SCOL_GREY);
	gfx->Line(hx, desc.loc.y1, hx, desc.loc.y2, SCOL_GREY);
	gfx->Line(desc.loc.x2, hy, desc.loc.x1, hy, SCOL_GREY);
	gfx->Line(hx, desc.loc.y2, hx, desc.loc.y1, SCOL_GREY);

	gfx->Rectangle(x-5, y-5, x+5, y+5, SCOL_WHITE);

	gfx->Line(desc.loc.x1, desc.loc.y1, desc.loc.x2, desc.loc.y1, SCOL_GREY);
	gfx->Line(desc.loc.x2, desc.loc.y1, desc.loc.x2, desc.loc.y2, SCOL_GREY);
	gfx->Line(desc.loc.x2, desc.loc.y2, desc.loc.x1, desc.loc.y2, SCOL_GREY);
	gfx->Line(desc.loc.x1, desc.loc.y2, desc.loc.x1, desc.loc.y1, SCOL_GREY);

}

void C_GUIJoyDisplay::Setup(C_GUI *g, GUI_BOX area)
{
	desc.loc = area;
	hx = (area.x1+area.x2) >> 1;
	hy = (area.y1+area.y2) >> 1;

	w = area.x2-area.x1;
	h = area.y2-area.y1;

	gui = g;
	gfx = g->GetPartialGFX();
	joyid = 0;
	input = g->GetElectron()->GetInput();
}

void C_GUIJoyDisplay::SetJoystick(int id)
{
	joyid = id;
}

#define AddBox(name, v)	\
	boxes[v].Setup(parent, box, SCOL_GREY);\
	AddToList(&boxes[v]);\
	box.x1 += 2;\
	box.y1 += 2;\
	names[v].Setup(parent, box, name, FCOL_WHITE | BCOL_BLACK, FONT_SIZE18);\
	AddToList(&names[v]);

#define Begin(v)\
	box = names[v].MinimiseShape();\
	box2 = boxes[v].GetShape();\
	box.y1 += (box.y2 - box.y1) + 5;\
	box.y2 = box2.y2;\
	box.x1 += 5;\
	box.x2 = box2.x2;

#define PMODE_IDS	-1
#define TEXT_ID		0
#define TEXTBUT_ID	1

#define JOYTABS_ID	2
#define JOYSET1_ID	3
#define JOYSET2_ID	4
#define DEADZONE_ID	5
#define CALIB_ID	6
#define JOYCLR1_ID	7
#define JOYCLR2_ID	8
#define JOYCLRF_ID	9
#define JOYSETF_ID	10

bool C_OutputMenu::Initialise(C_GUI *parent, int x1, int y1, int x2, int y2)
{
	int y, c;
	GUI_BOX box, box2;

	gui = parent;
	elk = parent->GetElectron();
	InitialiseStack();
	InitialiseList();

	/* printer box */
	box.x1 = x1+3;
	box.y1 = y1;
	box.x2 = x2-3;
	box.y2 = ((y2+y1) >> 1) - 3;
	AddBox("Printer", 0);

	/* joystick box */
	box = boxes[0].MinimiseShape();
	box.x2 = x2 - 3;
	box.y1 = box.y2 + 3;
	box.y2 = y2 - 3;
	AddBox("Joystick", 1);

	/* printer options */
	Begin(0);

	opthead.Setup(parent, box, "Mode :", FCOL_WHITE | BCOL_BLACK, FONT_SIZE12);
	AddToList(&opthead);
	box = opthead.MinimiseShape();
	box.y2 += box.y2 - box.y1;
	box.y1 = (box.y2 + box.y1) >> 1;
	box.x1 += 5;
	box.x2 = box2.x2;

	char *prnstrings[] = {"disabled", "pipe straight to PC printer", "save immediately to file", "save to file with fixed CR/LF's", "convert to RTF file"};

	for(c = 0; c < 5; c++)
	{
		pbuts[c].Setup(parent, this, PMODE_IDS, c, &pmode, box, prnstrings[c], FONT_SIZE12, FCOL_WHITE | BCOL_BLACK);
		AddToList(&pbuts[c]);
		box = pbuts[c].MinimiseShape();

		y = box.y2 - box.y1;
		box.y1 += y;
		box.y2 += y;
		box.x2 = box2.x2;
	}

	box.y1 += y;
	box.y2 += y;
	box.x1 -= 5;
	box.x2 -= 5;
	filename.Setup(parent, box, "Change or set file :", FCOL_WHITE | BCOL_BLACK, FONT_SIZE12);
	AddToList(&filename);

	box.y1 += y;
	box.y2 += y;
	name.Setup(gui, this, TEXT_ID, box, namestr, 256, FCOL_WHITE | BCOL_BLACK, FONT_SIZE12, false);
	box = name.MinimiseShape();
	sprintf(namestr, "");
	AddToList(&name);

	box.y2 += box.y2 - box.y1;
	box.y1 = (box.y1 + box.y2) >> 1;
	fset.Setup(gui, this, TEXTBUT_ID, box, "Open File", 0, FCOL_WHITE | BCOL_BLACK, FONT_SIZE12, true, false);
	box = fset.MinimiseShape();

	box.x1 = box2.x2 - box.x2 + box.x1 - 3;
	box.x2 = box2.x2 - 3;
	fset.SetShape(box);

	AddToList(&fset);

	/* joystick options */
	Begin(1);

	if(sticks = elk->GetInput()->GetNumJoysticks())
	{
		box.x2 -= 140;

		joyselect.Setup(parent, this, JOYTABS_ID, box, FONT_SIZE12, 0);
		joyselect.StartEntryList(sticks);

		char joytext[12];
		for(int c = 0; c < sticks; c++)
		{
			sprintf(joytext, "Stick %d", c+1);
			joyselect.AddEntry(joytext);
		}

		AddToList(&joyselect);

		box2 = box = joyselect.GetEnclosedArea();

		box2.x2 = box2.x1+(box.y2-box.y1);
		box.x1 = box2.x2+2;

		box2.x1 += 3; box2.y1 += 3;
		box2.x2 -= 3; box2.y2 -= 3;

		cwind.Setup(parent, box2);
		AddToList(&cwind);

		joy2.Setup(parent, this, JOYSET2_ID, box, "acts as Plus 1 second joystick", FONT_SIZE12, FCOL_WHITE | BCOL_BLACK);
		box2 = joy2.MinimiseShape();

		box2.y1 = box.y2 - (box2.y2 - box2.y1);
		box2.y2 = box.y2;
		joy2.SetShape(box2);
		AddToList(&joy2);

		box2.y1 -= box2.y2 - box2.y1;
		box2.y2 = (box2.y1 + box2.y2) >> 1;
		joy1.Setup(parent, this, JOYSET1_ID, box2, "acts as Plus 1 first joystick", FONT_SIZE12, FCOL_WHITE | BCOL_BLACK);
		joy1.MinimiseShape();
		AddToList(&joy1);

		box2.y1 -= box2.y2 - box2.y1;
		box2.y2 = (box2.y1 + box2.y2) >> 1;
		fbjoy.Setup(parent, this, JOYSETF_ID, box2, "acts as First Byte joystick", FONT_SIZE12, FCOL_WHITE | BCOL_BLACK);
		fbjoy.MinimiseShape();
		AddToList(&fbjoy);

		box2.y1 -= box2.y2 - box2.y1;
		box2.y2 = (box2.y1 + box2.y2) >> 1;
		box2.x1 = box.x1; box2.x2 = box.x2;
		deadzone.Setup(parent, this, DEADZONE_ID, box2, "Dead zone : ", FONT_SIZE12, FCOL_WHITE | BCOL_BLACK, 25, 0);
		AddToList(&deadzone);

		box2.y2 = box2.y1 - 7;
		box2.y1 = box.y1+5;
		box2.x1 = box.x1+5; box2.x2 = box.x2-5;
		calib.Setup(parent, this, CALIB_ID, box2, "Next Calibration Prompt", 0, FCOL_WHITE | BCOL_BLACK, FONT_SIZE12, true, false);

		ctext.Setup(parent, box2, "Joystick already calibrated, or must be calibrated by OS (e.g. control panel in Win, jscal in Linux)", FCOL_RED | BCOL_BLACK, FONT_SIZE12);

		cjoy = -1;
		ResetJoystick(0);

		y = (box.y2 - box.y1) / 3;

		box.x1 = box.x2 + 6;
		box.x2 = x2-6;
		box.y2 = box.y1+y;

		box.y2 -= 3;
		clrfb.Setup(parent, this, JOYCLRF_ID, box, "Set no First Byte stick", 0, FCOL_WHITE | BCOL_BLACK, FONT_SIZE8, true, false);
		AddToList(&clrfb);
		box.y2 += 3;

		box.y2 += y;
		box.y1 += y;
		box.y2 -= 3;
		clr1.Setup(parent, this, JOYCLR1_ID, box, "Set no first stick", 0, FCOL_WHITE | BCOL_BLACK, FONT_SIZE8, true, false);
		AddToList(&clr1);
		box.y2 += 3;

		box.y2 += y;
		box.y1 += y;

		box.y1 += 3;
		clr2.Setup(parent, this, JOYCLR2_ID, box, "Set no second stick", 0, FCOL_WHITE | BCOL_BLACK, FONT_SIZE8, true, false);
		AddToList(&clr2);
	}
	else
	{
		ctext.Setup(parent, box, "No joysticks connected", FCOL_WHITE | BCOL_BLACK, FONT_SIZE12);
		AddToList(&ctext);
		elk->GetPlus1()->SetJoystick(0, -1);
		elk->GetPlus1()->SetJoystick(1, -1);
	}

	return true;
}

void C_OutputMenu::Enter(void)
{
	switch(elk->GetPlus1()->PRNGetMode())
	{
		case PRNMODE_OFF :		pmode = 0; break;
		case PRNMODE_PARPORT :	pmode = 1; break;
		case PRNMODE_ASCII :	pmode = 2; break;
		case PRNMODE_ASCIILF :	pmode = 3; break;
		case PRNMODE_RTF :		pmode = 4; break;
	}
}

void C_OutputMenu::Exit(void)
{
	switch(pmode)
	{
		case 0 : elk->GetPlus1()->PRNSetMode(PRNMODE_OFF); break;
		case 1 : elk->GetPlus1()->PRNSetMode(PRNMODE_PARPORT); break;
		case 2 : elk->GetPlus1()->PRNSetMode(PRNMODE_ASCII); break;
		case 3 : elk->GetPlus1()->PRNSetMode(PRNMODE_ASCIILF); break;
		case 4 : elk->GetPlus1()->PRNSetMode(PRNMODE_RTF); break;
	}
}

void C_OutputMenu::Draw(void)
{
	C_GUIScreen::Draw();

	if(sticks)
	{
		if(cal)
			calib.Draw();
		else
			ctext.Draw();
	}
}

void C_OutputMenu::SendMsg(GUI_MSG msg)
{
	C_GUIScreen::SendMsg(msg);
	if(cal && sticks) calib.SendMsg(msg);

	BACK_MSG m;

	while(MessagesWaiting())
	{
		m = PopMessage();

		switch(m.owner)
		{
			case TEXTBUT_ID :
			case TEXT_ID :

				elk->GetPlus1()->PRNSetFile(namestr);

			break;

			case JOYTABS_ID :

				ResetJoystick(m.val);

			break;

			case JOYSET1_ID :
				if(joy1.GetTicked())
					elk->GetPlus1()->SetJoystick(0, cjoy);
				else
					elk->GetPlus1()->SetJoystick(0, -1);
			break;

			case JOYSET2_ID :
				if(joy2.GetTicked())
					elk->GetPlus1()->SetJoystick(1, cjoy);
				else
					elk->GetPlus1()->SetJoystick(1, -1);
			break;

			case JOYSETF_ID :
				if(fbjoy.GetTicked())
					elk->SetJoystickId(cjoy);
				else
					elk->SetJoystickId(-1);
			break;

			case JOYCLR1_ID :
				elk->GetPlus1()->SetJoystick(0, -1);
				joy1.SetTicked(false);
			break;

			case JOYCLR2_ID :
				elk->GetPlus1()->SetJoystick(1, -1);
				joy2.SetTicked(false);
			break;

			case JOYCLRF_ID :
				elk->SetJoystickId(-1);
				fbjoy.SetTicked(false);
			break;

			case DEADZONE_ID :

				elk->GetInput()->SetDeadZone(cjoy, deadzone.GetValue() / 4.0f);

			break;

			case CALIB_ID :

				elk->GetInput()->FlagJoystickDone();
				GetNextJoyPrompt();

			break;
		}
	}
}

void C_OutputMenu::ResetJoystick(int njoy)
{
	JOY_INFO i;

	cjoy = njoy;

	cwind.SetJoystick(cjoy);
	joy1.SetTicked(elk->GetPlus1()->GetJoystick(0) == cjoy);
	joy2.SetTicked(elk->GetPlus1()->GetJoystick(1) == cjoy);
	fbjoy.SetTicked(elk->GetJoystickId() == cjoy);
	deadzone.SetValue(elk->GetInput()->GetDeadZone(cjoy) * 4.0f);

	i = elk->GetInput()->GetJoystickInfo(cjoy);
	if(i.calibrate)
	{
		cal = true;

		elk->GetInput()->BeginCalibration(cjoy);
		GetNextJoyPrompt();
	}
	else
	{
		cal = false;
	}
}

#define EXTEXT "\nand click here to proceed"

void C_OutputMenu::GetNextJoyPrompt(void)
{
	char *t2;
	const char *t = elk->GetInput()->GetJoystickInstruction();

	if(t)
	{
		t2 = (char *)malloc(strlen(t) + strlen(EXTEXT) + 1);
		sprintf(t2, "%s%s", t, EXTEXT);
		calib.ChangeText(t2);

		free(t2);
	}
	else
	{
		cal = false;
	}
}

#undef EXTEXT

