/*

	ElectrEm (c) 2000/1 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for
	details

	gui/romsetup.cpp
	================

	The ROM setup screen & a special GUI element for displaying ROMs

*/
#include "romsetup.h"
#include "romclasses.h"

#include "../gfx/gfxdefs.h"
#include "../gfx/gfx.h"
#include "guidefs.h"

#include "../fs/fs.h"
#include "../misc/config.h"
#include "../ula/ula.h"

#include <stdio.h>
#include <malloc.h>
#include <string.h>

/* the ROM element, fresh for scroll fielding */
class C_GUIROMElement : public C_GUIElementBase, public C_GUIMessageCollector
{
	public :
		C_GUIROMElement(void);

		void Draw(void);
		void SendMsg(GUI_MSG msg);
		void Kill(void);

		void Setup(C_GUI *g, C_GUIMessageCollector *scrn, int id, GUI_BOX area, char *fname, char *rname, unsigned __int8 type);
		char *GetFName(void);
		char *GetRName(void);
		unsigned __int8 GetType(void);

		void SetPosition(int x, int y);
		GUI_BOX GetShape(void);

	private :
		static C_GUIMessageCollector *screen;
		BACK_MSG bmsg;
		GUI_BOX clip;

		int x, y;
		char *fname, *rname;
		unsigned __int8 type;
};

C_GUIMessageCollector *C_GUIROMElement::screen;

C_GUIROMElement::C_GUIROMElement(void)
{
	fname = rname = NULL;
}

#define NAME_OFFSET 300

void C_GUIROMElement::Draw(void)
{
	gfx->SetClipArea(desc.loc.x1, desc.loc.y1, desc.loc.x1+NAME_OFFSET, desc.loc.y2);
	gfx->WriteString(FONT_SIZE12, fname, x, y, FCOL_RED | BCOL_BLACK);

	gfx->SetClipArea(desc.loc.x1+NAME_OFFSET, desc.loc.y1, desc.loc.x2, desc.loc.y2);
	gfx->WriteString(FONT_SIZE12, rname, x+NAME_OFFSET, y, FCOL_WHITE | BCOL_BLACK);

	if(desc.mouseover)
	{
		gfx->Line(desc.loc.x1, desc.loc.y1, desc.loc.x2, desc.loc.y1, SCOL_GREY);
		gfx->Line(desc.loc.x2, desc.loc.y1, desc.loc.x2, desc.loc.y2, SCOL_GREY);
		gfx->Line(desc.loc.x2, desc.loc.y2, desc.loc.x1, desc.loc.y2, SCOL_GREY);
		gfx->Line(desc.loc.x1, desc.loc.y2, desc.loc.x1, desc.loc.y1, SCOL_GREY);
	}
}

void C_GUIROMElement::SendMsg(GUI_MSG msg)
{
	if(gui->PreHandling(&desc, msg))
	{
		if(desc.newclicks == 1)
			screen->PushMessage(bmsg);
	}
}

void C_GUIROMElement::Kill(void)
{
	if(fname)
	{
		free(fname);
		fname = NULL;
	}

	if(rname)
	{
		free(rname);
		rname = NULL;
	}
}

#define ROMELEMENT_CLICK 0

void C_GUIROMElement::Setup(C_GUI *g, C_GUIMessageCollector *scrn, int id, GUI_BOX area, char *fn, char *rn, unsigned __int8 typ)
{
	gui = g;
	screen = scrn;
	gfx = gui->GetPartialGFX();
	desc.loc = clip = area;
	gui->SetDefault(&desc);

	bmsg.owner = id;
	bmsg.val = ROMELEMENT_CLICK;

	fname = strdup(fn);
	rname = strdup(rn);
	type = typ;
}

char *C_GUIROMElement::GetFName(void)
{
	return fname;
}

char *C_GUIROMElement::GetRName(void)
{
	return rname;
}

unsigned __int8 C_GUIROMElement::GetType(void)
{
	return type;
}

void C_GUIROMElement::SetPosition(int xx, int yy)
{
	x = xx;
	y = yy;

	desc.loc.y1 = y;
	desc.loc.y2 = y+gfx->StrHeight(FONT_SIZE12, "t");

	if(desc.loc.y1 <= clip.y1)
		desc.loc.y1 = clip.y1;
	if(desc.loc.y2 > clip.y2)
		desc.loc.y2 = clip.y2;
}

GUI_BOX C_GUIROMElement::GetShape(void)
{
	GUI_BOX t;

	t = clip;
	t.y2 = t.y1 + gfx->StrHeight(FONT_SIZE12, "t");

	return t;
}

/* a ROM slot screen */

class C_GUIROMSlot : public C_GUIElementBase, public C_GUIMessageCollector
{
	public :
		C_GUIROMSlot();

		void Draw(void);
		void SendMsg(GUI_MSG msg);
		void Kill(void);

		void Setup(C_GUI *g, int *slotv, GUI_BOX area, int slotid, char *sname, int col1, int col2, int fnt);

		void SetROM(char *fname, char *rname);
		char *GetROM(void);
		void SetMode(int m);
		int GetMode(void);

	private :
		static C_ULA *elk;

		GUI_BOX carea;
		int font, colour1, colour2;

		char *romname, *slotname;

		C_GUIRadioButton rom, ram, off;
		int rmode, *cslot, slotnum;
};

C_ULA *C_GUIROMSlot::elk;

C_GUIROMSlot::C_GUIROMSlot(void)
{
	romname = slotname = NULL;
}

void C_GUIROMSlot::Kill(void)
{
	if(romname)
	{
		free(romname);
		romname = NULL;
	}

	if(slotname)
	{
		free(slotname);
		slotname = NULL;
	}

	rom.Kill();
	ram.Kill();
	off.Kill();
}

#define SLOT_NAMEWIDTH	100
#define SLOT_RNAMEWIDTH	220

void C_GUIROMSlot::Draw(void)
{
	gfx->SetClipArea(carea.x1, carea.y1, carea.x1+SLOT_NAMEWIDTH, carea.y2);

	gfx->Rectangle(carea.x1, carea.y1, carea.x1+SLOT_NAMEWIDTH, carea.y2, (*cslot == slotnum) ? colour2 : colour1);
	gfx->WriteString(font, slotname, carea.x1, carea.y1, (*cslot == slotnum) ? colour2 : colour1);

	gfx->SetClipArea(carea.x1+SLOT_NAMEWIDTH, carea.y1, carea.x1+SLOT_NAMEWIDTH+SLOT_RNAMEWIDTH, carea.y2);
	gfx->WriteString(font, romname, carea.x1+SLOT_NAMEWIDTH, carea.y1, colour1);

	off.Draw();
	rom.Draw();
	ram.Draw();

	if(desc.mouseover)
	{
		gfx->Line(desc.loc.x1, desc.loc.y1, desc.loc.x2, desc.loc.y1, SCOL_GREY);
		gfx->Line(desc.loc.x2, desc.loc.y1, desc.loc.x2, desc.loc.y2, SCOL_GREY);
		gfx->Line(desc.loc.x2, desc.loc.y2, desc.loc.x1, desc.loc.y2, SCOL_GREY);
		gfx->Line(desc.loc.x1, desc.loc.y2, desc.loc.x1, desc.loc.y1, SCOL_GREY);
	}
}

void C_GUIROMSlot::SendMsg(GUI_MSG msg)
{
	off.SendMsg(msg);
	rom.SendMsg(msg);
	ram.SendMsg(msg);

	if(gui->PreHandling(&desc, msg))
	{
		if(desc.newclicks >= 1)
		{
			*cslot = slotnum;
		}
	}

	BACK_MSG w;

	while(MessagesWaiting())
	{
		w = PopMessage();
		SetMode(rmode);
	}
}

void C_GUIROMSlot::Setup(C_GUI *g, int *slotv, GUI_BOX area, int slotid, char *sname, int col1, int col2, int fnt)
{
	gui = g;
	gfx = gui->GetPartialGFX();
	elk = g->GetElectron();
	carea = desc.loc = area;
	InitialiseStack();

	desc.loc.x2 = desc.loc.x1 + SLOT_NAMEWIDTH;

	gui->SetDefault(&desc);

	slotname = strdup(sname);

	cslot = slotv;
	slotnum = slotid;

	font = fnt;
	colour1 = col1;
	colour2 = col2;

	int w;

	area.x1 += (SLOT_RNAMEWIDTH + SLOT_NAMEWIDTH);
	w = (area.x2 - area.x1) >> 2;
	area.x2 = area.x1 + w;

	off.Setup(g, this, 0, ULAROMMODE_OFF, &rmode, area, "Disable", font, colour1);
	off.MinimiseShape();
	area.x1 += w;
	area.x2 += w;

	rom.Setup(g, this, 0, ULAROMMODE_ON, &rmode, area, "ROM", font, colour1);
	rom.MinimiseShape();
	area.x1 += w;
	area.x2 += w;

	ram.Setup(g, this, 0, ULAROMMODE_RAM, &rmode, area, "RAM", font, colour1);
	ram.MinimiseShape();
	area.x1 += w;
	area.x2 += w;

	rmode = ULAROMMODE_OFF;
}

void C_GUIROMSlot::SetROM(char *fname, char *rname)
{
	if(fname)
	{
		if(elk->LoadROM(fname, slotnum, fname))
		{
			if(romname) free(romname);
			romname = strdup(rname);
		}
	}
	else
	{
		if(romname) free(romname);
		romname = strdup("<<No ROM>>");
		elk->FreeROM(slotnum);

		SetMode(ULAROMMODE_OFF);
	}
}

void C_GUIROMSlot::SetMode(int m)
{
	elk->SetROMPresent(slotnum, m);
	rmode = elk->GetROMPresent(slotnum);
}

int C_GUIROMSlot::GetMode(void)
{
	return rmode;
}

char *C_GUIROMSlot::GetROM(void)
{
	return elk->GetROMString(slotnum);
}
/* the menu screen */

bool C_ROMMenu::IsROM(unsigned __int8 *data)
{
	/* this is exactly the same test as [the AUG claims] the OS uses */

	int offs;

	offs = data[7];

	if(data[offs++]) return false;
	if(data[offs++] != '(') return false;
	if(data[offs++] != 'C') return false;
	if(data[offs] != ')') return false;

	return true;
}

void C_ROMMenu::GetROMName(unsigned __int8 *inrom, char *outname, int namelen)
{
	int c2, c3;

	if(IsROM(inrom))
	{
		c2 = 0;
		c3 = 9;
		do
		{
			outname[c2] = inrom[c3];

			if(!outname[c2]) break;

			if(outname[c2] >= 16 && outname[c2] <= 126)
				c2++;

			c3++;
		}
		while(c2 < namelen);

		outname[namelen-1] = '\0';
	}
	else
	{
		sprintf(outname, "<???>");
	}
}

unsigned __int8 C_ROMMenu::GetROMType(unsigned __int8 *inrom)
{
	/* only language or not language can be determined this way */

	return (inrom[6]&0x40) ? ROMCLASS_LANG : 0;
}

#define SLOT_FONT FONT_SIZE12
#define SLOT_SPACING 2

#define FREE_ID -1
#define RESTORE_ID -2

bool C_ROMMenu::Initialise(C_GUI *parent, int x1, int y1, int x2, int y2)
{
	gui = parent;
	elk = parent->GetElectron();

	/* set up initial parts of the GUI dialogue screen */
	GUI_BOX area;

	InitialiseStack();
	InitialiseList();
	IgnoreReset = false;
	ExplicitId = 0;
	int c = NUM_ROMCLASSES;
	while(c--) ImplicitHints[c] = true;

	area.x1 = x1;
	area.y1 = y1;
	area.x2 = x2;
	area.y2 = y2 - ULA_NUMROMS*(SLOT_SPACING+elk->GetGFX()->StrHeight(SLOT_FONT, ""));

	romselector.Setup(gui, area);
	AddToList(&romselector);

	/* this is the fun bit - this class is actually responsible for loading
	and inserting the paged ROMs as described in the config file */

	FileTree *move, *newroms;
	unsigned __int8 newdata[16384];
	gzFile file;

	char *romexts[] =
	{
		"", "rom", "rgz", "rom.gz", NULL
	};

	move = newroms = elk->GetFS()->GetFileTree(NULL, "rompath", romexts);

	if(!newroms) return false;

	/* calculate upper limit number of ROMs */
	numroms = 0;
	while(move)
	{
		if(!move->dir) numroms++;
		move = move->next;
	}

	/* allocate enough space for upper limit */
	roms = new C_GUIROMElement[numroms];
	numroms = 0;

	char namebuf[256];
	move = newroms;
	area = romselector.GetEnclosedArea();
	while(move)
	{
		if(!move->dir)
		{
			if(file = elk->GetFS()->GetGZ(NULL, "rompath", move->fname, "rb"))
			{
				gzread(file, newdata, 16384);
				gzclose(file);

				if(IsROM(newdata))
				{
					GetROMName(newdata, namebuf, 256);

					roms[numroms].Setup(parent, this, numroms, area, move->fname, namebuf, GetROMType(newdata));
					numroms ++;
				}
			}
		}

		move = move->next;
	}

	elk->GetFS()->FreeFileTree(newroms);
	romselector.SetElements(roms, numroms, sizeof(C_GUIROMElement));

	/* do rest of screen setup */
	int ty = area.y1 = area.y2+2 + (SLOT_SPACING >> 1);

	area.y2 = ((area.y1+y2) >> 1) - 2;
	area.x2 = x2;
	area.x1 = x2 - ((x2-x1)/8);
	empty.Setup(parent, this, FREE_ID, area, "Free ROM", 0, FCOL_WHITE | BCOL_BLACK, FONT_SIZE12, true, false);
	AddToList(&empty);

	area.y1 = area.y2 + 4;
	area.y2 = y2;
	restore.Setup(parent, this, RESTORE_ID, area, "Defaults", 0, FCOL_WHITE | BCOL_BLACK, FONT_SIZE12, true, false);
	AddToList(&restore);

	/* generate the various slots */
	area.y1 = ty;
	area.x1 = x1;
	area.y2 = area.y1 + elk->GetGFX()->StrHeight(SLOT_FONT, "");

	slots = new C_GUIROMSlot[ULA_NUMROMS];

	char *snames[] =
	{
		"Slot 0",
		"Slot 1",
		"Slot 2",
		"Slot 3",
		"Slot 4",
		"Slot 5",
		"Slot 6",
		"Slot 7",
		"BASIC slot",
		"Slot 12",
		"Slot 13",
		"Slot 14",
		"Slot 15"
	};

	cslot = 0;

	for(c = 0; c < ULA_NUMROMS; c++)
	{
		slots[c].Setup(parent, &cslot, area, c, snames[c], FCOL_WHITE | BCOL_BLACK, FCOL_WHITE | BCOL_BLUE, SLOT_FONT);

		area.y2 += area.y2-area.y1;
		area.y1 = (area.y2+area.y1) >> 1;
		area.y2 += SLOT_SPACING;
		area.y1 += SLOT_SPACING;

		AddToList(&slots[c]);
		slots[c].SetROM(NULL, NULL);
	}

	/* check config file for current settings */
	LoadFromConfigFile();

	return true;
}

void C_ROMMenu::DeInitialise(void)
{
	char opttext[10];
	config_setting s;
	int rslot;

	/* write settings */
	int c = ULA_NUMROMS;
	while(c--)
	{
		rslot = elk->GetHardwareROMSlot(c);

		sprintf(opttext, "rom%dmode", rslot);
		elk->GetConfig()->FindSetting(opttext, ANY_RESULT);
		s.result_type = INTEGER_RESULT;
		s.result_int = slots[c].GetMode();
		elk->GetConfig()->WriteSetting(s);

		sprintf(opttext, "rom%dfile", rslot);
		elk->GetConfig()->FindSetting(opttext, ANY_RESULT);
		s.result_type = STRING_RESULT;
		s.result_string = slots[c].GetROM();
		if(!s.result_string) s.result_string = "";
		elk->GetConfig()->WriteSetting(s);

		slots[c].Kill();
	}

	/* free memory */
	delete[] slots;
	delete[] roms;

	while(ExplicitId--)
		free(ExplicitHints[ExplicitId].name);

	ExplicitId = 0;
}

void C_ROMMenu::SendMsg(GUI_MSG msg)
{
	C_GUIScreen::SendMsg(msg);

	BACK_MSG m;

	while(MessagesWaiting())
	{
		m = PopMessage();

		if(m.owner >= 0)
		{
			slots[cslot].SetROM(roms[m.owner].GetFName(), roms[m.owner].GetRName());
		}
		else
		{
			switch(m.owner)
			{
				case FREE_ID :
					if(m.val == GUIBUTTON_PERFORM)
						slots[cslot].SetROM(NULL, NULL);
				break;

				case RESTORE_ID :
					if(m.val == GUIBUTTON_PERFORM)
						SetRoms(false, true, false, false);
				break;
			}
		}
	}
}

void C_ROMMenu::LoadFromConfigFile(void)
{
	int rslot;
	char opttext[10];
	gzFile file;
	unsigned __int8 newdata[16384];
	char namebuf[256];

	int c = ULA_NUMROMS;
	while(c--)
	{
		rslot = elk->GetHardwareROMSlot(c);

		sprintf(opttext, "rom%dfile", rslot);
		if(elk->GetConfig()->FindSetting(opttext, STRING_RESULT))
		{

			if(file = elk->GetFS()->GetGZ(NULL, "rompath", elk->GetConfig()->ReadSetting().result_string, "rb"))
			{
				gzread(file, newdata, 16384);
				gzclose(file);

				if(IsROM(newdata))
				{
					GetROMName(newdata, namebuf, 256);

					elk->GetConfig()->FindSetting(opttext, STRING_RESULT);
					slots[c].SetROM(elk->GetConfig()->ReadSetting().result_string, namebuf);
				}
			}

			sprintf(opttext, "rom%dmode", rslot);
			if(elk->GetConfig()->FindSetting(opttext, INTEGER_RESULT))
			{
				slots[c].SetMode(elk->GetConfig()->ReadSetting().result_int);
			}
		}
	}

	/* check for 'restore default' option */
	if(elk->GetConfig()->FindSetting("resetroms", BOOLEAN_RESULT))
	{
		if(elk->GetConfig()->ReadSetting().result_bool)
		{
			config_setting s;

			s.result_type = BOOLEAN_RESULT;
			s.result_bool = !SetRoms(false, true, false, false);
			elk->GetConfig()->FindSetting("resetroms", ANY_RESULT);
			elk->GetConfig()->WriteSetting(s);
		}
	}
}

/*

ids :

	0 - tape, CHAIN""
	1 - tape, *RUN""

	4 - DFS, drive 0, *CAT
	5 - DFS, drive 0, *LOAD
	6 - DFS, drive 0, *RUN
	7 - DFS, drive 0, *EXEC

	8 - DFS, drive 1, *CAT
	9 - DFS, drive 1, *LOAD
	10 - DFS, drive 1, *RUN
	11 - DFS, drive 1, *EXEC

	12 - ADFS, drive 0, *CAT
	13 - ADFS, drive 0, *LOAD
	14 - ADFS, drive 0, *RUN
	15 - ADFS, drive 0, *EXEC

	16 - ADFS, drive 1, *CAT
	17 - ADFS, drive 1, *LOAD
	18 - ADFS, drive 1, *RUN
	19 - ADFS, drive 1, *EXEC

	ff - no change

*/

bool C_ROMMenu::SetAutoId(unsigned __int8 id)
{
	switch(id >> 2)
	{
		case 0 :
			/* tape */
			SetRoms(false, false, true, true);
		break;

		case 1 :
		case 2 :
			/* DFS */
			SetRoms(true, false, true, true);
		break;

		case 3 :
		case 4 :
			/* ADFS */
			SetRoms(false, true, true, true);
		break;
	}

	return true;
}

/*

FindAndAdd finds and adds a particular ROM name if a hint against that
particular type of ROM is not set

*/
bool C_ROMMenu::FindAndAdd(char *name, int slot, bool complete)
{
	char *t1, *t2;
	int c;
	unsigned __int8 type;

	/* check name against explicit ROM hints first */
	c = ExplicitId;
	while(c--)
	{
		t1 = ExplicitHints[ExplicitId].name;
		t2 = name;

		while(*t1 == *t2)
		{
			t1++;
			t2++;
		}

		if(!*t1 || !*t2) return false;
	}

	/* now see if name can be found with a non-disabled ROM */
	c = numroms;
	while(c--)
	{
		t1 = name;
		t2 = roms[c].GetRName();
		type = roms[c].GetType();

		if(!type || ImplicitHints[type])
		{
			while(*t1 && *t1 == *t2)
			{
				t1++;
				t2++;
			}

			if(!*t1)
			{
				if(!complete)
					break;
				else
					if(!*t2)
						break;
			}
		}
	}

	if(c >= 0)
	{
		slots[slot].SetROM(roms[c].GetFName(), roms[c].GetRName());
		slots[slot].SetMode(ULAROMMODE_ON);

		return true;
	}

	return false;
}

#define BASIC_SLOT		8
#define PLUS1_SLOT		9
#define AUTOROM_SLOT	5
#define DFS_SLOT		0
#define E00ADFS_SLOT1	2
#define E00ADFS_SLOT2	3
#define ADFS_SLOT		4

bool C_ROMMenu::SetRoms(bool dfs, bool adfs, bool persist, bool aload)
{
	int c;
	bool res;

	c = ULA_NUMROMS;
	while(c--)
		if(!aload || c != PLUS1_SLOT)
			slots[c].SetROM(NULL, NULL);

	if(!ImplicitHints[ROMCLASS_ALL]) return false;

	res = FindAndAdd("BASIC", BASIC_SLOT, false);

	if(!ImplicitHints[ROMCLASS_PAGED]) return res;

	if(!aload)
		FindAndAdd("Electron Expansion", PLUS1_SLOT, false);

	if(aload)
		FindAndAdd("Emulator autoload", AUTOROM_SLOT, false);

	if(ImplicitHints[ROMCLASS_FS])
	{
		if(dfs)
		{
			if(FindAndAdd("DFS", DFS_SLOT, true))
				slots[DFS_SLOT].SetMode(persist ? ULAROMMODE_RAM : ULAROMMODE_OFF);
		}

		if(adfs)
		{
			if(	!FindAndAdd("PRES E00 ADFS", E00ADFS_SLOT1, true) ||
				!FindAndAdd("PRES E00 ADFS L", E00ADFS_SLOT2, false) )
			{
				slots[E00ADFS_SLOT1].SetROM(NULL, NULL);
				slots[E00ADFS_SLOT2].SetROM(NULL, NULL);

				FindAndAdd("Acorn ADFS", ADFS_SLOT, false);
				slots[ADFS_SLOT].SetMode(persist ? ULAROMMODE_ON : ULAROMMODE_OFF);
			}
			else
			{
				slots[E00ADFS_SLOT1].SetMode(persist ? ULAROMMODE_RAM : ULAROMMODE_OFF);
				slots[E00ADFS_SLOT2].SetMode(persist ? ULAROMMODE_RAM : ULAROMMODE_OFF);
			}
		}
	}

	return res;
}

#undef BASIC_SLOT
#undef PLUS1_SLOT
#undef AUTOROM_SLOT
#undef DFS_SLOT
#undef E00ADFS_SLOT1
#undef E00ADFS_SLOT2
#undef ADFS_SLOT

void C_ROMMenu::PassROMHint(bool set, int id, char *name)
{
	bool checkroms = false;

	switch(id)
	{
		case ROMCLASS_ALL :		/* all ROMs including BASIC */
		case ROMCLASS_PAGED :	/* all ROMs except BASIC */
		case ROMCLASS_DFS :		/* DFS ROM(s) */
		case ROMCLASS_ADFS :	/* ADFS ROM(s) */
		case ROMCLASS_FS :		/* all FS ROM(s) */
		case ROMCLASS_LANG :	/* language ROM(s) */
			ImplicitHints[id] = set;
			checkroms = true;
		break;

		case ROMCLASS_SPECIFIC :
			if(ExplicitId < 256)
			{
				ExplicitHints[ExplicitId].name = strdup(name);
				ExplicitHints[ExplicitId].add = set;

				ExplicitId ++;
			}
		break;

		case ROMCLASS_IGNORERESET :
			IgnoreReset = true;
		break;

		case ROMCLASS_RESET :
			if(!IgnoreReset)
			{
				while(ExplicitId--)
					free(ExplicitHints[ExplicitId].name);

				ExplicitId = 0;

				int c = NUM_ROMCLASSES;
				while(c--)
					ImplicitHints[c] = true;
			}

			IgnoreReset = false;
		break;
	}

	if(checkroms)
	{
		/* check all ROMs against new criteria */
	}
}

