/*

	ElectrEm (c) 2000/1 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for
	details

	misc/Allegro/aplatipt.cpp
	=========================

	The input class - Allegro bits. Pitifully simple. Allegro even emulates
	digital mode for analogue joysticks & vice versa

*/
#include "../input.h"
#include "../../targets.h"

#ifdef TRG_IPT_ALLEGRO

#define JOYFILE "joy.dat"

#include <allegro.h>

void C_input::PlatformConstructor(void)
{
	sticks = NULL;
}

void C_input::PlatformDestructor(void)
{
	save_joystick_data(JOYFILE);

	if(sticks)
	{
		free(sticks);
		sticks = NULL;
	}
}

bool C_input::PlatformSetup(void)
{
	if(install_keyboard() < 0)
		return false;
	InitialiseJoysticks();

	array = key;
	set_leds(0);

	return true;
}

bool C_input::UpdateKeyStates(void)
{
	if(numsticks) poll_joystick();
	return true;
}

void C_input::UpdateLEDS(void)
{
	if((oldnum != num) || (oldcaps != caps) || (oldscroll != scroll))
	{
		set_leds( (num ? KB_NUMLOCK_FLAG : 0) | (caps ? KB_CAPSLOCK_FLAG : 0) | (scroll ? KB_SCROLOCK_FLAG : 0));
		oldnum = num; oldcaps = caps; oldscroll = scroll;
	}
}

void C_input::ClearBuffer(void)
{
	clear_keybuf();
}

bool C_input::NewKey(void)
{
	return (keypressed() == TRUE) ? true : false;
}

unsigned char C_input::GetPressedASCII(void)
{
	return readkey();
}

int C_input::PlatformGetNumKeys(void)
{
	return KEY_MAX;
}

/* joystick stuff */
#include <stdio.h>
void C_input::InitialiseJoysticks(void)
{
	if(load_joystick_data(JOYFILE))
	{
		install_joystick(JOY_TYPE_AUTODETECT);
	}

	if(joy_type == JOY_TYPE_NONE || num_joysticks == 0)
	{
		numsticks = 0;
		sticks = NULL;
	}
	else
	{
		numsticks = num_joysticks;
		sticks = (JOY_INFO *)malloc(sizeof(JOY_INFO)*numsticks);

		int c;
		c = numsticks;
		while(c--)
		{
			sticks[c].calibrate = joy[c].stick[0].flags & JOYFLAG_CALIBRATE;
			sticks[c].analogue = (joy[c].stick[0].flags & JOYFLAG_ANALOGUE) || (joy[c].stick[0].flags & JOYFLAG_CALIB_ANALOGUE);
		}
	}
}

JOY_INFO C_input::GetJoystickInfo(int id)
{
	return sticks[id];
}

JOY_STATE C_input::PlatformGetJoystickState(int id)
{
	JOY_STATE state;

	/*

	nice thing about Allegro : it does the digital and analogue emulation as
	necessary, without ElectrEm having to get involved

	*/

	if(id >= numsticks)
		return state;

	/* floating point numbers, range 0 -> 1 */
	state.x = -joy[id].stick[0].axis[0].pos;
	state.y = -joy[id].stick[0].axis[1].pos;

	if(!(joy[id].stick[0].flags&JOYFLAG_SIGNED))
	{
		state.x += 128;
		state.y += 128;
	}

	/* boolean values */
	state.up = joy[id].stick[0].axis[1].d1;
	state.down = joy[id].stick[0].axis[1].d2;
	state.left = joy[id].stick[0].axis[0].d1;
	state.right = joy[id].stick[0].axis[0].d2;

	state.button = false;
	int c = joy[id].num_buttons;
	while(c--)
		state.button = state.button || joy[id].button[c].b;

	return state;
}

void C_input::BeginCalibration(int id)
{
	cjoyid = id;
}

const char *C_input::GetJoystickInstruction(void)
{
	return calibrate_joystick_name(cjoyid);
}

void C_input::FlagJoystickDone(void)
{
	calibrate_joystick(cjoyid);
}

#endif
