/*

	ElectrEm (c) 2000/1 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for
	details

	misc/Allegro/timing.cpp
	=======================

	Handles the frame limiting and skipping in an Allegro way

*/
#include "../../targets.h"

#ifdef TRG_TMR_ALLEGRO

#include "../timing.h"
#include <allegro.h>
volatile int timevar;

/*

under Windows, Allegro can only offer 200 ticks per second accurately, and as
a result, the following #define'd values have been selected

*/

#define MULTIPLIER 4
#define MAXBEATS 50

void timing_func(void)
{
	timevar++;
}

END_OF_FUNCTION(timing_func);

C_timer::C_timer(void)
{
}

C_timer::~C_timer(void)
{
}

bool C_timer::Calibrate(int beats)
{
	if(beats > MAXBEATS)
	{
		subtotal = (beats<<8) / MAXBEATS;
		beats /= (subtotal>>8);
	}
	else
		subtotal = 1<<8;

	beats *= MULTIPLIER;
	install_int_ex(timing_func, BPS_TO_TIMER(beats));

	error = timevar = 0;

	return true;
}

float C_timer::GetMultiplier(void)
{
	return mul;
}

void C_timer::SetMultiplier(float mult)
{
	mul = mult;
	if(mul)
		Calibrate((int)(mult*(float)standard));
}

void C_timer::Resync(void)
{
	current = errc = error = timevar = 0;
}

bool C_timer::SetBeatsPerSecond(int beats)
{
	LOCK_FUNCTION(timing_func);
	LOCK_VARIABLE(timevar);
	install_timer();

	mul = 1.0f;
	standard = beats;

	return Calibrate(beats);
}

void C_timer::RegisterBeat(void)
{
	if(!mul)
		return;

	current+=256;
	if(current >= subtotal)
	{
		current -= subtotal;
		while((timevar+error) < MULTIPLIER);
		error += timevar - MULTIPLIER;
		timevar = 0;
	}
}

bool C_timer::FrameLate(void)
{
	if(error*subtotal >= MULTIPLIER)
	{
		errc++;
		if(errc > 10)
		{
			Resync();
			return false;
		}
		return true;
	}

	return false;
}

#endif
