/*

	ElectrEm (c) 2000/1 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for
	details

	misc/Posix/timing.cpp
	=====================

	Handles the frame limiting and skipping, POSIX kernel signal style. I'd
	like to come up with a better solution, but whenever you ask about
	mildly accurate timing, Linux people say 'get real time Linux (a different
	version of the OS) for that sort of thing'.

	An alternative for intel line Pentium and above Linux users would be the
	RDTSC instruction, which is something I shall very shortly look into
	properly

*/
#include "../timing.h"
#include "../../targets.h"

#ifdef TRG_TMR_POSIX

#include <sys/time.h>
#include <signal.h>
#include <stdio.h>
volatile sig_atomic_t timevar;

void timing_func(int sig)
{
	timevar++;
}

#define US 1000000
#define MULTIPLIER 2
#define MAXBEATS 25

C_timer::C_timer(void)
{
}

C_timer::~C_timer(void)
{
}

bool C_timer::Calibrate(int beats)
{
	struct itimerval itrm;
	int nbeats;

	subtotal = 1;

	//don't trust kernel above 50 beats per second . . .
	if(beats > MAXBEATS)
	{
		subtotal = (beats << 8) / MAXBEATS;
		nbeats = (beats << 8) / subtotal;
	}
	else
	{
		nbeats = beats << 8;
		subtotal = 1 << 8;
	}

	nbeats *= (MULTIPLIER << 8);

	itrm.it_interval.tv_sec = 0;
	itrm.it_interval.tv_usec = (US << 8)/nbeats;

	itrm.it_value = itrm.it_interval;

	setitimer(ITIMER_REAL, &itrm, NULL);

	return true;
}

float C_timer::GetMultiplier(void)
{
	return mul;
}

void C_timer::SetMultiplier(float mult)
{
	mul = mult;
	if(mul)
		Calibrate((int)(mult*(float)standard));
}

void C_timer::Resync(void)
{
	current = errc = error = timevar = 0;
}

bool C_timer::SetBeatsPerSecond(int beats)
{
	signal(SIGALRM, timing_func);

	mul = 1.0f;
	standard = beats;

	return Calibrate(beats);
}

void C_timer::RegisterBeat(void)
{
	if(!mul)
		return;

	current+=256;
	if(current >= subtotal)
	{
		current -= subtotal;

		while((timevar+error) < MULTIPLIER);

		error += timevar - MULTIPLIER;
		timevar = 0;
	}
}

bool C_timer::FrameLate(void)
{
	if(error*subtotal >= MULTIPLIER)
	{
		errc++;
		if(errc > 10)
			Resync();
		return true;
	}
	return false;
}

#endif
