/*

	ElectrEm (c) 2000/1 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for
	details

	misc/SVGALib/platipt.cpp
	========================

	The input class - SVGALib bits. With messed up ASCII buffer, and Linux
	console joystick device joystick support rather than SVGALib joystick
	support to avoid the draconian calibration methods - indeed to avoid any
	calibration!

*/
#include "../input.h"
#include "../../targets.h"

#ifdef TRG_IPT_SVGALIB

#include <unistd.h>
#include <sys/ioctl.h>
#include <linux/kd.h>
#include <linux/joystick.h>
#include <string.h>
#include <malloc.h>
#include <stdio.h>
#include <fcntl.h>

#include "../atables.h"

void C_input::PlatformConstructor(void)
{
	memset(oldstates, 0, MAX_ASCII);
}

void C_input::PlatformDestructor(void)
{
	keyboard_close();

	while(numsticks--)
		close(sticks[numsticks].fd);

	free(sticks);
}

bool C_input::PlatformSetup(void)
{
	keyboard_fd = keyboard_init_return_fd();
//	keyboard_clearstate();

	/* NB : on an Electron with Plus 1,

	CTRL+C = stop directing output to printer

	so it would be really nice if that key combination didn't also cause
	ElectrEm to quit */
//	keyboard_translatekeys(DONT_CATCH_CTRLC);

	/* check for up to 4 /dev/js?'s */
	char fname[9];
	sticks = (JOY_INFO *)malloc(sizeof(JOY_INFO)*4);
	numsticks = 0;

	for(int c = 0; c < 4; c++)
	{
		sprintf(fname, "/dev/js%d", c);

		sticks[numsticks].fd = open(fname, O_NONBLOCK);

		if(sticks[numsticks].fd >= 0)
		{
			sticks[numsticks].buttons = 0;
			numsticks++;
		}
	}

	return keyboard_fd != -1;
}

bool C_input::UpdateKeyStates(void)
{
	keyboard_update();
	array = keyboard_getstate();

	return true;
}

void C_input::UpdateLEDS(void)
{
	/* not sure about this */
	if((oldnum != num) || (oldcaps != caps) || (oldscroll != scroll))
	{
		ioctl(keyboard_fd, KDSETLED, (caps ? LED_CAP : 0) | (num ? LED_NUM : 0) | (scroll ? LED_SCR : 0));
		oldnum = num; oldcaps = caps; oldscroll = scroll;
	}
}

void C_input::ClearBuffer(void)
{
}

bool C_input::NewKey(void)
{
	int c;
	bool ret = false;

	c = MAX_ASCII;
	while(c--)
	{
		if((array[c] ^oldstates[c])&array[c])
		{
			ret = true;
		}
		oldstates[c] = array[c];
	}

	return ret;
}

unsigned char C_input::GetPressedASCII(void)
{
	unsigned char *stable;
	int c;

	stable = (array[SCANCODE_LEFTSHIFT] || array[SCANCODE_RIGHTSHIFT]) ? ascii_table_shift : ascii_table;
	c = MAX_ASCII;
	while(c--)
	{
		if(array[c] && stable[c])
			return stable[c];
	}

	return 0;
}

int C_input::PlatformGetNumKeys(void)
{
	return MAX_KEY;
}

/* joystick stuff, no joystick supported yet */

JOY_INFO C_input::GetJoystickInfo(int id)
{
	JOY_INFO f;

	f.calibrate = false;
	f.analogue = true;

	return f;
}

JOY_STATE C_input::PlatformGetJoystickState(int id)
{
	js_event e;

	if(id >= numsticks) return sticks[0].s;

	while(read(sticks[id].fd, &e, sizeof(js_event)) > 0)
	{
		switch(e.type)
		{
			case JS_EVENT_INIT | JS_EVENT_BUTTON :

				if(e.value)
					sticks[id].buttons |= (1 << e.number);
				else
					sticks[id].buttons &= ~(1 << e.number);
				sticks[id].s.button = sticks[id].buttons ? true : false;

			break;

			case JS_EVENT_BUTTON :

				sticks[id].buttons ^= (1 << e.number);
				sticks[id].s.button = sticks[id].buttons ? true : false;

			break;

			case JS_EVENT_AXIS | JS_EVENT_INIT :
			case JS_EVENT_AXIS :

				switch(e.number)
				{
					default : break;

					case 0 : //X
						sticks[id].s.x = (- e.value) >> 8;

						sticks[id].s.left = e.value < -64;
						sticks[id].s.right = e.value > 64;
					break;

					case 1 : //Y
						sticks[id].s.y = (- e.value) >> 8;

						sticks[id].s.up = e.value < -64;
						sticks[id].s.down = e.value > 64;
					break;
				}

			break;
		}
	}

	return sticks[id].s;
}

void C_input::BeginCalibration(int id)
{
}

const char *C_input::GetJoystickInstruction(void)
{
	return NULL;
}

void C_input::FlagJoystickDone(void)
{
}

#endif
