/*

	ElectrEm (c) 2000 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for
	details

	misc/Win32/timing.cpp
	=====================

	Handles the frame limiting and skipping in a Win32/PerformanceCounter way
*/
#include "../timing.h"
#include "../../targets.h"

#ifdef TRG_TMR_WIN32

bool skip;
int errc;

C_timer::C_timer(void)
{
}

C_timer::~C_timer(void)
{
}

bool C_timer::Calibrate(int beats)
{
	aim = 256;

	if(beats > 50)
	{
		aim = (beats <<8) / 50;
		beats = 50;
	}

	QueryPerformanceFrequency((LARGE_INTEGER *)&add);
	add /= beats;

	Resync();
	
	return true;
}

float C_timer::GetMultiplier(void)
{
	return mul;
}

void C_timer::SetMultiplier(float mult)
{
	mul = mult;
	if(mul)
	{
		Calibrate((int)(mult*(float)standard));
		skip = false;
	}
	else
		skip = true;
}

bool C_timer::SetBeatsPerSecond(int beats)
{
	mul = 1.0f;
	standard = beats;
	skip = false;

	return Calibrate(beats);
}

void C_timer::Resync(void)
{
	QueryPerformanceCounter((LARGE_INTEGER *)&target);
	target += add;
	late = false;
	subtotal = aim;
	errc = 0;
}

void C_timer::RegisterBeat(void)
{
	unsigned __int64 current;

	if(skip) return;

	subtotal-=256;
	if(subtotal<=0)
	{
		subtotal += aim;

		do
			QueryPerformanceCounter((LARGE_INTEGER *)&current);
		while(current < target);

		target += add;
		late = current > target;
	}
}

bool C_timer::FrameLate(void)
{
	if(late)
	{
		errc++;
		if(errc > 5)
		{
			Resync();
			return false;
		}
	}
	return late;
}

#endif