/*

	ElectrEm (c) 2000/1 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for
	details

	misc/config.cpp
	===============

	This works with ASCII based configuration / options files. And was
	completely rewritten for beta 7, chopping about 300 lines and several
	bugs in the process

*/
#include "../fs/fs.h"
#include "config.h"
#include <stdlib.h>
#include <malloc.h>
#include <string.h>
#include <ctype.h>

/* CONSTRUCTOR / DESTRUCTOR */
C_config::C_config(void)
{
	list = current = NULL;
	owner = NULL;
	req = NULL;
	fname = NULL;
	dirty = false;
}

C_config::~C_config(void)
{
	Close();
}

ConfigIndex *C_config::NewIndex(void)
{
	if(!list)
	{
		current = list = new ConfigIndex;
	}
	else
	{
		current = list;
		while(current->next)
		{
			current = current->next;
		}
		current->next = new ConfigIndex;
		current = current->next;
	}

	current->value.result_type = NO_RESULT;
	current->title = NULL;
	current->next = NULL;
	current->freestr = false;

	return current;
}

void C_config::GetNewLine(FILE *f)
{
	NewIndex();

	int len = 256, pos = 0, nextc;
	current->title = (char *)malloc(len);

	do
	{
		nextc = fgetc(f);

		if(nextc != '\r' && nextc != '\n' && nextc != EOF)
		{
			current->title[pos] = nextc;

			pos++;
			if(pos == len)
			{
				len += 256;
				current->title = (char *)realloc(current->title, len);
			}
		}
	}
	while(nextc != '\n' && nextc != EOF);

	current->title[pos] = '\0';
}

void C_config::RollLeft(void)
{
	int c = 0, l;

	l = strlen(current->title);

	while(l--)
	{
		current->title[c] = current->title[c+1];
		c++;
	}
}

void C_config::DetermineType(char *arg)
{
	char *end, *p, *pc;

	struct banswer
	{
		char *text;
		bool r;
	};
	banswer *b, banswers[] =
	{
		{"true", true},		{"yes", true},	{"on", true},
		{"false", false},	{"no", false},	{"off", false},
		{NULL}
	};

	//kill white space at end
	p = arg + strlen(arg) - 1;
	while(*p == ' ' || *p == '\t')
		p--;
	p[1] = '\0';

	//check if there actually is a setting here
	if(!strlen(arg))
	{
		current->value.result_type = NO_RESULT;
		return;
	}

	//check if this looks like a number
	current->value.result_int = strtol(arg, &end, 0);
	if(*end == '\0')
	{
		current->value.result_type = INTEGER_RESULT;
		return;
	}

	//now assume it is a string, but check for boolean values
	current->value.result_string = arg;
	current->value.result_type = STRING_RESULT;

	pc = p = strdup(arg);
	while(*pc)
	{
		*pc = tolower(*pc);
		pc++;
	}

	b = banswers;
	while(b->text)
	{
		if(!strcmp(b->text, p))
		{
			current->value.result_type = BOOLEAN_RESULT;
			current->value.result_bool = b->r;
			return;
		}

		b++;
	}
}

void C_config::FreeIndex(void)
{
	if(current == list)
	{
		list = list->next;
	}
	else
	{
		ConfigIndex *c = list;
		while(c->next != current)
			c = c->next;
		c->next = current->next;
	}

	if(current->title)
	{
		free(current->title);
		current->title = NULL;
	}

	if(current->freestr && current->value.result_string)
	{
		free(current->value.result_string);
		current->value.result_string = NULL;
	}

	delete current;
	current = list;
}

void C_config::Close(void)
{
	ConfigIndex *n;
	FILE *out;

	/* save file here, if one is open */
	if(fname)
	{
		if(dirty && (out = fopen(fname, "wt")))
		{
			current = list;
			while(current)
			{
				fputs(current->title, out);

				if(current->value.result_type != NO_RESULT)
				{
					fputs(" = ", out);

					switch(current->value.result_type)
					{
						case BOOLEAN_RESULT :
							fputs(current->value.result_bool ? "true" : "false", out);
						break;

						case INTEGER_RESULT :
							fprintf(out, "%d", current->value.result_int);
						break;

						case STRING_RESULT :
							if(current->value.result_string)
								fputs(current->value.result_string, out);
						break;
					}
				}

				if(current->next)
					fputc('\n', out);

				current = current->next;
			}

			fclose(out);
		}

		dirty = false;
		free(fname);
		fname = NULL;
	}

	/* free memory */
	current = list;
	while(current)
	{
		n = current->next;
		FreeIndex();
		current = n;
	}

	list = current = NULL;

	if(req)
	{
		free(req);
		req = NULL;
	}
}

bool C_config::Open(char *name, C_FSAccess *own)
{
	FILE *input;
	char *t;
	bool assign;

	owner = own;

	Close();
	dirty = false;

	fname = strdup(name);
	if(!(input = fopen(name, "r"))) return false;

	do
	{
		GetNewLine(input);

		//get rid of preceeding white space
		while(current->title[0] == ' ' || current->title[0] == '\t')
			RollLeft();

		//check now if is a comment
		if(current->title[0] == '#' || current->title[0] == '\0')
		{
			current->value.result_type = NO_RESULT;
		}
		else
		{
			//check an '=' is in there somewhere
			t = current->title;
			assign = false;
			while(*t)
			{
				if(*t == '=') assign = true;
				t++;
			}

			if(!assign)
			{
				current->value.result_type = NO_RESULT;
			}
			else
			{
				//find & mark end of identifier
				t = current->title;
				while(*t != ' ' && *t != '\t' && *t != '=')
					t++;

				*t = '\0'; //fix up title

				//negotiate assignment operator
				while(*t != '=')
					t++;
				t++;

				//get rid of white space preceeding setting
				while(*t == ' ' || *t == '\t')
					t++;

				DetermineType(t);
			}
		}
	}
	while(!feof(input));

	fclose(input);

	return true;
}

bool C_config::FindSetting(char *name, set_types style)
{
	found = false;
	current = list;

	if(req) free(req);
	req = strdup(name);

	while(current)
	{
		if(	!strcmp(current->title, name) &&
			( (style == ANY_RESULT) || (style == current->value.result_type) ))
		{
			found = true;
			break;
		}

		current = current->next;
	}

	return found;
}

config_setting C_config::ReadSetting(void)
{
	config_setting junk;

	return (current != NULL) ? current->value : junk;
}

void C_config::WriteSetting(struct config_setting setting)
{
	dirty = true;

	if(setting.result_type == STRING_RESULT && !setting.result_string)
		return;

	if(found)
	{
		free(current->title);
		if(current->freestr)
			free(current->value.result_string);
		current->freestr = false;
	}
	else
		NewIndex();

	current->title = strdup(req);
	current->value = setting;

	if(current->value.result_type == STRING_RESULT)
	{
		current->value.result_string = strdup(setting.result_string);
		current->freestr = true;
	}
}
