/*

	ElectrEm (c) 2000/1 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for
	details

	misc/input.cpp
	==============

	The platform independant parts of the increasingly badly named input
	class.

	Originally this class provided keyboard reading stuff, hence its name, but
	now keyboard & joystick reading & calibration, and 'parallel port' writing
	all occurs within mostly target specific parts of this class

*/
#include "input.h"
#include <string.h>
#include <stdio.h>
#include <malloc.h>
#include "../ula/ula.h"

#define NK NULL

char *ElkNormalKeyList[] =
{
	"Right cursor",		"Copy",			"No key",	"Space bar",
	"Left cursor",		"Down cursor",	"Return",	"Delete",
	"-",				"Up cursor",	":",		"No key",
	"0", 				"p",			";",		"/",
	"9", 				"o",			"l",		".",
	"8", 				"i",			"k",		",",
	"7",				"u",			"j",		"m",
	"6",				"y",			"h",		"n",
	"5",				"t",			"g",		"b",
	"4",				"r",			"f",		"v",
	"3",				"e",			"d",		"c",
	"2",				"w",			"s",		"x",
	"1",				"q",			"a",		"z",
	"Escape",			"FUNC",			"CTRL",		"Shift",
	"Superbreak shift",	"Break",		"Menu",		"Quit emulator"
};

char *ElkShiftKeyList[] =
{
	"|",				"[",			NK,			NK,
	"^",				"_",			NK,			NK,
	"=",				"",			"*",		NK,
	"@",				"P",			"+",		"?",
	")",				"O",			"L",		">",
	"(",				"I",			"K",		"<",
	"'",				"U",			"J",		"M",
	"&",				"Y",			"H",		"N",
	"%",				"T",			"G",		"B",
	"$",				"R",			"F",		"V",
	"#",				"E",			"D",		"C",
	"\"",				"W",			"S",		"X",
	"!",				"Q",			"A",		"Z",
	NK,					"CAPS LK",		NK,			NK,
	NK,					NK,				NK,			NK
};

char *ElkCtrlKeyList[] =
{
	"\\",				"]",			NK,			NK,
	"~",				"}",			NK,			NK,
	NK,					"{",			NK,			NK,
	NK,					NK,				NK,			NK,
	NK,					NK,				NK,			NK,
	NK,					"{",			NK,			NK,
	NK,					NK,				NK,			NK,
	NK,					NK,				NK,			NK,
	NK,					"{",			NK,			NK,
	NK,					NK,				NK,			NK,
	NK,					NK,				NK,			NK,
	NK,					"{",			NK,			NK,
	NK,					NK,				NK,			NK,
	NK,					NK,				NK,			NK,
	NK,					"{",			NK,			NK,
};

char *ElkFuncKeyList[] =
{
	NK,					NK,				NK,			NK,
	NK,					NK,				NK,			NK,
	NK,					NK,				NK,			NK,
	"F0",				NK,				"PLOT",		"PRINT",
	"F9",				"OLD [R]",		"LIST",		"SAVE",
	"F8",				"INPUT",		"CHAIN",	"LOAD",
	"F7",				"UNTIL",		"RAD",		"MODE",
	"F6",				"REPEAT",		"DEG",		"NEXT",
	"F5",				"THEN",			"GOTO",		"RENUM",
	"F4",				"RUN [R]",		"FOR",		"VDU",
	"F3",				"ELSE",			"DRAW",		"COLOUR",
	"F2",				"RESTORE",		"STEP",		"PROC",
	"F1",				"LOCAL",		"AUTO",		"END",
	NK,					NK,				NK,			NK,
	NK,					NK,				NK,			NK
};

char **ElkKeyLists[] =
{
	ElkNormalKeyList, ElkShiftKeyList, ElkCtrlKeyList, ElkFuncKeyList
};

C_input::C_input(C_ULA *elk)
{
	mapname = NULL;
	mapdirty =
	num = caps = scroll = oldnum = oldcaps = oldscroll = false;
	electron = elk;
	ClearKeyPressedDescriptor();

	for(int c = 0; c < 8; c++) Tables[c] = NULL;

	PlatformConstructor();
}

C_input::~C_input(void)
{
	CloseMap();

	ParportClose();
	PlatformDestructor();

	for(int c = 0; c < 8; c++)
	{
		if(Tables[c])
		{
			free(Tables[c]);
			Tables[c] = NULL;
		}
	}
}

bool C_input::Setup(void)
{
	numsticks = 0;
	if(!PlatformSetup()) return false;

	oldnum = oldcaps = oldscroll = false;
	memset(keystates, 0, 14);
	MaxKey = PlatformGetNumKeys();

	for(int tab = 0; tab < 8; tab++)
	{
		Tables[tab] = (struct SingleKey *)malloc(sizeof(struct SingleKey)*MaxKey);

		for(int key = 0; key < MaxKey; key++)
		{
			Tables[tab][key].shift = 0;
			Tables[tab][key].key = INPUT_NOKEY;
		}
	}

	Tables[0][EKEY_DEFQUIT].key = 0xe8;
	Tables[0][EKEY_DEFQUIT].shift = 0;
	Tables[0][EKEY_DEFMENU].key = 0xe4;
	Tables[0][EKEY_DEFMENU].shift = 0;

	ParportSetup();
	return true;
}

unsigned __int8 C_input::GetKeyboard(BrokenWord addr)
{
	return	((addr.a&0x0001) ? 0 : keystates[0]) |
			((addr.a&0x0002) ? 0 : keystates[1]) |
			((addr.a&0x0004) ? 0 : keystates[2]) |
			((addr.a&0x0008) ? 0 : keystates[3]) |
			((addr.a&0x0010) ? 0 : keystates[4]) |
			((addr.a&0x0020) ? 0 : keystates[5]) |
			((addr.a&0x0040) ? 0 : keystates[6]) |
			((addr.a&0x0080) ? 0 : keystates[7]) |
			((addr.a&0x0100) ? 0 : keystates[8]) |
			((addr.a&0x0200) ? 0 : keystates[9]) |
			((addr.a&0x0400) ? 0 : keystates[10]) |
			((addr.a&0x0800) ? 0 : keystates[11]) |
			((addr.a&0x1000) ? 0 : keystates[12]) |
			((addr.a&0x2000) ? 0 : keystates[13]);
}

void C_input::UpdateKeyboard(void)
{
	int c;
	int numkeys = 0;

	/* request platform update key array */
	UpdateKeyStates();

	/* assume no keys pressed */
	memset(keystates, 0, 15);

	/* first do a scan for shifts */
	tabnum = 0;
	c = MaxKey;
	while(c--)
	{
		if(array[c])
		{
			if(Tables[0][c].shift == 0x10)
			{
				switch(Tables[0][c].key)
				{
					case 0xd8 : tabnum |= 4; break;
					case 0xd4 : tabnum |= 2; break;
					case 0xd2 : tabnum |= 1; break;
				}
			}
			else numkeys++;
		}
	}

	/* if non-shift keys remain, do those */
	if(numkeys)
	{
		c = MaxKey;
		while(c--)
		{
			if(array[c] && Tables[0][c].shift != 0x10)
			{
				lastpressed = c;

				keystates[13] |= Tables[tabnum][c].shift;
				keystates[Tables[tabnum][c].key >> 4] |= Tables[tabnum][c].key&15;
			}
		}
	}
	else
	{
		lastpressed = -1;

		/* apply shift */
		keystates[13] |= tabnum << 1;
	}

	UpdateLEDS();

	/* fill in shifts information */
	oldshifts = shifts&(IPT_SHFT_MENU | IPT_SHFT_QUIT);
	shifts = tabnum | (keystates[14] << 3);
}

/* for calibration purposes */

void C_input::ClearKeyPressedDescriptor(void)
{
	pshift = pkey = -1;
	intable[0] = intable[1] = -1;
	noshifts = clear = allshifts = false;
}

char *C_input::AddToKeyPressedDescriptor(int id)
{
	char **ptr, *ret;

	intable[1] = intable[0];
	intable[0] = id;
	forceallshifts = false;

	switch(intable[0])
	{
		case 55 : //shift
		case 54 : //ctrl

			pkey = pshift = intable[0];

		break;

		case 53 : //func

			if(pkey == 55)
			{
				pshift = 55;
				pkey = intable[0];
			}
			else
				pshift = pkey = intable[0];

		break;

		case ADDKEY_SETALLSHIFTS : allshifts = true; noshifts = false; pshift = -1; break;
		case ADDKEY_SETCLEAR : clear = true; break;
		case ADDKEY_SETNOSHIFTS : if(!forceallshifts){noshifts = true; allshifts = false;} break;

		case 57 : //break
		case 52 : //escape
		case 7 : //delete
		case 6 : //return
		case 3 : //space bar

			forceallshifts = true;

		default :

			switch(intable[1])
			{
				case 55 : //shift
				case 54 : //ctrl
				case 53 : //func
					pshift = pkey;
					pkey = intable[0];
				break;

				default :
					pshift = -1;
					pkey = intable[0];
				break;
			}

		break;
	}

	if(forceallshifts) noshifts = false;

	switch(pshift)
	{
		case -1 : ptr = ElkKeyLists[0]; break;
		default : ptr = ElkKeyLists[56-pshift]; break;
	}

	ret = NULL;

	while(1)
	{
		if((pkey == -1) || (ret = ptr[pkey])) break;

		ptr = ElkKeyLists[0];

		if(pkey != pshift) pshift = -1;
	}

	char *t;

	ret = strdup((ret != NULL) ? ret : "");

	if(clear)
	{
		t = (char *)malloc(strlen(" [to clear]") + strlen(ret) + 1);
		sprintf(t, "%s [to clear]", ret);

		free(ret); ret = t;
	}

	if(allshifts || forceallshifts)
	{
		t = (char *)malloc(strlen(" [all shifts]") + strlen(ret) + 1);
		sprintf(t, "%s [all shifts]", ret);

		free(ret); ret = t;
	}
	else
		if(noshifts)
		{
			t = (char *)malloc(strlen(" [ignore shifts]") + strlen(ret) + 1);
			sprintf(t, "%s [ignore shifts]", ret);

			free(ret); ret = t;
		}

	return ret;
}

int *C_input::GetCurrentKeyPressedList(void)
{
	ptable[0] = pkey;
	if(pshift != pkey)
		ptable[1] = pshift;
	else
		ptable[1] = -1;
	ptable[2] = -1;

	return ptable;
}

bool C_input::GetBindDone(void)
{
	int c, tabc;

	if(!clear && pkey == -1) return false;

	/* is this a shift assignment, or a clear ? */
	if(pkey == pshift || clear)
	{
		c = MaxKey;
		while(c--)
		{
			if(array[c])
			{
				mapdirty = true;

				if(clear)
				{
					int ltab = 8;
					while(ltab--)
					{
						Tables[ltab][c].shift = 0;
						Tables[ltab][c].key = INPUT_NOKEY;
					}
				}
				else
				{
					int ltab = 8;
					while(ltab--)
					{
						Tables[ltab][c].shift = 0x10;
						Tables[ltab][c].key = ((pkey << 2) & 0xf0) | (1 << (pkey&3));
					}
				}

				ClearKeyPressedDescriptor();
				return true;
			}
		}
	}
	else
	{
		int desshift;

		switch(pshift)
		{
			default : desshift = 0; break;
			case 55 : desshift = 8; break;
			case 54 : desshift = 4; break;
			case 53 : desshift = 2; break;
		}

		/* if non-shift keys are pressed, do an assignment */
		if(lastpressed != -1)
		{
			mapdirty = true;

			if(allshifts || forceallshifts || noshifts)
			{
				for(tabc = 0; tabc < 8; tabc++)
				{
					Tables[tabc][lastpressed].key = ((pkey << 2) & 0xf0) | (1 << (pkey&3));
					Tables[tabc][lastpressed].shift = noshifts ? desshift : (tabc << 1);
				}
			}
			else
			{
				Tables[tabnum][lastpressed].key = ((pkey << 2) & 0xf0) | (1 << (pkey&3));
				Tables[tabnum][lastpressed].shift = desshift;
			}

			ClearKeyPressedDescriptor();
			return true;
		}
	}

	return false;
}

int C_input::GetKeyFlags(void)
{
	return (allshifts ? IPTFLG_ALLSHIFT : 0) | (noshifts ? IPTFLG_NOSHIFT : 0) | (clear ? IPTFLG_CLEAR : 0);
}

/* LEDs & error message */

unsigned __int8 C_input::GetShifts(void)
{
	return (pkey == -1) ? ((shifts^oldshifts)&shifts) : 0;
}

void C_input::SetCaps(bool on)
{
	caps = on;
}

void C_input::SetScroll(bool on)
{
	scroll = on;
}

void C_input::SetNum(bool on)
{
	num = on;
}

char *C_input::GetError(void)
{
	return errstring;
}

/* small parts of joystick interface */

JOY_STATE C_input::GetJoystickState(int id)
{
	JOY_STATE state;

	state = PlatformGetJoystickState(id);

	/* apply deadzone */

	if(abs(state.x) < sticks[id].dead)
	{
		state.x = 0;
	}
	else
	{
		if(state.x < 0)
			state.x = ((state.x + sticks[id].dead) << 7) / (130 - sticks[id].dead);
		else
			state.x = ((state.x - sticks[id].dead) << 7) / (130 - sticks[id].dead);
	}

	if(abs(state.y) < sticks[id].dead)
	{
		state.y = 0;
	}
	else
	{
		if(state.y < 0)
			state.y = ((state.y + sticks[id].dead) << 7) / (130 - sticks[id].dead);
		else
			state.y = ((state.y - sticks[id].dead) << 7) / (130 - sticks[id].dead);
	}

	/* adjust for expected unsigned 8bit result */

	state.x = state.x + 128;
	state.y = state.y + 128;

	return state;
}

void C_input::SetDeadZone(int id, float w)
{
	sticks[id].fldead = w;
	sticks[id].dead = (int)(w*128.0f);
}

float C_input::GetDeadZone(int id)
{
	return sticks[id].fldead;
}

int C_input::GetNumJoysticks(void)
{
	return numsticks;
}

