#ifndef INPUT_H
#define INPUT_H

#include "../types.h"
#include "../targets.h"

#ifndef C_ULA
class C_ULA;
#endif

#define EKEY_MENU EKEY_F11

#define TRG_IPT_GLOBALS
#include TRG_IPT_INCLUDE
#include TRG_PARPORT_INCLUDE
#undef TRG_IPT_GLOBALS

#define IPT_SHFT_SHIFT			0x1
#define IPT_SHFT_CTRL			0x2
#define IPT_SHFT_FUNC			0x4

#define IPT_SHFT_SPRBRK			0x8
#define IPT_SHFT_BREAK			0x10
#define IPT_SHFT_MENU			0x20
#define IPT_SHFT_QUIT			0x40

#define IPTFLG_ALLSHIFT 		1
#define IPTFLG_NOSHIFT			2
#define IPTFLG_CLEAR			4

#define ADDKEY_QUIT				59
#define ADDKEY_GUI				58
#define ADDKEY_SBREAK			56
#define ADDKEY_SETALLSHIFTS		255
#define ADDKEY_SETCLEAR			254
#define ADDKEY_SETNOSHIFTS		253

#define INPUT_NOKEY				0xf0

struct JOY_STATE
{
	bool up, down, left, right;
	int x, y;
	bool button;
};

struct JOY_INFO
{
	int dead;
	float fldead;

	bool analogue, calibrate;
#define TRG_IPT_JOYINFO_MEMBERS
#include TRG_IPT_INCLUDE
#undef TRG_IPT_JOYINFO_MEMBERS
};

class C_input
{
	public :
		/* intialisation */
		bool Setup(void);
		~C_input(void);
		C_input(C_ULA *elk);
		char *GetError(void);

		/* asynchronous PC-wise keyboard activities */
		bool UpdateKeyStates(void);
		unsigned __int8 GetShifts(void);

		/* ASCII buffer activities */
		bool NewKey(void);
		unsigned char GetPressedASCII(void);
		void ClearBuffer(void);

		/* asynchronous Electron-wise keyboard activities */
		unsigned __int8 GetKeyboard(BrokenWord addr);
		void UpdateKeyboard(void);

		/* LED setting things */
		void SetCaps(bool on);
		void SetScroll(bool on);
		void SetNum(bool on);

		/* keyboard map selection */
		bool LoadMap(char *name);
		void CloseMap(void);
		bool CreateMap(char *name);
		char *GetMapName(void);

		/* keyboard map manipulation */
		void ClearKeyPressedDescriptor(void);
		void SetKeyClear(void);
		char *AddToKeyPressedDescriptor(int id);
		int *GetCurrentKeyPressedList(void);
		bool GetBindDone(void);
		int GetKeyFlags(void);

		/* joystick information */
		int GetNumJoysticks(void);
		JOY_INFO GetJoystickInfo(int id);
		JOY_STATE GetJoystickState(int id);

		/* joystick calibration (if required) */
		void BeginCalibration(int id);
		const char *GetJoystickInstruction(void);
		void FlagJoystickDone(void);
		void SetDeadZone(int id, float w);
		float GetDeadZone(int id);

		/* parallel port */
		bool ParportFree(void);
		void ParportSend(unsigned __int8 v);
		void ParportClose(void);
		bool ParportOpen(void);

#define TRG_IPT_PUMEMBERS
#include TRG_IPT_INCLUDE
#include TRG_PARPORT_INCLUDE
#undef TRG_IPT_PUMEMBERS

	private :

		/* some platform specific private functions */
		void PlatformConstructor(void);
		void PlatformDestructor(void);
		bool PlatformSetup(void);
		int PlatformGetNumKeys(void);
		void UpdateLEDS(void);
		JOY_STATE PlatformGetJoystickState(int id);

		void ParportSetup(void);
#define TRG_IPT_PRMEMBERS
#include TRG_IPT_INCLUDE
#include TRG_PARPORT_INCLUDE
#undef TRG_IPT_PRMEMBERS

		char *errstring;
		C_ULA *electron;

		/*

		this array correlates closely to the Electron's view of the keyboard,
		but features an additional line holding ElectrEm special keys, and
		yet another line for correct 'nokey'ing

		*/
		unsigned __int8 keystates[16];

		/*

		these next six things hold the state of the keyboard LED's, as
		ElectrEm would like them to be. Not all OS's can act on this
		information

		*/
		bool num, caps, scroll;
		bool oldnum, oldcaps, oldscroll;

		/* now some bits related to the keyboard maps */
		char *mapname;
		struct SingleKey
		{
			unsigned char shift;
			unsigned char key;
		} *Tables[8];
		int MaxKey;

		bool mapdirty;

		/* and the keyboard map calibration . . . */
		int ptable[3], pshift, pkey;
		int intable[2];
		bool allshifts, forceallshifts, noshifts, clear;

		/*

		these two are filled in by UpdateKeyStates, and possibly later used
		in key calibrations - it just saves examining all the key states
		twice, and the associated code duplication issues

		*/
		int tabnum, lastpressed;

		/*

		these two are used for the rest of ElectrEm to check for the menu,
		quit, break and superbreak keys in a keyboard map friendly way

		*/
		unsigned __int8 shifts, oldshifts;

		/* now the joysticks */
		JOY_INFO *sticks;
		int numsticks;
};

#endif
