#include "input.h"
#include <zlib.h>
#include "../ula/ula.h"
#include "../fs/fs.h"
#include <string.h>
#include <malloc.h>

void C_input::CloseMap(void)
{
	if(mapdirty && mapname)
	{
		/* save new map */
		gzFile temp;
		if(temp = electron->GetFS()->GetGZ(NULL, "keybpath", mapname, "wb"))
		{
			int key, tab;

			for(tab = 0; tab < 8; tab++)
				for(key = 0; key < MaxKey; key++)
				{
					gzputc(temp, Tables[tab][key].shift);
					gzputc(temp, Tables[tab][key].key);
				}

			gzclose(temp);
		}
	}

	if(mapname)
	{
		free(mapname);
		mapname = NULL;
	}
}

bool C_input::CreateMap(char *name)
{
	/* save current file if necessary */
	CloseMap();

	/* generate new file */
	mapname = strdup(name); mapdirty = true;
	CloseMap();

	/* properly set state */
	mapdirty = false;
	mapname = strdup(name);

	return true;
}

bool C_input::LoadMap(char *name)
{
	gzFile temp;
	bool quitdef = false, guidef = false;

	CloseMap();
	mapdirty = false;

	if(temp = electron->GetFS()->GetGZ(NULL, "keybpath", name, "rb"))
	{
		mapname = strdup(name);

		int key, tab;

		for(tab = 0; tab < 8; tab++)
			for(key = 0; key < MaxKey; key++)
			{
				Tables[tab][key].shift = gzgetc(temp);
				Tables[tab][key].key = gzgetc(temp);

				if(Tables[tab][key].key == 0xe8) quitdef = true;
				if(Tables[tab][key].key == 0xe4) guidef = true;
			}

		gzclose(temp);

		/* make sure at least enter GUI & quit are defined */
		if(!quitdef)
		{
			//put quit on scroll lock as in previous versions
			Tables[0][EKEY_DEFQUIT].key = 0xe8;
			Tables[0][EKEY_DEFQUIT].shift = 0;
		}

		if(!guidef)
		{
			//put menu on F11
			Tables[0][EKEY_DEFMENU].key = 0xe4;
			Tables[0][EKEY_DEFMENU].shift = 0;
		}

		return true;
	}

	return false;
}

char *C_input::GetMapName(void)
{
	return mapname;
}

