/*

	ElectrEm (c) 2000/1 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for
	details

	plus1/print/print.cpp
	=====================

	Handles printer output, in one of these ways :

		- dumly passing on the characters (useful if in UNIX with a real
		printer and writing to the relevant /dev entry),

		-near dumly passing on the characters but interpreting carriage
		returns (useful for writing a text file in whatever OS), or

		- generating an approximate RTF file using Epson FX control codes
*/
#include "../plus1.h"
#include "../../misc/input.h"
#include "../../fs/fs.h"
#include <string.h>
#include <malloc.h>

/* top bits handle actual output */

void C_Plus1::PRNRollLeft(void)
{
	int c;
	for(c = 1; c < PRNbufpos; c++)
		PRNbuffer[c-1] = PRNbuffer[c];

	PRNbufpos--;
}

#define FX_set(x)	(!x || x == '0') ? false : true

void C_Plus1::PRNProcessToRTF(void)
{
	while(PRNbufpos)
	{
		switch(PRNbuffer[0])
		{
			default :

				fputc(PRNbuffer[0]&127, PRNout);

				/* top bit ? */

			break;

			case '\\' :	fputs("\\", PRNout);			break;

			case 0 :									break;
			case 1 :									break;
			case 2 :									break;
			case 3 :									break;
			case 4 :									break;
			case 5 :									break;
			case 6 :									break;
			case 7 :									break; // beeper
			case 8 :									break; // backspace
			case 9 :	fputs("\\tab ", PRNout);		break; // tab
			case 10 :	fputs("\\line ", PRNout);		break; // linefeed
			case 11 :									break; // vertical tab
			case 12 :	fputs("\\page ", PRNout);		break; // form feed
			case 13 :									break; // carriage return
			case 14 :									break; // double width to EOL
			case 15 :	fputs("\\scaps ", PRNout);		break; // condensed mode on
			case 16 :									break;
			case 17 :									break; // select printer
			case 18 :	fputs("\\scaps0 ", PRNout);		break; // end condensed mode
			case 19 :									break; // deselect printer
			case 20 :									break; // cancel one line double width
			case 21 :									break;
			case 22 :									break;
			case 23 :									break;
			case 24 :									break; // ignore text to EOL
			case 25 :									break;
			case 26 :									break;

			case 27 :	//ESC code

				/* don't even think about it if code may be incomplete */
				if(PRNbufpos < 4) return;

				PRNRollLeft();
				switch(PRNbuffer[0])
				{
					case 45 :	//underline mode select
						PRNRollLeft();

						if(FX_set(PRNbuffer[0]))
							fputs("\\ul ", PRNout);
						else
							fputs("\\ul0 ", PRNout);
					break;

					case 52 :	fputs("\\i ", PRNout);		break; // italics on
					case 53 :	fputs("\\i0 ", PRNout);		break; // italics off
					case 69 :	fputs("\\b ", PRNout);		break; // emphasised on
					case 70 :	fputs("\\b0 ", PRNout);		break; // emphasised off
					case 71 :	fputs("\\outl ", PRNout);	break; // double-strike on
					case 72 :	fputs("\\outl0 ", PRNout);	break; // double-strike off

					case 112 :	//proportional font
						PRNRollLeft();

						if(FX_set(PRNbuffer[0]))
							fputs("\\f1 ", PRNout);
						else
							fputs("\\f0 ", PRNout);
					break;

				}

			break;

			case 28 :									break;
			case 29 :									break;
			case 30 :									break;
			case 31 :									break;
		}
		PRNRollLeft();
	}
}

void C_Plus1::PRNSendChar(char v)
{
	switch(PRNMode)
	{
		case PRNMODE_PARPORT :
			ipt->ParportSend(v);
		break;

		case PRNMODE_RTF :
			if(PRNout)
			{
				PRNbuffer[PRNbufpos] = v;
				PRNbufpos++;

				if(PRNbufpos == 128) PRNProcessToRTF();
			}
		break;

		case PRNMODE_ASCII :
			if(PRNout)
			{
				fputc(v, PRNout);
				fflush(PRNout);
			}
		break;

		case PRNMODE_ASCIILF :
			if(PRNout)
			{
				switch(v)
				{
					case '\r' :								break;
					default :	fputc(v, PRNout);			break;
				}
			}
		break;
	}
}

/* now the state setting bits */

/* assumptions : setfile comes, then, if valid, set mode */

bool C_Plus1::PRNSetFile(char *name)
{
	PRNClose();
	PRNfname = strdup(name);

	switch(PRNMode)
	{
		default : break;

		case PRNMODE_ASCII :
			PRNout = fs->GetFile("", NULL, name, "wb");
		break;

		case PRNMODE_ASCIILF :
			PRNout = fs->GetFile("", NULL, name, "wt");
		break;

		case PRNMODE_RTF :
			PRNout = fs->GetFile("", NULL, name, "wt");
			fputs("{\\rtf1{\\fonttbl\\f0\\fmodern Courier;}{\\fonttbl\\f1\\froman Tms Rmn;}\\f0\\fs20 ", PRNout);
			PRNbufpos = 0;
		break;
	}

	return true;
}


void C_Plus1::PRNSetMode(int mde)
{
	char *t;

	if(PRNMode != mde)
	{
		PRNMode = mde;

		switch(PRNMode)
		{
			case PRNMODE_OFF :
				PRNClose();
			break;

			case PRNMODE_PARPORT :
				PRNClose();
				ipt->ParportOpen();
			break;

			default :
				PRNSetFile(t = strdup(PRNfname));
				free(t);
			break;
		}
	}
}

void C_Plus1::PRNClose(void)
{
	if(PRNout)
	{
		switch(PRNMode)
		{
			case PRNMODE_ASCII :
			case PRNMODE_ASCIILF :
			break;

			case PRNMODE_RTF :
				PRNProcessToRTF();
				fprintf(PRNout, "}");
			break;
		}

		fs->ReleaseFile(PRNout);
		PRNout = NULL;
	}
	else
	{
		switch(PRNMode)
		{
			case PRNMODE_PARPORT :
				ipt->ParportClose();
			break;
		}
	}

	if(PRNfname)
	{
		free(PRNfname);
		PRNfname = NULL;
	}
}

unsigned __int8 C_Plus1::PRNGetStatus(int time)
{
	/* top bit set if unable to accept a character */

	return (PRNMode != PRNMODE_PARPORT) ? 0x00 : (ipt->ParportFree() ? 0x00 : 0x80);
}

int C_Plus1::PRNGetMode(void)
{
	return PRNMode;
}

int C_Plus1::PRNGetCurrentMode(void)
{
	return PRNMode;
}

