/*

	ElectrEm (c) 2000 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for
	details

	screen/regular/big.cpp
	======================

	Draws Electron pixel & blank scanlines to a 640x512 bitmap.
	When Electron is at 320x256, 1 pixel mapped to 2x2 pixels.

*/
#include "../screen.h"
#include "../../gfx/gfx.h"
#include "../../ula/ula.h"
#include "big.h"
#include "rstd.h"
#include <string.h>

void C_ScPelsBig::DrawPelLine(int line)
{
	BrokenWord PelPtr;
	int x;
	unsigned __int64 *vptr, *vptr2;
	ConvElement *e;

	if(!locked) return;

	vptr = (unsigned __int64 *)img.GetDLinePtr(line << 1);
	vptr2 = (unsigned __int64 *)((unsigned __int8 *)vptr + img.pitch);
	PelPtr.a = LinePtr;

	if(CTable->wide)
	{
		x = 80;
		while(x--)
		{
			*vptr2++ = *vptr++ = *((unsigned __int64 *)&Table[MemPtr[PelPtr.a]].low);
			PelPtr.a = MemMoves[PelPtr.a];
		}
	}
	else
	{
		x = 40;
		while(x--)
		{
			e = &Table[MemPtr[PelPtr.a]];
			*vptr2++ = *vptr++ = *((unsigned __int64 *)&e->low);
			*vptr2++ = *vptr++ = *((unsigned __int64 *)&e->high);
			PelPtr.a = MemMoves[PelPtr.a];
		}
	}

	img.ReleaseLinePtr();
	EndOfLine();
}

void C_ScPelsBig::DrawMultiLine(int line, int *changes, int changecount, int *pchanges, int pchangecount)
{
	BrokenWord PelPtr;
	int x, t;
	unsigned __int64 *vptr, *vptr2;
	ConvElement *e;

	if(locked)
	{
		vptr = (unsigned __int64 *)img.GetDLinePtr(line<<1);
		vptr2 = (unsigned __int64 *)((unsigned __int8 *)vptr + img.pitch);
		PelPtr.a = LinePtr;

		t = PEL_START;

		x = 80;
		while(x)
		{
			if(!(x&1))
			{
				while((t >= *changes) && changecount)
				{
					changes++;

					SetMode(*changes, line);

					changes++;
					changecount--;
				}

				while((t >= *pchanges) && pchangecount)
				{
					scr->AffectPalEntry(pchanges);
					pchanges += 3;
					pchangecount--;
				}
			}

			if(CTable->wide)
			{
				*vptr2++ = *vptr++ = *((unsigned __int64 *)&Table[MemPtr[PelPtr.a]].low);

				x--;
				t++;
			}
			else
			{
				e = &Table[MemPtr[PelPtr.a]];
				*vptr2++ = *vptr++ = *((unsigned __int64 *)&e->low);
				*vptr2++ = *vptr++ = *((unsigned __int64 *)&e->high);

				x -= 2;
				t += 2;
			}

			PelPtr.a = MemMoves[PelPtr.a];
		}

		img.ReleaseLinePtr();
		EndOfLine();
	}

	if(changecount)
	{
		changes += 1+((changecount-1) << 1);
		SetMode(*changes, line);
	}

	while(pchangecount--)
	{
		scr->AffectPalEntry(pchanges);
		pchanges += 3;
	}
}

void C_ScPelsBig::DrawBlankLine(int line, unsigned __int8 col0)
{
	unsigned __int32 *vptr;

	if(!locked) return;

	vptr = (unsigned __int32 *)img.GetLinePtr(line << 1);
	memset(vptr, col0, 640);
	img.ReleaseLinePtr();

	vptr = (unsigned __int32 *)img.GetLinePtr(1|(line<<1));
	memset(vptr, col0, 640);
	img.ReleaseLinePtr();

	EndOfBlankLine();
}

bool C_ScPelsBig::Setup(C_ULA *electron, DSPSTRUCT screen, ModeTable *tabs)
{
	bpp = 4;
	img.Create(electron->GetGFX(), 640, 512, screen, PAL_ELK);

	StdSetup();

	return true;
}

