/*

	ElectrEm (c) 2000/1 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for
	details

	screen/regular/pal.cpp
	======================

	messes with the palette tables used by the regular graphics pixel
	display

*/
#include "pal.h"
#include <stdio.h>

#define ror(x)		x = (x >> 1) | (x << 7)

void C_ScreenPalModifierNormal::SetPalette(unsigned __int8 addr, unsigned __int8 value)
{
	vals[tab][addr-8] = set[addr-8] = value;

	if(sync)
	{
		if(addr <= 0x9)
			SyncPalette();
	}
	else
	{
		int c;
		unsigned __int8 cid;
		BrokenQuad *col0, *col1, *col2, *col3;
		unsigned int or0, or1, or2, or3;

		switch(addr)
		{
			case 0x8 :
			case 0x9 :
				col0 = &WCols[0];	col1 = &WCols[2];	col2 = &WCols[8];	col3 = &WCols[10];
				or0 = 0x00;		or1 = 0x08;		or2 = 0x80;		or3 = 0x88;
			break;

			case 0xa :
			case 0xb :
				col0 = &WCols[4];	col1 = &WCols[6];	col2 = &WCols[12];	col3 = &WCols[14];
				or0 = 0x20;			or1 = 0x28;			or2 = 0xa0;			or3 = 0xa8;
			break;

			case 0xc :
			case 0xd :
				col0 = &WCols[5];	col1 = &WCols[7];	col2 = &WCols[13];	col3 = &WCols[15];
				or0 = 0x22;		or1 = 0x2a;		or2 = 0xa2;		or3 = 0xaa;
			break;

			case 0xe :
			case 0xf :
				col0 = &WCols[1];	col1 = &WCols[3];	col2 = &WCols[9];	col3 = &WCols[11];
				or0 = 0x02;		or1 = 0x0a;		or2 = 0x82;		or3 = 0x8a;
			break;
		}

		switch(tab)
		{
			case Tab4B : //4bpp, 640x256 display, 80 pitch display mode [mode 2]
 				c = 16;
				while(c--)
				{
					cid = (c&5) | ((c&10) << 3);

					ColsTab4B[cid | or0].low.l = *col0;
					ColsTab4B[cid | or1].low.l = *col1;
					ColsTab4B[cid | or2].low.l = *col2;
					ColsTab4B[cid | or3].low.l = *col3;

					ror(cid);

					ColsTab4B[cid | (or0>>1)].low.h = *col0;
					ColsTab4B[cid | (or1>>1)].low.h = *col1;
					ColsTab4B[cid | (or2>>1)].low.h = *col2;
					ColsTab4B[cid | (or3>>1)].low.h = *col3;
				}
			break;

			case Tab4N : //4bpp, 320x256 display, 80 pitch display mode [mode 2]
				c = 16;
				while(c--)
				{
					cid = (c&5) | ((c&10) << 3);

					ColsTab4N[cid | or0].low.l.w.w0 = col0->w.w0;
					ColsTab4N[cid | or1].low.l.w.w0 = col1->w.w0;
					ColsTab4N[cid | or2].low.l.w.w0 = col2->w.w0;
					ColsTab4N[cid | or3].low.l.w.w0 = col3->w.w0;

					ror(cid);

					ColsTab4N[cid | (or0>>1)].low.l.w.w1 = col0->w.w0;
					ColsTab4N[cid | (or1>>1)].low.l.w.w1 = col1->w.w0;
					ColsTab4N[cid | (or2>>1)].low.l.w.w1 = col2->w.w0;
					ColsTab4N[cid | (or3>>1)].low.l.w.w1 = col3->w.w0;
				}
			break;
		}
	}
}

void C_ScreenPalModifierNormal::SyncPalette(void)
{
	int c;
	unsigned __int8 cid;

	c = 8;
	while(c--)
		vals[tab][c] = set[c];

	switch(tab)
	{
		case Tab1B :

			c = 128;
			while(c--)
			{
				cid = c;
				ColsTab1B[cid].low.l.b.b0			= WCols[0].b.b0;
				ColsTab1B[cid | 0x80].low.l.b.b0	= WCols[8].b.b0;

				ror(cid);
				ColsTab1B[cid].low.l.b.b1			= WCols[0].b.b0;
				ColsTab1B[cid | 0x40].low.l.b.b1	= WCols[8].b.b0;

				ror(cid);
				ColsTab1B[cid].low.l.b.b2			= WCols[0].b.b0;
				ColsTab1B[cid | 0x20].low.l.b.b2	= WCols[8].b.b0;

				ror(cid);
				ColsTab1B[cid].low.l.b.b3			= WCols[0].b.b0;
				ColsTab1B[cid | 0x10].low.l.b.b3	= WCols[8].b.b0;

				ror(cid);
				ColsTab1B[cid].low.h.b.b0			= WCols[0].b.b0;
				ColsTab1B[cid | 0x08].low.h.b.b0	= WCols[8].b.b0;

				ror(cid);
				ColsTab1B[cid].low.h.b.b1			= WCols[0].b.b0;
				ColsTab1B[cid | 0x04].low.h.b.b1	= WCols[8].b.b0;

				ror(cid);
				ColsTab1B[cid].low.h.b.b2			= WCols[0].b.b0;
				ColsTab1B[cid | 0x02].low.h.b.b2	= WCols[8].b.b0;

				ror(cid);
				ColsTab1B[cid].low.h.b.b3			= WCols[0].b.b0;
				ColsTab1B[cid | 0x01].low.h.b.b3	= WCols[8].b.b0;
			}

		break;

		case Tab1W :

			c = 128;
			while(c--)
			{
				cid = c;
				ColsTab1W[cid].low.l.w.w0			= WCols[0].w.w0;
				ColsTab1W[cid | 0x80].low.l.w.w0	= WCols[8].w.w0;

				ror(cid);
				ColsTab1W[cid].low.l.w.w1			= WCols[0].w.w0;
				ColsTab1W[cid | 0x40].low.l.w.w1	= WCols[8].w.w0;

				ror(cid);
				ColsTab1W[cid].low.h.w.w0			= WCols[0].w.w0;
				ColsTab1W[cid | 0x20].low.h.w.w0	= WCols[8].w.w0;

				ror(cid);
				ColsTab1W[cid].low.h.w.w1			= WCols[0].w.w0;
				ColsTab1W[cid | 0x10].low.h.w.w1	= WCols[8].w.w0;

				ror(cid);
				ColsTab1W[cid].high.l.w.w0			= WCols[0].w.w0;
				ColsTab1W[cid | 0x08].high.l.w.w0	= WCols[8].w.w0;

				ror(cid);
				ColsTab1W[cid].high.l.w.w1			= WCols[0].w.w0;
				ColsTab1W[cid | 0x04].high.l.w.w1	= WCols[8].w.w0;

				ror(cid);
				ColsTab1W[cid].high.h.w.w0			= WCols[0].w.w0;
				ColsTab1W[cid | 0x02].high.h.w.w0	= WCols[8].w.w0;

				ror(cid);
				ColsTab1W[cid].high.h.w.w1			= WCols[0].w.w0;
				ColsTab1W[cid | 0x01].high.h.w.w1	= WCols[8].w.w0;
			}

		break;

		case Tab1N :

			c = 256;
			while(c--)
			{
				ColsTab1N[c].low.l.b.b0 = (WCols[(c&0x80) >> 4].b.b0&7) | (WCols[(c&0x40) >> 3].b.b0 << 3);
				ColsTab1N[c].low.l.b.b1 = (WCols[(c&0x20) >> 2].b.b0&7) | (WCols[(c&0x10) >> 1].b.b0 << 3);
				ColsTab1N[c].low.l.b.b2 = (WCols[(c&0x08) >> 0].b.b0&7) | (WCols[(c&0x04) << 1].b.b0 << 3);
				ColsTab1N[c].low.l.b.b3 = (WCols[(c&0x02) << 2].b.b0&7) | (WCols[(c&0x01) << 3].b.b0 << 3);
			}

		break;

		case Tab2B :

			c = 256;
			while(c--)
			{
				ColsTab2B[c].low.l.w.w0 = WCols[((c&0x80) >> 4) | ((c&0x08) >> 2)].w.w0;
				ColsTab2B[c].low.l.w.w1 = WCols[((c&0x40) >> 3) | ((c&0x04) >> 1)].w.w0;
				ColsTab2B[c].low.h.w.w0 = WCols[((c&0x20) >> 2) | ((c&0x02) >> 0)].w.w0;
				ColsTab2B[c].low.h.w.w1 = WCols[((c&0x10) >> 1) | ((c&0x01) << 1)].w.w0;
			}

		break;

		case Tab2W :

			c = 256;
			while(c--)
			{
				ColsTab2W[c].low.l = WCols[((c&0x80) >> 4) | ((c&0x08) >> 2)];
				ColsTab2W[c].low.h = WCols[((c&0x40) >> 3) | ((c&0x04) >> 1)];
				ColsTab2W[c].high.l = WCols[((c&0x20) >> 2) | ((c&0x02) >> 0)];
				ColsTab2W[c].high.h = WCols[((c&0x10) >> 1) | ((c&0x01) << 1)];
			}

		break;

		case Tab2N :

			c = 256;
			while(c--)
			{
				ColsTab2N[c].low.l.b.b0 = WCols[((c&0x80) >> 4) | ((c&0x08) >> 2)].b.b0;
				ColsTab2N[c].low.l.b.b1 = WCols[((c&0x40) >> 3) | ((c&0x04) >> 1)].b.b0;
				ColsTab2N[c].low.l.b.b2 = WCols[((c&0x20) >> 2) | ((c&0x02) >> 0)].b.b0;
				ColsTab2N[c].low.l.b.b3 = WCols[((c&0x10) >> 1) | ((c&0x01) << 1)].b.b0;
			}

		break;

		case Tab4N :

			c = 256;
			while(c--)
			{
				ColsTab4N[c].low.l.w.w0 = WCols[((c&0x80) >> 4) | ((c&0x20) >> 3) | ((c&0x08) >> 2) | ((c&0x02) >> 1)].w.w0;
				ColsTab4N[c].low.l.w.w1 = WCols[((c&0x40) >> 3) | ((c&0x10) >> 2) | ((c&0x04) >> 1) | ((c&0x01) >> 0)].w.w0;
			}
		break;

		case Tab4B :

			c = 256;
			while(c--)
			{
				ColsTab4B[c].low.l = WCols[((c&0x80) >> 4) | ((c&0x20) >> 3) | ((c&0x08) >> 2) | ((c&0x02) >> 1)];
				ColsTab4B[c].low.h = WCols[((c&0x40) >> 3) | ((c&0x10) >> 2) | ((c&0x04) >> 1) | ((c&0x01) >> 0)];
			}

		break;
	}
}

#undef ror

#define BPP1		1
#define BPP2		2
#define BPP4		3
#define BPPFIELD	3

#define HIGHRES		4
#define LOWRES		0

#define COL80		8
#define COL40		0

void *C_ScreenPalModifierNormal::SetMode(int mode)
{
	sync = true;
	switch(mode)
	{
		case BPP1 | LOWRES | COL40 :
		case BPP1 | HIGHRES | COL80 :
			tab = Tab1B;
		break;

		case BPP1 | HIGHRES | COL40 :
			tab = Tab1W;
		break;

		case BPP1 | LOWRES | COL80 :
			tab = Tab1N;
		break;


		case BPP2 | LOWRES | COL40 :
		case BPP2 | HIGHRES | COL80 :
			tab = Tab2B;
		break;

		case BPP2 | HIGHRES | COL40 :
			tab = Tab2W;
		break;

		case BPP2 | LOWRES | COL80 :
			tab = Tab2N;
		break;


		case BPP4 | HIGHRES | COL80 :
			tab = Tab4B;
			sync = false;
		break;

		case BPP4 | LOWRES | COL80 :
			tab = Tab4N;
			sync = false;
		break;
	}

	int c;

	c = ((mode&BPPFIELD) == BPP4) ? 8 : 2;
	while(c--)
	{
		if(set[c] != vals[tab][c])
		{
			SyncPalette();
			break;
		}
	}

	switch(tab)
	{
		case Tab1B : return (void *)ColsTab1B;
		case Tab1W : return (void *)ColsTab1W;
		case Tab1N : return (void *)ColsTab1N;

		case Tab2B : return (void *)ColsTab2B;
		case Tab2W : return (void *)ColsTab2W;
		case Tab2N : return (void *)ColsTab2N;

		case Tab4B : return (void *)ColsTab4B;
		case Tab4N : return (void *)ColsTab4N;
	}

	return NULL;
}

void C_ScreenPalModifierNormal::Disable(void)
{
	delete ColsTab1B;
	delete ColsTab1W;
	delete ColsTab1N;

	delete ColsTab2B;
	delete ColsTab2W;
	delete ColsTab2N;

	delete ColsTab4B;
	delete ColsTab4N;
}

void C_ScreenPalModifierNormal::Enable(BrokenQuad *cols)
{
	int c, c2;

	WCols = cols;
	sync = true;
	tab = TabNone;

	ColsTab1B = new ConvElement[256];
	ColsTab1W = new ConvElement[256];
	ColsTab1N = new ConvElement[256];

	ColsTab2B = new ConvElement[256];
	ColsTab2W = new ConvElement[256];
	ColsTab2N = new ConvElement[256];

	ColsTab4B = new ConvElement[256];
	ColsTab4N = new ConvElement[256];

	/* assume a valid initial state */

	c = 256;
	while(c--)
	{
		ColsTab1B[c].high.l.a = ColsTab1W[c].high.l.a = ColsTab1N[c].high.l.a =
		ColsTab1B[c].high.h.a = ColsTab1W[c].high.h.a = ColsTab1N[c].high.h.a =
		ColsTab1B[c].low.l.a = ColsTab1W[c].low.l.a = ColsTab1N[c].low.l.a =
		ColsTab1B[c].low.h.a = ColsTab1W[c].low.h.a = ColsTab1N[c].low.h.a =

		ColsTab2B[c].high.l.a = ColsTab2W[c].high.l.a = ColsTab2N[c].high.l.a =
		ColsTab2B[c].high.h.a = ColsTab2W[c].high.h.a = ColsTab2N[c].high.h.a =
		ColsTab2B[c].low.l.a = ColsTab2W[c].low.l.a = ColsTab2N[c].low.l.a =
		ColsTab2B[c].low.h.a = ColsTab2W[c].low.h.a = ColsTab2N[c].low.h.a =

		ColsTab4B[c].high.l.a = ColsTab4N[c].high.l.a =
		ColsTab4B[c].high.h.a = ColsTab4N[c].high.h.a =
		ColsTab4B[c].low.l.a = ColsTab4N[c].low.l.a =
		ColsTab4B[c].low.h.a = ColsTab4N[c].low.h.a = 0;
	}

	c = 8;
	while(c--)
	{
		set[c] = 0xff;

		c2 = TabNone;
		while(c2--)
			vals[c2][c] = 0xff;
	}
}
#undef ror

