/*

	ElectrEm (c) 2000 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for
	details

	screen/small.cpp
	================

	Draws Electron pixel & blank scanlines to a 320x256 bitmap.
	When Electron is at 320x256, 1 pixel mapped to 1x1 pixels.

*/
#include "../screen.h"
#include "../../gfx/gfx.h"
#include "../../ula/ula.h"
#include "small.h"
#include "rstd.h"
#include <string.h>

void C_ScPelsSmall::DrawPelLine(int line)
{
	int x;
	BrokenWord PelPtr;
	unsigned __int32 *vptr;
	unsigned __int64 *vptr64;

	if(!locked) return;

	vptr64 = (unsigned __int64 *)(vptr = (unsigned __int32 *)img.GetLinePtr(line));
	PelPtr.a = LinePtr;

	if(CTable->wide)
	{
		x = 80;
		while(x--)
		{
			*vptr++ = Table[MemPtr[PelPtr.a]].low.l.a;
			PelPtr.a = MemMoves[PelPtr.a];
		}
	}
	else
	{
		x = 40;
		while(x--)
		{
			*vptr64++ = *((unsigned __int64 *)&Table[MemPtr[PelPtr.a]].low);
			PelPtr.a = MemMoves[PelPtr.a];
		}
	}

	img.ReleaseLinePtr();
	EndOfLine();
}

void C_ScPelsSmall::DrawMultiLine(int line, int *changes, int changecount, int *pchanges, int pchangecount)
{
	BrokenWord PelPtr;
	int x, t;
	unsigned __int32 *vptr;
	unsigned __int64 *vptr64;

	if(locked)
	{
		vptr64 = (unsigned __int64 *)(vptr = (unsigned __int32 *)img.GetLinePtr(line));
		PelPtr.a = LinePtr;

		t = PEL_START;

		x = 80;
		while(x)
		{
			if(!(x&1))
			{
				while((t >= *changes) && changecount)
				{
					changes++;

					SetMode(*changes, line);

					changes++;
					changecount--;
				}

				while((t >= *pchanges) && pchangecount)
				{
					scr->AffectPalEntry(pchanges);
					pchanges += 3;
					pchangecount--;
				}
			}

			if(CTable->wide)
			{
				*vptr++ = Table[MemPtr[PelPtr.a]].low.l.a;
				x--;
				t++;
			}
			else
			{
				*vptr64++ = *((unsigned __int64 *)&Table[MemPtr[PelPtr.a]].low);
				x -=2;
				t += 2;
			}

			PelPtr.a = MemMoves[PelPtr.a];
		}

		img.ReleaseLinePtr();
		EndOfLine();
	}

	if(changecount)
	{
		changes += 1+((changecount-1) << 1);
		SetMode(*changes, line);
	}

	while(pchangecount--)
	{
		scr->AffectPalEntry(pchanges);
		pchanges += 3;
	}
}

void C_ScPelsSmall::DrawBlankLine(int line, unsigned __int8 col0)
{
	unsigned __int32 *vptr;

	if(!locked) return;

	vptr = (unsigned __int32 *)img.GetLinePtr(line);
	memset(vptr, col0, 320);
	img.ReleaseLinePtr();

	EndOfBlankLine();
}

bool C_ScPelsSmall::Setup(C_ULA *electron, DSPSTRUCT screen, ModeTable *tabs)
{
	bpp = 0;
	img.Create(electron->GetGFX(), 320, 256, screen, PAL_ELK);

	StdSetup();

	return true;
}

