#ifndef ELKSCREEN_H
#define ELKSCREEN_H

#include "../types.h"
#include "../gfx/gfx.h"

#ifndef C_6502Base
class C_6502Base;
#endif
#ifndef C_ULA
class C_ULA;
#endif
#ifndef C_ScreenPels
class C_ScreenPels;
#endif
#ifndef C_ScreenPalModifier
class C_ScreenPalModifier;
#endif

class C_screen
{
	public :
		C_screen(C_gfx_api *api);
		~C_screen(void);

		void Kill(void);

		static int Keepalive(int time);
		void ResetCPU(void);

		bool Setup(C_ULA *electron, bool vidmode, bool cleanscale, int scalelevel, bool multiscr);
		void Reset(void);

		void SetPalette(unsigned __int8 addr, unsigned __int8 val, int time);
		void ChangeGFXMode(unsigned __int8 newmode, int time);

		void SaveImage(char *name);
		void SetSize(bool big, int scalelevel, bool scaletype);

		bool ValidMultiMode(unsigned __int8 mode);
		bool SetMultiMode(bool v, unsigned __int8 mult);
		bool GetMultiEnabled(void);

		void AlertToInterrupt(int time);

		bool GetSize(void);
		C_bitmap *GetBitmap(void);

		char *GetError(void);

		void MinimiseTimer(int sub);
		C_ScreenPalModifier *GetPal(void);

		void AffectPalEntry(int *palchange);

		static BrokenWord VidAddr;
		bool wide;

	private :
		static C_6502Base *cpu;

		void ResetMultiMode(void);
		static C_ScreenPels *peldraw;
		static C_ScreenPalModifier *palette;

		static int line;

		static int ModeChangeCount;
		static int ModeChanges[128], *ModeChangeCursor;
		static int PalChangeCount;
		static int PalChanges[384], *PalChangeCursor;
		static int PalCols[8], PalOldCols[8];

		static int TimeBasis;

		static C_ULA *ula;
		C_gfx_api *gfxptr;

		static unsigned __int8 Mode;
		static BrokenQuad WCols[16];

		DSPSTRUCT rsmall, rbig, rlong;

		char *errstring;

		unsigned __int8 multiMode, multiPresent;
		bool multiEnable;

		static bool blank;
		static int coltab, modecols;
};

#endif