/*

	ElectrEm (c) 2000 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for
	details

	screen/wide/wbig.cpp
	====================

	Draws multiplexed Electron pixel & blank scanlines to a 640x512 bitmap.
	When Electron is at 320x256, 1 pixel mapped to 2x2 pixels.

*/
#include "../screen.h"
#include "../../gfx/gfx.h"
#include "../../ula/ula.h"
#include "big1001.h"
#include "wstd.h"
#include <string.h>

void C_ScPelsBig1001::DrawPelLine(int line)
{
	BrokenWord PelPtr;
	int x;
	unsigned __int32 *vptr, *vptr2;
	BrokenQuad qcol;

	if(!locked) return;

	vptr = (unsigned __int32 *)img.GetDLinePtr(line<<1);
	vptr2 = vptr + (img.pitch >> 2);
	PelPtr.a = LinePtr;

	if(CTable->wide)
	{
		x = 80;
		switch(CTable->bpp&3)
		{
			case 1 :
				while(x--)
				{
					qcol.a = MemPtr[PelPtr.a];
					PelPtr.a += 8;
					if(PelPtr.a&32768) PelPtr.a -= CTable->pitch;

					*vptr2++ = *vptr++ = Table[qcol.w.w1.a].QTable1.a;
					*vptr2++ = *vptr++ = Table[qcol.w.w0.a].QTable1.a;
				}
			break;

			case 2 :
				while(x--)
				{
					qcol.a = MemPtr[PelPtr.a];
					PelPtr.a += 8;
					if(PelPtr.a&32768) PelPtr.a -= CTable->pitch;

					*vptr2++ = *vptr++ = Lines[qcol.b.b3][qcol.b.b1].QTable1.a;
					*vptr2++ = *vptr++ = Lines[qcol.b.b2][qcol.b.b0].QTable1.a;
				}
			break;

			case 3 :
				while(x--)
				{
					qcol.a = MemPtr[PelPtr.a];
					PelPtr.a += 8;
					if(PelPtr.a&32768) PelPtr.a -= CTable->pitch;

					*vptr2++ = *vptr++ = Table[qcol.w.w0.a].QTable1.a;
					*vptr2++ = *vptr++ = Table[qcol.w.w0.a].QTable2.a;
				}
			break;
		}
	}
	else
	{
		x = 40;
		switch(CTable->bpp&3)
		{
			case 1 :
				while(x--)
				{
					qcol.a = MemPtr[PelPtr.a];
					PelPtr.a += 8;
					if(PelPtr.a&32768) PelPtr.a -= CTable->pitch;

					*vptr2++ = *vptr++ = Table[qcol.w.w1.a].QTable1.a;
					*vptr2++ = *vptr++ = Table[qcol.w.w1.a].QTable2.a;
					*vptr2++ = *vptr++ = Table[qcol.w.w0.a].QTable1.a;
					*vptr2++ = *vptr++ = Table[qcol.w.w0.a].QTable2.a;
				}
			break;

			case 2 :
				while(x--)
				{
					qcol.a = MemPtr[PelPtr.a];
					PelPtr.a += 8;
					if(PelPtr.a&32768) PelPtr.a -= CTable->pitch;

					*vptr2++ = *vptr++ = Lines[qcol.b.b3][qcol.b.b1].QTable1.a;
					*vptr2++ = *vptr++ = Lines[qcol.b.b3][qcol.b.b1].QTable2.a;
					*vptr2++ = *vptr++ = Lines[qcol.b.b2][qcol.b.b0].QTable1.a;
					*vptr2++ = *vptr++ = Lines[qcol.b.b2][qcol.b.b0].QTable2.a;
				}
			break;
		}
	}

	img.ReleaseDLinePtr();
	EndOfLine();
}

void C_ScPelsBig1001::DrawBlankLine(int line, unsigned __int8 col0)
{
	unsigned __int32 *vptr;

	if(!locked) return;

	vptr = (unsigned __int32 *)img.GetLinePtr(line << 1);
	memset(vptr, col0, 640);
	img.ReleaseLinePtr();

	vptr = (unsigned __int32 *)img.GetLinePtr(1|(line<<1));
	memset(vptr, col0, 640);
	img.ReleaseLinePtr();
}

bool C_ScPelsBig1001::Setup(C_ULA *electron, DSPSTRUCT screen, ModeTable *tabs)
{
	bpp = 4;
	img.Create(electron->GetGFX(), 640, 512, screen, PAL_ELK);

	StdSetup();

	return true;
}
