#include "pal1001.h"
#include "../../types.h"

void C_ScreenPalModifier1001::SetPalette(unsigned __int8 addr, unsigned __int8 v)
{
	int c;
	int cid;
	BrokenQuad *col0, *col1, *col2, *col3;
	unsigned int or0, or1, or2, or3;

	if(bpp != -1)
	{
		//bits 0,1	: 1 = 1bpp, 2 = 2bpp, 3 = 4bpp
		//bit 2		: 1 = 640 width, 0 = 320 width
		//bit 3		: 1 = 80 byte pitch, 0 = 40
		//bit 4		: 1 = wide bpp, 0 = 1001 bpp

		if((bpp&3) == 3)
		{
			switch(addr)
			{
				case 0x8 :
				case 0x9 :
					col0 = &WCols[0];	col1 = &WCols[2];	col2 = &WCols[8];	col3 = &WCols[10];
					or0 = 0x00;		or1 = 0x02;		or2 = 0x08;		or3 = 0x0a;
				break;

				case 0xa :
				case 0xb :
					col0 = &WCols[4];	col1 = &WCols[6];	col2 = &WCols[12];	col3 = &WCols[14];
					or0 = 0x04;			or1 = 0x06;			or2 = 0x0c;			or3 = 0x0e;
				break;

				case 0xc :
				case 0xd :
					col0 = &WCols[5];	col1 = &WCols[7];	col2 = &WCols[13];	col3 = &WCols[15];
					or0 = 0x05;		or1 = 0x07;		or2 = 0x0d;		or3 = 0x0f;
				break;

				case 0xe :
				case 0xf :
					col0 = &WCols[1];	col1 = &WCols[3];	col2 = &WCols[9];	col3 = &WCols[11];
					or0 = 0x01;		or1 = 0x03;		or2 = 0x09;		or3 = 0x0b;
				break;
			}
		}

		switch(bpp)
		{
			case 0x1 : //1bpp, 320x256 display, 40 pitch display mode [modes 4 & 6]
			case (0x1 | 0x4 | 0x8) : //1bpp, 640x256 display, 80 pitch display mode [modes 0 & 3]
				if(addr <= 0x9)
				{
					c = 4096;
					while(c--)
					{
						cid = c;
						CArray.Table[cid | 0x0000].QTable1.b.b0 = WCols[0].b.b0;
						CArray.Table[cid | 0x1000].QTable1.b.b0 = WCols[8].b.b0;

						cid = ((cid >> 4) | (cid << 12)) & 65535;
						CArray.Table[cid | 0x0000].QTable1.b.b1 = WCols[0].b.b0;
						CArray.Table[cid | 0x0100].QTable1.b.b1 = WCols[8].b.b0;

						cid = ((cid >> 4) | (cid << 12)) & 65535;
						CArray.Table[cid | 0x0000].QTable1.b.b2 = WCols[0].b.b0;
						CArray.Table[cid | 0x0010].QTable1.b.b2 = WCols[8].b.b0;

						cid = ((cid >> 4) | (cid << 12)) & 65535;
						CArray.Table[cid | 0x0000].QTable1.b.b3 = WCols[0].b.b0;
						CArray.Table[cid | 0x0001].QTable1.b.b3 = WCols[8].b.b0;
					}
				}
			break;

			case (0x1 | 0x4) : //1bpp, 640x256 display, 40 pitch display mode [modes 4 & 6]
				if(addr <= 0x9)
				{
					c = 4096;
					while(c--)
					{
						cid = c;
						CArray.Table[cid | 0x0000].QTable1.w.w0 = WCols[0].w.w0;
						CArray.Table[cid | 0x1000].QTable1.w.w0 = WCols[8].w.w0;

						cid = ((cid >> 4) | (cid << 12)) & 65535;
						CArray.Table[cid | 0x0000].QTable1.w.w1 = WCols[0].w.w0;
						CArray.Table[cid | 0x0100].QTable1.w.w1 = WCols[8].w.w0;

						cid = ((cid >> 4) | (cid << 12)) & 65535;
						CArray.Table[cid | 0x0000].QTable2.w.w0 = WCols[0].w.w0;
						CArray.Table[cid | 0x0010].QTable2.w.w0 = WCols[8].w.w0;

						cid = ((cid >> 4) | (cid << 12)) & 65535;
						CArray.Table[cid | 0x0000].QTable2.w.w1 = WCols[0].w.w0;
						CArray.Table[cid | 0x0001].QTable2.w.w1 = WCols[8].w.w0;
					}
				}
			break;

			case (0x1 | 0x8) : //1bpp, 320x256 display, 80 pitch display mode [modes 0 & 3]
				if(addr <= 0x9)
				{
					c = 4096;
					while(c--)
					{
						cid = c;
						CArray.Table[cid | 0x0000].QTable1.b.b0 = WCols[0].b.b0;
						CArray.Table[cid | 0x1000].QTable1.b.b0 = WCols[8].b.b0;

						cid = ((cid >> 8) | (cid << 8)) & 65535;
						CArray.Table[cid | 0x0000].QTable1.b.b1 = WCols[0].b.b0;
						CArray.Table[cid | 0x0010].QTable1.b.b1 = WCols[8].b.b0;
					}
				}
			break;

			case 0x2 : //2bpp, 320x256 display, 40 pitch display mode [mode 5]
			case (0x2 | 0x4 | 0x8) : //2bpp, 640x256 display, 80 pitch display mode [mode 1]
				if(addr <= 0x9)
				{
					c = 256;
					while(c--)
					{
						CArray.Lines[c&0x0f][(c >> 4) | 0x00].QTable1.w.w0 = WCols[0].w.w0;
						CArray.Lines[c&0x0f][(c >> 4) | 0x10].QTable1.w.w0 = WCols[2].w.w0;
						CArray.Lines[c&0x0f][(c >> 4) | 0x20].QTable1.w.w0 = WCols[8].w.w0;
						CArray.Lines[c&0x0f][(c >> 4) | 0x30].QTable1.w.w0 = WCols[10].w.w0;

						CArray.Lines[c&0xf0][((c << 4)&0xf0) | 0x00].QTable1.w.w1 = WCols[0].w.w0;
						CArray.Lines[c&0xf0][((c << 4)&0xf0) | 0x01].QTable1.w.w1 = WCols[2].w.w0;
						CArray.Lines[c&0xf0][((c << 4)&0xf0) | 0x02].QTable1.w.w1 = WCols[8].w.w0;
						CArray.Lines[c&0xf0][((c << 4)&0xf0) | 0x03].QTable1.w.w1 = WCols[10].w.w0;
					}
				}
			break;

			case (0x2 | 0x4) : //2bpp, 640x256 display, 40 pitch display mode [mode 5]
				if(addr <= 0x9)
				{
					c = 256;
					while(c--)
					{
						CArray.Lines[c&0x0f][(c >> 4) | 0x00].QTable1 = WCols[0];
						CArray.Lines[c&0x0f][(c >> 4) | 0x10].QTable1 = WCols[2];
						CArray.Lines[c&0x0f][(c >> 4) | 0x20].QTable1 = WCols[8];
						CArray.Lines[c&0x0f][(c >> 4) | 0x30].QTable1 = WCols[10];

						CArray.Lines[c&0xf0][((c << 4)&0xf0) | 0x00].QTable2 = WCols[0];
						CArray.Lines[c&0xf0][((c << 4)&0xf0) | 0x01].QTable2 = WCols[2];
						CArray.Lines[c&0xf0][((c << 4)&0xf0) | 0x02].QTable2 = WCols[8];
						CArray.Lines[c&0xf0][((c << 4)&0xf0) | 0x03].QTable2 = WCols[10];
					}
				}
			break;

			case (0x2 | 0x8) : //2bpp, 320x256 display, 80 pitch display mode [mode 1]
				if(addr <= 0x9)
				{
					c = 256;
					while(c--)
					{
						CArray.Lines[c&0x0f][(c >> 4) | 0x00].QTable1.b.b0 = WCols[0].b.b0;
						CArray.Lines[c&0x0f][(c >> 4) | 0x10].QTable1.b.b0 = WCols[2].b.b0;
						CArray.Lines[c&0x0f][(c >> 4) | 0x20].QTable1.b.b0 = WCols[8].b.b0;
						CArray.Lines[c&0x0f][(c >> 4) | 0x30].QTable1.b.b0 = WCols[10].b.b0;

						CArray.Lines[c&0xf0][((c << 4)&0xf0) | 0x00].QTable1.b.b1 = WCols[0].b.b0;
						CArray.Lines[c&0xf0][((c << 4)&0xf0) | 0x01].QTable1.b.b1 = WCols[2].b.b0;
						CArray.Lines[c&0xf0][((c << 4)&0xf0) | 0x02].QTable1.b.b1 = WCols[8].b.b0;
						CArray.Lines[c&0xf0][((c << 4)&0xf0) | 0x03].QTable1.b.b1 = WCols[10].b.b0;
					}
				}
			break;

			case 15 : //4bpp, 640x256 display, 80 pitch display mode [mode 2]
				c = 256;
				while(c--)
				{
					cid = (c&0xf0) | ((c&0x0f) << 12);

					CArray.Table[cid | or0].QTable2 = *col0;
					CArray.Table[cid | or1].QTable2 = *col1;
					CArray.Table[cid | or2].QTable2 = *col2;
					CArray.Table[cid | or3].QTable2 = *col3;

					cid >>= 4;
					CArray.Table[cid | (or0<<4)].QTable1 = *col0;
					CArray.Table[cid | (or1<<4)].QTable1 = *col1;
					CArray.Table[cid | (or2<<4)].QTable1 = *col2;
					CArray.Table[cid | (or3<<4)].QTable1 = *col3;
				}
			break;

			case 11 : //4bpp, 320x256 display, 80 pitch display mode [mode 2]
				c = 256;
				while(c--)
				{
					cid = (c&0xf0) | ((c&0x0f) << 12);

					CArray.Table[cid | or0].QTable1.w.w1 = col0->w.w0;
					CArray.Table[cid | or1].QTable1.w.w1 = col1->w.w0;
					CArray.Table[cid | or2].QTable1.w.w1 = col2->w.w0;
					CArray.Table[cid | or3].QTable1.w.w1 = col3->w.w0;

					cid >>= 4;
					CArray.Table[cid | (or0<<4)].QTable1.w.w0 = col0->w.w0;
					CArray.Table[cid | (or1<<4)].QTable1.w.w0 = col1->w.w0;
					CArray.Table[cid | (or2<<4)].QTable1.w.w0 = col2->w.w0;
					CArray.Table[cid | (or3<<4)].QTable1.w.w0 = col3->w.w0;
				}
			break;
		}
	}
}

#undef ror

void C_ScreenPalModifier1001::SyncPalette(void)
{
	int c, cid;
	BrokenQuad bc;

	if(bpp != -1)
	{
		c = 256;
		switch(bpp)
		{
			case 0x1 : //1bpp, 320x256 display, 40 pitch display mode [modes 4 & 6]
			case (0x1 | 0x4 | 0x8) : //1bpp, 640x256 display, 80 pitch display mode [modes 0 & 3]
				c = 65536;
				while(c--)
				{
					CArray.Table[c].QTable1.b.b0 = (c&0xf000) >> 12;
					CArray.Table[c].QTable1.b.b1 = (c&0x0f00) >> 8;
					CArray.Table[c].QTable1.b.b2 = (c&0x00f0) >> 4;
					CArray.Table[c].QTable1.b.b3 = (c&0x000f) >> 0;
				}

				c = 4096;
				while(c--)
				{
					cid = c;
					CArray.Table[cid | 0x0000].QTable1.b.b0 = WCols[0].b.b0;
					CArray.Table[cid | 0x1000].QTable1.b.b0 = WCols[8].b.b0;

					cid = ((cid >> 4) | (cid << 12)) & 65535;
					CArray.Table[cid | 0x0000].QTable1.b.b1 = WCols[0].b.b0;
					CArray.Table[cid | 0x0100].QTable1.b.b1 = WCols[8].b.b0;

					cid = ((cid >> 4) | (cid << 12)) & 65535;
					CArray.Table[cid | 0x0000].QTable1.b.b2 = WCols[0].b.b0;
					CArray.Table[cid | 0x0010].QTable1.b.b2 = WCols[8].b.b0;

					cid = ((cid >> 4) | (cid << 12)) & 65535;
					CArray.Table[cid | 0x0000].QTable1.b.b3 = WCols[0].b.b0;
					CArray.Table[cid | 0x0001].QTable1.b.b3 = WCols[8].b.b0;
				}
			break;

			case (0x1 | 0x4) : //1bpp, 640x256 display, 40 pitch display mode [modes 4 & 6]
				c = 65536;
				while(c--)
				{
					CArray.Table[c].QTable1.b.b0 =
					CArray.Table[c].QTable1.b.b1 = (c&0xf000) >> 12;
					CArray.Table[c].QTable1.b.b2 =
					CArray.Table[c].QTable1.b.b3 = (c&0x0f00) >> 8;

					CArray.Table[c].QTable2.b.b0 =
					CArray.Table[c].QTable2.b.b1 = (c&0x00f0) >> 4;
					CArray.Table[c].QTable2.b.b2 =
					CArray.Table[c].QTable2.b.b3 = (c&0x000f) >> 0;
				}

				c = 4096;
				while(c--)
				{
					cid = c;
					CArray.Table[cid | 0x0000].QTable1.w.w0 = WCols[0].w.w0;
					CArray.Table[cid | 0x1000].QTable1.w.w0 = WCols[8].w.w0;

					cid = ((cid >> 4) | (cid << 12)) & 65535;
					CArray.Table[cid | 0x0000].QTable1.w.w1 = WCols[0].w.w0;
					CArray.Table[cid | 0x0100].QTable1.w.w1 = WCols[8].w.w0;

					cid = ((cid >> 4) | (cid << 12)) & 65535;
					CArray.Table[cid | 0x0000].QTable2.w.w0 = WCols[0].w.w0;
					CArray.Table[cid | 0x0010].QTable2.w.w0 = WCols[8].w.w0;

					cid = ((cid >> 4) | (cid << 12)) & 65535;
					CArray.Table[cid | 0x0000].QTable2.w.w1 = WCols[0].w.w0;
					CArray.Table[cid | 0x0001].QTable2.w.w1 = WCols[8].w.w0;
				}
			break;

			case (0x1 | 0x8) : //1bpp, 320x256 display, 80 pitch display mode [modes 0 & 3]
				c = 65536;
				while(c--)
				{
					CArray.Table[c].QTable1.b.b0 = (c&0xf000) >> 12;
					CArray.Table[c].QTable1.b.b1 = (c&0x00f0) >> 4;
				}

				c = 4096;
				while(c--)
				{
					cid = c;
					CArray.Table[cid | 0x0000].QTable1.b.b0 = WCols[0].b.b0;
					CArray.Table[cid | 0x1000].QTable1.b.b0 = WCols[8].b.b0;

					cid = ((cid >> 8) | (cid << 8)) & 65535;
					CArray.Table[cid | 0x0000].QTable1.b.b1 = WCols[0].b.b0;
					CArray.Table[cid | 0x0010].QTable1.b.b1 = WCols[8].b.b0;
				}
			break;

			case 0x2 : //2bpp, 320x256 display, 40 pitch display mode [mode 5]
			case (0x2 | 0x4 | 0x8) : //2bpp, 640x256 display, 80 pitch display mode [mode 1]
				bc.a = 65535;
				while(bc.a--)
				{
					CArray.Lines[bc.b.b1][bc.b.b0].QTable1.b.b0 =
					CArray.Lines[bc.b.b1][bc.b.b0].QTable1.b.b1 = (unsigned __int8)(((bc.a&0xf000) >> 8) | ((bc.a&0x00f0) >> 4));
					CArray.Lines[bc.b.b1][bc.b.b0].QTable1.b.b2 =
					CArray.Lines[bc.b.b1][bc.b.b0].QTable1.b.b3 = (unsigned __int8)(((bc.a&0x0f00) >> 4) | ((bc.a&0x000f) >> 0));
				}

				c = 256;
				while(c--)
				{
					CArray.Lines[c&0x0f][(c >> 4) | 0x00].QTable1.w.w0 = WCols[0].w.w0;
					CArray.Lines[c&0x0f][(c >> 4) | 0x10].QTable1.w.w0 = WCols[2].w.w0;
					CArray.Lines[c&0x0f][(c >> 4) | 0x20].QTable1.w.w0 = WCols[8].w.w0;
					CArray.Lines[c&0x0f][(c >> 4) | 0x30].QTable1.w.w0 = WCols[10].w.w0;

					CArray.Lines[c&0xf0][((c << 4)&0xf0) | 0x00].QTable1.w.w1 = WCols[0].w.w0;
					CArray.Lines[c&0xf0][((c << 4)&0xf0) | 0x01].QTable1.w.w1 = WCols[2].w.w0;
					CArray.Lines[c&0xf0][((c << 4)&0xf0) | 0x02].QTable1.w.w1 = WCols[8].w.w0;
					CArray.Lines[c&0xf0][((c << 4)&0xf0) | 0x03].QTable1.w.w1 = WCols[10].w.w0;
				}
			break;

			case (0x2 | 0x4) : //2bpp, 640x256 display, 40 pitch display mode [mode 5]
				bc.a = 65535;
				while(bc.a--)
				{
					CArray.Lines[bc.b.b1][bc.b.b0].QTable1.b.b0 =
					CArray.Lines[bc.b.b1][bc.b.b0].QTable1.b.b1 =
					CArray.Lines[bc.b.b1][bc.b.b0].QTable1.b.b2 =
					CArray.Lines[bc.b.b1][bc.b.b0].QTable1.b.b3 = (unsigned __int8)(((bc.a&0xf000) >> 8) | ((bc.a&0x00f0) >> 4));

					CArray.Lines[bc.b.b1][bc.b.b0].QTable2.b.b0 =
					CArray.Lines[bc.b.b1][bc.b.b0].QTable2.b.b1 =
					CArray.Lines[bc.b.b1][bc.b.b0].QTable2.b.b2 =
					CArray.Lines[bc.b.b1][bc.b.b0].QTable2.b.b3 = (unsigned __int8)(((bc.a&0x0f00) >> 4) | ((bc.a&0x000f) >> 0));
				}

				c = 256;
				while(c--)
				{
					CArray.Lines[c&0x0f][(c >> 4) | 0x00].QTable1 = WCols[0];
					CArray.Lines[c&0x0f][(c >> 4) | 0x10].QTable1 = WCols[2];
					CArray.Lines[c&0x0f][(c >> 4) | 0x20].QTable1 = WCols[8];
					CArray.Lines[c&0x0f][(c >> 4) | 0x30].QTable1 = WCols[10];

					CArray.Lines[c&0xf0][((c << 4)&0xf0) | 0x00].QTable2 = WCols[0];
					CArray.Lines[c&0xf0][((c << 4)&0xf0) | 0x01].QTable2 = WCols[2];
					CArray.Lines[c&0xf0][((c << 4)&0xf0) | 0x02].QTable2 = WCols[8];
					CArray.Lines[c&0xf0][((c << 4)&0xf0) | 0x03].QTable2 = WCols[10];
				}
			break;

			case (0x2 | 0x8) : //2bpp, 320x256 display, 80 pitch display mode [mode 1]
				bc.a = 65535;
				while(bc.a--)
				{
					CArray.Lines[bc.b.b1][bc.b.b0].QTable1.b.b0 = (unsigned __int8)(((bc.a&0xf000) >> 8) | ((bc.a&0x00f0) >> 4));
					CArray.Lines[bc.b.b1][bc.b.b0].QTable1.b.b1 = (unsigned __int8)(((bc.a&0x0f00) >> 4) | ((bc.a&0x000f) >> 0));
				}

				c = 256;
				while(c--)
				{
					CArray.Lines[c&0x0f][(c >> 4) | 0x00].QTable1.b.b0 = WCols[0].b.b0;
					CArray.Lines[c&0x0f][(c >> 4) | 0x10].QTable1.b.b0 = WCols[2].b.b0;
					CArray.Lines[c&0x0f][(c >> 4) | 0x20].QTable1.b.b0 = WCols[8].b.b0;
					CArray.Lines[c&0x0f][(c >> 4) | 0x30].QTable1.b.b0 = WCols[10].b.b0;

					CArray.Lines[c&0xf0][((c << 4)&0xf0) | 0x00].QTable1.b.b1 = WCols[0].b.b0;
					CArray.Lines[c&0xf0][((c << 4)&0xf0) | 0x01].QTable1.b.b1 = WCols[2].b.b0;
					CArray.Lines[c&0xf0][((c << 4)&0xf0) | 0x02].QTable1.b.b1 = WCols[8].b.b0;
					CArray.Lines[c&0xf0][((c << 4)&0xf0) | 0x03].QTable1.b.b1 = WCols[10].b.b0;
				}
			break;

			case 3 : //4bpp, 320x256 display, 40 pitch display mode [doesn't exist]
			case 11 : //4bpp, 320x256 display, 80 pitch display mode [mode 2]
				bc.a = 65536;
				while(bc.a--)
				{
					CArray.Table[bc.w.w0.a].QTable1.b.b0 =
					CArray.Table[bc.w.w0.a].QTable1.b.b1 = (bc.b.b1 & 0xf0) | (bc.b.b0 >> 4);
					CArray.Table[bc.w.w0.a].QTable1.b.b2 =
					CArray.Table[bc.w.w0.a].QTable1.b.b3 = (bc.b.b1 << 4) | (bc.b.b0 & 0x0f);
				}

				bc.a = 256;
				while(bc.a--)
				{
					cid = (bc.a&0xf0) | ((bc.a&0x0f) << 12);

					c = 16;
					while(c--)
					{
						CArray.Table[(cid >> 4) | (c << 4)].QTable1.w.w0 =
						CArray.Table[cid | c].QTable1.w.w1 = WCols[c].w.w0;
					}
				}
			break;

			case 7 : //4bpp, 640x256 display, 40 pitch display mode [doesn't exist]
			case 15 : //4bpp, 640x256 display, 80 pitch display mode [mode 2]
				bc.a = 65536;
				while(bc.a--)
				{
					CArray.Table[bc.w.w0.a].QTable1.b.b0 =
					CArray.Table[bc.w.w0.a].QTable1.b.b1 =
					CArray.Table[bc.w.w0.a].QTable1.b.b2 =
					CArray.Table[bc.w.w0.a].QTable1.b.b3 = (bc.b.b1 & 0xf0) | (bc.b.b0 >> 4);

					CArray.Table[bc.w.w0.a].QTable2.b.b0 =
					CArray.Table[bc.w.w0.a].QTable2.b.b1 =
					CArray.Table[bc.w.w0.a].QTable2.b.b2 =
					CArray.Table[bc.w.w0.a].QTable2.b.b3 = (bc.b.b1 << 4) | (bc.b.b0 & 0x0f);
				}

				bc.a = 256;
				while(bc.a--)
				{
					cid = (bc.a&0xf0) | ((bc.a&0x0f) << 12);

					c = 16;
					while(c--)
					{
						CArray.Table[(cid >> 4) | (c << 4)].QTable1 =
						CArray.Table[cid | c].QTable2 = WCols[c];
					}
				}
			break;
		}
	}
}

void *C_ScreenPalModifier1001::SetMode(int i)
{
	bpp = i;
	SyncPalette();
	return (void *)&CArray;
}

void C_ScreenPalModifier1001::Disable(void)
{
	delete CArray.Table;
}

void C_ScreenPalModifier1001::Enable(BrokenQuad *cols)
{
	int c;
	WCols = cols;

	CArray.Table = new ConvElement1001[65536];
	c = 256;
	while(c--)
		CArray.Lines[c] = &CArray.Table[c << 8];

	SyncPalette();
}

