/*

	ElectrEm (c) 2000/1 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for
	details.

	sound/DirectX/soundcard.cpp
	===========================

	DirectX sound card interface

*/
#include "../sounddevs.h"

#ifdef TRG_SND_DSOUND

#include <string.h>

#define BUFFER_SIZE		1024
#define CIRCULAR_SIZE	4096

void C_SoundCard::SetupInput(void)
{
	/* set up sound system, return on failure */
	if(FAILED(DirectSoundCaptureCreate(NULL, &lpdsc, NULL)))
		return;

	/* attempt to create output buffer */
	DSCBUFFERDESC dscbd;

	InputBuffer.length = BUFFER_SIZE;
	InputBuffer.buffer = (unsigned __int8 *)malloc(InputBuffer.length);

	dscbd.dwSize = sizeof(DSBUFFERDESC);
	dscbd.dwBufferBytes = CIRCULAR_SIZE;
	dscbd.lpwfxFormat = &pcmwf;
	dscbd.dwReserved = 0;

	ReadPos = 0;

	if(!FAILED(lpdsc->CreateCaptureBuffer(&dscbd, &inBuffer, NULL)))
	{
		inBuffer->Start(DSBPLAY_LOOPING);
		CanInput = true;
	}
}

void C_SoundCard::SetupOutput(void)
{
	/* set up sound system, return on failure */
	if(FAILED(DirectSoundCreate(NULL, &lpds, NULL)))
		return;

	if(FAILED(lpds->SetCooperativeLevel(winfo.hWnd, DSSCL_PRIORITY)))
		return;

	/* attempt to create output buffer */
	DSBUFFERDESC dsbd;
	unsigned __int8 *audio_ptr_1=NULL, *audio_ptr_2=NULL;
	DWORD audio_length_1=0, audio_length_2=0;

	memset(&pcmwf, 0, sizeof(WAVEFORMATEX));
	pcmwf.wFormatTag = WAVE_FORMAT_PCM;
	pcmwf.nChannels = 1;
	pcmwf.nSamplesPerSec = 44100;
	pcmwf.nBlockAlign = 1;
	pcmwf.nAvgBytesPerSec = pcmwf.nSamplesPerSec * pcmwf.nBlockAlign;
	pcmwf.wBitsPerSample = 8;
	pcmwf.cbSize = 0;

	dsbd.dwSize = sizeof(DSBUFFERDESC);
	dsbd.dwBufferBytes = CIRCULAR_SIZE;
	dsbd.lpwfxFormat = &pcmwf;
	dsbd.dwReserved = 0;

	OutputBuffer.buffer = (unsigned char *)malloc(dsbd.dwBufferBytes);
	OutputBuffer.length = BUFFER_SIZE;

	dsbd.dwFlags = 0;
	WritePos = 0;
	Waiting = false;

	if(!FAILED(lpds->CreateSoundBuffer(&dsbd, &outBuffer, NULL)))
	{
		outBuffer->Play(0, 0, DSBPLAY_LOOPING);
		CanOutput = true;
	}
}

void C_SoundCard::Setup(void)
{
	InputRemaining = InputAllocated =
	OutputRemaining = OutputAllocated = 0;

	CanOutput = CanInput = false;

	SetupOutput();
	SetupInput();
}

void C_SoundCard::Close(void)
{
	if(inBuffer)
	{
		inBuffer->Stop();
		inBuffer->Release();
		inBuffer = NULL;
	}

	if(lpdsc)
	{
		lpdsc->Release();
		lpdsc = NULL;
	}

	if(outBuffer)
	{
		outBuffer->Stop();
		outBuffer->Release();
		outBuffer = NULL;
	}

	if(lpds)
	{
		lpds->Release();
		lpds = NULL;
	}

	if(OutputBuffer.buffer)
	{
		free(OutputBuffer.buffer);
		OutputBuffer.buffer = NULL;
	}

	if(InputBuffer.buffer)
	{
		free(InputBuffer.buffer);
		InputBuffer.buffer = NULL;
	}
}

void C_SoundCard::Keepalive(int clock)
{
	if((OutputAllocated&~1) && !OutputRemaining && outBuffer)
	{
		/* attempt to dump some more sound */
		if(Waiting)
		{
			unsigned __int8 *AudioPtr1, *AudioPtr2;
			DWORD AudioLength1, AudioLength2;

			if(!FAILED(outBuffer->Lock(WritePos, BUFFER_SIZE, (void **)&AudioPtr1, &AudioLength1, (void **)&AudioPtr2, &AudioLength2, 0)))
			{
				WritePos = (WritePos+BUFFER_SIZE)&(CIRCULAR_SIZE-1);

				memcpy(AudioPtr1, OutputBuffer.buffer, AudioLength1);
				memcpy(AudioPtr2, (OutputBuffer.buffer+AudioLength1), AudioLength2);

				outBuffer->Unlock(AudioPtr1, AudioLength1, AudioPtr2, AudioLength2);
				Waiting = false;
			}
		}

		if(!Waiting)
		{
			DWORD PlayPos;
			int difference;

			outBuffer->GetCurrentPosition(&PlayPos, NULL);
			difference = CircularDifference(PlayPos, WritePos, CIRCULAR_SIZE);

			if(difference >= BUFFER_SIZE)
			{
				OutputRemaining = OutputAllocated;
				Waiting = true;
				memset(OutputBuffer.buffer, 127, BUFFER_SIZE);
			}
		}
	}

	if((InputAllocated&~1) && inBuffer)
	{
		DWORD PlayPos;
		int difference;

		inBuffer->GetCurrentPosition(&PlayPos, NULL);
		difference = CircularDifference(PlayPos, ReadPos, CIRCULAR_SIZE);

		if(difference >= BUFFER_SIZE)
		{
			unsigned __int8 *AudioPtr1, *AudioPtr2;
			DWORD AudioLength1, AudioLength2;

			if(!FAILED(inBuffer->Lock(ReadPos, BUFFER_SIZE, (void **)&AudioPtr1, &AudioLength1, (void **)&AudioPtr2, &AudioLength2, 0)))
			{
				ReadPos = (ReadPos+BUFFER_SIZE)&(CIRCULAR_SIZE-1);

				memcpy(InputBuffer.buffer, AudioPtr1, AudioLength1);
				memcpy(InputBuffer.buffer+AudioLength1, AudioPtr2, AudioLength2);

				inBuffer->Unlock(AudioPtr1, AudioLength1, AudioPtr2, AudioLength2);
				InputRemaining = InputAllocated;
			}
		}
	}
}

C_SoundCard::C_SoundCard(void)
{
	OutputBuffer.buffer = NULL;
	InputBuffer.buffer = NULL;
	lpds = NULL;
	lpdsc = NULL;
	outBuffer = NULL;
	inBuffer = NULL;
}

void C_SoundCard::Pause(int time)
{
	if(outBuffer && OutputAllocated&~1)
	{
		outBuffer->Stop();
	}

	if(inBuffer && InputAllocated&~1)
	{
		inBuffer->Stop();
	}
}

void C_SoundCard::Unpause(int time)
{
	if(outBuffer && OutputAllocated&~1)
	{
		outBuffer->Play(0, 0, DSBPLAY_LOOPING);
	}

	if(inBuffer && InputAllocated&~1)
	{
		inBuffer->Start(DSBPLAY_LOOPING);
		ReadPos = 0;
	}

}
#endif
