/*

	ElectrEm (c) 2000/1 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for
	details.

	sound/OSS/soundcard.cpp
	=======================

	Open Sound System sound card interface - doesn't yet support software
	resampling, so simply doesn't allow input or output if 8bit/44100Hz isn't
	available in either style
*/
#include "../sounddevs.h"

#ifdef TRG_SND_OSS

#include <sys/ioctl.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/soundcard.h>
#include <malloc.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include "../sound.h"

#define BUFFER_SIZE 1024
#define BUFFER_POWER 10

void C_SoundCard::ReEstablishFD(int newmode)
{
	int speed = 44100;
	unsigned long fragsize;

	fragsize = BUFFER_POWER | 0x20000;

	if(AudioFD != -1)
	{
		close(AudioFD);
		AudioFD = -1;
	}

	if(newmode == -1)
	{
		AccessMode = newmode;
		return;
	}

	if((AudioFD = open("/dev/dsp", newmode | O_NONBLOCK)) == -1)
	{
		if(AccessMode != -1)
		{
			AudioFD = open("/dev/dsp", AccessMode | O_NONBLOCK);
		}
		else
		{
			return;
		}
	}
	else
	{
		if(newmode == O_RDWR) ioctl(AudioFD, SNDCTL_DSP_SETDUPLEX, 0);
		AccessMode = newmode;
	}

	ioctl(AudioFD, SNDCTL_DSP_SETFRAGMENT, &fragsize);

	if(ioctl(AudioFD, SNDCTL_DSP_SPEED, &speed) == -1)
	{
		close(AudioFD);
		AudioFD = -1;
		AccessMode = -1;
	}
}

void C_SoundCard::Setup(void)
{
	InputRemaining = InputAllocated =
	InputPause = OutputPause =
	OutputRemaining = OutputAllocated = 0;

	CanOutput =
	CanInput = true;

	AcRead = AcWrite = false;

	OutputBuffer.length =
	InputBuffer.length = 0;
	OldMode = 0;

	OutputBuffer.buffer = (unsigned __int8 *)malloc(BUFFER_SIZE);
	InputBuffer.buffer = (unsigned __int8 *)malloc(BUFFER_SIZE);
}

void C_SoundCard::Close(void)
{
	if(OutputBuffer.buffer)
	{
		free(OutputBuffer.buffer);
		OutputBuffer.buffer = NULL;
	}

	if(InputBuffer.buffer)
	{
		free(InputBuffer.buffer);
		InputBuffer.buffer = NULL;
	}

	if(AudioFD != -1)
	{
		close(AudioFD);
		AudioFD = -1;
	}
}

void C_SoundCard::Keepalive(int clock)
{
	if(Paused) return;

	if(AcWrite)
	{
		if(OutputBuffer.length && !OutputRemaining)
		{
			int c;

			if((c = write(AudioFD, &OutputBuffer.buffer[BufStart], OutputBuffer.length)) != -1)
			{
				BufStart += c;
				OutputBuffer.length -= c;
			}
		}

		if(OutputAllocated && !OutputRemaining && !OutputBuffer.length && CircularDifference(clock, TimeBase, SOUND_MASK+1) >= BUFFER_SIZE)
		{
			memset(OutputBuffer.buffer, 127, BUFFER_SIZE);
			OutputRemaining = OutputAllocated;
			OutputBuffer.length = BUFFER_SIZE;
			BufStart = 0;
		}
	}

	if(AcRead)
		if(InputAllocated && !InputRemaining)
		{
			if((InputBuffer.length = read(AudioFD, InputBuffer.buffer, BUFFER_SIZE)) > 0)
				InputRemaining = InputAllocated;
		}
}

C_SoundCard::C_SoundCard(void)
{
	OutputBuffer.buffer = NULL;
	OutputBuffer.length = BUFFER_SIZE;
	AudioFD = -1;
}

bool C_SoundCard::ConsiderReallocation(int newid)
{
	int Mode;
	bool newread = AcRead, newwrite = AcWrite;

	/* detemine new mode */
	newread = (InputAllocated>>1) ? true : false;
	newwrite = (OutputAllocated>>1) ? true : false;

	if(newread == AcRead && newwrite == AcWrite || (!newread && !newwrite))
		return true;

	int NewMode;

	switch((newwrite ? 2 : 0) | (newread ? 1 : 0))
	{
		case 0 : NewMode = -1; break;
		case 1 : NewMode = O_RDONLY; break;
		case 2 : NewMode = O_WRONLY; break;
		case 3 :
			int caps;

			ioctl(AudioFD, SNDCTL_DSP_GETCAPS, &caps);

			if(caps&DSP_CAP_DUPLEX)
				NewMode = O_RDWR;
			else
			{
				newwrite = false;
				NewMode = O_RDONLY;
			}
		break;
	}

	ReEstablishFD(NewMode);

	if(AccessMode == NewMode)
	{
		AcRead = newread;
		AcWrite = newwrite;

		if(!AcRead) { InputRemaining = 0; }
		if(!AcWrite) { OutputRemaining = 0; }

		return true;
	}
	else
		return false;
}

void C_SoundCard::Pause(int time)
{
	Paused = true;
}

void C_SoundCard::Unpause(int time)
{
	Paused = false;
	TimeBase = time;
}
#endif
