/*

	ElectrEm (c) 2000/1 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for
	details.

	sound/SDL/soundcard.cpp
	=======================

	SDL sound card interface - SDL doesn't support input, but output is
	fully supported

*/
#include "../sounddevs.h"

#ifdef TRG_SND_SDL

#include "../sound.h"
#include <stdio.h>
#include <malloc.h>
#include <string.h>

#define SAMPLE_SIZE 2048
#define BUFFER_SIZE 8192

/*

SDL callback

*/
void C_SoundCard::FillBuffer(void *ud, Uint8 *ptr, int len)
{
	if(bufferpos+len > BUFFER_SIZE)
	{
		/* a two part copy */
		int slice = BUFFER_SIZE - bufferpos;

		memcpy(ptr, &bufmem[bufferpos], slice);
		len -= slice;
		bufferpos = len;
		memcpy(ptr+slice, bufmem, len);
	}
	else
	{
		/* a one part copy */
		memcpy(ptr, &bufmem[bufferpos], len);
		bufferpos += len;
	}

	if(((writepos-bufferpos)%BUFFER_SIZE) <= SAMPLE_SIZE)
	{
		dwanted = true;
	}
}

unsigned __int8 *C_SoundCard::bufmem;
int C_SoundCard::bufferpos, C_SoundCard::writepos;
bool C_SoundCard::dwanted, C_SoundCard::Locked;

void C_SoundCard::Setup(void)
{
	InputRemaining = InputAllocated =
	OutputRemaining = OutputAllocated = 0;
	CanOutput = CanInput = false;

	OutputBuffer.length = SAMPLE_SIZE;
	Locked = false;
	SDL_AudioSpec *req = (SDL_AudioSpec *)malloc(sizeof(SDL_AudioSpec));

	req->freq = 44100;
	req->format = AUDIO_U8;
	req->channels = 1;
	req->callback = FillBuffer;
	req->samples = SAMPLE_SIZE;
	req->userdata = NULL;

	int res = SDL_OpenAudio(req, NULL);

	if(!res)
	{
		bufmem = (unsigned __int8 *)calloc(BUFFER_SIZE, sizeof(unsigned __int8));
		bufferpos = BUFFER_SIZE - SAMPLE_SIZE;
		writepos = 0;
		SDL_PauseAudio(0);

		CanInput = true;
	}
}

void C_SoundCard::Close(void)
{
	if(bufmem)
	{
		free(bufmem);
		bufmem = NULL;

		SDL_PauseAudio(1);
		SDL_CloseAudio();
	}
}

void C_SoundCard::Keepalive(int clock)
{
	/* check for new input or output requirement */
	if(!OutputRemaining && (OutputAllocated&~1))
	{
		if(Locked)
		{
			SDL_UnlockAudio();
			Locked = false;
		}

		if(!Locked && dwanted)
		{
			OutputRemaining = OutputAllocated;
			SDL_LockAudio();
			Locked = true;

			OutputBuffer.buffer = &bufmem[writepos];
			writepos = (writepos + SAMPLE_SIZE)%BUFFER_SIZE;
			memset(OutputBuffer.buffer, 127, SAMPLE_SIZE);

			dwanted = false;
		}
	}
}

C_SoundCard::C_SoundCard(void)
{
	bufmem = NULL;
}

void C_SoundCard::Pause(int time)
{
	if(bufmem) SDL_PauseAudio(1);
}

void C_SoundCard::Unpause(int time)
{
	if(bufmem) SDL_PauseAudio(0);
}
#endif
