/*

	ElectrEm (c) 2000/1 Thomas Harte - an Acorn Electron Emulator

	This file at some time additionally worked upon by Paul Boddie

	This is open software, distributed under the GPL 2, see 'Copying' for
	details

	sound/apibase.cpp
	=================

	This code is the default method of handling all input/output to sound
	sources which does everything except actually dispatching and receiving
	buffers since that activity varies from sound and platform target to
	target.

	SoundID's are just integers with the lsb set if the ID refers to an
	output stream, and not set otherwise. Another one bit is set somewhere
	in the remaining 31, and that bit is unique to a particular ID.

	This base class is inherited to provide both sound file input and sound
	card input/output (the former only if supported)
*/
#include "apibase.h"
#include <stdio.h>

SoundID C_SoundAPIBase::RequestAccess(SoundMode m)
{
	int *Allocated;
	bool *Can;
	SoundID newid, ormask;

	switch(m)
	{
		case SoundMode_Input :
			Can = &CanInput;
			Allocated = &InputAllocated;
			ormask = 0;
		break;

		case SoundMode_Output :
			Can = &CanOutput;
			Allocated = &OutputAllocated;
			ormask = 1;
		break;
	}

	if(!(*Can))
	{
		Error = SoundError_NotSupported;
		return 0;
	}

	newid = 2;
	while( newid && (*Allocated)&newid)
	{
		newid <<= 1;
	}

	if(newid)
	{
		(*Allocated) |= newid;
		if(ConsiderReallocation(newid | ormask))
			return newid | ormask;
		else
			(*Allocated) &= ~newid;
	}
	else
		Error = SoundError_TooMany;

	return 0;
}

void C_SoundAPIBase::ReturnAccess(SoundID id)
{
	switch(id&1)
	{
		case 0 : InputPause &= ~id; InputRemaining &= ~id; InputAllocated &= ~id;	ConsiderReallocation(InputAllocated); break;
		case 1 : OutputPause &= ~id; OutputRemaining &= ~id; OutputAllocated &= ~id; ConsiderReallocation(OutputAllocated | 1);	break;
	}

	if(!InputAllocated) InputRemaining = 0;
	if(!OutputAllocated) OutputRemaining = 0;

	if(InputAllocated || OutputAllocated)
	{
		if(!InputPause && !OutputPause)
			Unpause(0);

		if((InputPause == InputAllocated) && (OutputPause == OutputAllocated))
			Pause(0);
	}
}

bool C_SoundAPIBase::IsInputId(SoundID id)
{
	return (id&1) ? false : true;
}

SoundBuffer *C_SoundAPIBase::PollNewBuffer(SoundID id, int samples)
{
	switch(id&1)
	{
		case 0 :
			if(InputRemaining&id)
				return &InputBuffer;
			else
				return NULL;
		break;
		case 1 :
			if((OutputRemaining&id) && (samples >= OutputBuffer.length))
				return &OutputBuffer;
			else
				return NULL;
		break;
	}

	return NULL;
	/*
	
	"return NULL;" must be included, or else MSVC issues a 'not all control
	paths return a value' compiler error
	
	*/
}

void C_SoundAPIBase::ReturnBuffer(SoundID id)
{
	switch(id&1)
	{
		case 0 :
			InputRemaining &= ~id;
		break;
		case 1 :
			OutputRemaining &= ~id;
		break;
	}
}

void C_SoundAPIBase::SetOutputVolume(float vol)
{
	int OutputSources = 0;
	SoundID omask = 2;

	Volume = vol;

	while(omask)
	{
		if(OutputAllocated&omask)
			OutputSources++;

		omask <<= 1;
	}

	OutputBuffer.contribution = (int)((vol / (float)OutputSources) * 255.0f);
}

float C_SoundAPIBase::GetOutputVolume(void)
{
	return Volume;
}

SoundErr C_SoundAPIBase::GetError(void)
{
	return Error;
}

bool C_SoundAPIBase::Control(int, int)
{
	return false;
}

bool C_SoundAPIBase::ConsiderReallocation(int newid)
{
	return true;
}

void C_SoundAPIBase::MinimiseTimer(int sub)
{
}

void C_SoundAPIBase::Pause(SoundID id, int time)
{
	if(id&1)
		OutputPause |= (id&~1);
	else
		InputPause |= id;

	if((InputPause == InputAllocated) && (OutputPause == OutputAllocated))
		Pause(time);
}

void C_SoundAPIBase::Unpause(SoundID id, int time)
{
	if(id&1)
		OutputPause &= ~id;
	else
		InputPause &= ~id;

	if(!InputPause && !OutputPause)
		Unpause(time);
}

