#ifndef SNDAPIBASE_H
#define SNDAPIBASE_H

#include "../types.h"

struct SoundBuffer
{
	unsigned __int8 *buffer;
	int length, contribution;
};

typedef unsigned __int32 SoundID;

#define SOUNDERR_TOOMANY		-1
#define SOUNDERR_NOTSUPPORTED	-2

enum SoundMode
{
	SoundMode_Input,
	SoundMode_Output
};

enum SoundErr
{
	SoundError_TooMany,
	SoundError_NotSupported
};

#define SOUNDCONTROL_SETTIME	0

class C_SoundAPIBase
{
	public :

		virtual void Close(void) = 0;

		virtual SoundID RequestAccess(SoundMode);
		virtual bool IsInputId(SoundID);
		virtual void ReturnAccess(SoundID);

		virtual SoundBuffer *PollNewBuffer(SoundID, int);
		virtual void ReturnBuffer(SoundID);

		virtual void SetOutputVolume(float);
		virtual float GetOutputVolume(void);

		virtual void Pause(SoundID, int);
		virtual void Unpause(SoundID, int);

		virtual SoundErr GetError(void);

		virtual void Keepalive(int clock) = 0;
		virtual void MinimiseTimer(int sub);
		virtual bool Control(int command, int value);

	protected :

		virtual bool ConsiderReallocation(int newid);
		virtual void Pause(int time) = 0;
		virtual void Unpause(int time) = 0;

		int OutputAllocated;
		int OutputRemaining;
		int OutputPause;
		SoundBuffer OutputBuffer;
		bool CanOutput;

		int InputAllocated;
		int InputRemaining;
		int InputPause;
		SoundBuffer InputBuffer;
		bool CanInput;

		float Volume;
		SoundErr Error;
};

#endif
