/*

	ElectrEm (c) 2000/1 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for
	details.

	sound/sound.cpp
	===============

	Don't get confused - this sound.cpp is nothing to do with the one that
	featured up to and including beta 8.

	This sound.cpp offers pointers to classes for reading audio output and
	input from files or the sound card.

	At this time ElectrEm doesn't actually offer sound output to sound files,
	and sound input from the sound card isn't supported across all targets,
	but that is nothing to do with this class.
*/

#include "sound.h"
#include "sounddevs.h"
#include <string.h>

C_SoundProvider::C_SoundProvider()
{
	list = NULL;
}

C_SoundProvider::~C_SoundProvider()
{
}

C_SoundProvider::SoundResource *C_SoundProvider::NewListMember(void)
{
	SoundResource *ret = list;

	if(!ret)
	{
		ret = list = new SoundResource;
	}
	else
	{
		while(ret->next)
			ret = ret->next;

		ret->next = new SoundResource;
		ret = ret->next;
	}

	ret->path = ret->name = ret->mode = NULL;
	ret->next = NULL;
	ret->references = 0;
	ret->freeable = true;

	return ret;
}

void C_SoundProvider::RemoveListMember(SoundResource *r)
{
	if(list == r)
	{
		list = list->next;
		delete r;
	}
	else
	{
		SoundResource *res = list;

		while(res->next)
		{
			if(res->next == r)
			{
				res->next = res->next->next;
				delete r;
				break;
			}

			res = res->next;
		}
	}
}

void C_SoundProvider::Setup(C_FSAccess *f)
{
	SoundResource *res;

	fs = f;

	/* create sound card entry */
	list = res = NewListMember();
	res->resource = new C_SoundCard;
	res->name = strdup(SOUNDCARD_NAME);
	res->freeable = false;

	((C_SoundCard *)res->resource)->Setup();
	res->resource->SetOutputVolume(1.0f);
}

void C_SoundProvider::Close(void)
{
	SoundResource *res = list, *nxt;

	while(res)
	{
		nxt = res->next;

		res->resource->Close();
		delete res->resource;
		delete res;

		res = nxt;
	}

	list = NULL;
}

C_SoundAPIBase *C_SoundProvider::GetSoundSource(char *path, char *name, char *mode)
{
	SoundResource *res = list;
	bool match;

	while(res)
	{
		match = true;

		if(path)
		{
			match = false;

			if(res->path)
				if(!strcmp(res->path, path)) match = true;
		}

		if(name)
		{
			match = false;

			if(res->name)
				if(!strcmp(res->name, name)) match = true;
		}

		if(match)
		{
			res->references++;
			return res->resource;
		}

		res = res->next;
	}

	/* not found, so open */
	res = NewListMember();
	res->resource = new C_SoundFile;
	res->name = strdup(name);
	res->path = strdup(path);

	((C_SoundFile *)res->resource)->Setup(path, name, fs);

	return res->resource;
}

void C_SoundProvider::ReleaseSoundSource(C_SoundAPIBase *r)
{
	SoundResource *res = list;

	while(res)
	{
		if(res->resource == r)
		{
			res->references--;

			if(!res->references && res->freeable)
			{
				res->resource->Close();
				delete res->resource;
				RemoveListMember(res);
			}

			break;
		}

		res = res->next;
	}
}

void C_SoundProvider::Keepalive(int time)
{
	SoundResource *res = list;

	while(res)
	{
		res->resource->Keepalive(time);
		res = res->next;
	}
}

