#ifndef TAPE_H
#define TAPE_H

#include "../uef/uef.h"

class C_ULA;
class C_Speaker;
class C_TapeBitProvider;
class C_TapeBitProviderUEF;
class C_TapeBitProviderT2P3;
class C_TapeBitProviderWHTape;
class C_TapeBitProviderWave;

struct FileDataBlock;

#define TAPE_INPUTMODE_FILE			0
#define TAPE_INPUTMODE_SOUND		1
#define TAPE_INPUTMODE_SPEEDUP		2

#define TAPE_OUTPUTMODE_UEF			3
#define TAPE_OUTPUTMODE_SOUND		4

#define TAPE_IO_PIPING				5

class C_tape
{
	public :
		/* initialisation / deinitialisation */
		C_tape(void);
		~C_tape(void);

		bool Setup(C_ULA *electron);
		void Closedown(void);

		/* file handling */
		bool Open(char *path, char *name, int samptime, int clocktime);
		bool New(char *path, char *name, int samptime, int clocktime);
		bool Useful(C_UEFChunkSelector *selector);
		void Close(void);

		/* keepalives & signals */
		void Keepalive(int samptime, int clocktime);
		void MinimiseTimer(int samp, int clock);
		void SignalReset(void);

		/* virtual tape handling */
		void Rewind(int samptime, int clocktime);

		/* auto load id responder */
		unsigned __int8 GetAutoloadID(void);

		/* fast tape hack*/
		bool EnterService(unsigned __int8 command);
		void HandleVOP(unsigned __int8 command, int samptime, int clocktime);
		void AtStringEnd(void);

		/* register access */
		void WriteByte(unsigned __int8 value, unsigned __int32 bv, int samptime, int clocktime);
		void ReadByte(unsigned __int8 *value, unsigned __int32 *bv, int samptime, int clocktime);
		void SetMode(bool counter, bool read, bool motor, int samptime, int clocktime);

		/* some state related stuff */
		void SetOperationMode(int mode, bool set, int samptime, int clocktime);
		bool GetOperationMode(int mode);

	private :
		struct FileBlock
		{
			char *name;
			unsigned __int8 *header;

			int blocknum;
			bool last, empty, locked;

			int offset;
			FileBlock *next;

		} *list, *current;

		int bitcount;

		C_ULA *elk;
		C_Speaker *speaker;

		unsigned __int16 TapeData8;
		unsigned __int32 TapeData32;
		bool TapeInData;

		bool tape_read;
		bool motor_on;
		bool counter;

		void BuildTable(void);
		void FreeTable(void);
		void ForgeFootprint(void);
		bool FileContinues(void);

		void InitialiseOutput(void);
		void AddString(char *output);
		void EndOutput(void);

		void NextFile(void);
		bool FindFile(char *name, bool);
		char *GetFilename(BrokenWord addr, char *terminators);

		void SetBitIn(C_TapeBitProvider *b, int samptime, int clocktime);

		BrokenWord GetCRC(unsigned __int8 *start, int len);
		bool StrCmp(char *one, char *two);
		int StrLen(char *str, int maxlen);

		char *outputtext, *outcursor, *outend;

		enum TapeCommands
		{
			NONE, LOAD, LOADEND, RUNEND
		} TapeCommand;

		void BeginLoad(unsigned __int8 *info, TapeCommands nxt);
		void LoadNextBlock(void);

		TapeCommands NextCmd;
		int FileLength;
		BrokenWord FileAddr;
		unsigned __int8 *FileInfo;
		unsigned __int8 LoadBuffer[22];
		int bitsubcounter;

		bool dirty;

		/* byte level read bits */
		void ReadOSFileByte(void);
		FileDataBlock *OSFileData;
		unsigned int OSFileOffset;
		bool OSFileEOF;

		/* the bit providers */
		C_TapeBitProviderWave *card, *file;
		C_TapeBitProviderUEF *uef;
		C_TapeBitProviderT2P3 *t2p3;
		C_TapeBitProviderWHTape *whtape;

		C_TapeBitProvider *bitin, *filein;
		bool UEFOut, WaveOut, IOLoop;
};

#endif