#ifndef UEF_H
#define UEF_H

#include "../types.h"
#include <zlib.h>

#define UEF_EOC -1
#define UEF_EOF -1

#ifndef C_UEFFile
class C_UEFFile;
#endif
#ifndef C_UEFChunkSelector
class C_UEFChunkSelector;
#endif

class C_UEFChunk
{
	public :
		C_UEFChunk(gzFile input, char *name, bool wrte);
		~C_UEFChunk(void);

		int Read(void *buf, int numbytes);
		int ReadSeek(long offset, int mode);
		int GetC(void);

		int Write(const void *buf, int numbytes);
		int WriteSeek(long offset, int mode);
		void PutC(int value);

		unsigned __int16 GetId(void);
		unsigned __int32 GetLength(void);
		bool EOC(void);

		void SetId(__int16 id);
		void SetLength(__int32 length);

	private :

		bool Create(gzFile input, char *name, bool wrte);
		bool HasChanged(void);
		bool Write(gzFile f, C_UEFFile *owner);
		void Kill(void);

		int Enable(C_UEFFile *owner);
		int Disable(void);
		int refcount;
		int GetRefCount(void);

		C_UEFChunk *next, *last;
		friend C_UEFChunkSelector;
		friend C_UEFFile;

		/*

		generalised seek

		*/

		int Seek(unsigned __int32 *variable, long offset, int mode);

		/*

		memory is used to store the chunk contents while this chunk is active,
		or until file write time if changes are made

		*/

		unsigned __int8 *memory;
		unsigned __int32 memorylen;
		bool dirty;

		/*

		read and write pointers

		*/

		unsigned __int32 readptr, writeptr;
		bool write;

		/*

		a file block is unique by virtue of the file its in, and its start
		location (in bytes offset) within that file

		*/

		char *fname;
		int offset;

		/*

		just a store of this chunk's id and length

		*/

		unsigned __int16 id;
		unsigned __int32 length;
};

class C_UEFFile
{
	public :
		C_UEFFile(void);
		~C_UEFFile(void);

		bool Open(char *name, unsigned __int16 version, char *mode);
		void Close(void);

		unsigned __int16 GetVersion(void);
		C_UEFChunkSelector *GetSelectorPtr(void);
		void ReleaseSelectorPtr(C_UEFChunkSelector *selt);

		bool HasFile(void);

	private :
		void BuildFileChain(gzFile file, char *fname);
		void Killing(C_UEFChunk *chunk);

		gzFile GetFile(char *name);
		friend C_UEFChunk;
		friend C_UEFChunkSelector;

		char *fname, *tempname;
		C_UEFChunk *list;

		bool read, write;
		unsigned __int16 newversion, oldversion;

		gzFile cinput;
		char *cname;
};

class C_UEFChunkSelector
{
	public :
		C_UEFChunkSelector(C_UEFFile *initialiser, C_UEFChunk *first, bool wrte);
		~C_UEFChunkSelector(void);

		int GetOffset(void);
		bool Seek(long offset, int mode);
		bool FindId(unsigned __int16 id);
		bool FindIdMajor(unsigned __int8 id);
		void Reset(void);

		C_UEFChunk *CurrentChunk(void);

		C_UEFChunk *GetChunkPtr(void);
		C_UEFChunk *EstablishChunk(void);
		void ReleaseChunkPtr(C_UEFChunk *chunk);

		bool RemoveChunk(C_UEFChunk *chunk);

		bool OverRan(void);
		void ResetOverRan(void);

		C_UEFFile *GetFather(void);

	private :

		bool Find(unsigned __int16 id, int shifter);
		void ReleaseCurrent(void);

		C_UEFChunk *list, *current;
		C_UEFFile *father;
		bool justopen, borrowed, overran, write, enabled;
};

#endif
