/*

	ElectrEm (c) 2000 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for
	details

	ula/roms.cpp
	============

	Bits and pieces for loading of ROMs and so on

*/
#include "../fs/fs.h"
#include "ula.h"
#include <malloc.h>
#include <stdio.h>
#include <string.h>
#include <zlib.h>

unsigned __int8 *C_ULA::GetROMPtr(int rnum)
{
	if(rnum <= ULA_NUMROMS)
		return RomPtrs[rnum];
	else
		return NULL;
}

void C_ULA::FreeROM(int rnum)
{
	if(rnum > ULA_NUMROMS) return;

	RomMode[rnum] = ULAROMMODE_OFF;
	RomLoaded[rnum] = false;
	if(RomPtrs[rnum])
	{
		free(RomPtrs[rnum]);
		RomPtrs[rnum] = NULL;
	}
	if(RomNames[rnum])
	{
		free(RomNames[rnum]);
		RomNames[rnum] = NULL;
	}
}

char *C_ULA::GetROMString(int rnum)
{
	return RomNames[rnum];
}

void C_ULA::SetROMPtr(int rnum, unsigned __int8 *ptr)
{
	if(rnum > ULA_NUMROMS) return;
	RomPtrs[rnum] = ptr;
}

void C_ULA::SetROMPresent(int rnum, int mode)
{
	if(rnum > ULA_NUMROMS) return;
	switch(mode)
	{
		case ULAROMMODE_RAM :

			if(!RomPtrs[rnum])
			{
				RomPtrs[rnum] = (unsigned __int8 *)malloc(16384);
			}

		default : RomMode[rnum] = mode; break;
		break;

		case ULAROMMODE_ON :
			RomMode[rnum] = RomLoaded[rnum] ? ULAROMMODE_ON : ULAROMMODE_OFF;
		break;
	}
}

int C_ULA::GetROMPresent(int rnum)
{
	if(rnum > ULA_NUMROMS) return false;

	return RomMode[rnum];
}

void C_ULA::SetROMLoaded(int rnum, bool truth)
{
	if(rnum > ULA_NUMROMS) return;

	RomLoaded[rnum] = truth;
}

bool C_ULA::GetROMLoaded(int rnum)
{
	if(rnum > ULA_NUMROMS) return false;

	return RomLoaded[rnum];
}

int C_ULA::GetHardwareROMSlot(int rnum)
{
	if(rnum >= ULA_NUMROMS) return -1;

	return cind[rnum];
}

bool C_ULA::LoadROM(char *tname, int rnum, char *newname)
{
	char *extensions[] =
	{
		"", ".rom", ".rom.gz", ".rgz", NULL
	};

	unsigned __int8 *ptr;
	char *loadname, **ext;
	bool loaded = false;
	gzFile stream;

	if(rnum > ULA_NUMROMS) return false;

	loadname = (char *)malloc(strlen(tname)+8);
	stream = NULL;
	ext = extensions;

	do
	{
		sprintf(loadname, "%s%s", tname, *ext);
		stream = fs->GetGZ(NULL, "rompath", loadname, "rb");
		ext++;
	}
	while(!stream && *ext);

	if(stream)
	{
		if(RomNames[rnum]) free(RomNames[rnum]);
		RomNames[rnum] = strdup(newname);

		if(!(ptr = GetROMPtr(rnum)))
		{
			ptr = (unsigned __int8 *)malloc(16384);
		}

		gzread(stream, ptr, 16384);
		gzclose(stream);
		loaded = true;

		SetROMPtr(rnum, ptr);
		SetROMLoaded(rnum, true);
	}

	return loaded;
}