/*

	ElectrEm (c) 2000/1 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for
	details

	ula/ulaio.cpp
	=============

 	This file redistributes fc, fd & fe page read/writes

*/

#include "ula.h"
#include <stdio.h>
#include "../plus1/plus1.h"
#include "../misc/input.h"
#include "../wd1770/wd1770.h"
#include "../tape/tape.h"
#include "../screen/screen.h"
#include "../sound/speaker.h"
#include "../misc/timing.h"

#define CheckInterrupts()	\
	if(!(IntStatus & (ULAIRQ_DISPLAY | ULAIRQ_RTC | ULAIRQ_HTONE | ULAIRQ_SEND | ULAIRQ_RECEIVE)))\
	{\
		ccpu->ClearIRQ();\
		IntStatus &= ~ULAIRQ_MASTER;\
	}

int C_ULA::ReadFrom(BrokenWord addr, unsigned __int8 *value, unsigned __int32 *bvalue, int time)
{
	unsigned __int32 t;

	//this just saves checking the NULL pointer at every other point in this
	//function
	if(bvalue == NULL) bvalue = &t;

	if(addr.b.h < 0xc0)
	{
		if(PagedRom == 8 || PagedRom == 9)
		{
			*value = input->GetKeyboard(addr);
			return 0;
		}
		else
		{
			*value = 0;
			return 0;
		}
	}
	else
	{
		bool t;

		switch(addr.b.h)
		{
			case 0xf0 :
			case 0xf1 :
			case 0xf3 :
			case 0xf4 :
			case 0xff :
				t = ConsiderTapeHack(addr);

				*value = t ? 0x60 : GetROMPtr(ULAROM_OS)[addr.a - 0xc000];
				if(bvalue) *bvalue = t ? 0 : CWideOSRom[addr.a - 0xc000];
			break;

			case 0xfc :
				switch(addr.b.l)
				{
					default :
//						fprintf(stderr, "Unrecognised read at %04x\n", addr.a);
						*value = GetROMPtr(ULAROM_OS)[addr.a - 0xc000];
					return 0;

					case 0xc4 :
					case 0xc5 :
					case 0xc6 :
					case 0xc7 :
						*value = disc->ReadFromRegister(this, addr.b.l - 0xc0, time);
					return 0;

					case 0x70 :
					case 0x71 :
					case 0x72 :
						plus1->ReadFrom(addr, value, bvalue, time);
					break;

					case 0xe0 : //TUBE
						*value = 0;
					return 0;

					case 0xc0 : //first byte joystick or Plus 3 drive control
						if(!joyon)
						{
							*value = disc->ReadFromRegister(this, addr.b.l - 0xc0, time);
							break;
						}
					case 0xfe : //relocated first byte joystick
						JOY_STATE i;

						if(joyid >= 0)
						{
							i = input->GetJoystickState(joyid);

							*value =	(i.up ? 0x00 : 0x01) |
										(i.down ? 0x00 : 0x02) |
										(i.left ? 0x00 : 0x04) |
										(i.right ? 0x00 : 0x08) |
										(i.button ? 0x00 : 0x10) |
										0x20;
						}
						else
							*value = 0;
					break;
				}
			return 0;

			case 0xfd :
				if(TapePrinting)
				{
					if(addr.b.l == 0xff)
					{
						*value = *textptr;
						if(*textptr)
						{
							textptr++;
							if(!(*textptr))
							{
								tape->AtStringEnd();
							}
						}
					}
					else
						*value = ccpu->ReadByte(addr.a);
				}
				else
				{
//					fprintf(stderr, "Unrecognised read at %04x\n", addr.a);
					*value = GetROMPtr(ULAROM_OS)[addr.a-0xc000];
				}
			return 0;

			case 0xfe :
//			fprintf(stderr, "r : %02x\n", addr.b.l&0xf);
				switch(addr.b.l&0xf)
				{
					case 0x0 :
						*value = IntStatus;// | ~(IntControl|3);

						IntStatus &= ~ULAIRQ_POWER;
						CheckInterrupts();

					return 0;

					case 0x4:
						tape->ReadByte(value, bvalue, TimeToSample(time, slowdown->GetMultiplier()), time);
					return 0;

					default :
						*value = GetROMPtr(ULAROM_OS)[addr.a - 0xc000];
					return 0;
				}
			break;
		}
	}

	return 0;
}

#define SB		0
#define SL		1
#define ST		0x100

int C_ULA::WriteTo(BrokenWord addr, unsigned __int8 value, unsigned __int32 bvalue, int time)
{
	switch(addr.b.h)
	{
		case 0xfc :
			switch(addr.b.l)
			{
				default :
//					fprintf(stderr, "Unrecognised write %02x -> %04x [%d]\n", value, addr.a, time);
				break;

				case 0xc0 :
					joyon = false;
				case 0xc4 :
				case 0xc5 :
				case 0xc6 :
				case 0xc7 :
					disc->WriteToRegister(this, addr.b.l - 0xc0, value, time);
				break;

				case 0x70 :
				case 0x71 :
				case 0x72 :
					plus1->WriteTo(addr, value, bvalue, time);
				break;

				case 0x7f : //Slogger MRB RAM paging*/
/*					switch(MRBMode)
					{
						case MRBMODE_OFF : break;

						case MRBMODE_SHADOW :

							if(value&0x80)
								ccpu->SetAddrPtrs(0, 0x80, RamPtr, WideRamPtr, true, true);
							else
								ccpu->SetAddrPtrs(0, 0x80, &RamPtr[0x8000], &WideRamPtr[0x8000], true, true);

						break;

						case MRBMODE_TURBO :
							if(value&0x80)
								ccpu->SetAddrPtrs(0, 0x20, RamPtr, WideRamPtr, true, true);
							else
								ccpu->SetAddrPtrs(0, 0x20, &RamPtr[0x8000], &WideRamPtr[0x8000], true, true);
						break;
					}*/
				return 0;

				case 0xff : //JIM
					PageJIM(value);
				break;
			}
		break;

		case 0xfd :
//			fprintf(stderr, "Unrecognised write %02x -> %04x [%d]\n", value, addr.a, time);
		break;

		case 0xfe :
			addr.b.l &= 0xf;
			Values[addr.b.l] = value;

			switch(addr.b.l)
			{
				case 0x0 :
					IntControl = value;
					IntStatus &= (value | 3);

					CheckInterrupts();
				break;

				case 0x3 :
					scr->VidAddr.b.h = (scr->VidAddr.b.h & 1) | ((value&63) << 1);
				break;

				case 0x4 :
					tape->WriteByte(value, bvalue, TimeToSample(time, slowdown->GetMultiplier()), time);
				break;

				case 0x2 :
					scr->VidAddr.b.h = (scr->VidAddr.b.h & ~1) | (value >> 7);
					scr->VidAddr.b.l = (value&96) << 1;
				break;

				case 0x5 :
					if(value & ~15)
					{
						if(value&ULACLEAR_HTONE)
							IntStatus &= ~ULAIRQ_HTONE;

						if(value&ULACLEAR_RTC)
							IntStatus &= ~ULAIRQ_RTC;

						if(value&ULACLEAR_DISPLAY)
							IntStatus &= ~ULAIRQ_DISPLAY;

						CheckInterrupts();
					}

					//if(!(PagedRom&8)) value |= 8;

					if(((PagedRom&12) != 8) || (value&8))
					{
						PagedRom = value&15;
						DoPage();
					}
				break;

				case 0x6 : //for counter (tape or sound)
					speaker->SetFrequency(value, TimeToSample(time, slowdown->GetMultiplier()));
				break;

				case 0x7 :
					scr->ChangeGFXMode((value >> 3)&7, time);

					SelectWROM((value >> 3)&7);
					FixWROM();

					input->SetCaps((value&0x80) ? true : false);

					switch((value >> 1)&3)
					{
						case 0 : //switching to casette input
							tape->SetMode(true, true, (value&0x40) ? true : false, TimeToSample(time, slowdown->GetMultiplier()), time);
							speaker->SetMode(SM_OFF, TimeToSample(time, slowdown->GetMultiplier()));
						break;

						case 1 : //switching to sound generator
							tape->SetMode(false, false, (value&0x40) ? true : false, TimeToSample(time, slowdown->GetMultiplier()), time);
							speaker->SetMode(SM_ON, TimeToSample(time, slowdown->GetMultiplier()));
						break;

						case 2 : //switching to casette output
							tape->SetMode(true, false, (value&0x40) ? true : false, TimeToSample(time, slowdown->GetMultiplier()), time);
							speaker->SetMode(SM_OFF, TimeToSample(time, slowdown->GetMultiplier()));
						break;

						case 3 : //counter off
							tape->SetMode(false, false, (value&0x40) ? true : false, TimeToSample(time, slowdown->GetMultiplier()), time);
							speaker->SetMode(SM_OFF, TimeToSample(time, slowdown->GetMultiplier()));
						break;
					}
				break;

				case 0x8 :
				case 0x9 :
				case 0xa :
				case 0xb :
				case 0xc :
				case 0xd :
				case 0xe :
				case 0xf :
					scr->SetPalette(addr.b.l, value, time);
				break;
			}
		break;
	}

	return 0;
}

void C_ULA::SelectWROM(unsigned __int8 mode)
{
	switch(mode)
	{
		case 0 :
		case 3 :
		case 4 :
		case 6 :
			CWideOSRom = WideOSRoms[0];
		break;

		case 1 :
		case 5 :
		case 7 :
			CWideOSRom = WideOSRoms[1];
		break;

		case 2 :
			CWideOSRom = WideOSRoms[2];
		break;
	}
}

void C_ULA::FixWROM(void)
{
	/* avoids the device pages */

	ccpu->SetAddrPtrs(0xc0, 0xfc-0xc0, RomPtrs[ULAROM_OS], CWideOSRom, true, false);
	ccpu->SetAddrPtrs(0xff, 0x100 - 0xff, &RomPtrs[ULAROM_OS][0xff00 - 0xc000], &CWideOSRom[0xff00 - 0xc000], true, false);

/*	switch(mode)
	{
		case 0 :
		case 3 :
		case 4 :
		case 6 :
			ccpu->SetAddrPtrs(192, 0xfc, RomPtrs[ULAROM_OS], WideOSRoms[0], true, false);
		break;

		case 1 :
		case 5 :
		case 7 :
			ccpu->SetAddrPtrs(192, 0xfc, RomPtrs[ULAROM_OS], WideOSRoms[1], true, false);
		break;

		case 2 :
			ccpu->SetAddrPtrs(192, 0xfc, RomPtrs[ULAROM_OS], WideOSRoms[2], true, false);
		break;
	}*/
}

