/*

	ElectrEm (c) 2000 Thomas Harte - an Acorn Electron Emulator

	This is open software, distributed under the GPL 2, see 'Copying' for
	details

	wd1770/wd1770.cpp
	=================

	Horrid little 'quick fix' WD1770 emulation

*/
#include "wd1770.h"
#include "drive.h"
#include "../ula/ula.h"
#include "../6502/6502.h"

#include <stdlib.h>
#include <malloc.h>
#include <stdio.h>

void C_WD1770::Reset(void)
{
	regs[STATUS]	= regs[TRACK] = regs[SECTOR] = regs[DATA] = 0;
	regs[CONTROL]	= 1;
	instr			= NONE;
}

void C_WD1770::WriteToRegister(C_ULA *electron, int offset, unsigned __int8 value, int time)
{
//	fprintf(stderr, "WD1770 write to %d [%d]\n", offset, value);

	switch(offset)
	{
		case 0 :	//drive control - assumed to be as on a B+
			regs[CONTROL] = value;

			/* bit 1 - select second drive */
			/* bit 0 - select first drive */
			if(value&1)
				electron->SetCurrentDrive(0);
			if(value&2)
				electron->SetCurrentDrive(1);

			if(!electron->GetCurrentDrive()->HasFile())
				regs[CONTROL] = 0xff;
			else
			{
				/* bit 5 - reset drive controller chip */
				/* bit 3 - double density select */
				/* bit 2 - side select */
				if(electron->GetCurrentDrive()->FirstSide() != ((value&4) ? false : true))
					electron->GetCurrentDrive()->Flip();
			}
		break;

		case 4 :	//instruction register
			if(!electron->GetCurrentDrive()->HasFile())
			{
				regs[STATUS] = WD_R_FLAG;
			}
			else
				switch(instr)
				{
					default :
/*						if((value >> 4) == 13)
							fprintf(temp, "ignoring force interrupt\n");
						else
							fprintf(temp, "what the?\n");*/
					break;

					case NONE :
						switch(value >> 4)
						{
							case 0 : //restore		[type 1]
								track = regs[TRACK] = 0;
								regs[STATUS] = WD_M_FLAG | WD_S_FLAG;
							break;

							case 1 : //seek			[type 1]
								track = regs[TRACK] = regs[DATA];
								regs[STATUS] = WD_M_FLAG | WD_S_FLAG;
								regs[STATUS] |= track ? WD_Z_FLAG : 0;
							break;

							case 2 :
							case 3 : //step			[type 1]
							break;

							case 4 :
							case 5 : //step in		[type 1]
								track++;
								if(value&16)
									regs[TRACK] = track;
								regs[STATUS] = WD_M_FLAG | WD_S_FLAG | WD_Z_FLAG;
							break;

							case 6 :
							case 7 : //step out		[type 1]
								track--;
								if(value&16)
									regs[TRACK] = track;

								regs[STATUS] = WD_M_FLAG | WD_S_FLAG;
								regs[STATUS] |= track ? WD_Z_FLAG : 0;
							break;

							case 8 :
							case 9 : //read sector		[type 2]
								instr = (value&16) ? READ_SECTOR_TRACK : READ_SECTOR;
								regs[STATUS] = WD_M_FLAG | WD_D_FLAG | WD_B_FLAG;
								sector = regs[SECTOR];
								sector_position = 0;
							break;

							case 10 :
							case 11 : //write sector	[type 2]
								if(electron->GetCurrentDrive()->GetWriteProtect())
									regs[STATUS] = WD_W_FLAG;
								else
								{
									instr = (value&16) ? WRITE_SECTOR_TRACK : WRITE_SECTOR;
									regs[STATUS] = WD_M_FLAG | WD_D_FLAG | WD_B_FLAG;
									sector = regs[SECTOR];
									sector_position = 0;
								}
							break;

							case 12 : //read address	[type 3]
							break;

							case 14 : //read track		[type 3]
							break;

							case 15 : //write track		[type 3]
								if(electron->GetCurrentDrive()->GetWriteProtect())
									regs[STATUS] = WD_W_FLAG;
								else
								{
								}
							break;

							case 13 : //force interrupt	[type 4]
							break;
						}
					break;
				}
		break;

		case 5 :	//track registers
		case 6 :
			regs[offset-4] = value;
		break;

		case 7 :	//data & sector register
			switch(instr)
			{
				default :
					regs[offset-4] = value;
				break;

				case WRITE_SECTOR :
					unsigned char *cside;

					regs[offset-4] = value;

					electron->GetCurrentDrive()->SetTrack(track);

					if(cside = electron->GetCurrentDrive()->GetSectorPtr(sector, true))
					{
						cside[sector_position] = value;
						electron->GetCurrentDrive()->ReleaseSectorPtr(sector);
					}

					sector_position++;

					if(sector_position == 256)
					{
						instr = NONE;
						regs[STATUS] = WD_M_FLAG;
					}
				break;
			}
		break;
	}
}

unsigned __int8 C_WD1770::ReadFromRegister(C_ULA *electron, int offset, int time)
{
	unsigned __int8 ret;

//	fprintf(stderr, "WD1770 read from %d\n", offset);

	switch(offset)
	{
		default :
		break;

		case 0 :
		return regs[CONTROL];

		case 4 :
		return regs[STATUS];

		case 5 :
		case 6 :
		return regs[offset-4];

		case 7 :
			switch(instr)
			{
				default :
				case NONE :
				return regs[DATA];

				case READ_SECTOR :
					unsigned char *cside;

					electron->GetCurrentDrive()->SetTrack(track);

					if(cside = electron->GetCurrentDrive()->GetSectorPtr(sector, false))
					{
						ret = cside[sector_position];
						electron->GetCurrentDrive()->ReleaseSectorPtr(sector);
					}
					else
					{
						ret = 255;
					}

					regs[DATA] = ret;

					sector_position++;

					if(sector_position == 256)
					{
						instr = NONE;
						regs[STATUS] = WD_M_FLAG;
					}
				return ret;
			}
		break;
	}

	return 0;
}